Package: effects
Version: 4.2-5
Date: 2026-01-28
Title: Effect Displays for Linear, Generalized Linear, and Other Models
Authors@R: c(person("John", "Fox", role = "aut", email = "jfox@mcmaster.ca"),
	person("Sanford", "Weisberg", role = "aut", email = "sandy@umn.edu"),
		person("Brad", "Price", role = c("aut", "cre"), email = "brad.price@mail.wvu.edu"),
    person("Michael", "Friendly", role = "aut", email = "friendly@yorku.ca"),
    person("Jangman", "Hong", role = "aut"),
    person("Robert", "Andersen", role = "ctb"),
    person("David", "Firth", role = "ctb"),
    person("Steve", "Taylor", role = "ctb"),
    person("R Core Team", role="ctb"))
Depends: R (>= 3.5.0), carData, datasets
Suggests: pbkrtest (>= 0.4-4), nlme, MASS, poLCA, heplots, splines,
        ordinal, car, knitr, betareg, alr4, robustlmm
Imports: lme4, nnet, lattice, grid, colorspace, graphics, grDevices,
        stats, survey, utils, estimability (>= 1.4.1), insight
Description: Graphical and tabular effect displays, e.g., of interactions, for 
  various statistical models with linear predictors.
License: GPL (>= 2)
URL: https://cran.r-project.org/package=effects,
        https://www.john-fox.ca/,
        https://github.com/bprice2652/effects_repo
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-02-13 16:06:40 UTC; user
Author: John Fox [aut],
  Sanford Weisberg [aut],
  Brad Price [aut, cre],
  Michael Friendly [aut],
  Jangman Hong [aut],
  Robert Andersen [ctb],
  David Firth [ctb],
  Steve Taylor [ctb],
  R Core Team [ctb]
Maintainer: Brad Price <brad.price@mail.wvu.edu>
Repository: CRAN
Date/Publication: 2026-02-17 06:10:59 UTC

# last modified 2025-07-29 by J. Fox

import(carData)

importFrom(lattice, barchart, current.panel.limits, densityplot, larrows, llines, lpoints, ltext,
           panel.abline, panel.barchart, #panel.grid,
           panel.polygon, panel.text, strip.custom, strip.default, trellis.par.get,
           trellis.par.set, xyplot)
importFrom(colorspace, rainbow_hcl, sequential_hcl)
importFrom(grid, grid.pretty, grid.segments, unit)
importFrom(lme4, fixef)
importFrom(nnet, multinom)
importFrom(graphics, plot)
importFrom(grDevices, gray, palette, rgb)
importFrom(survey, svymean)
importFrom(insight, find_formula,get_parameters,get_varcov)
importFrom(stats, as.formula, binomial, coef, coefficients, cov, delete.response, family, fitted,
    formula, glm, glm.control, lm, lm.fit, loess.smooth, mahalanobis, make.link, median,
    model.frame, model.matrix, model.offset,
    model.response, na.exclude, na.omit, nlm, predict, qf, qnorm, qt, quantile, residuals,
    spline, terms, update, vcov, weights, xtabs)
importFrom(utils, menu, browseURL)

export(effect, allEffects, Effect, effectsTheme)
export(predictorEffect, predictorEffects)
export(effectsHexsticker)
export(effCoef)
export(levels2dates)
S3method(plot, predictoreff)
S3method(plot, predictorefflist)
S3method(predictorEffects, default)
S3method(predictorEffects, poLCA)
S3method(predictorEffect, default)
S3method(predictorEffect, svyglm)
S3method(predictorEffect, poLCA)
S3method(effCoef, default)

S3method(Effect, default)
S3method(Effect, lm)
S3method(Effect, poLCA)
S3method(Effect, mlm)
S3method(Effect, svyglm)
S3method(Effect, multinom)
S3method(Effect, polr)
S3method(Effect, merMod)

export(predictorEffect.default,predictorEffects.default, 
       Effect.default, effect.default, allEffects.default)
S3method(effSources,betareg)
S3method(effSources,clm)
S3method(effSources,clm2) 
S3method(effSources,clmm)
S3method(effSources,default) 
S3method(effSources,glmmPQL)
S3method(effSources,gls)
S3method(effSources,rlmerMod)
export(effSources,effSources.default)
#,effSources.gls,effSources.glmmPQL,
#       effSources.rlmerMod,effSources.clm,effSources.clm2,
#       effSources.clmm,effSources.betareg)

S3method(print, eff)
S3method(print, efflist)
S3method(print, mlm.efflist)
S3method(print, summary.eff)
S3method(print, predictoreff)
S3method(print, predictorefflist)
S3method(summary, eff)
S3method(summary, efflist)
S3method(summary, mlm.efflist)
S3method(summary, predictorefflist)
S3method(as.data.frame, eff)
S3method(as.data.frame, efflist)
S3method(as.data.frame, effpoly)
S3method(as.data.frame, efflatent)
S3method(plot, eff)
S3method(print, plot.eff)
S3method(plot, efflist)
S3method(plot, mlm.efflist)
S3method(print, effpoly)
S3method(summary, effpoly)
S3method(plot, effpoly)
S3method(print, efflatent)
S3method(summary, efflatent)
S3method(allEffects, default)
S3method(allEffects, poLCA)
S3method(allEffects, mlm)
S3method(effect, default)
S3method(vcov, eff)
S3method(`[`, efflist)
S3method(levels2dates, effpoly)
S3method(levels2dates, eff)

## ----include=FALSE--------------------------------------------------------
library(knitr)
opts_chunk$set(
tidy=FALSE,fig.width=5,fig.height=5,cache=FALSE
)

## ----echo=FALSE, results='hide', include=FALSE----------------------------
#options(continue="+    ", prompt="R> ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)

## -------------------------------------------------------------------------
mvrunif <- function(n, R, min = 0, max = 1){
    # method (but not code) from E. Schumann,
    # "Generating Correlated Uniform Variates"
    # URL:
    # <http://comisef.wikidot.com/tutorial:correlateduniformvariates>
    # downloaded 2015-05-21
    if (!is.matrix(R) || nrow(R) != ncol(R) ||
    max(abs(R - t(R))) > sqrt(.Machine$double.eps))
    stop("R must be a square symmetric matrix")
    if (any(eigen(R, only.values = TRUE)$values <= 0))
    stop("R must be positive-definite")
    if (any(abs(R) - 1 > sqrt(.Machine$double.eps)))
    stop("R must be a correlation matrix")
    m <- nrow(R)
    R <- 2 * sin(pi * R / 6)
    X <- matrix(rnorm(n * m), n, m)
    X <- X %*% chol(R)
    X <- pnorm(X)
    min + X * (max - min)
}

gendata <- function(n = 5000, R, min = -2, max = 2, s = 1.5,
    model = expression(x1 + x2 + x3)){
    data <- mvrunif(n = n, min = min, max = max, R = R)
    colnames(data) <- c("x1", "x2", "x3")
    data <- as.data.frame(data)
    data$error <- s * rnorm(n)
    data$y <- with(data, eval(model) + error)
    data
}

R <- function(offdiag = 0, m = 3){
    R <- diag(1, m)
    R[lower.tri(R)] <- R[upper.tri(R)] <- offdiag
    R
}

## -------------------------------------------------------------------------
set.seed(682626)
Data.1 <- gendata(R = R(0), model = expression(x1 + x2 * x3))
round(cor(Data.1), 2)
summary(mod.1 <- lm(y ~ x1 + x2 + x3, data = Data.1))

## ----fig-contrived-1a,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
library(effects)
plot(predictorEffects(mod.1, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     rows=1, cols=3)

## ----fig-contrived-1b,include=TRUE, fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x2", "x3"), mod.1, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)))

## ----fig-contrived-1c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x1", "x2"), mod.1, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80"),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))

## -------------------------------------------------------------------------
set.seed(682626)
Data.2 <- gendata(R = R(0.5), model = expression(x1 + x2 * x3))
mod.2 <- lm(y ~ x1 + x2 + x3, data = Data.2)

## ----fig-contrived-2a,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(predictorEffects(mod.2, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80",fig.show='hide'),
     axes=list(x=list(rotate=45)),
     rows=1, cols=3)

## ----fig-contrived-2b,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x2", "x3"), mod.2, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)))

## ----fig-contrived-2c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x1", "x2"), mod.2, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80",fig.show='hide'),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))

## -------------------------------------------------------------------------
set.seed(682626)
Data.3 <- gendata(R = R(0.5), model = expression(x1^2 + x2 + x3))
mod.3 <- lm(y ~ x1 + x2 + x3, data = Data.3)

## ----fig-contrived-3a,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(predictorEffects(mod.3, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     rows=1, cols=3)

## ----fig-contrived-3b,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x2", "x3"), mod.3, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)))

## ----fig-contrived-3c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x1", "x2"), mod.3, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80"),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))

## -------------------------------------------------------------------------
set.seed(682626)
Data.4 <- gendata(R = R(0.5), model = expression(x1^2 + x2 * x3))
mod.4 <- lm(y ~ x1 + x2 + x3, data = Data.4)

## ----fig-contrived-4a,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(predictorEffects(mod.4, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     rows=1, cols=3)

## ----fig-contrived-4b,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x2", "x3"), mod.4, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
      axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)))

## ----fig-contrived-4c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x1", "x2"), mod.4, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80"),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))

## ----fig-contrived-5a,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'----
mod.5 <- lm(y ~ poly(x1, 2) + x2*x3, data=Data.4)
plot(Effect("x1", mod.5, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80", span=0.2))

## ----fig-contrived-5b,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x2", "x3"), mod.5, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)), span=0.5)

## ----fig-contrived-5c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'----
plot(Effect(c("x1", "x2"), mod.5, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80", span=0.35),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))

## ----setopts,echo=FALSE-------------------------------------------------------
library("knitr")
opts_chunk$set(fig.width=5,fig.height=5,tidy=TRUE,
               out.width="0.8\\textwidth",echo=TRUE)
options(prompt=" ")

## ----echo=FALSE, results='hide', include=FALSE--------------------------------
#options(continue="+    ", prompt="R> ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)
library(effects)

## ----include=FALSE------------------------------------------------------------
library(knitr)
opts_chunk$set(
tidy=FALSE,fig.width=5,fig.height=5,cache=FALSE,comment=NA, prompt=TRUE
)
render_sweave()

## ----echo=FALSE, results='hide', include=FALSE----------------------------
options(continue="    ", prompt=" ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)

## ----fig.height=4,fig.width=8---------------------------------------------
library("effects")
Prestige$type <- factor(Prestige$type, c("bc", "wc", "prof")) # reorder levels
g1 <- lm(prestige ~ education + type + education:type, data = Prestige)
  # equivalent to lm(prestige ~ education*type, data = Prestige)
plot(predictorEffects(g1), lines=list(multiline=TRUE))

## -------------------------------------------------------------------------
data(Orthodont, package="nlme")
g2 <- lme4::lmer(distance ~ age + Sex + (1 | Subject), data = Orthodont)
summary(g2)

## ----fig.height=4,fig.width=8---------------------------------------------
plot(predictorEffects(g2))

## ----setopts,echo=FALSE---------------------------------------------
library("knitr")
opts_chunk$set(fig.width=5,fig.height=5,#tidy=TRUE,
               out.width="0.8\\textwidth",echo=TRUE)
#options(prompt=" ")
options(continue="+    ", prompt="R> ", width=70)
options(show.signif.stars=FALSE, scipen=3)

## ----setup, include=FALSE, cache=FALSE, results='hide', echo=FALSE------------
library(car)
library(effects)
render_sweave()
options(width=80, digits=5, str=list(strict.width="cut"))
strOptions(strict.width="cut")

## -----------------------------------------------------------------------------
library("car") # also loads the carData package
Prestige$type <- factor(Prestige$type, levels=c("bc", "wc", "prof"))
lm1 <- lm(prestige ~ education + poly(women, 2) +
                     log(income)*type, data=Prestige)

## -----------------------------------------------------------------------------
S(lm1)

## ----fig11,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'--------------
library("effects")
e1.lm1 <- predictorEffect("education", lm1)
plot(e1.lm1)

## -----------------------------------------------------------------------------
brief(e1.lm1$model.matrix)

## -----------------------------------------------------------------------------
e1a.lm1 <- predictorEffect("education", lm1, focal.levels=5)
e1a.lm1
summary(e1a.lm1)
as.data.frame(e1a.lm1)

## -----------------------------------------------------------------------------
e2.lm1 <- predictorEffect("income", lm1, focal.levels=5)
as.data.frame(e2.lm1)

## ----fig12,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'--------------
plot(predictorEffect("income", lm1), 
     lines=list(multiline=TRUE))

## ----fig13,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'--------------
plot(predictorEffect("type", lm1), lines=list(multiline=TRUE))

## ----fig14,include=TRUE,fig.width=7,fig.height=8,fig.show='hide'--------------
eall.lm1 <- predictorEffects(lm1)
plot(eall.lm1)

## ----eval=FALSE---------------------------------------------------------------
# plot(eall.lm1)
# plot(predictorEffects(lm1))
# plot(predictorEffects(lm1, ~ income + education + women + type))

## ----eval=FALSE---------------------------------------------------------------
# plot(predictorEffects(lm1, ~ type + education))

## ----eval=FALSE---------------------------------------------------------------
# plot(predictorEffects(lm1, ~ women))
# plot(predictorEffects(lm1)[[2]])
# plot(predictorEffect("women", lm1))

## ----fig21a,include=TRUE,fig.width=5,fig.height=4.5,fig.show='hide'-----------
e3.lm1 <- predictorEffect("type", lm1)
plot(e3.lm1, lines=list(multiline=TRUE))

## ----fig21b,include=TRUE,fig.width=6,fig.height=5,fig.show='hide'-------------
plot(e3.lm1, lines=list(multiline=FALSE)) # the default

## ----fig22a,include=TRUE,fig.width=5,fig.height=4.5,fig.show='hide'-----------
e3.lm1 <- predictorEffect("type", lm1,
                          xlevels=list(income=c(5000, 15000, 25000)))
plot(e3.lm1, lines=list(multiline=TRUE),
     confint=list(style="bars"))

## ----fig22b,include=TRUE,fig.width=5.5,fig.height=5,fig.show='hide'-----------
plot(e3.lm1,
     lines=list(multiline=FALSE), # the default
     lattice=list(layout=c(3, 1)))

## ----fig23,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'--------------
e4.lm1 <- predictorEffect("education", lm1,
                          se=list(type="scheffe", level=.99), 
                          vcov.=hccm)
plot(e4.lm1)

## -----------------------------------------------------------------------------
lm2 <- lm(log(prestige) ~ log(income) + education + type, Prestige)

## ----fig30,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'--------------
plot(predictorEffects(lm2, ~ income))

## ----fig31,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'--------------
plot(predictorEffects(lm2, ~ income),
 axes=list(
   x=list(income=list(transform=list(trans=log, inverse=exp)))
   ))

## ----fig32,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'--------------
plot(predictorEffects(lm2, ~ income),
 main="Transformed Plot",
 axes=list(
    grid=TRUE,
    x=list(rotate=30,
           rug=FALSE,
           income=list(transform=list(trans=log, inverse=exp),
                       lab="income, log-scale",
                       ticks=list(at=c(2000, 5000, 10000, 20000)),
                       lim=c(1900, 21000))
    )))

## ----figdates1,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'----------
data("airquality", package="datasets")
airquality$Date <- with(airquality, as.Date(paste("1973", Month, Day, sep="-"), 
    format="%Y-%m-%d"))
airquality$Date.num <- as.numeric(airquality$Date)
m1.date <- lm(Ozone ~ Date.num + Solar.R + Wind + Temp, data=airquality)
eff.date.1 <- Effect("Date.num", m1.date)
plot(eff.date.1, axes=list(x=list(Date.num=list(lab="Date", 
    ticks=list(at=levels2dates(eff.date.1, "Date.num", "1970-01-01"))), 
    rotate=45)), main="Date Effect")

## ----figdates2,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'----------
plot(eff.date.1, axes=list(x=list(Date.num=list(lab="Date", 
    ticks=list(at=levels2dates(eff.date.1, "Date.num", "1970-01-01", n=4))))), 
    main="Date Effect")

## -----------------------------------------------------------------------------
eff.date.df <- as.data.frame(eff.date.1)
eff.date.df$Date <- as.Date(eff.date.df$Date.num, origin="1970-01-01")
eff.date.df

## ----figdates3,include=TRUE,fig.width=7.5,fig.height=7.5,fig.show='hide'------
m2.date <- lm(Ozone ~ Date.num*Temp + Solar.R + Wind, data=airquality)
eff.date.2 <- Effect(c("Date.num", "Temp"), m2.date, xlevels=6)
plot(eff.date.2, axes=list(x=list(Date.num=list(lab="Date", 
  ticks=list(at=levels2dates(eff.date.2, "Date.num", "1970-01-01", n=3))), 
  rotate=45)), main="Date Effect by Temperature")

## ----fig33,include=TRUE,fig.width=4,fig.height=4,fig.show='hide'--------------
# default:
plot(predictorEffects(lm2, ~ education),
     main="Default log(prestige)")
# Change only tick-mark labels to arithmetic scale:
plot(predictorEffects(lm2, ~ education),
     main="log(prestige), Arithmetic Ticks",
     axes=list(y=list(transform=list(trans=log, inverse=exp),
                      lab="prestige", type="rescale")))
# Replace log(presige) by prestige:
plot(predictorEffects(lm2, ~ education),
     main="Prestige in Arithmethic Scale",
     axes=list(y=list(transform=exp, lab="prestige")))

## -----------------------------------------------------------------------------
library("lme4") # for lmer()
Blackmore$tran.exercise <- bcnPower(Blackmore$exercise, 
                                    lambda=0.25, gamma=0.1)
mm1 <- lmer(tran.exercise ~ I(age - 8)*group +
              (I(age - 8) | subject), data=Blackmore)

## ----fig33a,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'-------------
e1.mm1 <- predictorEffect("age", mm1)
plot(e1.mm1, lines=list(multiline=TRUE), confint=list(style="auto"))

## ----fig33b,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'-------------
f.trans <- function(x) bcnPower(x, lambda=0.25, gamma=0.1)
f.inverse <- function(x) bcnPowerInverse(x, lambda=0.25, gamma=0.1)
plot(e1.mm1, lines=list(multiline=TRUE),
     confint=list(style="auto"),
     axes=list(x=list(age=list(lab="Age (years)")),
               y=list(transform=list(trans=f.trans, inverse=f.inverse),
                      type="response",
                      lab="Exercise (hours/week)")),
     lattice=list(key.args=list(x=.20, y=.75, corner=c(0, 0), 
                                padding.text=1.25)),
     main=""
)

## -----------------------------------------------------------------------------
data("Blowdown", package="alr4")
gm1 <- glm(y ~ log(d) + s + spp, family=binomial, data=Blowdown)

## ----fig34,include=TRUE,fig.width=6.5,fig.height=6.5,fig.show='hide'----------
plot(predictorEffects(gm1),
     axes=list(grid=TRUE, x=list(rug=FALSE, rotate=35)))

## ----fig35,include=TRUE,fig.width=3.5,fig.height=3.5,fig.show='hide'----------
e1.gm1 <- predictorEffect("spp", gm1)
plot(e1.gm1, main="type='rescale'",
     axes=list(y=list(type="rescale",
                      lab="logit scale, probability labels"),
               x=list(rotate=30),
               grid=TRUE))
plot(e1.gm1, main="type='link'",
     axes=list(y=list(type="link",
                      lab="logit scale, logit labels"),
               x=list(rotate=30),
               grid=TRUE))
plot(e1.gm1, main="type='response'",
     axes=list(y=list(type="response", grid=TRUE,
                      lab="probabilty scale, probability labels"),
               x=list(rotate=30),
               grid=TRUE))

## ----fig36,include=TRUE,fig.width=5.5,fig.height=4.5,fig.show='hide'----------
or <- order(as.data.frame(e1.gm1)$fit) # order smallest to largest
Blowdown$spp1 <- factor(Blowdown$spp,  # reorder levels of spp
                        levels=levels(Blowdown$spp)[or])
gm2 <- update(gm1, ~ . - spp + spp1)   # refit model
plot(predictorEffects(gm2, ~ spp1), main="type='response', ordered",
     axes=list(y=list(type="response",
                      lab="probabilty scale, probability labels"),
               x=list(rotate=30, spp=list(lab="Species")),
               grid=TRUE))

## ----fig37,include=TRUE,fig.width=9,fig.height=12,fig.show='hide'-------------
gm3 <- update(gm2, ~ . + s:log(d)) # add an interaction
plot(predictorEffects(gm3, ~ s + d),
     axes=list(x=list(rug=FALSE, rotate=90),
               y=list(type="response", lab="Blowdown Probability")),
     lattice=list(layout=c(1, 5)))

## ----fig38,include=TRUE,fig.width=9,fig.height=5,fig.show='hide'--------------
plot(predictorEffects(gm3, ~ s + d, 
                      xlevels=list(d=c(5, 40, 80), s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               x=list(rug=FALSE),
               y=list(type="response", lab="Blowdown probability")),
     lines=list(multiline=TRUE))

## ----fig39,include=TRUE,fig.width=7,fig.height=7,fig.show='hide'--------------
gm4 <- update(gm3, ~ . + spp:log(d))
plot(predictorEffects(gm4, ~ d, xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               y=list(type="response"),
               x=list(rug=FALSE)),
     lines=list(multiline=TRUE))

## ----fig310,include=TRUE,fig.width=7,fig.height=5,fig.show='hide'-------------
plot(predictorEffects(gm4, ~ d, xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE, 
               y=list(type="response"), 
               x=list(rug=FALSE)),
     lines=list(multiline=TRUE, z.var="spp", lty=1:9),
     lattice=list(layout=c(3, 1)))

## ----fig311,include=TRUE,fig.width=5.5,fig.height=5.5,fig.show='hide'---------
plot(predictorEffects(gm3, ~ d, 
                      xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               x=list(rug=FALSE),
               y=list(type="response")),
     lines=list(multiline=TRUE),
     confint=list(style="auto"))

## ----fig312,include=TRUE,fig.width=7,fig.height=6,fig.show='hide'-------------
gm5 <- update(gm2, ~ . + spp:s)
plot(predictorEffects(gm5, ~ spp, xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               y=list(type="response"),
               x=list(rug=FALSE, rotate=30)),
     lines=list(multiline=TRUE),
     confint=list(style="auto"))

## ----fig314,include=TRUE,fig.width=8,fig.height=6,fig.show='hide'-------------
plot(predictorEffects(gm5, ~ spp, xlevels=list(s=c(0.1, 0.5, 0.9))),
     rug=FALSE,
     axes=list(grid=TRUE,
               y=list(type="response"),
               x=list(rotate=30)),
     lines=list(multiline=TRUE),
     confint=list(style="auto"),
     lattice=list(key.args=list(space="right",
                                columns=1,
                                border=TRUE,
                                fontfamily="serif",
                                cex=1.25,
                                cex.title=1.5)))

## ----fig313,include=TRUE,fig.width=13,fig.height=5.5,fig.show='hide'----------
plot(predictorEffects(gm3, ~ s + d, xlevels=list(s=6, d=6)),
     axes=list(x=list(rug=FALSE, rotate=90), 
               y=list(ticks=list(at=c(.999, .99, .95, .8, .5, .2, .05)))),
     lattice=list(layout=c(3, 2)))

## ----fig313b,include=TRUE,fig.width=6,fig.height=10,fig.show='hide'-----------
plot(predictorEffect("s", gm3, xlevels=list(d=6)),
     axes=list(x=list(rug=FALSE, rotate=90),  
               y=list(ticks=list(at=c(.999, .99, .95, .8, .5, .2, .05)))),
     lattice=list(layout=c(3, 2),
                  array=list(row=1, col=1, nrow=2, ncol=1, more=TRUE)))
plot(predictorEffect("d", gm3, xlevels=list(s=6)),
     axes=list(x=list(rug=FALSE, rotate=90),  
               y=list(ticks=list(at=c(.999, .99, .95, .8, .5, .2, .05)))),
     lattice=list(layout=c(3, 2),
                  array=list(row=2, col=1, nrow=2, ncol=1, more=FALSE)))

## ----fig316,include=TRUE,fig.width=7,fig.height=5,fig.show='hide'-------------
plot(predictorEffects(gm4, ~ d, xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               x=list(rug=FALSE),
               y=list(type="response")),
     lines=list(multiline=TRUE, z.var="spp", lty=1:9),
     lattice=list(layout=c(3, 1),
                  strip=list(factor.names=TRUE,
                             values=TRUE,
                             cex=1.5)))

## ----fig315,include=TRUE,fig.width=7,fig.height=6,fig.show='hide'-------------
gm5 <- update(gm2, ~ . + spp:s)
plot(predictorEffects(gm5, ~ spp, xlevels=list(s=c(0.1, 0.5, 0.9))),
     symbols=list(pch=15:17, cex=1.5),
     axes=list(grid=TRUE,
               y=list(type="response"),
               x=list(rotate=30)),
     lines=list(multiline=TRUE),
     confint=list(style="auto"),
     lattice=list(key.args=list(cex=1.5, cex.title=1.5)))

## ----fig51,include=TRUE,fig.width=10,fig.height=9,fig.show='hide'-------------
lm5 <- lm(prestige ~ log(income) + education + women + type,
          Prestige)
plot(predictorEffects(lm5, residuals=TRUE),
     axes=list(grid=TRUE,
               x=list(rotate=30)),
     partial.residuals=list(smooth=TRUE,
                            span=0.75,
                            lty="dashed"))

## ----fig52,include=TRUE,fig.width=10,fig.height=5,fig.show='hide'-------------
options(scipen=10) # suppress scientific notation 
lm6 <- lm(infantMortality ~ group*ppgdp, data=UN)
plot(predictorEffects(lm6, ~ ppgdp, partial.residuals=TRUE),
     axes=list(x=list(rotate=25), 
               y=list(lim=c(0, 150))),
    id=list(n=1),
    lattice=list(layout=c(3, 1)))

## ----fig53,include=TRUE,fig.width=10,fig.height=5,fig.show='hide'-------------
lm7 <- lm(log(infantMortality) ~ group*log(ppgdp), data=UN)
plot(predictorEffects(lm7, ~ ppgdp, partial.residuals=TRUE),
     axes=list(x=list(rotate=25)),
     id=list(n=1),
     lattice=list(layout=c(3, 1)))

## ----fig54,include=TRUE,fig.width=10,fig.height=5,fig.show='hide'-------------
plot(predictorEffects(lm7, ~ ppgdp, partial.residuals=TRUE),
     axes=list(x=list(rotate=25),
               y=list(transform=list(trans=log, inverse=exp),
                      type="response",
                      lab="Infant Mortality")),
     id=list(n=1),
     lattice=list(layout=c(3, 1)))

## -----------------------------------------------------------------------------
S(lm2)

## ----fig55,include=TRUE,fig.width=8,fig.height=4,fig.show='hide'--------------
plot(Effect(c("income", "type"), lm2, residuals=TRUE),
     axes=list(x=list(rotate=30)),
     partial.residuals=list(span=0.9), 
     layout=c(3, 1))

## -----------------------------------------------------------------------------
library("MASS") # for polr()
Womenlf$partic <- factor(Womenlf$partic,
    levels=c("not.work", "parttime", "fulltime")) # order response levels
or1 <- polr(partic ~ log(hincome) + children, data=Womenlf)
S(or1)

## ----fig41,include=TRUE,fig.width=6.5,fig.height=6.5,fig.show='hide'----------
plot(predictorEffects(or1),
     axes=list(grid=TRUE),
     lattice=list(key.args=list(columns=1)))

## ----fig62,include=TRUE,fig.width=6,fig.height=4,fig.show='hide'--------------
plot(predictorEffects(or1),
     axes=list(grid=TRUE, y=list(style="stacked")),
     lattice=list(key.args=list(columns=1)))

## -----------------------------------------------------------------------------
library("nnet") # for multinom()
mr1 <- multinom(vote ~ age + gender + economic.cond.national +
                       economic.cond.household + Blair + Hague + Kennedy +
                       Europe*political.knowledge, data=BEPS)

## ----fig42,include=TRUE,fig.width=6.5,fig.height=6.5,fig.show='hide'----------
plot(predictorEffects(mr1, ~ age + Blair + Hague + Kennedy),
     axes=list(grid=TRUE, x=list(rug=FALSE)),
     lattice=list(key.args=list(columns=1)),
     lines=list(multiline=TRUE, col=c("blue", "red", "orange")))

## ----fig43,include=TRUE,fig.width=10,fig.height=5,fig.show='hide'-------------
plot(predictorEffects(mr1, ~ Europe + political.knowledge,
                      xlevels=list(political.knowledge=0:3,
                                   Europe=c(1, 6, 11))),
     axes=list(grid=TRUE, 
               x=list(rug=FALSE, 
                      Europe=list(ticks=list(at=c(1, 6, 11))),
                      political.knowledge=list(ticks=list(at=0:3))),
               y=list(style="stacked")),
     lines=list(col=c("blue", "red", "orange")),
     lattice=list(key.args=list(columns=1),
                  strip=list(factor.names=FALSE)))

## ----eval=FALSE---------------------------------------------------------------
# effectsTheme()

# 2014-12-11 Effects plots for ordinal and ordinal mixed models from the 'ordinal' package
# 2014-12-11 effect.clm built from effect.mer as modified 2014-12-07,  by S. Weisberg
# 2015-06-10: requireNamespace("MASS") rather than require("MASS")
# 2016-02-12: added support for clmm and clm objects from 'ordinal' S. Weisberg
# 2016-08-16: added ... argument to effect() and Effect() methods. J. Fox
# 2017-03-28: fixed bug to allow data=m S. Weisberg

###
###  clm2
###
clm2.to.polr <- function(mod) {
  if (requireNamespace("MASS", quietly=TRUE)){
    polr <- MASS::polr
  }
  else stop("The MASS package is needed for this function")
  cl <- mod$call
  present <- match(c("scale", "nominal", "link", "threshold"), names(cl), 0L)
  if(any(present != 0)) {
    if(present[3] != 0){if(cl$link != "logit") stop("'link' must be 'logisitic' for use with effects")}
    if(present[4] != 0){if(cl$threshold != "flexible") stop("'threshold' must be 'flexible' for use with effects")}
    if(present[1] != 0){if(!is.null(cl$scale)) stop("'scale' must be NULL for use with effects")}
    if(present[2] != 0){if(!is.null(cl$nominal)) stop("'nominal' must be NULL for use with effects")}
  }
  if(is.null(mod$Hessian)){
    message("\nRe-fitting to get Hessian\n")
    mod <- update(mod, Hess=TRUE)
  }
  cl$formula <- cl$location
  cl$method <- cl$link
  .m <- match(c("formula", "data", "subset","weights",
                "na.action",  "contrasts", "method"), names(cl), 0L)
  cl <- cl[c(1L, .m)]
  cl$start <- c(mod$beta, mod$Theta)
  cl[[1L]] <- as.name("polr")
  mod2 <- eval(cl)
  mod2$coefficients <- mod$beta
  # get vcov
  numTheta <- length(mod$Theta)
  numBeta <- length(mod$beta)
  or <- c( (numTheta+1):(numTheta + numBeta), 1:(numTheta))
  mod2$vcov <- as.matrix(vcov(mod)[or, or])
  class(mod2) <- c("fakeclm2", class(mod2))
  mod2
}

#method for 'fakeglm' objects. Do not export
vcov.fakeclm2 <- function(object, ...) object$vcov

#The next three functions should be exported

effect.clm2 <- function(term, mod, ...) {
  effect(term, clm.to.polr(mod), ...)
}

allEffects.clm2 <- function(mod, ...){
  allEffects(clm.to.polr(mod), ...)
}

Effect.clm2 <- function(focal.predictors, mod, ...){
  Effect(focal.predictors, clm.to.polr(mod), ...)
}

###
###   clmm
###
clmm.to.polr <- function(mod) {
  if (requireNamespace("MASS", quietly=TRUE)){
    polr <- MASS::polr
  }
  else stop("The MASS package is needed for this function")
  cl <- mod$call
  present <- match(c("scale", "nominal", "link", "threshold"), names(cl), 0L)
  if(any(present != 0)) {
    if(present[3] != 0){if(cl$link != "logit") stop("'link' must be 'logit' for use with effects")}
    if(present[4] != 0){if(cl$threshold != "flexible") stop("'threshold' must be 'flexible' for use with effects")}
    if(present[1] != 0){if(!is.null(cl$scale)) stop("'scale' must be NULL for use with effects")}
    if(present[2] != 0){if(!is.null(cl$nominal)) stop("'nominal' must be NULL for use with effects")}
  }
  if(is.null(mod$Hessian)){
    message("\nRe-fitting to get Hessian\n")
    mod <- update(mod, Hess=TRUE)
  }
  cl$formula <- fixmod(mod$formula)  # changed for clm2
  cl$method <- cl$link
  .m <- match(c("formula", "data", "subset","weights",
                "na.action",  "contrasts", "method"), names(cl), 0L)
  cl <- cl[c(1L, .m)]
  cl$start <- c(mod$beta, mod$Theta)
  cl[[1L]] <- as.name("polr")
  mod2 <- eval(cl)
  mod2$coefficients <- mod$beta
  # get vcov
  numTheta <- length(mod$Theta)
  numBeta <- length(mod$beta)
  or <- c( (numTheta+1):(numTheta + numBeta), 1:(numTheta))
  mod2$vcov <- as.matrix(vcov(mod)[or, or])
  class(mod2) <- c("fakeclmm", class(mod2))
  mod2
}

#method for 'fakeglm' objects. Do not export
vcov.fakeclmm <- function(object, ...) object$vcov

#The next three functions should be exported

effect.clmm <- function(term, mod, ...) {
  effect(term, clmm.to.polr(mod), ...)
}

allEffects.clmm <- function(mod, ...){
  allEffects(clmm.to.polr(mod), ...)
}

Effect.clmm <- function(focal.predictors, mod, ...){
  Effect(focal.predictors, clmm.to.polr(mod), ...)
}


###
### clm
###
clm.to.polr <- function(mod) {
  if (requireNamespace("MASS", quietly=TRUE)){
    polr <- MASS::polr
  }
  else stop("The MASS package is needed for this function")
  cl <- mod$call
  present <- match(c("scale", "nominal", "link", "threshold"), names(cl), 0L)
  if(any(present != 0)) {
    if(present[3] != 0){if(cl$link != "logit") stop("'link' must be 'logit' for use with effects")}
    if(present[4] != 0){if(cl$threshold != "flexible") stop("'threshold' must be 'flexible' for use with effects")}
    if(present[1] != 0){if(!is.null(cl$scale)) stop("'scale' must be NULL for use with effects")}
    if(present[2] != 0){if(!is.null(cl$nominal)) stop("'nominal' must be NULL for use with effects")}
  }
  if(is.null(mod$Hessian)){
    message("\nRe-fitting to get Hessian\n")
    mod <- update(mod, Hess=TRUE)
  }
  # cl$formula <- cl$location
  cl$method <- cl$link
  .m <- match(c("formula", "data", "subset","weights",
                "na.action",  "contrasts", "method"), names(cl), 0L)
  cl <- cl[c(1L, .m)]
  cl$start <- c(mod$beta, mod$Theta)
  cl[[1L]] <- as.name("polr")
  mod2 <- eval(cl)
  mod2$coefficients <- mod$beta
  # get vcov
  numTheta <- length(mod$Theta)
  numBeta <- length(mod$beta)
  or <- c( (numTheta+1):(numTheta + numBeta), 1:(numTheta))
  mod2$vcov <- as.matrix(vcov(mod)[or, or])
  class(mod2) <- c("fakeclm", class(mod2))
  mod2
}

#method for 'fakeglm' objects. Do not export
vcov.fakeclm <- function(object, ...) object$vcov

#The next three functions should be exported

effect.clm <- function(term, mod, ...) {
  effect(term, clm.to.polr(mod), ...)
}

allEffects.clm <- function(mod, ...){
  allEffects(clm.to.polr(mod), ...)
}

Effect.clm <- function(focal.predictors, mod, ...){
  Effect(focal.predictors, clm.to.polr(mod), ...)
}

# Effect generic and methods
# John Fox and Sanford Weisberg
# 2012-12-21: Allow for empty cells in factor interactions, S. Weisberg
# 2012-03-05: Added .merMod method for development version of lme4, J. Fox
# 2012-04-06: Added support for lme4.0, J. Fox
# 2013-07-15:  Changed default xlevels and default.levels
# 2013-10-15: Added Effect.default(). J. Fox
# 2013-10-22: fixed bug in Effect.lm() when na.action=na.exclude. J. Fox
# 2013-10-29: code to handle "valid" NAs in factors. J. Fox
# 2013-11-06: fixed bug in Effect.multinom() in construction of effect object
# 2014-03-13: modified Effect.lm() to compute partial residuals. J. Fox
# 2014-05-06: fixed bug in Effect.gls() when cor or var structure depends on variables in the data set. J. Fox
# 2014-08-02: added vcov.=vcov argument to allow other methods of estimating var(coef.estimates)
# 2014-09-25: added KR argument to Effect.mer() and Effect.merMod(). J. Fox
# 2014-12-07: don't assume that pbkrtest is installed. J. Fox
# 2015-03-25: added "family" element to eff objects returned by Effect.lm(). J. Fox
# 2016-02-16: fixed problem in handling terms like polynomials for non-focal predictors. J. Fox
# 2016-03-01: recoded calculation of partial residuals. J. Fox
# 2016-07-19: added checkFormula(). J. Fox
# 2017-08-18: removed default.levels argument. J. Fox
# 2017-08-26: introduced confint list argument, including Scheffe intervals. J. Fox
# 2017-08-29: reintroduce legacy se and confidence.level arguments.
# 2017-09-07: added Effect.svyglm()
# 2017-09-14: no partial residuals for Effect.svyglm()
# 2017-11-03: correct handling of rank deficient models, now using `estimability` package
# 2017-11-22: modified checkFormula to work with clm2 models that don't have a 'formula' argument
# 2017-12-10: Effect.default. Effect.mer, .merMod, .lme, gls have been replaced to use the default.
# 2018-01-22: allow given.values="equal" or given.values="default" 
# 2018-01-25: substitute se for confint arg; make confint a legacy arg
# 2018-05-06: allow for complete=FALSE arg in potential calls to vcov.lm() and vcov.glm.
# 2018-05-13: allow partial residuals to be computed when the x.var is a factor.
# 2018-06-05: Effect.default now makes sure family$aic is 
#             set, for use with non-standard families.
# 2018-06-05: A test has been added to Effect.default to chech if family$variance
#             has one parameter.  If not, the function is stopped and an error is
#             returned.
# 2018-06-12: Fixed bug with vcov in Effect.default
# 2018-06-20: Added a check to Effect.default to handle family args that
#             are character or an unevaluated function
# 2018-10-01: Avoid warnings when testing given.values == "equal" or "default".
# 2018-10-08: transformation argument changed to legacy
# 2018-10-08: new returned value 'link' = family(mod) 
# 2019-04-20: made Effect.default() more robust in fitting fake glm by setting epsilon=Inf.
# 2019-04-20: fixed bug in .set.given.equal() in tests for model class.
# 2019-07-05: clm, clm2 and clmm were not passing threshholds to the fake polr object, now corrected.
# 2019-09-04: handle xlevels=n argument correctly
# 2020-05-22: Removed fixFormula function.  
# 2020-05-27: Added effCoef generic that uses the 'insight' package to find the formula, coef estimates and vcov for methods supported by insight
# 2020-06-23: Added effSources to gather sources for new regression methods.  
#             Old mechanism of using Effect.method will still work
# 2020-12-02: Allow cov. to be a matrix, not just a function.
# 2022-01-29: Added warning or note about unestimable effects.
# 2022-02-16: Make computation of residual df more robust.
# 2025-07-27: transformation is no longer a "legacy" argument to Effect.lm().

### Non-exported function added 2018-01-22 to generalize given.values to allow for "equal" weighting of factor levels for non-focal predictors.
.set.given.equal <- function(m){
  if(inherits(m, "lm") & !("(Intercept)" %in% names(coef(m))))
      stop("Seting given.vales='equal' requires an intercept in the model formula")
  terms <- terms(m)
  classes <- attr(terms, "dataClasses")
  response <- attr(terms, "response")
  classes <- classes[-response]
  factors <- names(classes)[classes=="factor"]
  out <- NULL
  for (f in factors){
    form <- as.formula(paste( "~", f, collapse=""))
    .m0 <- if(inherits(m, "glm")) 
              {update(m, form, control=glm.control(epsilon=Inf, maxit=1))} else {
    if(inherits(m, "polr"))
              {update(m, form, control=list(maxit=1))} else {
    if(inherits(m, "multinom"))    
              {update(m, form, maxit=0, trace=FALSE)} else
        update(m, form)}}
    names <- colnames(model.matrix(.m0))[-1]
    vals <- rep(1/(length(names)+1), length(names))
    names(vals) <- names
    out <- c(out, vals)
  }
  out
}

# 2020-05-29 Use insight::get_parameters to get a vector of parameter estimates
# for any model supported by insight.

effCoef <- function(mod, ...){UseMethod("effCoef", mod)}
effCoef.default <- function(mod, ...){
  est1 <- insight::get_parameters(mod, ...)
  est <- est1[,2]
  names(est) <- est1[,1]
  est
}

### end of non-exported function

checkFormula <- function(object){
# clm2 does not have a formula,
#  if(inherits(object, "clm2")) formula <- function(x) x$call$location
  if (!inherits(object, "formula")){
    object <- insight::find_formula(object)$conditional
  }
  formula <- as.character(object)
  rhs <- formula[length(formula)]
  res <- regexpr("as.factor\\(|factor\\(|as.ordered\\(|ordered\\(|as.numeric\\(|as.integer\\(",
                 rhs)
  res == -1 || attr(res, "match.length") == 0
}

Effect <- function(focal.predictors, mod, ...){
  if (!checkFormula(mod)) stop("model formula should not contain calls to",
                               "\n  factor(), as.factor(), ordered(), as.ordered(),",
                               " as.numeric(), or as.integer();",
                               "\n  see 'Warnings and Limitations' in ?Effect")
  UseMethod("Effect", mod)
}

# 2017-12-04 new Effect.default that actually works
# 2017-12-07 added Effects.lme, .mer, gls that work

Effect.default <- function(focal.predictors, mod, ..., sources){ 
# 2020/05/23 ... uses 'insight' package, else
# if sources is null, try to construct it
  sources <- if(missing(sources)) effSources(mod) else sources
## formula 
   formula <- if(is.null(sources$formula)) 
     insight::find_formula(mod)$conditional else sources$formula
  # the next line returns the formula if focal.predictors is null
  if(is.null(focal.predictors)) return(formula)
## call
  cl <- if(is.null(sources$call)) {if(isS4(mod)) 
        mod@call else mod$call} else sources$call
  # insert formula into the call
  cl$formula <- formula
## type == 'glm' unless it is set in sources
  type <- if(is.null(sources$type)) "glm" else sources$type
# family
  fam <- try(family(mod), silent=TRUE)
  if(inherits(fam, "try-error")) fam <- NULL
  if(!is.null(sources$family)){fam <- sources$family}
  if(!is.null(fam)){
    fam$aic <- function(...) NULL
    # check to be sure the variance function in the family has one argument only,
    # otherwise this method won't work
    if(!is.null(fam$variance)){
      if(length(formals(fam$variance)) > 1)
        stop("Effect plots are not implemented for families with more than
             one parameter in the variance function (e.g., negative binomial).")}
  }
  cl$family <- fam
# get the coefficient estimates and vcov from sources if present
  coefficients <- if(is.null(sources$coefficients)) 
    effCoef(mod) else sources$coefficients
    vcov <- if(is.null(sources$vcov)) 
      as.matrix(vcov(mod, complete=TRUE)) else sources$vcov
# added 7/5/2019, next line, for models that use polr (e.g, clm, clm2)
  zeta <- if(is.null(sources$zeta)) NULL else sources$zeta
# set control parameters: suggested by Nate TeGrotenhuis
  cl$control <- switch(type,
          glm = glm.control(epsilon=Inf, maxit=1),
          polr = list(maxit=1),
          multinom = c(maxit=1))
  cl$method <- sources$method # NULL except for type=="polr"
  .m <- switch(type,
               glm=match(c("formula", "data", "family", "contrasts",  "subset",
                "control", "offset"), names(cl), 0L),
               polr=match(c("formula", "data", "family", "contrasts",  "subset",
                "control", "method"), names(cl), 0L),
               multinom=match(c("formula", "data", "family", "contrasts", "subset",
                "family", "maxit", "offset"), names(cl), 0L))
  cl <- cl[c(1L, .m)]
#  if(!is.null(fam)) cl$family <- fam
#  if (is.character(cl$family)) 
#    cl$family <- get(cl$family, mode = "function", envir = parent.frame())
#  if (is.function(cl$family)) 
#    cl$family <- family()
  cl[[1L]] <- as.name(type)
# The following eval creates on object of class glm, polr or multinom.  
# These are crated to avoid writing an Effects method for every type of model.  
# The only information used from this "fake" object are the coefficients and 
# the variance-covariance matrix, and these are copied from the original 
# object so Effects plots the right things.
  mod2 <- eval(cl)
  mod2$coefficients <- coefficients
  mod2$vcov <- vcov
  if(!is.null(zeta)) mod2$zeta <- zeta # added 7/5/2019
  if(type == "glm"){
       mod2$weights <- as.vector(with(mod2,
                        prior.weights * (family$mu.eta(linear.predictors)^2 /
                                        family$variance(fitted.values))))}
  class(mod2) <- c("fakeeffmod", class(mod2))
  Effect(focal.predictors, mod2, ...)  # call the glm/polr/multinom method
}

vcov.fakeeffmod <- function(object, ...) object$vcov

## This function removes terms with "|" or "||" in the formula, assuming these
## correspond to random effects.  As of 2020-05-22 this function is never used.

fixFormula <- function (term)
{
  if (!("|" %in% all.names(term)) && !("||" %in% all.names(term)))
    return(term)
  if ((is.call(term) && term[[1]] == as.name("|")) ||
      (is.call(term) && term[[1]] == as.name("||")))
    return(NULL)
  if (length(term) == 2) {
    nb <- fixFormula(term[[2]])
    if (is.null(nb))
      return(NULL)
    term[[2]] <- nb
    return(term)
  }
  nb2 <- fixFormula(term[[2]])
  nb3 <- fixFormula(term[[3]])
  if (is.null(nb2))
    return(nb3)
  if (is.null(nb3))
    return(nb2)
  term[[2]] <- nb2
  term[[3]] <- nb3
  term
}



Effect.lm <- function(focal.predictors, mod, xlevels=list(), fixed.predictors,
        vcov. = vcov, se=TRUE,
        residuals=FALSE, quantiles=seq(0.2, 0.8, by=0.2),
        x.var=NULL, transformation, ...,
        #legacy arguments:
        given.values, typical, offset, confint, confidence.level, 
        partial.residuals){ 
  
  if (is.numeric(xlevels)){
    if (length(xlevels) > 1 || round(xlevels != xlevels)) stop("xlevels must be a single whole number or a list")
    form <- Effect.default(NULL, mod) #returns the fixed-effects formula
    terms <- attr(terms(form), "term.labels")
    predictors <- all.vars(parse(text=terms))
    xlevs <- list()
    for (pred in predictors){
      xlevs[[pred]] <- xlevels
    }
    xlevels <- xlevs
  }
  
  if (!missing(partial.residuals)) residuals <- partial.residuals
  partial.residuals <- residuals
  if (missing(transformation)) 
    transformation <- list(link = family(mod)$linkfun, 
                           inverse = family(mod)$linkinv)
  if (missing(fixed.predictors)) fixed.predictors <- NULL
  fixed.predictors <- applyDefaults(fixed.predictors,
                                    list(given.values=NULL, typical=mean,
                                         apply.typical.to.factors=FALSE, offset=mean),
                                    arg="fixed.predictors")
  if (missing(given.values)) given.values <- fixed.predictors$given.values
# new 1/22/18 to allow for automatical equal weighting of factor levels
  if(!is.null(given.values)){
   if (given.values[1] == "default") given.values <- NULL
   if (given.values[1] == "equal") given.values <- .set.given.equal(mod)}
# end new code
  if (missing(typical)) typical <- fixed.predictors$typical
  if (missing(offset)) offset <- fixed.predictors$offset
  apply.typical.to.factors <- fixed.predictors$apply.typical.to.factors
  if (!missing(confint)) se <- confint
  confint <- applyDefaults(se, list(compute=TRUE, level=.95, type="pointwise"),
                           onFALSE=list(compute=FALSE, level=.95, type="pointwise"),
                           arg="se")
  se <- confint$compute
  if (missing(confidence.level)) confidence.level <- confint$level
  confidence.type <- match.arg(confint$type, c("pointwise", "Scheffe", "scheffe"))
  default.levels <- NULL # just for backwards compatibility
  data <- if (partial.residuals){
    all.vars <- all.vars(formula(mod))
    expand.model.frame(mod, all.vars)[, all.vars]
  }
  else NULL
  if (!is.null(given.values) && !all(which <- names(given.values) %in% names(coef(mod))))
    stop("given.values (", names(given.values[!which]), ") not in the model")
  off <- if (is.numeric(offset) && length(offset) == 1) offset
  else if (is.function(offset)) {
    mod.off <- model.offset(model.frame(mod))
    if (is.null(mod.off)) 0 else offset(mod.off)
  }
  else stop("offset must be a function or a number")
  formula.rhs <- formula(mod)[[3]]
  if (!missing(x.var)){
    if (!is.numeric(x.var)) {
      x.var.name <- x.var
      x.var <- which(x.var == focal.predictors)
    }
    if (length(x.var) == 0) stop("'", x.var.name, "' is not among the focal predictors")
    if (length(x.var) > 1) stop("x.var argument must be of length 1")
  }
  model.components <- Analyze.model(focal.predictors, mod, xlevels, default.levels, formula.rhs,
                                    partial.residuals=partial.residuals, quantiles=quantiles, x.var=x.var, data=data, typical=typical)
  excluded.predictors <- model.components$excluded.predictors
  predict.data <- model.components$predict.data
  predict.data.all.rounded <- predict.data.all <- if (partial.residuals) na.omit(data[, all.vars(formula(mod))]) else NULL
  factor.levels <- model.components$factor.levels
  factor.cols <- model.components$factor.cols
  n.focal <- model.components$n.focal
  x <- model.components$x
  X.mod <- model.components$X.mod
  cnames <- model.components$cnames
  X <- model.components$X
  x.var <- model.components$x.var
  formula.rhs <- formula(mod)[c(1, 3)]
  Terms <- delete.response(terms(mod))
  mf <- model.frame(Terms, predict.data, xlev = factor.levels, na.action=NULL)
  mod.matrix <- model.matrix(formula.rhs, data = mf, contrasts.arg = mod$contrasts)
  if (is.null(x.var)) partial.residuals <- FALSE
  factors <- sapply(predict.data, is.factor)
  if (partial.residuals){
    for (predictor in focal.predictors[-x.var]){
      if (!factors[predictor]){
        values <- unique(predict.data[, predictor])
        predict.data.all.rounded[, predictor] <- values[apply(outer(predict.data.all[, predictor], values, function(x, y) (x - y)^2), 1, which.min)]
      }
    }
  }
  mod.matrix.all <- model.matrix(mod)
  wts <- weights(mod)
  if (is.null(wts))
    wts <- rep(1, length(residuals(mod)))
  mod.matrix <- Fixup.model.matrix(mod, mod.matrix, mod.matrix.all,
                                   X.mod, factor.cols, cnames, focal.predictors, 
                                   excluded.predictors, typical, given.values, 
                                   apply.typical.to.factors) 
# 11/3/2017.  Check to see if the model is full rank
  # Compute a basis for the null space, using estimability package
  null.basis <- estimability::nonest.basis(mod)  # returns basis for null space
  # check to see if each row of mod.matrix is estimable
  is.estimable <- estimability::is.estble(mod.matrix, null.basis) # TRUE if effect is estimable else FALSE
  if (!any(is.estimable)) {
    warning("none of the values of the ",
                               paste(focal.predictors, collapse="*"),
                               " effect are estimable")
  } else if ((n.not.estimable <- sum(!is.estimable)) > 0) {
    message("Note:\n  ", n.not.estimable,
            if (n.not.estimable > 1) " values" else " value",
            " in the ",  paste(focal.predictors, collapse="*"),
            " effect are not estimable")
    }
    # substitute 0 for NA in coef vector and compute effects
  scoef <- ifelse(is.na(mod$coefficients), 0L, mod$coefficients)
  effect <- off + mod.matrix %*% scoef
  effect[!is.estimable] <- NA  # set all non-estimable effects to NA
# end estimability check
  if (partial.residuals){
    res <- na.omit(residuals(mod, type="working"))
    fitted <- na.omit(if (inherits(mod, "glm")) predict(mod, type="link") else predict(mod))
    partial.residuals.range <- range(fitted + res)
  }
  else {
    res <- partial.residuals.range <- NULL
  }
  result <- list(term = paste(focal.predictors, collapse="*"),
                 formula = formula(mod), response = response.name(mod),
                 variables = x, fit = effect, x = predict.data[, 1:n.focal, drop=FALSE],
                 x.all=predict.data.all.rounded[, focal.predictors, drop=FALSE],
                 model.matrix = mod.matrix,
                 data = X,
                 discrepancy = 0, offset=off,
                 residuals=res, partial.residuals.range=partial.residuals.range,
                 x.var=x.var)
  if (se) {
    if (any(family(mod)$family == c("binomial", "poisson"))) {
      z <- if (confidence.type == "pointwise") {
        qnorm(1 - (1 - confidence.level)/2)
      } else {
        p <- length(na.omit(coef(mod)))
        scheffe(confidence.level, p)
      }
    }
    else {
      df.residual <- df.residual(mod)
      if (is.null(df.residual) || is.na(df.residual)) df.residual <- Inf
      z <- if (confidence.type == "pointwise") {
        qt(1 - (1 - confidence.level)/2, df = df.residual)
      } else {
        p <- length(na.omit(coef(mod)))
        scheffe(confidence.level, p, df.residual)
      }
    }
    V <- if(inherits(vcov., "matrix")) vcov. else {
            if(inherits(vcov., "function")) vcov.(mod, complete=FALSE) 
            else stop("vcov. must be a function or matrix")}
    use <- !is.na(mod$coefficients) # new
    # mmat <- mod.matrix[, !is.na(mod$coefficients)] # remove non-cols with NA coeffs
    mmat <- mod.matrix[, use] # remove non-cols with NA coeffs # new
    if (any(is.na(V))) V <- V[use, use] # new
    eff.vcov <- mmat %*% V %*% t(mmat)
    rownames(eff.vcov) <- colnames(eff.vcov) <- NULL
    var <- diag(eff.vcov)
    result$vcov <- eff.vcov
    result$se <- sqrt(var)
    result$se[!is.estimable] <- NA
    result$lower <- effect - z * result$se
    result$upper <- effect + z * result$se
    result$confidence.level <- confidence.level
  }
  if (is.null(transformation$link) && is.null(transformation$inverse)) {
    transformation$link <- I
    transformation$inverse <- I
  }
  result$transformation <- transformation
  result$family <- family(mod)$family
# 2018-10-08 result$family kept to work with legacy code  
  result$link <- family(mod) 
  class(result) <- "eff"
  result
}

Effect.multinom <- function(focal.predictors, mod,
                            xlevels=list(), fixed.predictors,
                            vcov. = vcov, se=TRUE, ...,
                            #legacy arguments:
                            confint, confidence.level, given.values, typical){
  
  if (is.numeric(xlevels)){
    if (length(xlevels) > 1 || round(xlevels != xlevels)) stop("xlevels must be a single whole number or a list")
    form <- Effect.default(NULL, mod) #returns the fixed-effects formula
    terms <- attr(terms(form), "term.labels")
    predictors <- all.vars(parse(text=terms))
    xlevs <- list()
    for (pred in predictors){
      xlevs[[pred]] <- xlevels
    }
    xlevels <- xlevs
  }
  
  if (missing(fixed.predictors)) fixed.predictors <- NULL
  fixed.predictors <- applyDefaults(fixed.predictors,
                                    list(given.values=NULL, typical=mean),
                                    arg="fixed.predictors")
  if (missing(given.values)) given.values <- fixed.predictors$given.values
  # new 1/22/18 to allow for automatical equal weighting of factor levels
  if(!is.null(given.values)){
    if (given.values[1] == "default") given.values <- NULL
    if (given.values[1] == "equal") given.values <- .set.given.equal(mod)}
  # end new code
  # end new code
  if (missing(typical)) typical <- fixed.predictors$typical
  if (!missing(confint)) se <- confint
  confint <- applyDefaults(se, list(compute=TRUE, level=.95, type="pointwise"),
                           onFALSE=list(compute=FALSE, level=.95, type="pointwise"),
                           arg="se")
  se <- confint$compute
  if (missing(confidence.level)) confidence.level <- confint$level
  confidence.type <- match.arg(confint$type, c("pointwise", "Scheffe", "scheffe"))
  default.levels <- NULL # just for backwards compatibility
  if (length(mod$lev) < 3) stop("effects for multinomial logit model only available for response levels > 2")
  if (missing(given.values)) given.values <- NULL
  else if (!all(which <- colnames(given.values) %in% names(coef(mod))))
    stop("given.values (", colnames(given.values[!which]),") not in the model")
  formula.rhs <- formula(mod)[c(1, 3)]
  model.components <- Analyze.model(focal.predictors, mod, xlevels, default.levels, formula.rhs, typical=typical)
  excluded.predictors <- model.components$excluded.predictors
  predict.data <- model.components$predict.data
  factor.levels <- model.components$factor.levels
  factor.cols <- model.components$factor.cols
  #    n.focal <- model.components$n.focal
  x <- model.components$x
  X.mod <- model.components$X.mod
  cnames <- model.components$cnames
  X <- model.components$X
  formula.rhs <- formula(mod)[c(1, 3)]
  Terms <- delete.response(terms(mod))
  mf <- model.frame(Terms, predict.data, xlev = factor.levels)
  mod.matrix <- model.matrix(formula.rhs, data = mf, contrasts.arg = mod$contrasts)
  X0 <- Fixup.model.matrix(mod, mod.matrix, model.matrix(mod),
                           X.mod, factor.cols, cnames, focal.predictors, excluded.predictors, typical, given.values)
  resp.names <- make.names(mod$lev, unique=TRUE)
  resp.names <- c(resp.names[-1], resp.names[1]) # make the last level the reference level
  B <- t(coef(mod))
  V <- if(inherits(vcov., "matrix")) vcov. else {
         if(inherits(vcov., "function")) vcov.(mod) 
         else stop("vcov. must be a function or matrix")}
  m <- ncol(B) + 1
  p <- nrow(B)
  r <- p*(m - 1)
  n <- nrow(X0)
  P <- Logit <- matrix(0, n, m)
  colnames(P) <-  paste("prob.", resp.names, sep="")
  colnames(Logit) <-  paste("logit.", resp.names, sep="")
  if (se){
    z <- if (confidence.type == "pointwise") {
      qnorm(1 - (1 - confidence.level)/2)
    } else {
      scheffe(confidence.level, p)
    }
    Lower.P <- Upper.P <- Lower.logit <- Upper.logit <- SE.P <- SE.logit <- matrix(0, n, m)
    colnames(Lower.logit) <-  paste("L.logit.", resp.names, sep="")
    colnames(Upper.logit) <-  paste("U.logit.", resp.names, sep="")
    colnames(Lower.P) <-  paste("L.prob.", resp.names, sep="")
    colnames(Upper.P) <-  paste("U.prob.", resp.names, sep="")
    colnames(SE.P) <-  paste("se.prob.", resp.names, sep="")
    colnames(SE.logit) <-  paste("se.logit.", resp.names, sep="")
  }
  for (i in 1:n){
    res <- eff.mul(X0[i,], B, se, m, p, r, V) # compute effects
    #        P[i,] <- prob <- res$p # fitted probabilities
    P[i,] <- res$p # fitted probabilities
    Logit[i,] <- logit <- res$logits # fitted logits
    if (se){
      #            SE.P[i,] <- se.p <- res$std.err.p # std. errors of fitted probs
      SE.P[i,] <- res$std.err.p # std. errors of fitted probs
      SE.logit[i,] <- se.logit <- res$std.error.logits # std. errors of logits
      Lower.P[i,] <- logit2p(logit - z*se.logit)
      Upper.P[i,] <- logit2p(logit + z*se.logit)
      Lower.logit[i,] <- logit - z*se.logit
      Upper.logit[i,] <- logit + z*se.logit
    }
  }
  resp.levs <- c(m, 1:(m-1)) # restore the order of the levels
  P <- P[, resp.levs]
  Logit <- Logit[, resp.levs]
  if (se){
    Lower.P <- Lower.P[, resp.levs]
    Upper.P <- Upper.P[, resp.levs]
    Lower.logit <- Lower.logit[, resp.levs]
    Upper.logit <- Upper.logit[, resp.levs]
    SE.P <- SE.P[, resp.levs]
    SE.logit <- SE.logit[, resp.levs]
  }
  result <- list(term=paste(focal.predictors, collapse="*"), formula=formula(mod), response=response.name(mod),
                 y.levels=mod$lev, variables=x, x=predict.data[, focal.predictors, drop=FALSE],
                 model.matrix=X0, data=X, discrepancy=0, model="multinom",
                 prob=P, logit=Logit)
  if (se) result <- c(result, list(se.prob=SE.P, se.logit=SE.logit,
                                   lower.logit=Lower.logit, upper.logit=Upper.logit,
                                   lower.prob=Lower.P, upper.prob=Upper.P,
                                   confidence.level=confidence.level))
  # find empty cells, if any, and correct
## 11/3/17:  The code until the next comment is surely incorrect, but
## generally harmless.  One must learn if the notion of estimablilty applied
## to multinomial models and figure out the right thing to do
  whichFact <- unlist(lapply(result$variables, function(x) x$is.factor))
  zeroes <- NULL
  if(sum(whichFact) > 1){
    nameFact <- names(whichFact)[whichFact]
    counts <- xtabs(as.formula( paste("~", paste(nameFact, collapse="+"))),
                    model.frame(mod))
    zeroes <- which(counts == 0)
  }
  if(length(zeroes) > 0){
    levs <- expand.grid(lapply(result$variables, function(x) x$levels))
    good <- rep(TRUE, dim(levs)[1])
    for(z in zeroes){
      good <- good &
        apply(levs, 1, function(x) !all(x == levs[z, whichFact]))
    }
    result$prob[!good, ] <- NA
    result$logit[!good, ] <- NA
    if (se){
      result$se.prob[!good, ] <- NA
      result$se.logit[!good, ] <- NA
      result$lower.prob[!good, ] <- NA
      result$upper.prob[!good, ] <- NA
    }
  }
## End of unnecessary code
  class(result) <-'effpoly'
  result
}

Effect.polr <- function(focal.predictors, mod,
                        xlevels=list(), fixed.predictors,
                        vcov.=vcov, se=TRUE, latent=FALSE, ...,
                        #legacy arguments:
                        confint, confidence.level, given.values, typical){
  
  if (is.numeric(xlevels)){
    if (length(xlevels) > 1 || round(xlevels != xlevels)) stop("xlevels must be a single whole number or a list")
    form <- Effect.default(NULL, mod) #returns the fixed-effects formula
    terms <- attr(terms(form), "term.labels")
    predictors <- all.vars(parse(text=terms))
    xlevs <- list()
    for (pred in predictors){
      xlevs[[pred]] <- xlevels
    }
    xlevels <- xlevs
  }
  
  if (missing(fixed.predictors)) fixed.predictors <- NULL
  fixed.predictors <- applyDefaults(fixed.predictors,
                                    list(given.values=NULL, typical=mean),
                                    arg="fixed.predictors")
  if (missing(given.values)) given.values <- fixed.predictors$given.values
  # new 1/22/18 to allow for automatical equal weighting of factor levels
  # new 1/22/18 to allow for automatical equal weighting of factor levels
  if(!is.null(given.values)){
    if (given.values[1] == "default") given.values <- NULL
    if (given.values[1] == "equal") given.values <- .set.given.equal(mod)}
  # end new code
  if (missing(typical)) typical <- fixed.predictors$typical
  if (!missing(confint)) se <- confint
  confint <- applyDefaults(se, list(compute=TRUE, level=.95, type="pointwise"),
                           onFALSE=list(compute=FALSE, level=.95, type="pointwise"),
                           arg="se")
  se <- confint$compute
  if (missing(confidence.level)) confidence.level <- confint$level
  confidence.type <- match.arg(confint$type, c("pointwise", "Scheffe", "scheffe"))
  default.levels <- NULL # just for backwards compatibility
  if (mod$method != "logistic") stop('method argument to polr must be "logistic"')
  if (missing(given.values)) given.values <- NULL
  else if (!all(which <- names(given.values) %in% names(coef(mod))))
    stop("given.values (", names(given.values[!which]),") not in the model")
  formula.rhs <- formula(mod)[c(1, 3)]
  model.components <- Analyze.model(focal.predictors, mod, xlevels, default.levels, formula.rhs, typical=typical)
  excluded.predictors <- model.components$excluded.predictors
  predict.data <- model.components$predict.data
  factor.levels <- model.components$factor.levels
  factor.cols <- model.components$factor.cols
  #    n.focal <- model.components$n.focal
  x <- model.components$x
  X.mod <- model.components$X.mod
  cnames <- model.components$cnames
  X <- model.components$X
  Terms <- delete.response(terms(mod))
  mf <- model.frame(Terms, predict.data, xlev = factor.levels, na.action=NULL)
  mod.matrix <- model.matrix(formula.rhs, data = mf, contrasts.arg = mod$contrasts)
  X0 <- Fixup.model.matrix(mod, mod.matrix, model.matrix(mod),
                           X.mod, factor.cols, cnames, focal.predictors, excluded.predictors, typical, given.values)
  resp.names <- make.names(mod$lev, unique=TRUE)
  X0 <- X0[,-1, drop=FALSE]
  b <- coef(mod)
  p <- length(b)  # corresponds to p - 1 in the text
  alpha <- - mod$zeta  # intercepts are negatives of thresholds
  z <- if (confidence.type == "pointwise") {
    qnorm(1 - (1 - confidence.level)/2)
  } else {
    scheffe(confidence.level, p + length(alpha))
  }
  result <- list(term=paste(focal.predictors, collapse="*"), formula=formula(mod), response=response.name(mod),
                 y.levels=mod$lev, variables=x,
                 x=predict.data[, focal.predictors, drop=FALSE],
                 model.matrix=X0, data=X, discrepancy=0, model="polr")
  if (latent){ 
    V <- if(inherits(vcov., "matrix")) vcov.[1:p, 1:p] else {
            if(inherits(vcov., "function")) vcov.(mod)[1:p, 1:p] 
            else stop("vcov. must be a function or matrix")}
    res <- eff.latent(X0, b, V, se)
    result$fit <- res$fit
    if (se){
      result$se <- res$se
      result$lower <- result$fit - z*result$se
      result$upper <- result$fit + z*result$se
      result$confidence.level <- confidence.level
    }
    transformation <- list()
    transformation$link <- I
    transformation$inverse <- I
    result$transformation <- transformation
    result$thresholds <- -alpha
    class(result) <- c("efflatent", "eff")
    return(result)
  }
  m <- length(alpha) + 1
  r <- m + p - 1
  indices <- c((p+1):r, 1:p)
  V <- if(inherits(vcov., "matrix")) vcov.[indices, indices] else {
         if(inherits(vcov., "function")) vcov.(mod)[indices, indices] 
         else stop("vcov. must be a function or matrix")}  
  for (j in 1:(m-1)){  # fix up the signs of the covariances
    V[j,] <- -V[j,]  #  for the intercepts
    V[,j] <- -V[,j]}
  n <- nrow(X0)
  P <- Logit <- matrix(0, n, m)
  colnames(P) <-  paste("prob.", resp.names, sep="")
  colnames(Logit) <-  paste("logit.", resp.names, sep="")
  if (se){
    Lower.logit <- Upper.logit <- Lower.P <- Upper.P <- SE.P <- SE.Logit <- matrix(0, n, m)
    colnames(Lower.logit) <-  paste("L.logit.", resp.names, sep="")
    colnames(Upper.logit) <-  paste("U.logit.", resp.names, sep="")
    colnames(Lower.P) <-  paste("L.prob.", resp.names, sep="")
    colnames(Upper.P) <-  paste("U.prob.", resp.names, sep="")
    colnames(SE.P) <-  paste("se.prob.", resp.names, sep="")
    colnames(SE.Logit) <-  paste("se.logit.", resp.names, sep="")
  }
  for (i in 1:n){
    res <- eff.polr(X0[i,], b, alpha, V, m, r, se) # compute effects
    P[i,] <- res$p # fitted probabilities
    Logit[i,] <- logit <- res$logits # fitted logits
    if (se){
      SE.P[i,] <- res$std.err.p # std. errors of fitted probs
      SE.Logit[i,] <- se.logit <- res$std.error.logits # std. errors of logits
      Lower.P[i,] <- logit2p(logit - z*se.logit)
      Upper.P[i,] <- logit2p(logit + z*se.logit)
      Lower.logit[i,] <- logit - z*se.logit
      Upper.logit[i,] <- logit + z*se.logit
    }
  }
  result$prob <- P
  result$logit <- Logit
  if (se) result <- c(result,
                      list(se.prob=SE.P, se.logit=SE.Logit,
                           lower.logit=Lower.logit, upper.logit=Upper.logit,
                           lower.prob=Lower.P, upper.prob=Upper.P,
                           confidence.level=confidence.level))
  class(result) <-'effpoly'
  result
}

# merMod -- included here to allow addtional KR argument
Effect.merMod <- function(focal.predictors, mod, ..., KR=FALSE){
  if (KR && !requireNamespace("pbkrtest", quietly=TRUE)){
    KR <- FALSE
    warning("pbkrtest is not available, KR set to FALSE")}
  fam <- family(mod)
  args <- list(
    family=fam,
    vcov = if (fam$family == "gaussian" && fam$link == "identity" && KR)
      as.matrix(pbkrtest::vcovAdj(mod)) else insight::get_varcov(mod))
  Effect.default(focal.predictors, mod, ..., sources=args)
}

# svyglm
Effect.svyglm <- function(focal.predictors, mod, fixed.predictors, ...){
  Svymean <- function(x){
    svymean(x, design=mod$survey.design)
  }
  ellipses.list <- list(...)
  if ((!is.null(ellipses.list$residuals) && !isFALSE(residuals)) || 
      (!is.null(ellipses.list$partial.residuals) && !isFALSE(ellipses.list$partial.residuals))){
    stop("partial residuals are not available for svyglm models")
  }
  if (missing(fixed.predictors)) fixed.predictors <- NULL
  fixed.predictors <- applyDefaults(fixed.predictors,
                                    list(given.values=NULL, typical=Svymean,
                                         apply.typical.to.factors=TRUE, offset=Svymean),
                                    arg="fixed.predictors")
  typical <- fixed.predictors$typical
  apply.typical.to.factors <- fixed.predictors$apply.typical.to.factors
  offset <- fixed.predictors$offset
  mod$call <- list(mod$call, data=mod$data)
  Effect.lm(focal.predictors, mod, typical=typical,
            apply.typical.to.factors=apply.typical.to.factors, offset=offset, ...)
}

# plot, summary, and print methods for effects package
# John Fox and Jangman Hong
#  last modified 2012-11-30 by J. Fox
#  29 June 2011 added grid, rotx and roty arguments to the two plot methods
#   by S. Weisberg
#  21 Dec 2012 modest modification of empty cells with crossed factors
#  2013-01-17: Added factor.ci.style arg to plot.eff() and plot.effpoly(). J. Fox
#  2013-01-18: Added CI bars to multiline plots with factor.ci.style="bars"
#  2013-01-19: Renamed 'factor.ci.style' to 'ci.style'.  Added a 'none' option
#   extended to variate terms if multiline=TRUE, ci.style="bars"
#  2013-01-30: scale arrow "heads" for error bars relative to cex
#  2013-05-31: fixed symbol colors in legends in plot.eff(). J. Fox
#  2013-08-14: fixed bug in restoring warn option. J. Fox
#  2013-08-27: fixed symbols argument for multiline plot in plot.eff(), reported by Ulrike Gromping. J. Fox
#  2013-08-31: fixed handling of ticks.x argument. John
#  2013-09-25: moved plot.eff methods to plot.methods.R for easier work. Michael
#  2013-10-17: added use.splines argument to plot.effpoly.  Sandy
#  2025-07-22: fix summary.eff when transformation is inverse rather than direct. John


summary.eff <- function(object, type=c("response", "link"), ...){
  
  effect <- as.vector(object$fit)
  trans.effect <- object$transformation$inverse(effect)
  check.order  <- if (all(order(effect) == order(trans.effect))){
    "direct"
  } else if (all(order(effect) == order(- trans.effect))){ 
    "inverse"
  } else {
    "inconsistent"
  }
  if (check.order == "inconsistent") {
    warning("the response transformation appears to be non-monotone")
  }
  
  result <- list()
  result$header <- paste("\n", gsub(":", "*", object$term), 'effect\n')
  result$offset <- object$offset
  type <- match.arg(type)
  if (type == "response") {
    object$fit <- object$transformation$inverse(object$fit)
    if (!is.null(object$confidence.level)){
      if (check.order == "inverse"){
        save.upper <- object$upper
        object$upper <- object$transformation$inverse(object$lower)
        object$lower <- object$transformation$inverse(save.upper)
      } else {
        object$lower <- object$transformation$inverse(object$lower)
        object$upper <- object$transformation$inverse(object$upper)
      }
    }
  }
  result$effect <- array(object$fit,     
                         dim=sapply(object$variables, function(x) length(x$levels)),
                         dimnames=lapply(object$variables, function(x) x$levels))
  if (!is.null(object$se)){
    result$lower.header <- paste('\n Lower', round(100*object$confidence.level, 2), 
                                 'Percent Confidence Limits\n')
    result$lower <- array(object$lower,   
                          dim=sapply(object$variables, function(x) length(x$levels)),
                          dimnames=lapply(object$variables, function(x) x$levels))
    result$upper.header <- paste('\n Upper', round(100*object$confidence.level, 2),
                                 'Percent Confidence Limits\n')
    result$upper <- array(object$upper,   
                          dim=sapply(object$variables, function(x) length(x$levels)),
                          dimnames=lapply(object$variables, function(x) x$levels))
  }
  if (object$discrepancy > 1e-3) result$warning <- paste("\nWarning: There is an average discrepancy of", 
                                                         round(object$discrepancy, 3),
                                                         "percent \n     in the 'safe' predictions for effect", object$term, '\n')
  class(result) <- "summary.eff"
  result
}

print.summary.eff <- function(x, ...){
  cat(x$header)
  if (x$offset != 0) cat("\noffset = ", x$offset, "\n\n")
  print(x$effect, ...)
  if (!is.null(x$lower)){
    cat(x$lower.header)
    print(x$lower, ...)
    cat(x$upper.header)
    print(x$upper, ...)
  }
  if (!is.null(x$thresholds)){
    cat("\nThresholds:\n")
    print(x$thresholds, ...)
  }
  if (!is.null(x$warning)) cat(x$warning)
  invisible(x)
}

print.eff <- function(x, type=c("response", "link"), ...){
  cat(paste("\n", gsub(":", "*", x$term), 'effect\n'))
  if (x$offset != 0) cat("\noffset = ", x$offset, "\n\n")
  type <- match.arg(type)
  if (type == "response") x$fit <- x$transformation$inverse(x$fit)
  table <- array(x$fit,     
                 dim=sapply(x$variables, function(x) length(x$levels)),
                 dimnames=lapply(x$variables, function(x) x$levels))
  print(table, ...)
  if (x$discrepancy > 1e-3) cat(paste("\nWarning: There is an average discrepancy of", 
                                      round(x$discrepancy, 3),
                                      "percent \n     in the 'safe' predictions for effect", x$term, '\n'))
  invisible(x)
}

print.efflist <- function(x, ...){
  cat(" model: ")
  form <- x[[1]]$formula
  attributes(form) <- NULL
  print(form)
  for (effect in names(x)){
    print(x[[effect]], ...)
  }
  invisible(x) 
}

summary.efflist <- function(object, ...){
  cat(" model: ")
  form <- object[[1]]$formula
  attributes(form) <- NULL
  print(form)
  for (effect in names(object)){
    print(summary(object[[effect]], ...))
  }
  invisible(NULL) 
}


print.effpoly <- function(x, type=c("probability", "logits"), ...){
  type <- match.arg(type)
  x.frame <-as.data.frame(x)
  n.predictors <- length(names(x$x))
  predictors <- names(x.frame)[1:n.predictors]
  y.lev <- x$y.lev
  ylevel.names <- make.names(paste("prob",y.lev))
  colnames(x$prob) <- colnames(x$logit) <- ylevel.names
  y.categories <- matrix(0, nrow=length(x.frame[,predictors[1]]), ncol=length(y.lev))
  for (i in 1:length(y.lev)){
    level <- which(colnames(x$prob)[i] == ylevel.names)
    y.categories[,i] <-  rep(y.lev[level], length(y.categories[,i]))
  }
  y.categories <- as.vector(y.categories)
  y.categories <- factor(y.categories)
  for (i in 1:length(y.lev)){
    cat(paste("\n", gsub(":", "*", x$term), " effect (", type,") for ", y.lev[i], "\n", sep=""))    
    table <- array(if (type == "probability") {x$prob[y.categories==y.lev[i]]}
                   else {x$logit[y.categories==y.lev[i]]},     
                   dim=sapply(x$variables, function(x) length(x$levels)),
                   dimnames=lapply(x$variables, function(x) x$levels))
    print(table, ...)
  }
  if (x$discrepancy > 0.1) cat(paste("\nWarning: There is an average discrepancy of", 
                                     round(x$discrepancy, 2),
                                     "percent \n     in the 'safe' predictions for effect", x$term, '\n'))
  invisible(x)
}

summary.effpoly <- function(object, type=c("probability", "logits"), ...){
  type <- match.arg(type)
  x.frame <-as.data.frame(object)
  n.predictors <- length(names(object$x))
  predictors <- names(x.frame)[1:n.predictors]
  y.lev <- object$y.lev
  ylevel.names <- make.names(paste("prob",y.lev))
  colnames(object$prob) <- colnames(object$logit) <- 
    colnames(object$lower.logit) <- colnames(object$upper.logit) <- 
    colnames(object$lower.prob) <- colnames(object$upper.prob)<- ylevel.names
  y.categories <-matrix(0, nrow=length(x.frame[,predictors[1]]), ncol=length(y.lev))
  for (i in 1:length(y.lev)){
    level <- which(colnames(object$prob)[i] == ylevel.names)
    y.categories[,i] <- rep(y.lev[level], length(y.categories[,i]))
  }
  y.categories <- as.vector(y.categories)
  y.categories <- factor(y.categories)
  for (i in 1:length(y.lev)){
    cat(paste("\n", gsub(":", "*", object$term), " effect (" , type, ") for ", y.lev[i], "\n", sep=""))    
    table <- array(if (type == "probability") {object$prob[y.categories==y.lev[i]]}
                   else {object$logit[y.categories==y.lev[i]]},     
                   dim=sapply(object$variables, function(x) length(x$levels)),
                   dimnames=lapply(object$variables, function(x) x$levels))
    print(table, ...)
  }
  if (is.null(object$confidence.level)) return(invisible(NULL))
  for (i in 1:length(y.lev)){
    cat(paste("\n", 'Lower', object$confidence.level*100, 'Percent Confidence Limits for'
              , y.lev[i],'\n'))
    table <- if (type == "probability") object$lower.prob else object$lower.logit
    table <- array(table[y.categories==y.lev[i]],     
                   dim=sapply(object$variables, function(x) length(x$levels)),
                   dimnames=lapply(object$variables, function(x) x$levels))
    print(table, ...)
  }
  for (i in 1:length(y.lev)){
    cat(paste("\n", 'Upper', object$confidence.level*100, 'Percent Confidence Limits for'
              , y.lev[i],'\n'))
    table <- if (type == "probability") object$upper.prob else object$upper.logit
    table <- array(table[y.categories==y.lev[i]],     
                   dim=sapply(object$variables, function(x) length(x$levels)),
                   dimnames=lapply(object$variables, function(x) x$levels))
    print(table, ...)
  }
  if (object$discrepancy > 0.1) cat(paste("\nWarning: There is an average discrepancy of", 
                                          round(object$discrepancy, 2),
                                          "percent \n     in the 'safe' predictions for effect", object$term, '\n'))
  invisible(NULL)
}

print.efflatent <- function(x, ...){
  cat(paste("\n", gsub(":", "*", x$term), 'effect\n'))
  table <- array(x$fit,     
                 dim=sapply(x$variables, function(x) length(x$levels)),
                 dimnames=lapply(x$variables, function(x) x$levels))
  print(table, ...)
  cat("\nThresholds:\n")
  print(x$thresholds, ...)
  if (x$discrepancy > 0.1) cat(paste("\nWarning: There is an average discrepancy of", 
                                     round(x$discrepancy, 3),
                                     "percent \n     in the 'safe' predictions for effect", x$term, '\n'))
  invisible(x)
}

summary.efflatent <- function(object, ...){
  result <- list()
  result$header <- paste("\n", gsub(":", "*", object$term), 'effect\n')
  result$effect <- array(object$fit,     
                         dim=sapply(object$variables, function(x) length(x$levels)),
                         dimnames=lapply(object$variables, function(x) x$levels))
  if (!is.null(object$se)){
    result$lower.header <- paste('\n Lower', round(100*object$confidence.level, 2), 
                                 'Percent Confidence Limits\n')
    result$lower <- array(object$lower,   
                          dim=sapply(object$variables, function(x) length(x$levels)),
                          dimnames=lapply(object$variables, function(x) x$levels))
    result$upper.header <- paste('\n Upper', round(100*object$confidence.level, 2),
                                 'Percent Confidence Limits\n')
    result$upper <- array(object$upper,   
                          dim=sapply(object$variables, function(x) length(x$levels)),
                          dimnames=lapply(object$variables, function(x) x$levels))
  }
  result$thresholds <- object$thresholds
  if (object$discrepancy > 0.1) result$warning <- paste("\nWarning: There is an average discrepancy of", 
                                                        round(object$discrepancy, 3),
                                                        "percent \n     in the 'safe' predictions for effect", object$term, '\n')
  class(result) <- "summary.eff"
  result
}

## ----setopts,echo=FALSE-------------------------------------------------------
library("knitr")
opts_chunk$set(fig.width=5,fig.height=5,tidy=TRUE,
               out.width="0.8\\textwidth",echo=TRUE)
options(prompt=" ")

## ----echo=FALSE, results='hide', include=FALSE--------------------------------
#options(continue="+    ", prompt="R> ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)

## ----include=FALSE------------------------------------------------------------
library(knitr)
opts_chunk$set(
tidy=FALSE,fig.width=5,fig.height=5,cache=FALSE,comment=NA, prompt=TRUE
)
render_sweave()

## ----echo=FALSE, results='hide', include=FALSE----------------------------
options(continue="    ", prompt=" ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)


## ----eval=FALSE-----------------------------------------------------------
#  Effect.gls <- function(focal.predictors, mod, ...){
#    cl <- mod$call
#    cl$weights <- NULL
#    args <- list(
#      type = "glm",
#      call = cl,
#      formula = formula(mod),
#      family = NULL,
#      coefficients = coef(mod),
#      vcov = as.matrix(vcov(mod)),
#      method=NULL)
#    Effect.default(focal.predictors, mod, ..., sources=args)
#  }

## ----fig.height=4,fig.width=8---------------------------------------------
library(effects)
require(nlme)
g <- gls(Employed ~ GNP + Population,
         correlation=corAR1(form= ~ Year), data=longley)
plot(predictorEffects(g))

## -------------------------------------------------------------------------
print(Effect.lme)

## -------------------------------------------------------------------------
data(Orthodont, package="nlme")
m1 <- nlme::lme(distance ~ age + Sex, data=Orthodont, 
                random= ~ 1 | Subject)
as.data.frame(Effect("age", m1))

## -------------------------------------------------------------------------
print(Effect.merMod)

## ----fig.height=4,fig.width=8---------------------------------------------
fm2 <- lme4::lmer(distance ~ age + Sex + (1 |Subject), data
                  = Orthodont)
plot(allEffects(fm2))

## -------------------------------------------------------------------------
data(cbpp, package="lme4")
gm1 <- lme4::glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
              data = cbpp, family = binomial)
as.data.frame(predictorEffect("period", gm1))

## -------------------------------------------------------------------------
print(Effect.rlmerMod)

## ----eval=FALSE,fig.height=4, fig.width=8---------------------------------
#  require(lme4)
#  fm3 <- robustlmm::rlmer(distance ~ age * Sex + (1 |Subject),
#                          data = Orthodont)
#  plot(predictorEffects(fm3))

## -------------------------------------------------------------------------
print(Effect.betareg)

## ----echo=FALSE,results='hide', include=FALSE-----------------------------
require(lme4)

## ----fig.height=4,fig.width=8,cache=FALSE---------------------------------
require(betareg)
require(lme4)
data("GasolineYield", package = "betareg")
gy_logit <- betareg(yield ~ batch + temp, data = GasolineYield)
summary(gy_logit)
plot(predictorEffects(gy_logit))

## -------------------------------------------------------------------------
print(Effect.clm)

## ----echo=FALSE,results='hide', include=FALSE-----------------------------
require(ordinal)
require(MASS)

## ----fig.height=6,fig.width=6---------------------------------------------
require(ordinal)
require(MASS)
mod.wvs1 <- clm(poverty ~ gender + religion + degree + country*poly(age,3),
    data=WVS)
plot(Effect(c("country", "age"), mod.wvs1), 
     lines=list(multiline=TRUE), layout=c(2, 2))

## -------------------------------------------------------------------------
print(Effect.clm2)

## ----fig.height=6,fig.width=8---------------------------------------------
v2 <- clm2(poverty ~ gender + religion + degree + country*poly(age,3),data=WVS)
plot(emod2 <- Effect(c("country", "age"), v2), 
     lines=list(multiline=TRUE), layout=c(2,2))

## -------------------------------------------------------------------------
print(Effect.clmm)

## ----fig.height=4,fig.width=4,cache=FALSE---------------------------------
require(ordinal)
require(MASS)
mm1 <- clmm(SURENESS ~ PROD + (1|RESP) + (1|RESP:PROD), 
        data = soup, link = "logit", threshold = "flexible")
plot(Effect("PROD", mm1),lines=list(multiline=TRUE))

## ----fig.height=6,fig.width=6---------------------------------------------
data(Baumann, package="carData")
b1 <- lm(cbind(post.test.1, post.test.2, post.test.3) ~ group + 
    pretest.1 + pretest.2, data = Baumann)
plot(Effect("group", b1))

regression.tests <- FALSE

if (regression.tests && require("carData") && require("effects")){

  load("regression-tests.RData")
  
  regressionTest <- function(old, new){
    test.name <- deparse(substitute(new))
    if (!isTRUE(all.equal(old, as.data.frame(new), tolerance=1e-5))) {
      stop("failed regression test ", test.name)
    } else {
      cat("\n", test.name, ": OK", sep="")
    }
  }
  
  m.lm <- lm(prestige~(income + education)*type, data=Duncan)
  eff.lm <- Effect(c("income", "type"), m.lm)
  regressionTest(df.lm, eff.lm)
  
  m.glm <- glm(volunteer ~ sex + neuroticism*extraversion, data=Cowles, family=binomial)
  eff.glm <- Effect(c("neuroticism", "extraversion"), m.glm)
  regressionTest(df.glm, eff.glm)
  
  m.mlm <- lm(cbind(post.test.1, post.test.2, post.test.3) ~ 
                group*(pretest.1 + pretest.2), data = Baumann)
  eff.mlm <- Effect(c("group", "pretest.1"), m.mlm)
  regressionTest(df.mlm, eff.mlm)
  
  if (require(nnet)){
    m.multinom <- multinom(vote ~ age + gender + economic.cond.national +
                           economic.cond.household + Blair + Hague + Kennedy +
                           Europe*political.knowledge, data=BEPS)
    
    eff.multinom <- Effect(c("Europe", "political.knowledge"), m.multinom,
                xlevels=list(political.knowledge=0:3))
    regressionTest(df.multinom, eff.multinom)
  }
  
  if (require(MASS)){
    m.polr <- polr(poverty ~ gender + religion + degree + country*poly(age,3),
                    data=WVS)
    
    eff.polr <- Effect(c("country", "age"), m.polr)
    regressionTest(df.polr, eff.polr)
  }
  
  if (require(nlme)){
    m.lme <- lme(distance ~ age*Sex, data = Orthodont, random = ~ 1)
    eff.lme <- Effect(c("age", "Sex"), m.lme)
    m.gls <- gls(Employed ~ GNP*Population,
        correlation=corAR1(form= ~ Year), data=longley)
    eff.gls <- Effect(c("GNP", "Population"), m.gls)
    regressionTest(df.gls, eff.gls)
  }
  
  if (require(lme4) && require(nlme)){
     data("Orthodont", package="nlme")
     m.lmer <- lmer(distance ~ age*Sex + (1 |Subject), data = Orthodont)
     eff.lmer <- Effect(c("age", "Sex"), m.lmer)
     regressionTest(df.lmer, eff.lmer)
  }
  
  if (require(lme4)){
    m.glmer <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                data = cbpp, family = binomial)
    eff.glmer <- Effect("period", m.glmer)
    regressionTest(df.glmer, eff.glmer)
  }
  
  if (require(lme4) && require(robustlmm)){
      m.rlmer <- rlmer(distance ~ age*Sex + (1 |Subject), data = Orthodont)
      eff.rlmer <- Effect(c("age", "Sex"), m.rlmer)
      regressionTest(df.rlmer, eff.rlmer)
  }
  
  if (require(betareg) && require(lme4)){
     data("GasolineYield", package = "betareg")
     m.betareg <- betareg(yield ~ batch*temp, data = GasolineYield)
     eff.betareg <- Effect(c("batch", "temp"), m.betareg)
     regressionTest(df.betareg, eff.betareg)
  }
  
  if (require(ordinal) && require(MASS)){
    m.clm <- clm(poverty ~ gender + religion + degree + country*poly(age,3),
        data=WVS)
    eff.clm <- Effect(c("country", "age"), m.clm)
    regressionTest(df.clm, eff.clm)
    
    m.clm2 <- clm2(poverty ~ gender + religion + degree + country*poly(age,3),
                 data=WVS)
    eff.clm2 <- Effect(c("country", "age"), m.clm2)
    regressionTest(df.clm2, eff.clm2)
    
    m.clmm <- clmm(SURENESS ~ PROD + (1|RESP) + (1|RESP:PROD), 
                data = soup, link = "logit", threshold = "flexible")
    eff.clmm <- Effect("PROD", m.clmm)
    regressionTest(df.clmm, eff.clmm)
  }
  
  if (require(poLCA)){
    set.seed(123)
    data(election, package="poLCA")
    f <- cbind(MORALG,CARESG,KNOWG,LEADG,DISHONG,INTELG,
               MORALB,CARESB,KNOWB,LEADB,DISHONB,INTELB)~PARTY
    m.poLCA <- poLCA(f, election, nclass=3, nrep=5)
    eff.poLCA <- Effect("PARTY", m.poLCA)
    regressionTest(df.poLCA, eff.poLCA)
  }
  
  if (require(survey)){
    data(api, package="survey")
    dstrat <-svydesign(id=~1,strata=~stype, weights=~pw, data=apistrat, fpc=~fpc)
    m.svyglm <- svyglm(api00~ell*meals+mobility, design=dstrat)
    eff.svyglm <- Effect(c("ell", "meals"), m.svyglm)
    regressionTest(df.svyglm, eff.svyglm)
  }
  
  if(require(nlme) && require(MASS)){
    m.glmmPQL <- glmmPQL(y ~ trt*I(week > 2), random = ~ 1 | ID,
                    family = binomial, data = bacteria)
    eff.glmmPQL <- Effect(c("trt", "week"), m.glmmPQL)
    regressionTest(df.glmmPQL, eff.glmmPQL)
  }

}

# the following commented-out code creates the comparison objects:
#
# df.betareg <- as.data.frame(eff.betareg )
# df.clm <- as.data.frame(eff.clm) 
# df.clm2 <- as.data.frame(eff.clm2)
# df.clmm <- as.data.frame(eff.clmm) 
# df.glm <- as.data.frame(eff.glm) 
# df.glmer <- as.data.frame(eff.glmer) 
# df.gls <- as.data.frame(eff.gls) 
# df.lm <- as.data.frame(eff.lm) 
# df.lme <- as.data.frame(eff.lme) 
# df.lmer <- as.data.frame(eff.lmer) 
# df.mlm <- as.data.frame(eff.mlm) 
# df.multinom <- as.data.frame(eff.multinom) 
# df.poLCA <- as.data.frame(eff.poLCA) 
# df.polr <- as.data.frame(eff.polr) 
# df.rlmer <- as.data.frame(eff.rlmer) 
# df.svyglm <- as.data.frame(eff.svyglm)
# df.glmmPQL <- as.data.frame(eff.glmmPQL)
# 
# 
# save(df.betareg, df.clm, df.clm2, df.clmm, df.glm,
#      df.glmer, df.gls, df.lm, df.lme, df.lmer, df.mlm,
#      df.multinom, df.poLCA, df.polr, df.rlmer, df.svyglm,
#      df.glmmPQL,
#      file="regression-tests.RData")

# To add to regression tests first
# load("regression-tests.RData")

# 2017-08-14 fixed bug in plot.predictoreff on passing 'multiline' to lines list
# 2017-08-30 for compatibility with other effect plots, default
#            is now multiline=FALSE
# 2017-11-09 fixed bug in setting the class for multinom models, and possibly others
# 2017-11-17 added methods for clm, clm2, clmm in the file effectsclmm.R
# 2017-12-08 modified predictorEffect.default and predictorEffects.default for compatibility to Effect.default
# 2018-01-09 fixed bug in predictorEffects.default with log() in a formula.
# 2018-01-24 fixed bug with minus sign in a formula predictorEffects.default
# 2018-05-14 predictorEffect.default() calls Effect() with x.var=1
# 2018-06-07 predictorEffects now works with offsets.
# 2018-08-09 removed explicit 'xlevels' argument from predictorEffects, so the argument is correctly passed with ...
# 2018-10-19: changed class of predictorefflist to c("predictorefflist", "efflist", "list")
# 2018-11-19: added xlevels argument with default 5 to be applied to conditioning predictors and
#             focal.levels argument with default 50 to be applied to focal predictor. J. Fox
# 2019-04-13: changed behavior of xlevels default to match Effect.lm() when residuals=TRUE. J. Fox
# 2020-05-29: use find_formula in the 'insight' package to find formulas.  S. Weisberg
# 2020-06-23: modified predictorEffect.default to get formula using effSources. S Weisberg

# removed xlevels argument 8/9/18
predictorEffect <- function(predictor, mod, focal.levels=50, xlevels=5, ...){  
  UseMethod("predictorEffect", mod)
}

# removed xlevels argument 8/9/18
predictorEffect.svyglm <- function(predictor, mod, focal.levels=50, xlevels=5, ...){
  mod$call <- list(mod$call, data=mod$data)
  NextMethod(object=mod, ...)
}

#simplified 12/10/17
# removed xlevels argument 8/9/18
predictorEffect.default <- function(predictor, mod, focal.levels=50, xlevels=5, ..., sources){
  dots <- list(...)
  which.residuals <- which(!is.na(sapply(names(dots), 
                                         function(x) pmatch(x, c("residuals", "partial.residuals")))))
  if (length(which.residuals) != 0){
    if (isTRUE(dots[[which.residuals]]) && missing(xlevels)) xlevels <- list()
  }
  sources <- if(missing(sources)) effSources(mod) else sources
  form <- if(is.null(sources$formula)) 
    find_formula(mod)$conditional else sources$formula
  all.vars <- all.vars(parse(text=form))
# all.vars <- find_terms(mod, flatten=TRUE)
  # find the right effect to use
  terms <- attr(terms(form), "term.labels")
  # get the predictor names:
  predictors <- all.vars(parse(text=terms))
# predictors <- find_terms(m2)$conditional
  sel <- which(predictors == predictor)
  if(length(sel) != 1) stop("First argument must be the quoted name of one predictor in the formula")
  if (is.numeric(xlevels)){
    if (length(xlevels) > 1 || round(xlevels != xlevels)) stop("xlevels must be a single whole number or a list")
    xlevs <- list()
    for (pred in predictors[-sel]){
      xlevs[[pred]] <- xlevels
    }
    xlevels <- xlevs
  }
  xlevels[[predictor]] <- focal.levels
  # create correspondence table
  decode <- function(name) all.vars(parse(text=unlist(strsplit(name, ":"))))
  tab <- rep(FALSE, length(terms))
  for(j in 1:length(terms)){if(predictor %in% decode(terms[j])) tab[j] <- TRUE}
  ans <- unlist(strsplit(paste(terms[tab], collapse=":"), ":"))
  ans <- unique(all.vars(parse(text=ans)))
  ans <- unique(c(predictor, ans)) # guarantees focal predictor is first
  args <- names(list(...))
  result <- if ("x.var" %in% args) Effect(ans, mod, xlevels=xlevels, ...) else Effect(ans, mod, x.var=1, xlevels=xlevels, ...)
  class(result) <- c("predictoreff", class(result))
  result
}

predictorEffects <- function(mod, predictors, focal.levels=50, xlevels=5, ...){
  UseMethod("predictorEffects", mod)
}


# rewritten, simplified, 12/08/17, bug in formulas fixed 01/24/2018
predictorEffects.default <- function(mod, predictors = ~ ., focal.levels=50, xlevels=5, ..., sources) {
  dots <- list(...)
  which.residuals <- which(!is.na(sapply(names(dots), 
                           function(x) pmatch(x, c("residuals", "partial.residuals")))))
  if (length(which.residuals) != 0){
    if (isTRUE(dots[[which.residuals]]) && missing(xlevels)) xlevels <- list()
  }
# The next function removes offset(s) from a formula, used for mform and cform
  no.offset <- function(x, preserve = NULL) {
    k <- 0
    proc <- function(x) {
      if (length(x) == 1) return(x)
      if (x[[1]] == as.name("offset") && !((k<<-k+1) %in% preserve)) return(x[[1]])
      replace(x, -1, lapply(x[-1], proc))
    }
    update(proc(x), . ~ . - offset)}
  sources <- if(missing(sources)) effSources(mod) else sources
  form <- if(is.null(sources$formula)) {find_formula(mod)$conditional} else sources$formula
  mform <- no.offset(form) # replacement for next line
#  mform <- no.offset(Effect.default(NULL, mod))  # returns the fixed-effect formula for any method
  cform <- if(is.character(predictors)) 
    as.formula(paste("~", paste(predictors, collapse="+"))) else
      predictors
  cform <- update(as.formula(paste(". ~", 
                paste(all.vars(formula(mform)[[3]]), collapse="+"))), 
                cform)
  cform <- no.offset(cform)
  mvars <- all.vars(mform[[3]])
  cvars <- all.vars(cform[[3]])
  if (is.list(focal.levels)){
    for(cvar in cvars){
      if (!is.null(focal.levels[[cvar]])) next
      focal.levels[[cvar]] <- 50
    }
  } else{
    if (!is.vector(focal.levels) || !is.numeric(focal.levels) || length(focal.levels) > 1 || round(focal.levels) != focal.levels)
      stop("focal.levels must be a length 1 positive\nwhole-number, numeric vector or a list")
  }
  if (length(xlevels) > 0){
    if (is.list(xlevels)){
      for(mvar in mvars){
        if (!is.null(xlevels[[mvar]])) next
        xlevels[[mvar]] <- 5
      }
    } else{
      if (!is.vector(xlevels) || !is.numeric(xlevels) || length(xlevels) > 1 || round(xlevels) != xlevels)
        stop("xlevels must be a length 1 positive\nwhole-number, numeric vector or a list")
    }
  }
# check that 'cvars' is a subset of 'mvars'. If so apply predictorEffect
  if(!all(cvars %in% mvars)){
    stop("argument 'predictors' not a subset of the predictors in the formula") 
  } else {
    result <- list()
    for(p in cvars){
      flevs <- if (is.numeric(focal.levels)) focal.levels else focal.levels[[p]]
      result[[p]] <- predictorEffect(p, mod, focal.levels=flevs, xlevels=xlevels, ...,
                                     sources=sources)
    }
  }
  class(result) <- c("predictorefflist", "efflist", "list")
  result
}

# plot methods

plot.predictoreff <- function(x, x.var,
                              main = paste(names(x$variables)[1], "predictor effect plot"), ...){
  if(missing(x.var)) x.var <- names(x$variables)[1]
  NextMethod(x, x.var=x.var, main=main, ...)
}


plot.predictorefflist <- function(x, selection, rows, cols, ask=FALSE, graphics=TRUE,
                                  lattice, ...){
  # Next line added 8/23/17 along with lattice, also lattice arg above
  lattice <- if(missing(lattice)) list() else lattice
  if (!missing(selection)){
    if (is.character(selection)) selection <- gsub(" ", "", selection)
    return(plot(x[[selection]], ...))
  }
  effects <- gsub(":", "*", names(x))
  if (ask){
    repeat {
      selection <- menu(effects, graphics=graphics, title="Select Term to Plot")
      if (selection == 0) break
      else print(plot(x[[selection]], ...))
    }
  }
  else {
    neffects <- length(x)
    mfrow <- mfrow(neffects)
    if (missing(rows) || missing(cols)){
      rows <- mfrow[1]
      cols <- mfrow[2]
    }
    for (i in 1:rows) {
      for (j in 1:cols){
        if ((i-1)*cols + j > neffects) break
        more <- !((i-1)*cols + j == neffects)
        lattice[["array"]] <- list(row=i, col=j, nrow=rows, ncol=cols, more=more)
        print(plot(x[[(i-1)*cols + j]], lattice=lattice,
                   ...))
      }
    }
  }
}


# print and summary methods

print.predictorefflist <- function(x, ...){
  for (eff in x){
    print(eff, ...)
  }
  invisible(x)
}

print.predictoreff <- function(x, ...){
  cat("\n", names(x$variables)[1], "predictor effect\n")
  NextMethod()
}

summary.predictorefflist <- function(object, ...){
  for (eff in object){
    cat("\n", names(eff$variables)[1], "predictor effect\n")
    print(summary(eff, ...))
  }
}

# 12/11/2017:  S. Weisberg.  This file contains all the Effect methods that call
# Effect.default.  Excluded are Effect.lm, Effect.polr, and Effect.multinom, 
# and for now Effect.svyglm.
# 06/08/2018: rewrote method for betareg, removing the 'link' argument from sources
# 11/28/2018:  modified Effect.gls to ignore the weights argument by 
#              deleting it from sources$call.
# 11/30/2018: fixed bug in Effect.merMod() specifying fam$family explicitly.
# 7/5/2019:  clm clm2 and clmm were not passing the estimated threshholds to polr
# 3/22/2020:  added Effect.glmmPQL (from MASS package)
# 4/27/2020:  require 'insight' package for find_formula and get_coefficients
#             so formula and coefficients are generally not needed
# 2020-06-13: fix typo (omitted ') in an error message
# 2020-06-23: All the Effect.* methods previously in this file have been removed
#             and replaced by effSources.* methods.  

effSources <- function(mod){
  UseMethod("effSources", mod)
}

effSources.default <- function(mod){NULL}

# lme, nlme package - default works

# gls, nlme package
effSources.gls <- function(mod){
  cl <- mod$call
  cl$weights <- NULL
  list(call = cl)
}

# glmmPQL method 3/22/2020
effSources.glmmPQL <- function(mod) {list(family = mod$family)}

# lme4 -- handled via an Effect method to allow for KR argument
# effSources.merMod <- function(mod){NULL}

# rlmer in robustlmm package, not really needed
effSources.rlmerMod <- function(mod){NULL}

# clm in the ordinal package. clm is not supported by insight package
effSources.clm <- function(mod){
  if (requireNamespace("MASS", quietly=TRUE)){
    polr <- MASS::polr} else stop("MASS package is required")
  polr.methods <- c("logistic", "probit", "loglog", 
                    "cloglog", "cauchit")
  method <- mod$link
  if(method == "logit") method <- "logistic"
  if(!(method %in% polr.methods)) 
    stop("'link' must be a 'method' supported by polr; see help(polr)")
  if(mod$threshold != "flexible") 
    stop("Effects only supports the 'flexible' threshold")
  numTheta <- length(mod$Theta)
  numBeta <- length(mod$beta)
  or <- c( (numTheta+1):(numTheta + numBeta), 1:(numTheta))
  list(
    type = "polr",
    coefficients = mod$beta,
    zeta = mod$alpha,  
    method=method,
    vcov = as.matrix(vcov(mod)[or, or]))
}

# clm2, this is supported by insight package
effSources.clm2 <- function(mod){
  if (requireNamespace("MASS", quietly=TRUE)){
      polr <- MASS::polr}
  polr.methods <- c("logistic", "probit", "loglog", 
                    "cloglog", "cauchit")
  method <- mod$link
  if(!(method %in% polr.methods)) 
    stop("'link' must be a 'method' supported by polr; see help(polr)")
  if(is.null(mod$Hessian)){
     message("\nRe-fitting to get Hessian\n")
     mod <- update(mod, Hess=TRUE)}
  if(mod$threshold != "flexible") 
    stop("Effects only supports the flexible threshold")
  numTheta <- length(mod$Theta)
  numBeta <- length(mod$beta)
  or <- c( (numTheta+1):(numTheta + numBeta), 1:(numTheta))
  list(
    type = "polr",
    formula = mod$call$location,
    coefficients = mod$beta,  
    zeta = mod$Theta,
    method=method,
    vcov = as.matrix(vcov(mod)[or, or]))
}

#clmm in ordinal package
effSources.clmm <- function(mod){ 
  if (requireNamespace("MASS", quietly=TRUE)){
    polr <- MASS::polr}
  else stop("The MASS package must be installed")
  polr.methods <- c("logistic", "probit", "loglog", 
                    "cloglog", "cauchit")
  method <- mod$link
  if(method == "logit") method <- "logistic"
  if(!(method %in% polr.methods)) 
    stop("'link' must be a 'method' supported by polr; see help(polr)")
  if(is.null(mod$Hessian)){
    message("\nRe-fitting to get Hessian\n")
    mod <- update(mod, Hess=TRUE)}
  if(mod$threshold != "flexible") 
    stop("Only threshold='flexible' is supported by effects")
  numTheta <- length(mod$Theta)
  numBeta <- length(mod$beta)
  or <- c( (numTheta+1):(numTheta + numBeta), 1:(numTheta))
  Vcov <- as.matrix(vcov(mod)[or, or])
  list(
    type = "polr",
    formula = insight::find_formula(mod)$conditional,
    coefficients = mod$beta,
    zeta=mod$alpha,
    method=method,
    vcov = as.matrix(Vcov))
}

# betareg from the betareg package
effSources.betareg <- function(mod){
  coef <- mod$coefficients$mean
  vco <- vcov(mod)[1:length(coef), 1:length(coef)]
# betareg uses beta errors with mean link given in mod$link$mean.  
# Construct a family based on the binomial() family
  fam <- binomial(link=mod$link$mean)
# adjust the variance function to account for beta variance
  fam$variance <- function(mu){
    f0 <- function(mu, eta) (1-mu)*mu/(1+eta)
    do.call("f0", list(mu, mod$coefficient$precision))}
# adjust initialize
  fam$initialize <- expression({mustart <- y})
# collect arguments
  args <- list(
    call = mod$call,
    formula = formula(mod),
    family=fam,
    coefficients = coef,
    vcov = vco)
  args
}

# utilities and common functions for effects package
# John Fox, Jangman Hong, and Sanford Weisberg

# 7-25-2013 S. Weisberg modified analyze.model and Analyze.model to ignore
#     default.levels, and use xlevels to set default.  Use grid.pretty by default
# 11-09-2013: fixed error message in Analyze.model(), bug reported by Joris Meys. J. Fox
# 2013-10-15: eliminated functions not needed after effect() methods removed. J. Fox
# 2013-10-29: fixed as.data.frame.*() to handle NA levels. J. Fox
# 2014-03-13: modified Fixup.model.matrix() and Analyze.model() to handle partial residuals; 
#     added is.factor.predictor() and is.numeric.predictor(). J. Fox
# 2014-03-14: error message for non-factor, non-numeric predictor
# 2014-07-08: if no numeric predictor, partial residuals suppressed with warning rather than an error
# 2014-10-09: namespace fixes. J. Fox
# 2015-04-08: added setStrip(), restoreStrip(). J. Fox
# 2015-07-07: fixed matchVarName() so that it handles periods in names properly. J. Fox
# 2015-09-10: added a fix for class = 'array' in Analyze.model.  S. Weisberg
# 2016-02-16: fix Analyze.model(), Fixup.model.matrix() to handle non-focal terms like polynomials correctly; clean up code. J. Fox
# 2016-03-01: correct and improve computation of partial residuals
# 2017-07-10: fix warnings about 1 x 1 arrays produced in eff.mul() and eff.polr() in R 3.4.0 (reported by Stefan Th. Gries). J. Fox
# 2017-07-14: added applyDefaults() and isFALSE(). J. Fox
# 2017-07-27: added effectsTheme(); removed setStrip(), restoreStrip(). J. Fox
# 2017-08-08: added .onAttach() to set lattice theme. J. Fox
# 2017-08-26: added scheffe() to compute multipler for Scheffe-type confidence bounds. J. Fox
# 2017-08-29: enhanced applyDefaults() with onFALSE argument. J. Fox
# 2017-09-02: added nice()
# 2017-09-08: small changes to accommodate Effect.svyglm()
# 2017-09-10: added replacement for ticksGrid()
# 2018-05-09: fix typo in startup message
# 2018-05-13: modified Analyze.model() to support partial-residual plots against factors.
# 2018-08-17: modified .onAttach() so that trellis device doesn't open, suggestion of Kurt Hornik.
# 2018-10-06: modified as.data.frame, adding a 'type' argument and deleting the 'transformation' argument, using the mu.eta function from the defining family
# 2018-10-19: added as.data.frame.efflist
# 2018-10-25: as.data.frame.eff() fixed so that deletion of the transformation argument doesn't break plot.eff(). J. Fox
# 2018-12-19: accommodate character and logical predictors. J. Fox
# 2019-08-27: correctly handle logical or character predictor with residuals
# 2019-08-30: further fixes to character and logical predictors
# 2019-10-24: add color options (e.g., for colorblind palette, suggestion of ) to effectsTheme(). J. Fox
# 2019-11-14: change class(x) == "y" to inherits(x, "y")
# 2022-02-18: insure that levels of focal predictor in returned objects are in proper order (bug reported by Christoph Scherber, didn't affect plots or tables).
# 2023-02-19: added levels2dates() and methods. J. Fox (request of Christoph Scherber).
# 2025-07-21: rename to is_factor_predictor() and is_numeric_predictor() to avoid confusion. J. Fox

has.intercept <- function(model, ...) any(names(coefficients(model))=="(Intercept)")

term.names <- function (model, ...) {
  term.names <- gsub(" ", "", labels(terms(model)))
  if (has.intercept(model)) c("(Intercept)", term.names)
  else term.names
}

response.name <- function (model, ...) deparse(attr(terms(model), "variables")[[2]])

mfrow <- function(n, max.plots=0){
  # number of rows and columns for array of n plots
  if (max.plots != 0 & n > max.plots)
    stop(paste("number of plots =",n," exceeds maximum =", max.plots))
  rows <- round(sqrt(n))
  cols <- ceiling(n/rows)
  c(rows, cols)
}

expand.model.frame <- function (model, extras, envir = environment(formula(model)),
                        na.expand = FALSE){  # modified version of R base function
  f <- formula(model)
  data <- eval(model$call$data, envir)
  ff <- foo ~ bar + baz
  if (is.call(extras)) 
    gg <- extras
  else gg <- parse(text = paste("~", paste(extras, collapse = "+")))[[1]]
  ff[[2]] <- f[[2]]
  ff[[3]][[2]] <- f[[3]]
  ff[[3]][[3]] <- gg[[2]]
  if (!na.expand) {
    naa <- model$call$na.action
    subset <- model$call$subset
    rval <- if (is.null(data)) eval(call("model.frame", ff, # modified
                                         subset = subset, na.action = naa), envir)           #  lines
    else eval(call("model.frame", ff, data = data,          #
                   subset = subset, na.action = naa), envir)           #
  }
  else {
    subset <- model$call$subset
    rval <- eval(call("model.frame", ff, data = data, subset = subset, 
                      na.action = I), envir)
    oldmf <- model.frame(model)
    keep <- match(rownames(oldmf), rownames(rval))
    rval <- rval[keep, ]
    class(rval) <- "data.frame"
  }
  return(rval)
}

is.relative <- function(term1, term2, factors) {
  all(!(factors[,term1]&(!factors[,term2])))
}

descendants <- function(term, mod, ...){
  names <- term.names(mod)
  if (has.intercept(mod)) names <- names[-1]
  if(length(names)==1) return(NULL)
  which.term <- which(term == names)
  if (length(which.term) == 0){
    factors <- attr(terms(...), "factors")
    rownames(factors) <- gsub(" ", "", rownames(factors))
    colnames(factors) <- gsub(" ", "", colnames(factors))
    (1:length(names))[sapply(names,
                             function(term2) is.relative(term, term2, factors))]
  }
  else {
    factors <- attr(terms(mod), "factors")
    rownames(factors) <- gsub(" ", "", rownames(factors))
    colnames(factors) <- gsub(" ", "", colnames(factors))
    (1:length(names))[-which.term][sapply(names[-which.term],
                                          function(term2) is.relative(term, term2, factors))]
  }
}

is.high.order.term <- function(term, mod,...){
  0 == length(descendants(term, mod, ...))
}

subscripts <- function(index, dims){
  subs <- function(dims, index){
    dim <- length(dims)
    if (dim == 0) return(NULL)
    cum <- c(1,cumprod(dims))[dim]
    i <- index %/% cum
    if (index %% cum != 0) i <- i + 1
    c(i, subs(dims[-dim], index - (i - 1)*cum))
  }
  rev(subs(dims, index))
}

matrix.to.df <- function(matrix, colclasses){
  opt <- options(warn = -1)
  on.exit(options(opt))
  ncol <- ncol(matrix)
  colnames <- colnames(matrix)
  colclasses[sapply(colclasses, function(x) "integer" %in% x)] <- "numeric"
  result <- vector(mode="list", length=ncol)
  names(result) <- colnames
  for (j in 1:ncol){
    result[[j]] <- matrix[, j]
    class <- colclasses[[colnames[j]]]
    result[[colnames[j]]] <- if ("numeric" %in% class) {
      decChar <- getOption('OutDec')
      if (decChar == '.') as.numeric(result[[colnames[j]]])
      else as.numeric(gsub(decChar, '.', matrix[,j]))
    }
    else if ("ordered" %in% class) ordered(result[[colnames[j]]])
    else if ("factor" %in% class) factor(result[[colnames[j]]]) 
    else result[[colnames[j]]]
  }
  as.data.frame(result)
}

# the following function is a modification of code contributed by Steve Taylor
# as.data.frame rewritten, 2018-10-06
# fixed 2018-10-25 so that plot.eff() isn't broken by the rewrite

as.data.frame.eff <- function(x, row.names=NULL, optional=TRUE, type=c("response", "link"), ...){ 
  type <- match.arg(type)
  linkinv <- if (is.null(x$link$linkinv)) I else x$link$linkinv
  linkmu.eta <- if(is.null(x$link$mu.eta)) function(x) NA else x$link$mu.eta
  xx <- x$x
  for (var in names(xx)){
    if (is.factor(xx[[var]])){
      xx[[var]] <- addNA(xx[[var]]) # handle factors with "valid" NA level
    }
  }
  x$x <- xx
  result <- switch(type, 
                   response= { if (is.null(x$se)) 
                     data.frame(x$x, fit=transform(x$fit))
                     else 
                       data.frame(x$x, 
                                  fit=linkinv(x$fit),
                                  se = linkmu.eta(x$fit) * x$se,
                                  lower=linkinv(x$lower), 
                                  upper=linkinv(x$upper))},
                   link = { if (is.null(x$se)) 
                     data.frame(x$x, fit=x$fit)
                     else 
                       data.frame(x$x, fit=x$fit, 
                                  se=x$se, 
                                  lower=x$lower, 
                                  upper= x$upper)})
  attr(result, "type") <- type
  result 
}

as.data.frame.efflist <- function(x, row.names=NULL, optional=TRUE, type, ...){
  lapply(x, as.data.frame, type)
}

as.data.frame.effpoly <- function(x, row.names=NULL, optional=TRUE, ...){
  factors <- sapply(x$variables, function(x) x$is.factor)
  factor.levels <- lapply(x$variables[factors], function(x) x$levels)
  if (!length(factor.levels) == 0){
    factor.names <- names(factor.levels)
    for (fac in factor.names){
      x$x[[fac]] <- factor(x$x[[fac]], levels=factor.levels[[fac]], exclude=NULL)
    }
  }
  result <- data.frame(x$x, x$prob, x$logit)
  if (!is.null(x$confidence.level)) result <- cbind(result,
                                                    x$se.prob, x$se.logit, x$lower.prob, x$upper.prob, x$lower.logit, x$upper.logit)
  result
}

as.data.frame.efflatent <- function(x, row.names=NULL, optional=TRUE, ...){
  xx <- x$x
  for (var in names(xx)){
    if (is.factor(xx$var)){
      xx$var <- addNA(xx$var) # handle factors with "valid" NA level
    }
  }
  x$x <- xx
  if (is.null(x$se)) data.frame(x$x, fit=x$fit)
  else data.frame(x$x, fit=x$fit, se=x$se, lower=x$lower, upper=x$upper)
}

logit2p <- function(logit) 1/(1 + exp(-logit))

p2logit <- function(p) log(p/(1 - p))


lrug <- function(x) {
  if (length(unique(x)) < 0.8 * length(x)) x <- jitter(x)
  grid.segments(x, unit(0, "npc"), x, unit(0.5, "lines"),
                default.units="native")
}

## model.response not generic
model.response.gls <- function(model){
  model.response(model.frame(as.formula(model$call$model), data=eval(model$call$data)))
}

terms.gls <- function(x, ...) terms(formula(x))

## vcov method for eff objects

vcov.eff <- function(object, ...) object$vcov

## [ method for efflist objects

`[.efflist` <- function(x, ...){
  y <- NextMethod("[")
  class(y) <- class(x)
  y
}

### the following functions are for use by Effect() methods

Analyze.model <- function(focal.predictors, mod, xlevels, default.levels=NULL, formula.rhs, 
                          partial.residuals=FALSE, quantiles, x.var=NULL, data=NULL, typical=mean){
  if ((!is.null(mod$nan.action)) && inherits(mod$na.action, "exclude"))
    class(mod$na.action) <- "omit"
  all.predictors <- all.vars(formula.rhs)
  check.vars <- !(focal.predictors %in% all.predictors)
  excluded.predictors <- setdiff(all.predictors, focal.predictors)
  number.bad <- sum(check.vars)
  if (any(check.vars)) {
    message <- if (number.bad == 1) paste("the following predictor is not in the model:", 
                                          focal.predictors[check.vars])
    else paste("the following predictors are not in the model:", 
               paste(focal.predictors[check.vars], collapse=", "))
    stop(message)
  }
  X.mod <- model.matrix(mod)
  cnames <- colnames(X.mod)
  factor.cols <- rep(FALSE, length(cnames))
  names(factor.cols) <- cnames
  for (name in all.predictors){
    if (is_factor_predictor(name, mod)) {
      factor.cols[grep(paste("^", name, sep=""), cnames)] <- TRUE
    }
  }
  factor.cols[grep(":", cnames)] <- FALSE   
  X <- na.omit(expand.model.frame(mod, all.predictors))
  
  which.matrices <- sapply(X, function(x) is.matrix(x) && ncol(x) == 1)
  if (any(which.matrices)){
    nms <- names(which.matrices[which.matrices])
    msg <- if (length(nms) > 1){
      paste("the predictors", paste(nms, collapse=", "), "are one-column matrices that were converted to vectors")
    } else {
      paste("the predictor", nms, "is a one-column matrix that was converted to a vector")
    }
    warning(msg)
    for (nm in nms){
      X[, nm] <- as.vector(X[, nm])
    }
  }
  
  for (name in all.predictors){
    if (is_factor_predictor(name, mod) && is.null(xlevels[[name]])) {
      xlevels[[name]] <- levels(X[, name]) # accomodate logical predictor
    }
  }
  bad <- sapply(X[, all.predictors, drop=FALSE], function(x) !(is.factor(x) || is.numeric(x)))
  if (any(bad)){
    message <- if (sum(bad) == 1) paste("the following predictor isn't a factor, logical, character, or numeric:", 
                                        all.predictors[bad])
    else paste("the following predictors aren't factors, logical, character, or numeric:", 
               paste(all.predictors[bad], collapse=", "))
    stop(message)
  }
  x <- list()
  factor.levels <- list()
  if(length(xlevels)==0 & length(default.levels) == 1L) xlevels <- default.levels
  if(is.numeric(xlevels) & length(xlevels) == 1L){
    levs <- xlevels
    for(name in focal.predictors) xlevels[[name]] <- levs
  }
  for (name in focal.predictors){
    levels <- mod$xlevels[[name]] ## reverted levels <- levels(X[, name]) 
    if(is.null(levels)) levels <- mod$xlevels[[paste("factor(",name,")",sep="")]] ##reverted deleted
    fac <- !is.null(levels)
    if (!fac) {    
      levels <- if (is.null(xlevels[[name]])){
        if (partial.residuals){
          quantile(X[, name], quantiles)
        }
        else{
          #         grid.pretty(range(X[, name]))
          nice(seq(min(X[, name]), max(X[, name]), length.out=5))
        }
      }
      else {
        if(length(xlevels[[name]]) == 1L) { 
          nice(seq(min(X[, name]), max(X[,name]), length=xlevels[[name]]))} else
            xlevels[[name]]}
    }
    else factor.levels[[name]] <- levels
#    x[[name]] <- list(name=name, is.factor=fac, levels=levels)
    x[[name]] <- list(name=name, is.factor=is.factor(X[, name]), levels=levels)
  }
  if (partial.residuals){
    numeric.predictors <- sapply(focal.predictors, function(predictor) is_numeric_predictor(predictor, mod))
    if (is.null(x.var)){
      x.var <- if (any(numeric.predictors)) which(numeric.predictors)[1]
      else 1
    }
    x.var.name <- focal.predictors[x.var]
    if (is.numeric(X[, x.var.name]) && is.null(xlevels[[x.var.name]])){
      x.var.range <- range(X[, focal.predictors[x.var]])
      x[[x.var]][["levels"]] <- seq(from=x.var.range[1], to=x.var.range[2], length=100)
    }
  }
  x.excluded <- list()
  for (name in excluded.predictors){
    levels <- mod$xlevels[[name]] ##reverted levels <- levels(X[, name])
    if (is.logical(X[, name])) levels <- c("FALSE", "TRUE")
    fac <- !is.null(levels)
    level <- if (fac) levels[1] else typical(X[, name])
    if (fac) factor.levels[[name]] <- levels
    x.excluded[[name]] <- list(name=name, is.factor=fac,
                               level=level)
  }
  dims <- sapply(x, function(x) length(x$levels))
  len <- prod(dims)
  n.focal <- length(focal.predictors)
  n.excluded <- length(excluded.predictors)
  n.vars <- n.focal + n.excluded
  predict.data <-matrix('', len, n.vars)
  excluded <- sapply(x.excluded, function(x) x$level)
  for (i in 1:len){
    subs <- subscripts(i, dims)
    for (j in 1:n.focal){
      predict.data[i,j] <- x[[j]]$levels[subs[j]]
    }
    if (n.excluded > 0)
      predict.data[i, (n.focal + 1):n.vars] <- excluded
  }
  colnames(predict.data) <- c(sapply(x, function(x) x$name),
                              sapply(x.excluded, function(x) x$name))
  colclasses <- lapply(X, class)
  colclasses[colclasses == "matrix"] <- "numeric"
  colclasses[colclasses == "array"] <- "numeric"
  predict.data <-  matrix.to.df(predict.data, colclasses=colclasses)
  for (i in 1:length(x)){
    if (x[[i]]$is.factor){
      predict.data[[x[[i]]$name]] <- if (is.ordered(predict.data[[x[[i]]$name]])){
        ordered(predict.data[[x[[i]]$name]], levels=x[[i]]$levels)
      } else {
        factor(predict.data[[x[[i]]$name]], levels=x[[i]]$levels)
      }
    }
  }
  list(predict.data=predict.data, 
       factor.levels=factor.levels, 
       factor.cols=factor.cols, focal.predictors=focal.predictors, n.focal=n.focal,
       excluded.predictors=excluded.predictors, n.excluded=n.excluded,
       x=x, X.mod=X.mod, cnames=cnames, X=X, x.var=x.var)   
}

Fixup.model.matrix <- function(mod, mod.matrix, mod.matrix.all, X.mod,
                               factor.cols, cnames, focal.predictors, excluded.predictors, 
                               typical, given.values, apply.typical.to.factors=FALSE){
  attr(mod.matrix, "assign") <- attr(mod.matrix.all, "assign")
  if (length(excluded.predictors) > 0){
    strangers <- Strangers(mod, focal.predictors, excluded.predictors)
    stranger.cols <-  
      apply(outer(strangers, attr(mod.matrix,'assign'), '=='), 2, any)
  }
  else stranger.cols <- rep(FALSE, ncol(mod.matrix))
  if (has.intercept(mod)) stranger.cols[1] <- TRUE
  if (any(stranger.cols)) {
    facs <- factor.cols & stranger.cols
    covs <- (!factor.cols) & stranger.cols
    if (has.intercept(mod)) covs[1] <- FALSE
    if (any(facs)){ 
      mod.matrix[,facs] <-  matrix(apply(as.matrix(X.mod[,facs]), 2, 
                                         if (apply.typical.to.factors) typical else mean), 
                                   nrow=nrow(mod.matrix), ncol=sum(facs), byrow=TRUE)
    }
    if (!is.null(given.values)){
      stranger.names <- cnames[stranger.cols]
      given <- stranger.names %in% names(given.values)
      if (any(given)) {
        mod.matrix[,stranger.names[given]] <- matrix(given.values[stranger.names[given]], nrow=nrow(mod.matrix), 
                                                     ncol=length(stranger.names[given]), byrow=TRUE)
      } 
    }
    for (name in cnames){
      components <- unlist(strsplit(name, ':'))
      components <- components[components %in% cnames]
      if (length(components) > 1) {
        mod.matrix[,name] <- apply(mod.matrix[,components], 1, prod)
      }
    }
  }
  mod.matrix
}

matchVarName <- function(name, expressions){
  scratch <- "zAMIjw4RN3" # randomly generated string
  name <- gsub("\\.", scratch, name)
  expressions <- gsub("\\.", scratch, as.character(expressions))
  a <- !grepl(paste("[.]+", name, sep=""), expressions)
  b <- !grepl(paste(name, "[.]+", sep=""), expressions)
  c <- grepl(paste("\\b", name, "\\b", sep=""), expressions)
  a & b & c
}

Strangers <- function(mod, focal.predictors, excluded.predictors){
  names <- term.names(mod)
  if (has.intercept(mod)) names <- names[-1]
  sel <- apply(sapply(excluded.predictors, matchVarName, expressions=names), 1, any)
  (1:length(sel))[sel]
}

# the following is used by effect.multinom() and Effect.multinom()

eff.mul <- function(x0, B, se, m, p, r, V){
  mu <- exp(x0 %*% B)
  mu <- mu/(1 + sum(mu))
  mu[m] <- 1 - sum(mu)
  logits <- log(mu/(1 - mu))
  if (!se) return(list(p=mu, logits=logits))
  d <- array(0, c(m, m - 1, p))
  exp.x0.B <- as.vector(exp(x0 %*% B))
  sum.exp.x0.B <- sum(exp.x0.B)
  for (j in 1:(m-1)){
    d[m, j,] <- - exp.x0.B[j]*x0
    for (jj in 1:(m-1)){
      d[j, jj,] <- if (jj != j)
        - exp(as.vector(x0 %*% (B[,jj] + B[,j])))*x0
      else exp.x0.B[j]*(1 + sum.exp.x0.B - exp.x0.B[j])*x0
    }
  }
  d <- d/(1 + sum.exp.x0.B)^2
  V.mu <- rep(0, m)
  for (j in 1:m){
    dd <- as.vector(t(d[j,,]))
    for (s in 1:r){
      for (t in 1:r){
        V.mu[j] <- V.mu[j] + V[s,t]*dd[s]*dd[t]
      }
    }
  }
  V.logits <- V.mu/(mu^2 * (1 - mu)^2)
  list(p=mu, std.err.p=sqrt(V.mu), logits=logits,
       std.error.logits=sqrt(V.logits))
}

# the following are used by effect.polr() and Effect.polr()

eff.polr <- function(x0, b, alpha, V, m, r, se){
  eta0 <- as.vector(x0 %*% b)
  mu <- rep(0, m)
  mu[1] <- 1/(1 + exp(alpha[1] + eta0))
  for (j in 2:(m-1)){
    mu[j] <- exp(eta0)*(exp(alpha[j - 1]) - exp(alpha[j]))/
      ((1 + exp(alpha[j - 1] + eta0))*(1 + exp(alpha[j] + eta0)))
  }
  mu[m] <- 1 - sum(mu)
  logits <- log(mu/(1 - mu))
  if (!se) return(list(p=mu, logits=logits))
  d <- matrix(0, m, r)
  d[1, 1] <- - exp(alpha[1] + eta0)/(1 + exp(alpha[1] + eta0))^2
  d[1, m:r] <- - exp(alpha[1] + eta0)*x0/(1 + exp(alpha[1] + eta0))^2
  for (j in 2:(m-1)){
    d[j, j-1] <- exp(alpha[j-1] + eta0)/(1 + exp(alpha[j-1] + eta0))^2
    d[j, j]   <- - exp(alpha[j] + eta0)/(1 + exp(alpha[j] + eta0))^2
    d[j, m:r] <- exp(eta0)*(exp(alpha[j]) - exp(alpha[j-1]))*
      (exp(alpha[j-1] + alpha[j] + 2*eta0) - 1) * x0 /
      (((1 + exp(alpha[j-1] + eta0))^2)*
         ((1 + exp(alpha[j] + eta0))^2))
  }
  d[m, m-1] <- exp(alpha[m-1] + eta0)/(1 + exp(alpha[m-1] + eta0))^2
  d[m, m:r] <- exp(alpha[m-1] + eta0)*x0/(1 + exp(alpha[m-1] + eta0))^2
  V.mu <- rep(0, m)
  for (j in 1:m){
    dd <- d[j,]
    for (s in 1:r){
      for (t in 1:r){
        V.mu[j] <- V.mu[j] + V[s,t]*dd[s]*dd[t]
      }
    }
  }
  V.logits <- V.mu/(mu^2 * (1 - mu)^2)
  list(p=mu, std.err.p=sqrt(V.mu), logits=logits,
       std.error.logits=sqrt(V.logits))
}

eff.latent <- function(X0, b, V, se){
  eta <- X0 %*% b
  if (!se) return(list(fit=eta))
  var <- diag(X0 %*% V %*% t(X0))
  list(fit=eta, se=sqrt(var))
}

# determine class of a predictor

# is_factor_predictor <- function(predictor, model) {
#   !is.null(model$xlevels[[predictor]])
# }

is_factor_predictor <- function(predictor, model) {
  predictor %in% names(attr(model.matrix(model), "contrasts"))
}

is_numeric_predictor <- function(predictor, model) {
  is.null(model$xlevels[[predictor]])
}

# custom lattice theme

# effectsTheme <- function(strip.background=list(col=gray(seq(0.95, 0.5, length=3))),
#                          strip.shingle=list(col="black"), clip=list(strip="off"),
#                          superpose.line=list(lwd=c(2, rep(1, 6)))){
#   
#   current <- sapply(c("strip.background", "strip.shingle", "clip", "superpose.line"),
#                     trellis.par.get)
#   result <- list(strip.background=strip.background, strip.shingle=strip.shingle, clip=clip,
#                  superpose.line=superpose.line)
#   attr(result, "current") <- current
#   result
# }

effectsTheme <- function (strip.background = list(col = gray(seq(0.95, 0.5, length = 3))), 
                          strip.shingle = list(col = "black"), clip = list(strip = "off"), 
                          superpose.line = list(lwd = c(2, rep(1, 6))), col){
  car.palette <- c("blue", "magenta", "cyan", "orange", "gray", "green3", "red")
  colorblind.palette <- rgb(red = c(230, 86, 0, 240, 0, 213, 204),
                            green = c(159, 180, 158, 228, 114, 94, 121),
                            blue  = c(0, 233, 115, 66, 178, 0, 167),
                            names = c("orange", "sky.blue", "bluish.green", "yellow", 
                                      "blue", "vermillion", "reddish.purple"),
                            maxColorValue = 255)
  # colorblind palette from https://jfly.uni-koeln.de/color/ (ignoring "black")
  current <- sapply(c("strip.background", "strip.shingle", 
                      "clip", "superpose.line"), lattice::trellis.par.get)
  if (!missing(col)){
    superpose.line$col <- if (col[1] == "colorblind"){
      colorblind.palette
    } else if (col[1] == "car") {
      car.palette
    } else if (col[1] == "R") {
      palette()[-1]
    } else {
      col
    }
  }
  result <- list(strip.background = strip.background, strip.shingle = strip.shingle, 
                 clip = clip, superpose.line = superpose.line)
  attr(result, "current") <- current
  result
}

.onAttach <- function(libname, pkgname){
  if (!"package:lattice" %in% search()){
    lattice::lattice.options(default.theme = effectsTheme)
    packageStartupMessage("lattice theme set by effectsTheme()",
                          "\nSee ?effectsTheme for details.")
  }
  else packageStartupMessage("Use the command",
                             "\n    lattice::trellis.par.set(effectsTheme())",
                             "\n  to customize lattice options for effects plots.",
                             "\nSee ?effectsTheme for details.")
}

# to handle defaults for list-style arguments

applyDefaults <- function(args, defaults, onFALSE, arg=""){
  if (is.null(args)) return(defaults)
  if (isFALSE(args)) {
    if (missing(onFALSE)) return(FALSE)
    else return(onFALSE)
  }
  names <- names(args)
  names <- names[names != ""]
  if (!isTRUE(args) && length(names) != length(args)) warning("unnamed ", arg, " arguments, will be ignored")
  if (isTRUE(args) || is.null(names)) defaults
  else defaults[names] <- args[names]
  as.list(defaults)
}

isFALSE <- function(x){ 
  length(x) == 1 && is.logical(x) && !isTRUE(x)
}

# compute multiplier for Scheffe-type confidence bounds

scheffe <- function(level, p, df=Inf){
  sqrt(p*qf(level, p, df))
}

# function to compute "nice" numbers

nice <- function (x, direction = c("round", "down", "up"), lead.digits = 1) {
  direction <- match.arg(direction)
  if (length(x) > 1){
    result <- sapply(x, nice, direction = direction, lead.digits = lead.digits)
    if (anyDuplicated(result)) result <- nice(x, direction=direction, lead.digits = lead.digits + 1)
    return(result)
  }
  if (x == 0) 
    return(0)
  power.10 <- floor(log(abs(x), 10))
  if (lead.digits > 1) 
    power.10 <- power.10 - lead.digits + 1
  lead.digit <- switch(direction, round = round(abs(x)/10^power.10), 
                       down = floor(abs(x)/10^power.10), up = ceiling(abs(x)/10^power.10))
  sign(x) * lead.digit * 10^power.10
}

ticksGrid <- function(x, y, col=reference.line$col){
  reference.line <- trellis.par.get("reference.line")
  panel.abline(h=y, v=x, col=col, lty=reference.line$lty)
}


# for character and logical predictors

is.factor <- function(x) inherits(x, "factor") || ((is.character(x) || is.logical(x)) && is.vector(x)) 

levels.character <- function(x) {
  levs <- unique(x)
  sort(levs[!is.na(levs)])
}

levels.logical <- function(x) {
  c("FALSE", "TRUE")
}

# to support Date variables

levels2dates <- function(effect, ...){
  UseMethod("levels2dates")
}

levels2dates.eff <- function(effect, predictor, origin, evenly.spaced=TRUE, n, ...){
  levels <- effect$variables[[predictor]]$levels
  if (evenly.spaced) {
    if (missing(n)) n <- length(levels)
    levels <- seq(min(levels), max(levels), length=n)
  }
  as.Date(levels, origin)
}

levels2dates.effpoly <- function(effect, predictor, origin, evenly.spaced=TRUE, n, ...){
  levels2dates.eff(effect, predictor, origin, evenly.spaced, n, ...)
}

# effect.mer and effect.lme built from effect.lm by S. Weisberg 29 June 2011
# 2012-03-08 to require() lme4 or nlme. J. Fox
# 2012-10-05 effect.lme didn't work with 'weights', now corrected.  S. Weisberg
# 2013-03-05: introduced merMod methods for development version of lme4. J. Fox
# 2013-04-06: added support for lme4.0, J. Fox
# 2013-07-30: added 'data' argument to lme.to.glm and mer.to.glm to allow
#   calling effect from within a subroutine.
# 2013-09-25:  removed the 'data' argument as it makes the functions fail with
#   logs, splines and polynomials
# 2014-09-24: added option for KR cov matrix to mer.to.glm(). J. Fox
# 2014-12-07: don't assume that pbkrtest is installed. J. Fox
# 2014-12-20: mer.to.glm failed for negative.binomial() because the link has an argument
#   that was handled incorrectly by the family.glmResp function.  This function is no longer
#   used by mer.to.glm.  The same error will recur in any link with an argument.
# 2015-06-10: requireNamespace("pbkrtest") rather than require("pbkrtest)
# 2015-07-02: fixed bug when the name of the data frame was the name of a function (e.g., sort, or lm)
# 2015-12-13: make it work with pbkrtest 0.4-3. J. Fox
# 2016-01-07: modified 'fixmod' to allow "||" in variance formulae
# 2016-01-19: Fixed bug in glm.to.mer when 'poly' is used in a model.
# 2016-06-08: Fixed bug handling the 'start' argument in glm.to.mer.  Fix by Ben Bolker, bug from Mariano Devoto
# 2016-11-18: Change to mer.to.glm and lme.to.glm for stability in unusual glms. By Nate TeGrotenhuis.
# 2017-03-28: in mer.to.glm, changed a name from m to .m.  The fake glm is created from the mer model's
#             call slot, which included the name of the data.frame, if any.  A data.frame named 'm'
#             therefore did not work.  Same bug fixed in lme.to.glm

# the function lm.wfit  gets the hessian wrong for mer's.  Get the variance
# from the vcov method applied to the mer object.


fixmod <- function (term)
{
  if (!("|" %in% all.names(term)) && !("||" %in% all.names(term)))
    return(term)
  if ((is.call(term) && term[[1]] == as.name("|")) ||
      (is.call(term) && term[[1]] == as.name("||")))
    return(NULL)
  if (length(term) == 2) {
    nb <- fixmod(term[[2]])
    if (is.null(nb))
      return(NULL)
    term[[2]] <- nb
    return(term)
  }
  nb2 <- fixmod(term[[2]])
  nb3 <- fixmod(term[[3]])
  if (is.null(nb2))
    return(nb3)
  if (is.null(nb3))
    return(nb2)
  term[[2]] <- nb2
  term[[3]] <- nb3
  term
}

# lme.to.glm evaluates a 'glm' model that is as similar to a given 'lme'
# model, in the same pattern as mer.to.glm.  This could be speeded up
# slightly by using 'lm' rather than 'glm' but I use 'glm' to parallel
# mer.to.glm more closely.  The differences are:  (1) match fewer args
# in the call; (2) different def of mod2$coefficients; no other
# changes

# The argument 'data' to lme.to.glm and to mer.to.glm copies the data
# from the object into the local environment and makes it visible when 'effect'
# is called from within another function.

lme.to.glm <- function(mod) {
  cl <- mod$call
  cl$formula <- cl$fixed
  cl$control <- glm.control(epsilon=1) # suggested by Nate TeGrotenhuis
  .m <- match(c("formula", "data", "subset",
                "na.action",  "contrasts"), names(cl), 0L)
  cl <- cl[c(1L, .m)]
  cl[[1L]] <- as.name("glm")
  mod2 <- eval(cl)
  pw <- attr(mod$modelStruct$varStruct, "weights")
  if(!is.null(pw)) mod2$prior.weights <- pw
  mod2$coefficients <- mod$coefficients$fixed
  mod2$vcov <- as.matrix(vcov(mod))
  mod2$linear.predictors <- model.matrix(mod2) %*% mod2$coefficients
  mod2$fitted.values <- mod2$family$linkinv(mod2$linear.predictors)
  mod2$weights <- as.vector(with(mod2,
                                 prior.weights * (family$mu.eta(linear.predictors)^2 /
                                                    family$variance(fitted.values))))
  mod2$residuals <- with(mod2,
                         prior.weights * (y - fitted.values)/weights )
  class(mod2) <- c("fakeglm", class(mod2))
  mod2
}

# mer.to.glm evaluates a 'glm' model that is as similar to a given 'mer'
# model as possible.  It is of class c("fakeglm", "glm", "lm")
# several items are added to the created objects. Do not export

mer.to.glm <- function(mod, KR=FALSE) {
  if (KR && !requireNamespace("pbkrtest", quietly=TRUE)){
    KR <- FALSE
    warning("pbkrtest is not available, KR set to FALSE")
  }
  # object$family$family doesn't work correctly with the negative binomial family because of the
  # argument in the family function, so the old line
  #   family <- family(mod)
  # returns an error message for these models.  The following kluge fixes this.
  # If this bug is fixed in lme4, this code may break because it expects resp$family$family
  # to return "Link Name(arg)" with ONE argument, and so spaces between Name and "(arg)"
  family1 <- function(object, ...) {UseMethod("family1", object@resp)}
  family1.lmResp <- function(object, ...) family(object, ...)
  family1.glmResp <- function(object, ...){
    famname <- object@resp$family$family
    open.paren <- regexpr("\\(", famname)
    if(open.paren==-1) {
      name <- famname
      arg <- list()
    } else {
      name <- sub(" ", ".", tolower(substr(famname, 1, -1 + open.paren)))
      arg <- list(as.numeric(gsub("\\)", "", substr(famname, 1 + open.paren, 100))))
    }
    if(is.null(object@resp$family$initialize)) do.call(name, arg) else object@resp$family
  }
  family <- family1(mod)
  # end
  link <- family$link
  family <- family$family
  cl <- mod@call
  cl$control <- glm.control(epsilon=1) # suggested by Nate TeGrotenhuis
  if(cl[[1]] =="nlmer") stop("effects package does not support 'nlmer' objects")
  .m <- match(c("formula", "family", "data", "weights", "subset",
                "na.action", "offset",
                "model", "contrasts"), names(cl), 0L)
  cl <- cl[c(1L, .m)]
  cl[[1L]] <- as.name("glm")
  cl$formula <- fixmod(as.formula(cl$formula))
  #    cl$data <- mod@frame # caused bug with a 'poly' in the formula
  mod2 <- eval(cl)
  mod2$coefficients <- lme4::fixef(mod) #mod@fixef
  mod2$vcov <- if (family == "gaussian" && link == "identity" && KR) as.matrix(pbkrtest::vcovAdj(mod)) else as.matrix(vcov(mod))
  mod2$linear.predictors <- model.matrix(mod2) %*% mod2$coefficients
  mod2$fitted.values <- mod2$family$linkinv(mod2$linear.predictors)
  mod2$weights <- as.vector(with(mod2,
                                 prior.weights * (family$mu.eta(linear.predictors)^2 /
                                                    family$variance(fitted.values))))
  mod2$residuals <- with(mod2,
                         prior.weights * (y - fitted.values)/weights )
  class(mod2) <- c("fakeglm", class(mod2))
  mod2
}



#method for 'fakeglm' objects. Do not export
vcov.fakeglm <- function(object, ...) object$vcov

#The next six functions should be exported as S3 methods

effect.mer <- function(term, mod, vcov.=vcov, KR=FALSE, ...) {
  result <- effect(term, mer.to.glm(mod, KR=KR), vcov., ...)
  result$formula <- as.formula(formula(mod))
  result
}

effect.merMod <- function(term, mod, vcov.=vcov, KR=FALSE, ...){
  effect.mer(term, mod, vcov.=vcov, KR=KR, ...)
}

effect.lme <- function(term, mod, ...) {
  mod1 <- lme.to.glm(mod)
  result <- effect(term, mod1)
  result$formula <- as.formula(formula(mod))
  result
}

allEffects.mer <- function(mod, KR=FALSE,...){
  allEffects(mer.to.glm(mod,KR=KR), ...)
}

allEffects.merMod <- function(mod, KR=FALSE,...){
  allEffects(mer.to.glm(mod,KR=KR), ...)
}

allEffects.lme <- function(mod, ...){
  allEffects(lme.to.glm(mod), ...)
}

if (requireNamespace("carData") && require("effects")){
  
  data(Duncan, package="carData")

  mi <- with(Duncan, mean(income))
  me <- with(Duncan, mean(education))
  med <- with(Duncan, median(education))
  
  # (1) focal: factor, constant: polynomial
  
  mod.1 <- lm(prestige ~ type + poly(income, degree=2, raw=TRUE), data=Duncan)
  
  X <- matrix(c(1, 0, 0, mi, mi^2,
                1, 1,  0, mi, mi^2,
                1, 0, 1, mi, mi^2),
              nrow=3, ncol=5, byrow=TRUE)
  if (!isTRUE(all.equal(as.vector(matrix(X %*% coef(mod.1))), as.vector(Effect("type", mod.1)$fit)))) 
    stop("failed Test 1-1")
 
  
  # (2) focal: polynomial, constant: factor
  
  X <- matrix(c(1, 0.4, 2/15, 10, 10^2,
                  1, 0.4, 2/15, 40, 40^2,
                  1, 0.4, 2/15, 70, 70^2),
                nrow=3, ncol=5, byrow=TRUE)
  if (!isTRUE(all.equal(as.vector(Effect("income", mod.1, xlevels=list(income=c(10, 40, 70)))$fit), 
            as.vector(matrix(X %*% coef(mod.1))))))
    stop("failed test 1-2")
  
  # (2a) As in (2), but without specifying xlevels
  X <- matrix(c(1, 0.4, 2/15, 7, 7^2,
                1, 0.4, 2/15, 30, 30^2,
                1, 0.4, 2/15, 40, 40^2,
                1, 0.4, 2/15, 60, 60^2,
                1, 0.4, 2/15, 80, 80^2),
              nrow=5, ncol=5, byrow=TRUE)
  if (!isTRUE(all.equal(as.vector(Effect("income", mod.1)$fit), 
                        as.vector(matrix(X %*% coef(mod.1)))))) 
    stop("failed test 1-2a")
  
  # (3) focal: factor*polynomial, constant: polynomial
  
  mod.2 <- lm(prestige ~ type*poly(income, degree=2, raw=TRUE) + 
                poly(education, degree=2, raw=TRUE), data=Duncan)
  
  X <- matrix(c(1, 0, 0, 10, 10^2, me, me^2, 0, 0, 0, 0,
                1, 1, 0, 10, 10^2, me, me^2, 10, 0, 10^2, 0,
                1, 0, 1, 10, 10^2, me, me^2, 0, 10, 0,  10^2,
                1, 0, 0, 70, 70^2, me, me^2, 0, 0, 0, 0,
                1, 1, 0, 70, 70^2, me, me^2, 70, 0, 70^2, 0,
                1, 0, 1, 70, 70^2, me, me^2, 0, 70, 0,  70^2),
              nrow=6, ncol=11, byrow=TRUE)
  if (!isTRUE(all.equal(as.vector(Effect(c("type", "income"), mod.2, xlevels=list(income=c(10, 70)))$fit),
            as.vector(matrix(X %*% coef(mod.2), 3, 2)))))
    stop("failed test 1-3")
  
  # (4) focal: polynomial, constant: factor*polynomial
  
  X <- matrix(c(1, 0.4, 2/15, mi, mi^2, 10, 10^2, 0.4*mi, 2/15*mi, 0.4*mi^2, 2/15*mi^2,
                1, 0.4, 2/15, mi, mi^2, 40, 40^2, 0.4*mi, 2/15*mi, 0.4*mi^2, 2/15*mi^2,
                1, 0.4, 2/15, mi, mi^2, 70, 70^2, 0.4*mi, 2/15*mi, 0.4*mi^2, 2/15*mi^2),
              nrow=3, ncol=11, byrow=TRUE)
  if (!isTRUE(all.equal(as.vector(Effect("education", mod.2, xlevels=list(education=c(10, 40, 70)))$fit),
            as.vector(X %*% coef(mod.2)))))
    stop("failed test 1-4")
  
  # (5) repeat of (3) with medians rather than means
  
  X <- matrix(c(1, 0, 0, 10, 10^2, med, med^2, 0, 0, 0, 0,
                1, 1, 0, 10, 10^2, med, med^2, 10, 0, 10^2, 0,
                1, 0, 1, 10, 10^2, med, med^2, 0, 10, 0,  10^2,
                1, 0, 0, 70, 70^2, med, med^2, 0, 0, 0, 0,
                1, 1, 0, 70, 70^2, med, med^2, 70, 0, 70^2, 0,
                1, 0, 1, 70, 70^2, med, med^2, 0, 70, 0,  70^2),
              nrow=6, ncol=11, byrow=TRUE)
  if (!isTRUE(all.equal(as.vector(Effect(c("type", "income"), mod.2, 
                             xlevels=list(income=c(10, 70)), typical=median)$fit),
            as.vector(X %*% coef(mod.2)))))
    stop("failed test 1-5")
  
  # (6) focal: factor*polynomial, constant: polynomial, using predict() & orthog. polys.
  
  mod.3 <- lm(prestige ~ type*poly(income, degree=2) + poly(education, degree=2), data=Duncan)
  
  if (!isTRUE(all.equal(as.vector(predict(mod.3, 
                              newdata=data.frame(income=c(10, 10, 10, 70, 70, 70), 
                                                type=factor(c("bc", "prof", "wc", "bc", "prof", "wc")),
                                                education=mean(Duncan$education)))),
            as.vector(Effect(c("type", "income"), mod.3, xlevels=list(income=c(10, 70)))$fit))))
    stop("failed test 1-6")
  
  # (7) focal: factor, constant: poly*poly
  
  mod.4 <- lm(prestige ~ type + poly(income, 2)*poly(education, 2), data=Duncan)
  
  if (!isTRUE(all.equal(as.vector(Effect("type", mod.4)$fit),
            as.vector(predict(mod.4, newdata=data.frame(type=c("bc", "prof", "wc"), 
                                                        income=rep(mi, 3), education=rep(me, 3)))))))
    stop("failed test 1-7")
  
  # (8) focal: factor, constant: 2nd deg polynomial in 2 Xs
  
  mod.5 <- lm(prestige ~ type + poly(income, education, degree=2), data=Duncan)
  
  if (!isTRUE(all.equal(as.vector(Effect("type", mod.5)$fit),
            as.vector(predict(mod.5, newdata=data.frame(type=c("bc", "prof", "wc"), 
                                    income=rep(mi, 3), education=rep(me, 3)))))))
    stop("failed test 1-8")
  
  # (9) focal: covariate, constant: 2 factors and 1 covariate, 3-way interaction
  
  data(Mroz, package="carData")
  mod.6 <- lm(lwg ~ inc + age*hc*wc, data=Mroz)
  mage <- with(Mroz, mean(age))
  mhc <- with(Mroz, mean(hc == "yes"))
  mwc <- with(Mroz, mean(wc == "yes"))
  hc <- rep(mhc, 3)
  wc <- rep(mwc, 3)
  age <- rep(mage, 3)
  X <- cbind(1, c(10, 40, 80), age, hc, wc, age*hc, age*wc, hc*wc, age*hc*wc)
  if (!isTRUE(all.equal(as.vector(Effect("inc", mod.6, xlevels=list(inc=c(10, 40, 80)))$fit),
                        as.vector(X %*% coef(mod.6)))))
      stop("failed test 1-8")
}

# Plot method for effpoly objects

# modified by Michael Friendly: added ci.style="bands" & alpha.band= arg
# modified by Michael Friendly: added lwd= argument for llines (was lwd=2)
# 2013-11-06: fixed drop dimension when only one focal predictor. John
# 2014-10-10: namespace fixes. John
# 2014-12-05: made key.args more flexible. John
# 2014-03-22: use wide columns by default only when x for legend not set. J. Fox
# 2016-09-08: added show.strip.values argument to plot.effpoly(). J. Fox
# 2017-08-16: modified plot.effpoly() to consolidate arguments and use lattice theme. J. Fox
# 2017-08-20: reintroduce legacy arguments for plot.effpoly()
# 2017-08-20: introduced multiline argument under lines argument and as a "legacy" argument
# 2017-09-10: use replacement for grid.panel()
# 2017-11-22: added a check for non-estimable factor combinations with style="stacked"
# 2018-01-02, 2018-01-30: changed defaults for key.args, lines 140-141
# 2018-02-09: Use one-column key for stacked plot.
# 2018-02-28: Fix handling of rug arg (error reported by Dave Armstrong).
# 2018-07-08: add cex sub-args for x and y axes (suggestion of Charles Leger).
# 2018-07-08: add cex sub-arg for strips.
# 2018-10-05: modified plot.effpoly() so that multiline plots don't show confidence limits 
#             by default, and so that confidence bars for a factor are staggered.
# 2020-06-01: plot.effpoly() no longer ignores lty.
# 2025-07-21: rename range.adj() to range_adj to avoid generic confusion.

plot.effpoly <- function(x, x.var=which.max(levels), 
                         main=paste(effect, "effect plot"),
                         symbols=TRUE, lines=TRUE, axes, confint, lattice, ...,
                         # legacy arguments:
                         type, multiline, rug, xlab, ylab, colors, cex, lty, lwd, 
                         factor.names, show.strip.values,
                         ci.style, band.colors, band.transparency, style, 
                         transform.x, ticks.x, xlim,
                         ticks, ylim, rotx, roty, alternating, grid, layout, 
                         key.args, use.splines){
  
  if (!is.logical(lines) && !is.list(lines)) lines <- list(lty=lines)
  lines <- applyDefaults(lines, 
              defaults=list(lty=trellis.par.get("superpose.line")$lty, 
                            lwd=trellis.par.get("superpose.line")$lwd[1], 
                            col=NULL, splines=TRUE, multiline=FALSE), 
              arg="lines")
  if (missing(multiline)) multiline <- lines$multiline
  if (missing(lwd)) lwd <- lines$lwd
  if (missing(lty)) lty <- lines$lty
  if (missing(use.splines)) use.splines <- lines$splines
  lines.col <- lines$col
  lines <- if (missing(lty)) lines$lty else lty
 
  if (!is.logical(symbols) && !is.list(symbols)) symbols <- list(pch=symbols)
  symbols <- applyDefaults(symbols,
                           defaults= list(
                             pch=trellis.par.get("superpose.symbol")$pch, 
                             cex=trellis.par.get("superpose.symbol")$cex[1]),
                           arg="symbols")
  cex <- symbols$cex
  symbols <- symbols$pch
  
  if (missing(axes)) axes <- NULL
  axes <- applyDefaults(axes, defaults=list(
    x=list(rotate=0, cex=1, rug=TRUE),
    y=list(lab=NULL, lim=c(NA, NA), ticks=list(at=NULL, n=5), 
           type="probability", rotate=0, cex=1),
    alternating=TRUE, grid=FALSE),
    arg="axes")
  
  x.args <- applyDefaults(axes$x, defaults=list(rotate=0, cex=1, rug=TRUE), 
                          arg="axes$x")
  
  if (missing(xlab)) {
    xlab.arg <- FALSE
    xlab <- list()
  }
  if (missing(xlim)) {
    xlim.arg <- FALSE
    xlim <- list()
  }
  if (missing(ticks.x)) {
    ticks.x.arg <- FALSE
    ticks.x <- list()
  }
  if (missing(transform.x)) {
    transform.x.arg <- FALSE
    transform.x <- list()
  }
  if (missing(rotx)) rotx <- x.args$rotate
  if (missing(rug)) rug <- x.args$rug
  cex.x <- x.args$cex
  x.args$rotate <- NULL
  x.args$rug <- NULL
  x.args$cex <- NULL
  x.pred.names <- names(x.args)
  if (length(x.pred.names) > 0){
    for (pred.name in x.pred.names){
      x.pred.args <- applyDefaults(x.args[[pred.name]], 
                                   defaults=list(lab=NULL, lim=NULL, ticks=NULL, transform=NULL), 
                                   arg=paste0("axes$x$", pred.name))
      if (!xlab.arg) xlab[[pred.name]] <- x.pred.args$lab
      if (!xlim.arg) xlim[[pred.name]] <- x.pred.args$lim
      if (!ticks.x.arg) ticks.x[[pred.name]] <- x.pred.args$ticks
      if (!transform.x.arg) transform.x[[pred.name]] <- x.pred.args$transform
    }
  }
  if (length(xlab) == 0) xlab <- NULL
  if (length(xlim) == 0) xlim <- NULL
  if (length(ticks.x) == 0) ticks.x <- NULL
  if (length(transform.x) == 0) transform.x <- NULL
  
  y.args <- applyDefaults(axes$y, defaults=list(lab=NULL, lim=c(NA, NA), ticks=list(at=NULL, n=5), type="probability", style="lines", rotate=0, cex=1), arg="axes$y")
  if (missing(ylim)) ylim <- y.args$lim
  if (missing(ticks)) ticks <- y.args$ticks
  if (missing(type)) type <- y.args$type
  type <- match.arg(type, c("probability", "logit"))
  if (missing(ylab)) ylab <- y.args$lab
  if (is.null(ylab)) ylab <- paste0(x$response, " (", type, ")")
  if (missing(roty)) roty <- y.args$rotate
  cex.y <- y.args$cex
  if (missing(alternating)) alternating <- axes$alternating
  if (missing(grid)) grid <- axes$grid
  if (missing(style)) style <- match.arg(y.args$style, c("lines", "stacked"))
  
  if (missing(colors)) colors <- if (is.null(lines.col)){
    if (style == "lines" || x$model == "multinom")
      trellis.par.get("superpose.line")$col
    else sequential_hcl(length(x$y.levels))
  } else {
    lines.col
  }
  
  if (missing(confint)) confint <- NULL
  confint <- applyDefaults(confint,
                           defaults=list(style=if (style == "lines" && !multiline && !is.null(x$se.prob)) "auto" else "none", alpha=0.15, col=colors),
                           onFALSE=list(style="none", alpha=0, col="white"),
                           arg="confint")
  if (missing(ci.style)) ci.style <- confint$style
  if (missing(band.transparency)) band.transparency <- confint$alpha
  if (missing(band.colors)) band.colors <- confint$col
  if(!is.null(ci.style)) ci.style <- match.arg(ci.style, c("auto", "bars", "lines", "bands", "none")) 
  confint <- confint$style != "none"
  
  if (is.null(multiline)) multiline <- if (confint) FALSE else TRUE
  
  effect.llines <- llines
  has.se <- !is.null(x$confidence.level) 
  if (confint && !has.se) stop("there are no confidence limits to plot")
  if (style == "stacked"){
    if (type != "probability"){
      type <- "probability"
      warning('type set to "probability" for stacked plot')
    }
    if (confint){
      confint <- FALSE
      warning('confint set to FALSE for stacked plot')
    }
    ylim <- c(0, 1)
  }
  
  if (missing(lattice)) lattice <- NULL
  lattice <- applyDefaults(lattice, defaults=list(
    layout=NULL, #key.args=list(),  #New default added 1/2/2017 by sw
    strip=list(factor.names=TRUE, values=TRUE, cex=1),
    array=list(row=1, col=1, nrow=1, ncol=1, more=FALSE),
    arg="lattice"
  ))
  lattice$key.args <- applyDefaults(lattice$key.args, defaults=list(
    space="top", border=FALSE, fontfamily="sans", cex=.75, cex.title=1, 
    arg="key.args"
  ))
  if (missing(layout)) layout <- lattice$layout
  if (missing(key.args)) key.args <- lattice$key.args
  strip.args <- applyDefaults(lattice$strip, defaults=list(factor.names=TRUE, values=TRUE, cex=1), arg="lattice$strip")
  factor.names <- strip.args$factor.names
  if (missing(show.strip.values)) show.strip.values <- strip.args$values
  cex.strip <- strip.args$cex
  height.strip <- max(1, cex.strip)
  array.args <- applyDefaults(lattice$array, defaults=list(row=1, col=1, nrow=1, ncol=1, more=FALSE), arg="lattice$array")
  row <- array.args$row
  col <- array.args$col
  nrow <- array.args$nrow
  ncol <- array.args$ncol
  more <- array.args$more
  
  .mod <- function(a, b) ifelse( (d <- a %% b) == 0, b, d)
  .modc <- function(a) .mod(a, length(colors))
  .mods <- function(a) .mod(a, length(symbols))
  .modl <- function(a) .mod(a, length(lines))
  effect <- paste(sapply(x$variables, "[[", "name"), collapse="*")
  split <- c(col, row, ncol, nrow)
  n.predictors <- length(names(x$x))
  y.lev <- x$y.lev
  n.y.lev <- length(y.lev)
  ylevel.names <- make.names(paste("prob",y.lev))
  colnames(x$prob) <- colnames(x$logit) <- ylevel.names
  if (has.se){
    colnames(x$lower.logit) <- colnames(x$upper.logit) <- 
      colnames(x$lower.prob) <- colnames(x$upper.prob)<- ylevel.names
  }
  x.frame <-as.data.frame(x)
  predictors <- names(x.frame)[1:n.predictors]
  levels <- if (n.predictors==1) length (x.frame[,predictors])
  else sapply(apply(x.frame[, predictors, drop=FALSE], 2, unique), length)
  if (is.character(x.var)) {
    which.x <- which(x.var == predictors)
    if (length(which.x) == 0) stop(paste("x.var = '", x.var, "' is not in the effect.", sep=""))
    x.var <- which.x
  }
  x.vals <- x.frame[, names(x.frame)[x.var]]    
  response <- matrix(0, nrow=nrow(x.frame), ncol=n.y.lev)
  for (i in 1:length(x$y.lev)){
    level <- which(colnames(x$prob)[i] == ylevel.names)
    response[,i] <- rep(x$y.lev[level], length(response[,i]))
  }
  prob <- as.vector(x$prob)
  logit <- as.vector(x$logit)
  response <- as.vector(response)
  if (has.se){
    lower.prob <- as.vector(x$lower.prob)
    upper.prob <- as.vector(x$upper.prob)
    lower.logit <- as.vector(x$lower.logit)
    upper.logit <- as.vector(x$upper.logit)
  }
  response <- factor(response, levels=y.lev)
  Data <- data.frame(prob, logit)
  if (has.se) Data <- cbind(Data, data.frame(lower.prob, upper.prob, lower.logit, upper.logit))
  Data[[x$response]] <- response
  for (i in 1:length(predictors)){
    Data <- cbind(Data, x.frame[predictors[i]])
  }
  levs <- levels(x$data[[predictors[x.var]]])
  n.predictor.cats <- sapply(Data[, predictors[-c(x.var)], drop=FALSE], 
                             function(x) length(unique(x)))
  if (length(n.predictor.cats) == 0) n.predictor.cats <- 1
  ci.style <- if(is.null(ci.style) || ci.style == "auto") {
    if(is.factor(x$data[[predictors[x.var]]])) "bars" else "bands"} else ci.style
  if( ci.style=="none" ) confint <- FALSE
  ### no confidence intervals if confint == FALSE or ci.style=="none"
  if (!confint){ # plot without confidence bands
    if (style == "lines"){ # line plot
      if (!multiline){
        layout <- if(is.null(layout)) c(prod(n.predictor.cats), length(levels(response)), 1) 
        else layout
        ### factor
        if (is.factor(x$data[[predictors[x.var]]])){ # x-variable a factor
          range <- if (type=="probability") range(prob, na.rm=TRUE) else range(logit, na.rm=TRUE)
          ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                     range[2] + .025*(range[2] - range[1]))
          tickmarks <- make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
          levs <- levels(x$data[[predictors[x.var]]])
          if (show.strip.values){
            for (pred in predictors[-x.var]){
              Data[[pred]] <- as.factor(Data[[pred]])
            }
          }
          result <- xyplot(eval(if (type=="probability") 
            parse(text=if (n.predictors==1) 
              paste("prob ~ as.numeric(", predictors[x.var],") |", x$response)
              else paste("prob ~ as.numeric(", predictors[x.var],") |", 
                         paste(predictors[-x.var], collapse="*"), 
                         paste("*", x$response)))
            else parse(text=if (n.predictors==1) 
              paste("logit ~ as.numeric(", predictors[x.var],") |", x$response)
              else paste("logit ~ as.numeric(", predictors[x.var],")|", 
                         paste(predictors[-x.var], collapse="*"), 
                         paste("*", x$response)))),
            par.strip.text=list(cex=0.8),							
            strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                               par.strip.text=list(cex=cex.strip)),
            panel=function(x, y, subscripts, x.vals, rug, ... ){
              if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
              good <- !is.na(y)
              effect.llines(x[good], y[good], lwd=lwd, lty=lty, type="b", pch=19, col=colors[1], cex=cex, ...)
              subs <- subscripts+as.numeric(rownames(Data)[1])-1		
            },
            
            
            ylab=ylab,
            ylim=if (is.null(ylim))
              if (type == "probability") range(prob) else range(logit)
            else ylim,
            xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
            main=main,
            x.vals=x$data[[predictors[x.var]]],
            rug=rug,
            scales=list(x=list(at=1:length(levs), labels=levs, rot=rotx, cex=cex.x), 
                        y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y), alternating=alternating),
            layout=layout,
            data=Data, ...)
          result$split <- split
          result$more <- more
          class(result) <- c("plot.eff", class(result))
        }
        else { # x-variable numeric
          if(use.splines) effect.llines <- spline.llines # added 10/17/13
          range <- if (type=="probability") range(prob, na.rm=TRUE) else range(logit, na.rm=TRUE)
          ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                     range[2] + .025*(range[2] - range[1]))
          tickmarks <- make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
          nm <- predictors[x.var]
          x.vals <- x$data[[nm]]   
          if (nm %in% names(ticks.x)){
            at <- ticks.x[[nm]]$at
            n <- ticks.x[[nm]]$n
          }
          else{
            at <- NULL
            n <- 5
          }
          xlm <- if (nm %in% names(xlim)){
            xlim[[nm]]
          }
          else range_adj(Data[nm]) # range(x.vals)
          tickmarks.x <- if ((nm %in% names(transform.x)) && !(is.null(transform.x))){
            trans <- transform.x[[nm]]$trans
            make.ticks(trans(xlm), link=transform.x[[nm]]$trans, inverse=transform.x[[nm]]$inverse, at=at, n=n)
          }
          else {
            trans <- I
            make.ticks(xlm, link=I, inverse=I, at=at, n=n)
          }
          if (show.strip.values){
            for (pred in predictors[-x.var]){
              Data[[pred]] <- as.factor(Data[[pred]])
            }
          }
          result <- xyplot(eval(if (type=="probability") 
            parse(text=if (n.predictors==1) 
              paste("prob ~ trans(", predictors[x.var],") |", x$response)
              else paste("prob ~ trans(", predictors[x.var],") |", 
                         paste(predictors[-x.var], collapse="*"), 
                         paste("*", x$response)))
            else parse(text=if (n.predictors==1) 
              paste("logit ~ trans(", predictors[x.var],") |", x$response)
              else paste("logit ~ trans(", predictors[x.var],") |", 
                         paste(predictors[-x.var], collapse="*"), 
                         paste("*", x$response)))
          ),
          par.strip.text=list(cex=0.8),							
          strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                             par.strip.text=list(cex=cex.strip)),
          panel=function(x, y, subscripts, x.vals, rug, ... ){
            if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
            if (rug) lrug(trans(x.vals))
            good <- !is.na(y)
            effect.llines(x[good], y[good], lwd=lwd, lty=lty, col=colors[1], ...)
            subs <- subscripts+as.numeric(rownames(Data)[1])-1	
          },
          ylab=ylab,
          xlim=suppressWarnings(trans(xlm)),
          ylim= if (is.null(ylim))
            if (type == "probability") range(prob) else range(logit)
          else ylim,
          xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
          main=main,
          x.vals=x$data[[predictors[x.var]]],
          rug=rug,
          scales=list(y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y), 
                      x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx, cex=cex.x),
                      alternating=alternating),
          layout=layout,
          data=Data, ...)
          result$split <- split
          result$more <- more
          class(result) <- c("plot.eff", class(result))
        }
        
      } else {
        layout <- if (is.null(layout)){
          lay <- c(prod(n.predictor.cats[-(n.predictors - 1)]), 
                   prod(n.predictor.cats[(n.predictors - 1)]), 1)
          if (lay[1] > 1) lay else lay[c(2, 1, 3)]
        }
        else layout
        if (n.y.lev > min(c(length(colors), length(lines), length(symbols))))
          warning('Colors, lines and symbols may have been recycled')
        range <- if (type=="probability") range(prob, na.rm=TRUE) else range(logit, na.rm=TRUE)
        ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                   range[2] + .025*(range[2] - range[1]))
        tickmarks <- make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
        if (is.factor(x$data[[predictors[x.var]]])){ # x-variable a factor
          key <- list(title=x$response, cex.title=1, border=TRUE,
                      text=list(as.character(unique(response))),
                      lines=list(col=colors[.modc(1:n.y.lev)], lty=lines[.modl(1:n.y.lev)], lwd=lwd),
                      points=list(pch=symbols[.mods(1:n.y.lev)], col=colors[.modc(1:n.y.lev)]),
                      columns = if ("x" %in% names(key.args)) 1 else 
                        find.legend.columns(length(n.y.lev), 
                              space=if("x" %in% names(key.args)) "top" else key.args$space))
          for (k in names(key.args)) key[k] <- key.args[k]
          if (show.strip.values){
            for (pred in predictors[-x.var]){
              Data[[pred]] <- as.factor(Data[[pred]])
            }
          }
          result <- xyplot(eval(if (type=="probability") 
            parse(text=if (n.predictors==1) 
              paste("prob ~ as.numeric(", predictors[x.var], ")")
              else paste("prob ~ as.numeric(", predictors[x.var],") | ", 
                         paste(predictors[-x.var], collapse="*")))
            else parse(text=if (n.predictors==1) 
              paste("logit ~ as.numeric(", predictors[x.var], ")")
              else paste("logit ~ as.numeric(", predictors[x.var],") | ", 
                         paste(predictors[-x.var], collapse="*")))), 
            strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                               par.strip.text=list(cex=cex.strip)),
            panel=function(x, y, subscripts, rug, z, x.vals, ...){
              if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
              for (i in 1:n.y.lev){
                sub <- z[subscripts] == y.lev[i]
                good <- !is.na(y[sub])
                effect.llines(x[sub][good], y[sub][good], lwd=lwd, type="b", col=colors[.modc(i)], lty=lines[.modl(i)],
                              pch=symbols[i], cex=cex, ...)
              }
            },
            ylab=ylab,
            ylim= if (is.null(ylim))
              if (type == "probability") range(prob) else range(logit)
            else ylim,
            xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
            x.vals=x$data[[predictors[x.var]]], 
            rug=rug,
            z=response,
            scales=list(x=list(at=1:length(levs), labels=levs, rot=rotx, cex=cex.x),
                        y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),  
                        alternating=alternating),
            main=main,
            key=key,
            layout=layout,
            data=Data, ...)
          result$split <- split
          result$more <- more
          class(result) <- c("plot.eff", class(result))    		
        }
        else { # x-variable numeric
          if(use.splines) effect.llines <- spline.llines # added 10/17/13
          range <- if (type=="probability") range(prob, na.rm=TRUE) else range(logit, na.rm=TRUE)
          ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                     range[2] + .025*(range[2] - range[1]))
          tickmarks <- make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
          nm <- predictors[x.var]
          x.vals <- x$data[[nm]]   
          if (nm %in% names(ticks.x)){
            at <- ticks.x[[nm]]$at
            n <- ticks.x[[nm]]$n
          }
          else{
            at <- NULL
            n <- 5
          }
          xlm <- if (nm %in% names(xlim)){
            xlim[[nm]]
          }
          else range_adj(Data[nm]) # range(x.vals)
          tickmarks.x <- if ((nm %in% names(transform.x)) && !(is.null(transform.x))){
            trans <- transform.x[[nm]]$trans
            make.ticks(trans(xlm), link=transform.x[[nm]]$trans, inverse=transform.x[[nm]]$inverse, at=at, n=n)
          }
          else {
            trans <- I
            make.ticks(xlm, link=I, inverse=I, at=at, n=n)
          }
          key <- list(title=x$response, cex.title=1, border=TRUE,
                      text=list(as.character(unique(response))), 
                      lines=list(col=colors[.modc(1:n.y.lev)], lty=lines[.modl(1:n.y.lev)], lwd=lwd),
                      columns = if ("x" %in% names(key.args)) 1 else 
                        find.legend.columns(length(n.y.lev), 
                                space=if("x" %in% names(key.args)) "top" else key.args$space))
          for (k in names(key.args)) key[k] <- key.args[k]
          if (show.strip.values){
            for (pred in predictors[-x.var]){
              Data[[pred]] <- as.factor(Data[[pred]])
            }
          }
          result <- xyplot(eval(if (type=="probability") 
            parse(text=if (n.predictors==1) paste("prob ~ trans(", predictors[x.var], ")")
                  else paste("prob ~ trans(", predictors[x.var],") |", 
                             paste(predictors[-x.var], collapse="*")))
            else parse(text=if (n.predictors==1) paste("logit ~ trans(", predictors[x.var], ")")
                       else paste("logit ~ trans(", predictors[x.var],") | ", 
                                  paste(predictors[-x.var], collapse="*")))), 
            strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                               par.strip.text=list(cex=cex.strip)),
            panel=function(x, y, subscripts, rug, z, x.vals, ...){
              if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
              if (rug) lrug(trans(x.vals))
              for (i in 1:n.y.lev){
                sub <- z[subscripts] == y.lev[i]
                good <- !is.na(y[sub])
                effect.llines(x[sub][good], y[sub][good], lwd=lwd, type="l", col=colors[.modc(i)], lty=lines[.modl(i)], ...)
              }
            },
            ylab=ylab,
            xlim=suppressWarnings(trans(xlm)),
            ylim= if (is.null(ylim))
              if (type == "probability") range(prob) else range(logit)
            else ylim,
            xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
            x.vals=x$data[[predictors[x.var]]], 
            rug=rug,
            z=response,
            scales=list(x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx, cex=cex.x), 
                        y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),
                        alternating=alternating),
            main=main,
            key=key,
            layout=layout,
            data=Data, ...)
          result$split <- split
          result$more <- more
          class(result) <- c("plot.eff", class(result))			
        }
      }
    }
    else { # stacked plot
      tickmarks <- make.ticks(c(0, 1), link=I, inverse=I, at=ticks$at, n=ticks$n)
      layout <- if (is.null(layout)){
        lay <- c(prod(n.predictor.cats[-(n.predictors - 1)]), 
                 prod(n.predictor.cats[(n.predictors - 1)]), 1)
        if (lay[1] > 1) lay else lay[c(2, 1, 3)]
      }
      else layout
      if (n.y.lev > length(colors))
        stop(paste('Not enough colors to plot', n.y.lev, 'regions'))
      key <- list(text=list(lab=rev(y.lev)), 
                  rectangle=list(col=rev(colors[1:n.y.lev])),
                  columns = 1)
                    # if ("x" %in% names(key.args)) 1 else 
                    # find.legend.columns(length(n.y.lev), 
                    #       space=if("x" %in% names(key.args)) "top" else key.args$space))
      for (k in names(key.args)) key[k] <- key.args[k]
      if (is.factor(x$data[[predictors[x.var]]])){ # x-variable a factor
# 11/22/17 check for rank deficient models and if found stop
        if(any(is.na(Data$prob))) stop("At least one combination of factor levels is not estimable.\n  Stacked plots are misleading, change to style='lines'")
        result <- barchart(eval(parse(text=if (n.predictors == 1) 
          paste("prob ~ ", predictors[x.var], sep="")
          else paste("prob ~ ", predictors[x.var]," | ", 
                     paste(predictors[-x.var], collapse="*")))), 
          strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                             par.strip.text=list(cex=cex.strip)),
          panel=function(x, y, ...){
            panel.barchart(x, y, ...)
            if (grid) ticksGrid(x=NA, y=tickmarks$at, col="white")
          },
          groups = response,
          col=colors,
          horizontal=FALSE, 
          stack=TRUE, 
          data=Data, 
          ylim=ylim, # if (is.null(ylim)) 0:1 else ylim,
          ylab=ylab, 
          xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
          scales=list(x=list(rot=rotx, at=1:length(levs), labels=levs, cex=cex.x), 
                      y=list(rot=roty, at=tickmarks$at, labels=tickmarks$labels, cex=cex.y), 
                      alternating=alternating),
          main=main,
          key=key,
          layout=layout)
        result$split <- split
        result$more <- more
        class(result) <- c("plot.eff", class(result))			
      }
      else { # x-variable numeric
        if(use.splines) effect.llines <- spline.llines # added 10/17/13
        nm <- predictors[x.var]
        x.vals <- x$data[[nm]]   
        if (nm %in% names(ticks.x)){
          at <- ticks.x[[nm]]$at
          n <- ticks.x[[nm]]$n
        }
        else{
          at <- NULL
          n <- 5
        }
        xlm <- if (nm %in% names(xlim)){
          xlim[[nm]]
        }
        else range_adj(Data[nm]) # range(x.vals)
        tickmarks.x <- if ((nm %in% names(transform.x)) && !(is.null(transform.x))){
          trans <- transform.x[[nm]]$trans
          make.ticks(trans(xlm), link=transform.x[[nm]]$trans, inverse=transform.x[[nm]]$inverse, at=at, n=n)
        }
        else {
          trans <- I
          make.ticks(xlm, link=I, inverse=I, at=at, n=n)
        }
        if (show.strip.values){
          for (pred in predictors[-x.var]){
            x$x[[pred]] <- as.factor(x$x[[pred]])
          }
        }
        result <- densityplot(eval(parse(text=if (n.predictors == 1)
          paste("~ trans(", predictors[x.var], ")", sep="")
          else paste("~ trans(", predictors[x.var], ") | ",
                     paste(predictors[-x.var], collapse="*")))),
          probs=x$prob,
          strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                             par.strip.text=list(cex=cex.strip)),
          panel =  function(x, subscripts, rug, x.vals, probs=probs, col=colors, ...){
            fill <- function(x, y1, y2, col){
              if (length(y2) == 1) y2 <- rep(y2, length(y1))
              if (length(y1) == 1) y1 <- rep(y1, length(y2))
              panel.polygon(c(x, rev(x)), c(y1, rev(y2)), col=col)
            }
            n <- ncol(probs)
            Y <- t(apply(probs[subscripts,], 1, cumsum))
            fill(x, 0, Y[,1], col=col[1])
            for (i in 2:n){
              fill(x, Y[,i-1], Y[,i], col=col[i])
            }
            if (rug) lrug(trans(x.vals))
            if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at, col="white")
          },
          rug=rug,
          x.vals=x$data[[predictors[x.var]]],
          data=x$x,
          xlim=suppressWarnings(trans(xlm)),
          ylim= c(0, 1), # if (is.null(ylim)) 0:1 else ylim,
          ylab=ylab,
          xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
          scales=list(x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx, cex=cex.x), 
                      y=list(rot=roty, at=tickmarks$at, labels=tickmarks$labels, cex=cex.y),
                      alternating=alternating),
          main=main,
          key=key,
          layout=layout, ...)
        result$split <- split
        result$more <- more
        class(result) <- c("plot.eff", class(result))
      }
    }
  }
  ### with confidence bands
  else{ # plot with confidence bands
    if (type == "probability"){
      lower <- lower.prob
      upper <- upper.prob
    }
    else {
      lower <- lower.logit
      upper <- upper.logit
    }
    if (!multiline){
      layout <- if(is.null(layout)) c(prod(n.predictor.cats), length(levels(response)), 1) 
      else layout
      ### factor
      if (is.factor(x$data[[predictors[x.var]]])){ # x-variable a factor
        range <- range(c(lower, upper), na.rm=TRUE)
        ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                   range[2] + .025*(range[2] - range[1]))
        tickmarks <- make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
        levs <- levels(x$data[[predictors[x.var]]])
        if (show.strip.values){
          for (pred in predictors[-x.var]){
            Data[[pred]] <- as.factor(Data[[pred]])
          }
        }
        result <- xyplot(eval(if (type=="probability") 
          parse(text=if (n.predictors==1) 
            paste("prob ~ as.numeric(", predictors[x.var],") |", x$response)
            else paste("prob ~ as.numeric(", predictors[x.var],") |", 
                       paste(predictors[-x.var], collapse="*"), 
                       paste("*", x$response)))
          else parse(text=if (n.predictors==1) 
            paste("logit ~ as.numeric(", predictors[x.var],") |", x$response)
            else paste("logit ~ as.numeric(", predictors[x.var],")|", 
                       paste(predictors[-x.var], collapse="*"), 
                       paste("*", x$response)))),
          par.strip.text=list(cex=0.8),							
          strip=strip.custom(..., strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                             par.strip.text=list(cex=cex.strip)),
          panel=function(x, y, subscripts, x.vals, rug, lower, upper, ... ){
            if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
            good <- !is.na(y)
            effect.llines(x[good], y[good], lwd=lwd, lty=lty, type="b", pch=19, col=colors[1], cex=cex, ...)
            subs <- subscripts+as.numeric(rownames(Data)[1])-1		
            if (ci.style == "bars"){
              larrows(x0=x[good], y0=lower[subs][good], 
                      x1=x[good], y1=upper[subs][good], 
                      angle=90, code=3, col=colors[.modc(2)], length=0.125*cex/1.5)
            }
            else  if(ci.style == "lines"){
              effect.llines(x[good], lower[subs][good], lty=2, col=colors[.modc(2)])
              effect.llines(x[good], upper[subs][good], lty=2, col=colors[.modc(2)])
            }
            else { if(ci.style == "bands") {		
              panel.bands(x[good], y[good],
                          lower[subs][good], upper[subs][good],
                          fill=band.colors[1], alpha=band.transparency)
            }}
          },
          
          
          ylab=ylab,
          ylim= if (is.null(ylim)) c(min(lower), max(upper)) else ylim,
          xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
          main=main,
          x.vals=x$data[[predictors[x.var]]],
          rug=rug,
          lower=lower,
          upper=upper, 
          scales=list(x=list(at=1:length(levs), labels=levs, rot=rotx, cex=cex.x), 
                      y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y), 
                      alternating=alternating),
          layout=layout,
          data=Data, ...)
        result$split <- split
        result$more <- more
        class(result) <- c("plot.eff", class(result))
      }
      else { # x-variable numeric
        if(use.splines) effect.llines <- spline.llines # added 10/17/13
        range <- range(c(lower, upper), na.rm=TRUE)
        ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                   range[2] + .025*(range[2] - range[1]))
        tickmarks <- make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
        nm <- predictors[x.var]
        x.vals <- x$data[[nm]]   
        if (nm %in% names(ticks.x)){
          at <- ticks.x[[nm]]$at
          n <- ticks.x[[nm]]$n
        }
        else{
          at <- NULL
          n <- 5
        }
        xlm <- if (nm %in% names(xlim)){
          xlim[[nm]]
        }
        else range_adj(Data[nm]) # range(x.vals)
        tickmarks.x <- if ((nm %in% names(transform.x)) && !(is.null(transform.x))){
          trans <- transform.x[[nm]]$trans
          make.ticks(trans(xlm), link=transform.x[[nm]]$trans, inverse=transform.x[[nm]]$inverse, at=at, n=n)
        }
        else {
          trans <- I
          make.ticks(xlm, link=I, inverse=I, at=at, n=n)
        }
        if (show.strip.values){
          for (pred in predictors[-x.var]){
            Data[[pred]] <- as.factor(Data[[pred]])
          }
        }
        result <- xyplot(eval(if (type=="probability") 
          parse(text=if (n.predictors==1) 
            paste("prob ~ trans(", predictors[x.var],") |", x$response)
            else paste("prob ~ trans(", predictors[x.var],") |", 
                       paste(predictors[-x.var], collapse="*"), 
                       paste("*", x$response)))
          else parse(text=if (n.predictors==1) 
            paste("logit ~ trans(", predictors[x.var],") |", x$response)
            else paste("logit ~ trans(", predictors[x.var],") |", 
                       paste(predictors[-x.var], collapse="*"), 
                       paste("*", x$response)))
        ),
        par.strip.text=list(cex=0.8),							
        strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                           par.strip.text=list(cex=cex.strip)),
        panel=function(x, y, subscripts, x.vals, rug, lower, upper, ... ){
          if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
          if (rug) lrug(trans(x.vals))
          good <- !is.na(y)
          effect.llines(x[good], y[good], lwd=lwd, lty=lty, col=colors[1], ...)
          subs <- subscripts+as.numeric(rownames(Data)[1])-1	
          if (ci.style == "bars"){
            larrows(x0=x[good], y0=lower[subs][good], 
                    x1=x[good], y1=upper[subs][good], 
                    angle=90, code=3, col=colors[.modc(2)], length=0.125*cex/1.5)
          }
          else  if(ci.style == "lines"){
            effect.llines(x[good], lower[subs][good], lty=2, col=colors[.modc(2)])
            effect.llines(x[good], upper[subs][good], lty=2, col=colors[.modc(2)])
          } 
          else { if(ci.style == "bands") {	
            panel.bands(x[good], y[good],
                        lower[subs][good], upper[subs][good],
                        fill=band.colors[1], alpha=band.transparency)
          }}
        },
        ylab=ylab,
        xlim=suppressWarnings(trans(xlm)),
        ylim= if (is.null(ylim)) c(min(lower), max(upper)) else ylim,
        xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
        main=main,
        x.vals=x$data[[predictors[x.var]]],
        rug=rug,
        lower=lower,
        upper=upper, 
        scales=list(y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y), 
                    x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx, cex=cex.x),
                    alternating=alternating),
        layout=layout,
        data=Data, ...)
        result$split <- split
        result$more <- more
        class(result) <- c("plot.eff", class(result))
      }
    } else {
      
      layout <- if (is.null(layout)){
        lay <- c(prod(n.predictor.cats[-(n.predictors - 1)]), 
                 prod(n.predictor.cats[(n.predictors - 1)]), 1)
        if (lay[1] > 1) lay else lay[c(2, 1, 3)]
      }
      else layout
      if (n.y.lev > min(c(length(colors), length(lines), length(symbols))))
        warning('Colors, lines and symbols may have been recycled')
      if (is.factor(x$data[[predictors[x.var]]])){ # x-variable a factor
        range <- range(c(lower, upper), na.rm=TRUE)
        ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                   range[2] + .025*(range[2] - range[1]))
        tickmarks <- make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
        key <- list(title=x$response, cex.title=1, border=TRUE,
                    text=list(as.character(unique(response))),
                    lines=list(col=colors[.modc(1:n.y.lev)], lty=lines[.modl(1:n.y.lev)], lwd=lwd),
                    points=list(pch=symbols[.mods(1:n.y.lev)], col=colors[.modc(1:n.y.lev)]),
                    columns = if ("x" %in% names(key.args)) 1 else 
                      find.legend.columns(length(n.y.lev), 
                            space=if("x" %in% names(key.args)) "top" else key.args$space))

        for (k in names(key.args)) key[k] <- key.args[k]
        if (show.strip.values){
          for (pred in predictors[-x.var]){
            Data[[pred]] <- as.factor(Data[[pred]])
          }
        }
        result <- xyplot(eval(if (type=="probability") 
          parse(text=if (n.predictors==1) 
            paste("prob ~ as.numeric(", predictors[x.var], ")")
            else paste("prob ~ as.numeric(", predictors[x.var],") | ", 
                       paste(predictors[-x.var], collapse="*")))
          else parse(text=if (n.predictors==1) 
            paste("logit ~ as.numeric(", predictors[x.var], ")")
            else paste("logit ~ as.numeric(", predictors[x.var],") | ", 
                       paste(predictors[-x.var], collapse="*")))), 
          strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                             par.strip.text=list(cex=cex.strip)),
          panel=function(x, y, subscripts, rug, z, x.vals, lower, upper, ...){
            if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
            for (i in 1:n.y.lev){
              os <- if (ci.style == "bars"){
                (i - (n.y.lev + 1)/2) * (2/(n.y.lev-1)) * .01 * (n.y.lev - 1)
              } else {
                0
              }
              sub <- z[subscripts] == y.lev[i]
              good <- !is.na(y[sub])
              effect.llines(x[sub][good] + os, y[sub][good], lwd=lwd, type="b", col=colors[.modc(i)], lty=lines[.modl(i)],
                            pch=symbols[i], cex=cex, ...)
              if (ci.style == "bars"){
                larrows(x0=x[sub][good] + os, y0=lower[ ][sub][good], 
                        x1=x[sub][good] + os, y1=upper[subscripts][sub][good], 
                        angle=90, code=3, col=colors[.modc(i)], length=0.125*cex/1.5)
              }
              else  if(ci.style == "lines"){
                effect.llines(x[sub][good], lower[subscripts][sub][good], lty=lines[.modl(i)], col=colors[.modc(i)])
                effect.llines(x[sub][good], upper[subscripts][sub][good], lty=lines[.modl(i)], col=colors[.modc(i)])
              }
              else { if(ci.style == "bands") {		
                panel.bands(x[sub][good], y[sub][good],
                            lower[subscripts][sub][good], upper[subscripts][sub][good],
                            fill=colors[.modc(i)], alpha=band.transparency)
              }}
            }
          },
          ylab=ylab,
          ylim= if (is.null(ylim)) c(min(lower), max(upper)) else ylim,
          xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
          x.vals=x$data[[predictors[x.var]]], 
          rug=rug,
          z=response,
          lower=lower,
          upper=upper,
          scales=list(x=list(at=1:length(levs), labels=levs, rot=rotx, cex=cex.x),
                      y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),  
                      alternating=alternating),
          main=main,
          key=key,
          layout=layout,
          data=Data, ...)
        result$split <- split
        result$more <- more
        class(result) <- c("plot.eff", class(result))    		
      }
      else { # x-variable numeric
        if(use.splines) effect.llines <- spline.llines # added 10/17/13
        range <- range(c(lower, upper), na.rm=TRUE)
        ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                   range[2] + .025*(range[2] - range[1]))
        tickmarks <- make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
        nm <- predictors[x.var]
        x.vals <- x$data[[nm]]   
        if (nm %in% names(ticks.x)){
          at <- ticks.x[[nm]]$at
          n <- ticks.x[[nm]]$n
        }
        else{
          at <- NULL
          n <- 5
        }
        xlm <- if (nm %in% names(xlim)){
          xlim[[nm]]
        }
        else range_adj(Data[nm]) # range(x.vals)
        tickmarks.x <- if ((nm %in% names(transform.x)) && !(is.null(transform.x))){
          trans <- transform.x[[nm]]$trans
          make.ticks(trans(xlm), link=transform.x[[nm]]$trans, inverse=transform.x[[nm]]$inverse, at=at, n=n)
        }
        else {
          trans <- I
          make.ticks(xlm, link=I, inverse=I, at=at, n=n)
        }
        key <- list(title=x$response, cex.title=1, border=TRUE,
                    text=list(as.character(unique(response))), 
                    lines=list(col=colors[.modc(1:n.y.lev)], lty=lines[.modl(1:n.y.lev)], lwd=lwd),
                    columns = if ("x" %in% names(key.args)) 1 else 
                      find.legend.columns(length(n.y.lev), 
                            space=if("x" %in% names(key.args)) "top" else key.args$space))
        for (k in names(key.args)) key[k] <- key.args[k]
        if (show.strip.values){
          for (pred in predictors[-x.var]){
            Data[[pred]] <- as.factor(Data[[pred]])
          }
        }
        result <- xyplot(eval(if (type=="probability") 
          parse(text=if (n.predictors==1) paste("prob ~ trans(", predictors[x.var], ")")
                else paste("prob ~ trans(", predictors[x.var],") |", 
                           paste(predictors[-x.var], collapse="*")))
          else parse(text=if (n.predictors==1) paste("logit ~ trans(", predictors[x.var], ")")
                     else paste("logit ~ trans(", predictors[x.var],") | ", 
                                paste(predictors[-x.var], collapse="*")))), 
          strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ", par.strip.text=list(cex=cex.strip),
                             par.strip.text=list(cex=cex.strip)),
          panel=function(x, y, subscripts, rug, z, x.vals, lower, upper, ...){
            if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
            if (rug) lrug(trans(x.vals))
            for (i in 1:n.y.lev){
              sub <- z[subscripts] == y.lev[i]
              good <- !is.na(y[sub])
              effect.llines(x[sub][good], y[sub][good], lwd=lwd, type="l", col=colors[.modc(i)], lty=lines[.modl(i)], ...)
              if (ci.style == "bars"){
                larrows(x0=x[sub][good], y0=lower[subscripts][sub][good], 
                        x1=x[sub][good], y1=upper[subscripts][sub][good], 
                        angle=90, code=3, col=colors[.modc(i)], length=0.125*cex/1.5)
              }
              else  if(ci.style == "lines"){
                effect.llines(x[sub][good], lower[subscripts][sub][good], lty=lines[.modl(i)], col=colors[.modc(i)])
                effect.llines(x[sub][good], upper[subscripts][sub][good], lty=lines[.modl(i)], col=colors[.modc(i)])
              }
              else { if(ci.style == "bands") {		
                panel.bands(x[sub][good], y[sub][good],
                            lower[subscripts][sub][good], upper[subscripts][sub][good],
                            fill=colors[.modc(i)], alpha=band.transparency)
              }}
            }
          },
          ylab=ylab,
          xlim=suppressWarnings(trans(xlm)),
          ylim= if (is.null(ylim)) c(min(lower), max(upper)) else ylim,
          xlab=if (is.null(xlab)) predictors[x.var] else xlab[[x.var]],
          x.vals=x$data[[predictors[x.var]]], 
          rug=rug,
          z=response,
          lower=lower,
          upper=upper,
          scales=list(x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx, cex=cex.x), 
                      y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),
                      alternating=alternating),
          main=main,
          key=key,
          layout=layout,
          data=Data, ...)
        result$split <- split
        result$more <- more
        class(result) <- c("plot.eff", class(result))			
      }
      
    }
  }
  result
}

# effect generic and methods; allEffects
# John Fox, Sanford Weisberg, and Jangman Hong
#  last modified 2012-12-08 by J. Fox
#  10/31/2012 modifed effect.lm to use z distn for ses with mer and nlme objects
# 12-21-2012 Allow for empty cells in factor interactions, S. Weisberg
# 7-15-2013:  S. Weisberg:  deleted 'default.levels' argument.  Changed and
#    generalized xlevels argument to include the function of default.levels.
# 2013-10-15: eliminated generic effect() and all its methods. J. Fox
# 2014-07-02: added vcov. argument to effect 
# 2014-12-10: Changed 'effect' back to a generic function.  S. Weisberg
# 2017-12-08: For compatibility with Effect.default, changed test for itercept i effect.default.  SW
# 2017-12-08: Removed unneeded allEffects.gls

effect <- function(term, mod, vcov.=vcov, ...){
  UseMethod("effect", mod)
}

effect.default <- function(term, mod, vcov.=vcov, ...){ 
  term <- gsub(" ", "", gsub("\\*", ":", term))
  terms <- term.names(mod)
  if ( terms[1] == "(Intercept)") terms <- terms[-1]
#  if (has.intercept(mod)) terms <- terms[-1]
  which.term <- which(term == terms)
  mod.aug<- list()
  if (length(which.term) == 0){
    message("NOTE: ", term, " does not appear in the model")
    mod.aug <- update(formula(mod), eval(parse(text=paste(". ~ . +", term))))
  }
  if (!is.high.order.term(term, mod, mod.aug))
    message("NOTE: ", term, " is not a high-order term in the model")
  predictors <- all.vars(parse(text=term)) 
  Effect(predictors, mod, vcov.=vcov., ...)
}


allEffects <- function(mod, ...) UseMethod("allEffects")

allEffects.default <- function(mod, ...){
  high.order.terms <- function(mod){
    names <- term.names(mod)
    if (has.intercept(mod)) names<-names[-1]
    rel <- lapply(names, descendants, mod=mod)
    (1:length(names))[sapply(rel, function(x) length(x)==0)]
  }
  names <- term.names(mod)
  if (has.intercept(mod)) names <- names[-1]
  if (length(names) == 0) stop("the model contains no terms (beyond a constant)")
  terms <- names[high.order.terms(mod)]
  result <- lapply(terms, effect, mod=mod, ...)
  names(result) <- terms
  class(result) <- 'efflist'
  result
}

#  Calculate Effects for term(s) in a Multivariate Linear Model 
#  2014-03-12: Introduced allEffects.mlm(). J. Fox


Effect.mlm <- function(focal.predictors, mod, response, ...) {
  if (missing(response)) {
    mod.frame <- model.frame(mod)
    response <- colnames(model.response(mod.frame))
  }
  else if (is.numeric(response)) {
    mod.frame <- model.frame(mod)
    response.names <- colnames(model.response(mod.frame))
    response <- response.names[response]
  }
  
  if (length(response)==1) {
    mod.1 <- update(mod, as.formula(paste(response, " ~ .")))
    result <- Effect(focal.predictors, mod.1,  ...)
  }
  else { 
    result <- as.list(NULL)
    for (resp in response) {
      mod.1 <- update(mod, as.formula(paste(resp, " ~ .")))
      lab <- resp
      result[[lab]] <- Effect(focal.predictors, mod.1,  ...)
    }
    class(result) <- "efflist"
  }
  result
}

allEffects.mlm <- function(mod, ...){
  result <- NextMethod()
  class(result) <- "mlm.efflist"
  result
}

plot.mlm.efflist <- function(x, ...){
  x <- do.call(c, x)
  class(x) <- "efflist"
  plot(x, ...)
}

summary.mlm.efflist <- function(object, ...){
  object <- do.call(c, object)
  for (effect in names(object)){
    cat("\n\nResponse:", object[[effect]]$response, "\n")
    print(summary(object[[effect]], ...))
  }
}

print.mlm.efflist <- function(x, ...){
  x <- do.call(c, x)
  for (effect in names(x)){
    cat("\n\nResponse:", x[[effect]]$response, "\n")
    print(x[[effect]], ...)
  }
  invisible(x) 
}

# plot.eff method for effects package, moved here from plot-summary-print-methods.R
# The plot.effpoly method remains there for now.
# 2013-10-17: Added use.splines keyword to plot.eff. Sandy
# 2013-10-17: Made ci.style="bands" default for variates; allow "bands" if multiline=TRUE
# 2013-10-29: fixed plot.eff() to handle factors with "valid" NA level. J. Fox
# 2014-03-03: modified plot.eff() to handle partial residuals. J. Fox
# 2014-09-20: fixed plot.eff() to work with partial residuals when rescale.axis=FALSE;
#             added smooth.residuals argument. J. Fox
# 2014-10-10: namespace fixes. J. Fox
# 2014-12-05: made key.args more flexible. J. Fox
# 2015-03-22: use wide columns by default only when x for legend not set. J. Fox
# 2015-03-25: use non-robust loess smooth for partial residuals for non-Gaussian families. J. Fox
# 2015-03-25: rationalized type and rescale.axis args to plot.eff(); deprecated rescale.axis arg. J. Fox
# 2015-05-28: added residuals.smooth.color argument. J. Fox
# 2015-08-28: added residuals.cex argument. J. Fox
# 2016-03-01: move computation of partial residuals to the plot.eff() method. J. Fox
# 2016-05-22: modified make.ticks() to avoid possible failure due to floating-point inaccuracy. J. Fox
# 2016-08-31: fixed plotting with partial residuals with various scalings of y-axis and x-axis. J. Fox
# 2016-09-16: added show.strip.values argument to plot.eff(). J. Fox
# 2017-06-12: fixed bug in plot.eff() for multiline displays with many conditioning variables. J. Fox
# 2017-07-15: modified plot.eff() to consolidate arguments and use lattice theme. J. Fox
# 2017-08-09: small bug fixes, reorganized axes=list(x=list()) argument. J. Fox
# 2017-08-17: tweaked layout. J. Fox
# 2017-08-23: Fixed bug with the lattice=list(array()) argument in plot.efflist --- lattice was as
#              an argument to the next method twice
# 2017-08-23: plot.eff, in key.args, set default for between.columns=0
# 2017-08-20: reintroduce legacy arguments for plot.eff()
# 2017-09-10: use replacement for grid.panel()
# 2017-11-03: Added a test to assume that at least one point will be plotted in a tile, else
#                draw a blank tile. Needed for rank-deficient models.  S. Weisberg.
# 2018-01-02: Changed the default key:  see lines 240-241
# 2018-01-02: Rewrote find.legend columns, lines 41-44
# 2018-01-30: enlarged text in key titles
# 2018-05-14: support plotting partial residuals against a factor on the horizontal axis in plot.lm()
# 2018-05-29: lty was ignored for multiplot with factor on x-axis; fixed (reported by Krisztian Magori)
# 2018-05-30: don't use hard-coded pch=19 when plotting a factor on the x-axis.
# 2018-06-30: add cex sub-args for x and y axes (suggestion of Charles Leger).
# 2018-07-04: add cex sub-arg for strips.
# 2018-10-09: moved transform arg from Effect to axes=list(y=list(transform=))
# 2018-10-15: moved z.var to lines=list(z.var)
# 2018-10-25: check number of points used for spline interpolation
# 2018-10-25: fixed bug in plot.eff() introduced by previous modification to as.data.frame.eff().
# 2018-11-03: fixed bug in plotting partial residuals when a factor focal predictor had empty levels.
# 2019-02-13: made sure lty not ignored.
# 2019-08-27: correctly handle character or logical predictor
# 2020-11-11: fixed bug in plot.eff when x.var is set for multiline plot.
# 2025-07-21: rename range_adj() to range_adj to avoid generic confusion.

# the following functions aren't exported

#find.legend.columns <- function(n, target=min(4, n)){
#  rem <- n %% target
#  if (rem != 0 && rem < target/2) target <- target - 1
#  target
#}
# new version 1/2/2017 by sw
find.legend.columns <- function(n, space="top"){
  if(space == "right") 1 else {
  if(n <= 2) 2 else {
   if(n == 3) 1 else 
     {if (n <= 6) 2 else 3}}}
}


make.ticks <- function(range, link, inverse, at, n) {
  warn <- options(warn=-1)
  on.exit(options(warn))
  link <- if (is.null(link))
    function(x) nlm(function(y) (inverse(y) - x)^2,
                    mean(range))$estimate
  else link
  if (is.null(n)) n <- 5
  labels <- if (is.null(at)){
    range.labels <- sapply(range, inverse)
    labels <- grid::grid.pretty(range.labels)
  }
  else at
  ticks <- try(sapply(labels, link), silent=TRUE)
  if (inherits(ticks, "try-error")){
    ticks <- seq(range[1], range[2], length=n)
  }
  list(at=ticks, labels=format(labels))
}

range_adj <- function(x){
  range <- range(x, na.rm=TRUE)
  c(range[1] - .025*(range[2] - range[1]),
    range[2] + .025*(range[2] - range[1]))
}

# added, modified from http://www.r-bloggers.com/confidence-bands-with-lattice-and-r/

panel.bands <- function(x, y, upper, lower, fill, col,
                        subscripts, ..., font, fontface, use.splines=FALSE)
{
  if(!missing(subscripts)) {
    upper <- upper[subscripts]
    lower <- lower[subscripts]
  }
  if (use.splines){
    if (length(x) < 5) warning("spline interpolation may be unstable with only ", length(x), " points")
    up <- spline(x, upper)
    down <- spline(x, lower)
    x <- up$x
    upper <- up$y
    lower <- down$y
  }
  panel.polygon(c(x, rev(x)), c(upper, rev(lower)),
                col = fill, fill=fill, border = FALSE,
                ...)
}


# modified by Michael Friendly: added key.args:
# modified by Michael Friendly: added ci.style="bands"
# modified by Michael Friendly: added lwd= argument for llines (not used elsewhere)
# modified by Michael Friendly: added alpha.band= argument for ci.style="bands"

spline.llines <- function(x, y, ...) {
  if (length(x) < 5) warning("spline interpolation may be unstable with only ", length(x), " points")
  llines(spline(x, y), ...)
}

plot.eff <- function(x, x.var,  
          main=paste(effect, "effect plot"),
          symbols=TRUE, lines=TRUE, axes, confint, partial.residuals, id, lattice,
          ...,
        # legacy arguments:
          multiline, z.var, rug, xlab, ylab, colors, cex, lty, lwd, ylim, xlim, 
          factor.names, ci.style, band.transparency, band.colors, type, ticks, 
          alternating, rotx, roty, grid, layout,
          rescale.axis, transform.x, ticks.x, show.strip.values, key.args, 
          use.splines, residuals.color, residuals.pch, residuals.cex, smooth.residuals,
          residuals.smooth.color, show.fitted, span)
{ 
  closest <- function(x, x0) apply(outer(x, x0, FUN=function(x, x0) abs(x - x0)), 1, which.min)
  .mod <- function(a, b) ifelse( (d <- a %% b) == 0, b, d)
  .modc <- function(a) .mod(a, length(colors))
  .mods <- function(a) .mod(a, length(symbols))
  .modl <- function(a) .mod(a, length(lines))
  .modb <- function(a) .mod(a, length(band.colors))
  
  if (!is.logical(lines) && !is.list(lines)) lines <- list(lty=lines)
  levels <- sapply(x$variables, function(z) length(as.vector(z[["levels"]])))
  lines <- applyDefaults(lines,
              defaults=list(multiline=is.null(x$se), 
                            z.var=which.min(levels),
                            lty=trellis.par.get("superpose.line")$lty,
                            lwd=trellis.par.get("superpose.line")$lwd[1], 
                            col=trellis.par.get("superpose.line")$col, 
                            splines=TRUE),
                         onFALSE=list(multiline=FALSE, lty=0, lwd=0, col=rgb(1, 1, 1, alpha=0), splines=FALSE),
                         arg="lines")
  if (missing(multiline)) multiline <- lines$multiline
  if (missing(z.var)) z.var <- lines$z.var
  if (missing(lwd)) lwd <- lines$lwd
  if (missing(colors)) colors <- lines$col
  if (missing(use.splines)) use.splines <- lines$splines
  lines <- if (missing(lty)) lines$lty else lty
  
  if (!is.logical(symbols) && !is.list(symbols)) symbols <- list(pch=symbols)
  symbols <- applyDefaults(symbols,
                           defaults=list(pch=trellis.par.get("superpose.symbol")$pch, cex=trellis.par.get("superpose.symbol")$cex[1]),
                           onFALSE=list(pch=NA_integer_, cex=0),
                           arg="symbols")
  cex <- symbols$cex
  symbols <- symbols$pch
  
  if (missing(axes)) axes <- NULL
  axes <- applyDefaults(axes, defaults=list(
    x=list(rotate=0, rug=TRUE, cex=1),
    y=list(lab=NA, lim=NA, cex=1, ticks=list(at=NULL, n=5), type="rescale", rotate=0, transform=NULL),
    alternating=TRUE, grid=FALSE),
    arg="axes")
  x.args <- applyDefaults(axes$x, defaults=list(rotate=0, rug=TRUE, cex=1), arg="axes$x")
  
  if (missing(xlab)) {
    xlab.arg <- FALSE
    xlab <- list()
  }
  if (missing(xlim)) {
    xlim.arg <- FALSE
    xlim <- list()
  }
  if (missing(ticks.x)) {
    ticks.x.arg <- FALSE
    ticks.x <- list()
  }
  if (missing(transform.x)) {
    transform.x.arg <- FALSE
    transform.x <- list()
  }
  if (missing(rotx)) rotx <- x.args$rotate
  if (missing(rug)) rug <- x.args$rug
  cex.x <- x.args$cex
  x.args$rotate <- NULL
  x.args$rug <- NULL
  x.args$cex <- NULL
  x.pred.names <- names(x.args)
  if (length(x.pred.names) > 0){
    for (pred.name in x.pred.names){
      x.pred.args <- applyDefaults(x.args[[pred.name]],
                                   defaults=list(lab=NULL, lim=NULL, ticks=NULL, transform=NULL),
                                   arg=paste0("axes$x$", pred.name))
      if (!xlab.arg) xlab[[pred.name]] <- x.pred.args$lab
      if (!xlim.arg) xlim[[pred.name]] <- x.pred.args$lim
      if (!ticks.x.arg) ticks.x[[pred.name]] <- x.pred.args$ticks
      if (!transform.x.arg) transform.x[[pred.name]] <- x.pred.args$transform
    }
  }
  if (length(xlab) == 0) xlab <- NA
  if (length(xlim) == 0) xlim <- NA
  if (length(ticks.x) == 0) ticks.x <- NA
  if (length(transform.x) == 0) transform.x <- NA
  
  y.args <- applyDefaults(axes$y, defaults=list(lab=NA, lim=NA, cex=1, ticks=list(at=NULL, n=5), type="rescale", rotate=0, transform=NULL), arg="axes$y")
  if (missing(ylab)) ylab <- y.args$lab
  if (missing(ylim)) ylim <- y.args$lim
  if (missing(ticks)) ticks <- y.args$ticks
  if (missing(type)) type <- y.args$type
  if (!missing(rescale.axis)) type <- if (rescale.axis) "rescale" else "response"
  type <- match.arg(type, c("rescale", "response", "link"))
  if (missing(roty)) roty <- y.args$rotate
  cex.y <- y.args$cex
  custom <- y.args$transform
  if(inherits(custom, "function")){
    custom <- list(trans=I, inverse=custom)
    type <- "response"
  }
#  if(!is.null(custom)) type="response" 
  if (missing(alternating)) alternating <- axes$alternating
  if (missing(grid)) grid <- axes$grid
  
  if (missing(confint) || isTRUE(confint)) confint <- NULL
  confint <- applyDefaults(confint,
                           defaults=list(style=NULL, alpha=0.15, col=colors),
                           onFALSE=list(style="none", alpha=0, col=NA_integer_),
                           arg="confint")
  if (missing(ci.style)) ci.style <- confint$style
  if (missing(band.transparency)) band.transparency <- confint$alpha
  if (missing(band.colors)) band.colors <- confint$col
  if(!is.null(ci.style)) ci.style <- match.arg(ci.style, c("auto", "bars", "lines", "bands", "none"))
  
  if (missing(partial.residuals)) partial.residuals <- NULL
  if (is.logical(partial.residuals)) partial.residuals <- list(plot=partial.residuals)
  partial.residuals <- applyDefaults(partial.residuals, defaults=list(
    plot=!is.null(x$residuals), fitted=FALSE, col=colors[2], pch=1, cex=1, smooth=TRUE, 
    span=2/3, smooth.col=colors[2], lty=lines[1], lwd=lwd),
    arg="partial.residuals")
  if (missing(show.fitted)) show.fitted <- partial.residuals$fitted
  if (missing(residuals.color)) residuals.color <- partial.residuals$col
  if (missing(residuals.pch)) residuals.pch <- partial.residuals$pch
  if (missing(residuals.cex)) residuals.cex <- partial.residuals$cex
  if (missing(smooth.residuals)) smooth.residuals <- partial.residuals$smooth
  if (missing(residuals.smooth.color)) residuals.smooth.color <- partial.residuals$smooth.col
  residuals.lty <- partial.residuals$lty
  residuals.lwd <- partial.residuals$lwd
  if (missing(span)) span <- partial.residuals$span
  partial.residuals <- partial.residuals$plot
  
  if (missing(id) || isFALSE(id)) {
    id.n <- 0
    id.cex <- 0
    id.col <- NULL
    id.labels <- NULL
  }
  else {
    id <- applyDefaults(id, list(
      n=2, cex=0.75, col=residuals.color, labels=NULL
    ), arg="id")
    id.n <- id$n
    id.col <- id$col
    id.cex <- id$cex
    id.labels <- id$labels
  }
  if (missing(lattice)) lattice <- NULL
  lattice <- applyDefaults(lattice, defaults=list(
    layout=NULL, #key.args=list(),  
    strip=list(factor.names=TRUE, values=!partial.residuals, cex=1),
    array=list(row=1, col=1, nrow=1, ncol=1, more=FALSE),
    arg="lattice"
  ))
  lattice$key.args <- applyDefaults(lattice$key.args, defaults=list(
    space="top", border=FALSE, fontfamily="sans", cex=.75, cex.title=1, 
    arg="key.args"
  ))
  if("x" %in% names(lattice$key.args)) lattice$key.args[["space"]] <- NULL
  if (missing(layout)) layout <- lattice$layout
  if (missing(key.args)){
    lattice$key.args[["between.columns"]] <- 
      if(is.null(lattice$key.args[["between.columns"]])) 0 else
      lattice$key.args[["between.columns"]]
    key.args <- lattice$key.args
  }
  strip.args <- applyDefaults(lattice$strip, defaults=list(factor.names=TRUE, values=!partial.residuals, cex=1), arg="lattice$strip")
  if (missing(factor.names)) factor.names <- strip.args$factor.names
  if (missing(show.strip.values)) show.strip.values <- strip.args$values
  cex.strip <- strip.args$cex
  height.strip <- max(1, cex.strip)
  array.args <- applyDefaults(lattice$array, defaults=list(row=1, col=1, nrow=1, ncol=1, more=FALSE), arg="lattice$array")
  row <- array.args$row
  col <- array.args$col
  nrow <- array.args$nrow
  ncol <- array.args$ncol
  more <- array.args$more
  
  if (smooth.residuals && !is.null(x$family)){
    loess.family <- if (x$family == "gaussian") "symmetric" else "gaussian"
    average.resid <- if (loess.family == "gaussian") mean else median
  }
  switch(type,
         rescale = {
           type <- "response"
           rescale.axis <- TRUE
         },
         response = {
           type <- "response"
           rescale.axis <- FALSE
         },
         link = {
           type <- "link"
           rescale.axis <- TRUE
         }
  )
#  levels <- sapply(x$variables, function(z) length(as.vector(z[["levels"]])))
  thresholds <- x$thresholds
  has.thresholds <- !is.null(thresholds)
  effect.llines <- llines
  if (length(ylab) == 1 && is.na(ylab)){
    ylab <- if (has.thresholds) paste(x$response, ": ", paste(x$y.levels, collapse=", "), sep="")
    else x$response
  }
  if (has.thresholds){
    threshold.labels <- abbreviate(x$y.levels, minlength=1)
    threshold.labels <- paste(" ",
                              paste(threshold.labels[-length(threshold.labels)], threshold.labels[-1], sep=" - "),
                              " ", sep="")
  }
  original.link <- trans.link <- 
    if(!is.null(custom)) custom$trans else x$transformation$link
  original.inverse <- trans.inverse <-
    if(!is.null(custom)) custom$inverse else  x$transformation$inverse
  residuals <- if (partial.residuals) x$residuals else NULL
  if (!is.null(residuals) && !is.null(id.labels)) names(residuals) <- id.labels
  partial.residuals.range <- x$partial.residuals.range
 
  if (!rescale.axis){
    x$lower[!is.na(x$lower)] <- trans.inverse(x$lower[!is.na(x$lower)])
    x$upper[!is.na(x$upper)] <- trans.inverse(x$upper[!is.na(x$upper)])
    x$fit[!is.na(x$fit)] <- trans.inverse(x$fit)[!is.na(x$fit)]
    trans.link <- trans.inverse <- I
  }
  x.all <- x$x.all
  if (!is.null(x.all)){
    for (i in 1:ncol(x.all)){
      if (inherits(x.all[, i], "factor"))  x.all[, i] <- droplevels(x.all[, i])
    }
  }
   split <- c(col, row, ncol, nrow)
  if (missing(x.var)) x.var <- x$x.var
  if (!is.null(x.var) && is.numeric(x.var)) x.var <- colnames(x$x)[x.var] 
  x.data <- x$data
  for (i in 1:ncol(x.data)){
    if (inherits(x.data[, i], "factor"))  x.data[, i] <- droplevels(x.data[, i])
  }
  effect <- paste(sapply(x$variables, "[[", "name"), collapse="*")
  vars <- x$variables
  x <- as.data.frame(x, type="link")
  for (i in 1:length(vars)){
    if (!(vars[[i]]$is.factor)) next
    x[, i] <- factor(x[,i], levels=vars[[i]]$levels, exclude=NULL)
    x[, i] <- droplevels(x[, i])
  }
  has.se <- !is.null(x$se)
  n.predictors <- ncol(x) - 1 - 3*has.se
  if (n.predictors == 1){
    predictor <- names(x)[1]
    if (is.list(xlab)) xlab <- xlab[[predictor]]
    ### factor no other predictors
    if (is.factor(x[,1])){
      ci.style <- if(is.null(ci.style) || ci.style == "auto") "bars" else ci.style
      range <- if(has.se & ci.style!="none")
        range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
      ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                 range[2] + .025*(range[2] - range[1]))
      if (!is.null(partial.residuals.range)){
        ylim[1] <- min(ylim[1], partial.residuals.range[1])
        ylim[2] <- max(ylim[2], partial.residuals.range[2])
      }
      tickmarks <- if (type == "response" && rescale.axis) 
        make.ticks(ylim, link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
      else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
      levs <- levels(x[,1])
      n.lev <- length(levs)
      plot <- xyplot(eval(parse(
        text=paste("fit ~ as.numeric(", names(x)[1], ")"))),
        strip=strip.custom(strip.names=c(factor.names, TRUE), 
          par.strip.text=list(cex=cex.strip)),
        par.settings=list(layout.heights=list(strip=height.strip)),
        panel=function(x, y, lower, upper, has.se, ...){ 
          if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
          good <- !is.na(y)
          if(!all(!good)){
          if (has.se){
            if (ci.style == "bars"){
              larrows(x0=x[good], y0=lower[good], x1=x[good], y1=upper[good], angle=90,
                      code=3, col=if (partial.residuals) band.colors[1] else colors[.modc(2)], 
                      length=0.125*cex/1.5)
            }
            else if(ci.style == "lines") {
              effect.llines(x[good], lower[good], lty=2, col=colors[.modc(2)])
              effect.llines(x[good], upper[good], lty=2, col=colors[.modc(2)])
            }
            else{ if(ci.style == "bands") {
              panel.bands(x[good], y[good], upper[good], lower[good], fill=band.colors[1],
                          alpha=band.transparency, use.splines=FALSE)
            }}
          }
          if (partial.residuals){
            x.fit <- as.numeric(as.factor(x.data[good, predictor]))
            partial.res <- y[x.fit] + residuals[good]
            lpoints(jitter(x.fit, factor=0.5), partial.res, col=residuals.color, pch=residuals.pch, cex=residuals.cex)
            if (smooth.residuals && length(partial.res) != 0) {
              lpoints(1:n.lev, tapply(partial.res, x.fit, average.resid), pch=16, cex=residuals.cex*1.25, col=residuals.color)
            }
          }
          effect.llines(x[good], y[good], lwd=lwd, col=colors[1], lty=lines, type='b', pch=symbols[1], cex=cex, ...)
          if (has.thresholds){
            panel.abline(h=thresholds, lty=3)
            panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)),
                       thresholds, threshold.labels, adj=c(0,0), cex=0.75)
            panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                       thresholds, threshold.labels, adj=c(1,0), cex=0.75)
          }
        }},
        ylim=ylim,
        ylab=ylab,
        xlab=if (length(xlab) == 1 && is.na(xlab)) names(x)[1] else xlab,
        scales=list(x=list(at=1:length(levs), labels=levs, rot=rotx, cex=cex.x),
                    y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),
                    alternating=alternating, y=roty),
        main=main,
        lower=x$lower, upper=x$upper, has.se=has.se, data=x, ...)
      result <- update(plot, layout = if (is.null(layout)) c(0, prod(dim(plot)))
                       else layout)
      result$split <- split
      result$more <- more
      class(result) <- c("plot.eff", class(result))
    }
    ### variate, no other predictors  ***
    else {
      effect.llines <- if(use.splines) spline.llines else effect.llines
      ci.style <- if(is.null(ci.style) || ci.style == "auto") "bands" else ci.style
      range <- if(has.se && ci.style!="none")
        range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
      
      ylim <- if (!any(is.na(ylim))) ylim
      else if (is.null(residuals)) c(range[1] - .025*(range[2] - range[1]), range[2] + .025*(range[2] - range[1]))
      else if (rescale.axis) c(min(partial.residuals.range[1], range[1] - .025*(range[2] - range[1])),
                               max(partial.residuals.range[2], range[2] + .025*(range[2] - range[1])))
      else c(min(original.inverse(partial.residuals.range[1]), range[1] - .025*(range[2] - range[1])),
             max(original.inverse(partial.residuals.range[2]), range[2] + .025*(range[2] - range[1])))
      tickmarks <- if (type == "response" && rescale.axis) 
              make.ticks(ylim, link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
      else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
      nm <- names(x)[1]
      x.vals <- x.data[, nm]
      if (nm %in% names(ticks.x)){
        at <- ticks.x[[nm]]$at
        n <- ticks.x[[nm]]$n
      }
      else{
        at <- NULL
        n <- 5
      }
      xlm <- if (nm %in% names(xlim)){
        xlim[[nm]]
      }
      else range_adj(x[nm]) # range(x.vals)
      tickmarks.x <- if ((nm %in% names(transform.x)) && !(is.null(transform.x))){
        trans <- transform.x[[nm]]$trans
        make.ticks(trans(xlm), link=transform.x[[nm]]$trans, inverse=transform.x[[nm]]$inverse, at=at, n=n)
      }
      else {
        trans <- I
        make.ticks(xlm, link=I, inverse=I, at=at, n=n)
      }
      
      if (is.null(x.var)){
        if (!is.null(residuals)){
          x.var <- names(x)[1]
        }
        else x.var <-  which.max(levels)
      }
      if (!is.null(residuals)) x.fit <- x.data[, predictor]
      if (is.numeric(x.var)) x.var <- predictor
      plot <- xyplot(eval(parse(
        text=paste("fit ~ trans(", x.var, ")"))),
        strip=strip.custom(strip.names=c(factor.names, TRUE),
          par.strip.text=list(cex=cex.strip)),
        par.settings=list(layout.heights=list(strip=height.strip)),
        panel=function(x, y, x.vals, rug, lower, upper, has.se, ...){ 
          if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
          good <- !is.na(y)
          if(!all(!good)){
          axis.length <- diff(range(x))
          effect.llines(x[good], y[good], lwd=lwd, col=colors[1], lty=lines,  ...)
          if (rug && is.null(residuals)) lrug(trans(x.vals))
          if (has.se){
            if (ci.style == "bars"){
              larrows(x0=x[good], y0=lower[good],
                      x1=x[good], y1=upper[good],
                      angle=90, code=3, col=if (partial.residuals) band.colors[1] else colors[.modc(2)],
                      length=.125*cex/1.5)
            }
            else if(ci.style == "lines") {
              effect.llines(x[good], lower[good], lty=2, col=colors[.modc(2)])
              effect.llines(x[good], upper[good], lty=2, col=colors[.modc(2)])
            }
            else{ if(ci.style == "bands") {
              panel.bands(x[good], y[good], upper[good], lower[good], fill=band.colors[1],
                          alpha=band.transparency, use.splines=use.splines)
            }}
          }
          if (has.thresholds){
            panel.abline(h=thresholds, lty=3)
            panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)),
                       thresholds, threshold.labels, adj=c(0,0), cex=0.75)
            panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                       thresholds, threshold.labels, adj=c(1,0), cex=0.75)
          }
          if (!is.null(residuals)){
            fitted <- y[good][closest(trans(x.fit), x[good])]
            partial.res <- if (!rescale.axis) original.inverse(original.link(fitted) + residuals)
            else fitted + residuals
            lpoints(trans(x.fit), partial.res, col=residuals.color, pch=residuals.pch, cex=residuals.cex)
            if (show.fitted) lpoints(trans(x.fit), fitted, pch=16, col=residuals.color)  # REMOVE ME
            if (smooth.residuals){
              llines(loess.smooth(trans(x.fit), partial.res, span=span, family=loess.family), lwd=residuals.lwd, lty=residuals.lty, col=residuals.smooth.color)
            }
            if (id.n > 0){
              M <- cbind(trans(x.fit), partial.res)
              md <- mahalanobis(M, colMeans(M), cov(M))
              biggest <- order(md, decreasing=TRUE)[1:id.n]
              pos <- ifelse(trans(x.fit[biggest]) > mean(current.panel.limits()$xlim), 2, 4)
              ltext(trans(x.fit[biggest]), partial.res[biggest], 
                    names(partial.res)[biggest], pos=pos, col=id.col, cex=id.cex)
            }
          }
          
        }},
        ylim=ylim,
        xlim=suppressWarnings(trans(xlm)),
        ylab=ylab,
        xlab=if (length(xlab) == 1 && is.na(xlab)) names(x)[1] else xlab,
        x.vals=x.vals, rug=rug,
        main=main,
        lower=x$lower, upper=x$upper, has.se=has.se, data=x,
        scales=list(y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),
                    x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx, cex=cex.x), alternating=alternating), ...)
      result <- update(plot, layout = if (is.null(layout)) c(0, prod(dim(plot)))
                       else layout)
      result$split <- split
      result$more <- more
      class(result) <- c("plot.eff", class(result))
    }
    return(result)
  }
  ###  more than one predictor
  predictors <- names(x)[1:n.predictors]
  levels <- sapply(apply(x[,predictors], 2, unique), length)
  if (is.null(x.var)){
    if (!is.null(residuals)){
      x.var <- names(x)[1]
    }
    else x.var <-  which.max(levels)
  }
  if (is.list(xlab)) xlab <- xlab[[x.var]]
  if (!is.null(residuals)) x.fit <- x.data[, x.var]
  if (is.character(x.var)) {
    which.x <- which(x.var == predictors)
    if (length(which.x) == 0) stop(paste("x.var = '", x.var, "' is not in the effect.", sep=""))
    x.var <- which.x
  }
  if (is.character(z.var)) {
    which.z <- which(z.var == predictors)
    if (length(which.z) == 0) stop(paste("z.var = '", z.var, "' is not in the effect.", sep=""))
    z.var <- which.z
  }
  if (x.var == z.var){ # z.var <- z.var + 1
    levels.not.x <- levels
    levels.not.x[x.var] <- Inf
    z.var <- which.min(levels.not.x)
  } 
  
  ### multiline
  if (multiline){
    if (!is.null(residuals)) warning("partial residuals are not displayed in a multiline plot")
    ci.style <- if(is.null(ci.style)) "none" else ci.style
    if(ci.style == "lines") {
      cat("Confidence interval style 'lines' changed to 'bars'\n")
      ci.style <- "bars"}
    range <- if (has.se && ci.style !="none")
      range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
    ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                               range[2] + .025*(range[2] - range[1]))
    tickmarks <- if (type == "response" && rescale.axis) make.ticks(ylim, link=trans.link,
                                                                    inverse=trans.inverse, at=ticks$at, n=ticks$n)
    else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
    zvals <- unique(x[, z.var])
    ### multiline factor
    if (is.factor(x[,x.var])){
      if (ci.style == "auto") ci.style <- "bars"
      levs <- levels(x[,x.var])
      key <- list(title=predictors[z.var], #cex.title=1, border=TRUE,
                  text=list(as.character(zvals)),
                  lines=list(col=colors[.modc(1:length(zvals))], 
                             lty=lines[.modl(1:length(zvals))], lwd=lwd),
                  points=list(col=colors[.modc(1:length(zvals))], 
                              pch=symbols[.mods(1:length(zvals))]),
                  columns = if ("x" %in% names(key.args)) 1 else 
                    find.legend.columns(length(zvals), 
                        space=if("x" %in% names(key.args)) "top" else key.args$space))
      for (k in names(key.args)) key[k] <- key.args[k]
      if (show.strip.values && n.predictors > 2){
        for (pred in predictors[-c(x.var, z.var)]){
          x[[pred]] <- as.factor(x[[pred]])
        }
      }
      plot <- xyplot(eval(parse(
        text=paste("fit ~ as.numeric(", predictors[x.var], ")",
                   if (n.predictors > 2) paste(" |",
                                               paste(predictors[-c(x.var, z.var)], collapse="*"))))),
        strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ",
          par.strip.text=list(cex=cex.strip)),
        par.settings=list(layout.heights=list(strip=height.strip)),
        panel=function(x, y, subscripts, z, lower, upper, show.se, ...){ 
          if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
          for (i in 1:length(zvals)){
            sub <- z[subscripts] == zvals[i]
            good <- !is.na(y[sub])
            if(!all(!good)){
            os <- if(show.se)
              (i - (length(zvals) + 1)/2) * (2/(length(zvals)-1)) *
              .01 * (length(zvals) - 1) else 0
            effect.llines(x[sub][good]+os, y[sub][good], lwd=lwd, type='b', col=colors[.modc(i)],
                          pch=symbols[.mods(i)], lty=lines[.modl(i)], cex=cex, ...)
            if (show.se){
              larrows(x0=x[sub][good]+os, y0=lower[subscripts][sub][good],
                      x1=x[sub][good]+os, y1=upper[subscripts][sub][good],
                      angle=90, code=3, col=eval(colors[.modc(i)]),
                      length=.125*cex/1.5)
            }
          }}
          if (has.thresholds){
            panel.abline(h=thresholds, lty=3)
            panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)),
                       thresholds, threshold.labels, adj=c(0,0), cex=0.75)
            panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                       thresholds, threshold.labels, adj=c(1,0), cex=0.75)
          }
        },
        ylim=ylim,
        ylab=ylab,
        xlab=if (length(xlab) == 1 && is.na(xlab)) predictors[x.var] else xlab,
        z=x[,z.var],
        scales=list(x=list(at=1:length(levs), labels=levs, rot=rotx, cex=cex.x),
                    y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),
                    alternating=alternating),
        zvals=zvals,
        main=main,
        key=key,
        lower=x$lower, upper=x$upper,
        show.se=has.se && ci.style=="bars",
        data=x, ...)
      result <- update(plot, layout = if (is.null(layout))
        c(0, prod(dim(plot))) else layout)
      result$split <- split
      result$more <- more
      class(result) <- c("plot.eff", class(result))
    }
    ### multiline variate
    else{
      if (ci.style == "auto") ci.style <- "bands"
      effect.llines <- if(use.splines) spline.llines else effect.llines
      nm <- names(x)[x.var]
      x.vals <- x.data[, nm]
      if (nm %in% names(ticks.x)){
        at <- ticks.x[[nm]]$at
        n <- ticks.x[[nm]]$n
      }
      else{
        at <- NULL
        n <- 5
      }
      xlm <- if (nm %in% names(xlim)){
        xlim[[nm]]
      }
      else range_adj(x[nm])
      tickmarks.x <- if ((nm %in% names(transform.x)) && !(is.null(transform.x))){
        trans <- transform.x[[nm]]$trans
        make.ticks(trans(xlm), link=transform.x[[nm]]$trans, inverse=transform.x[[nm]]$inverse, at=at, n=n)
      }
      else {
        trans <- I
        make.ticks(xlm, link=I, inverse=I, at=at, n=n)
      }
      key <- list(title=predictors[z.var], #cex.title=1, border=TRUE,
                  text=list(as.character(zvals)),
                  lines=list(col=colors[.modc(1:length(zvals))], 
                             lty=lines[.modl(1:length(zvals))], lwd=lwd),
                  columns = if ("x" %in% names(key.args)) 1 else 
                             find.legend.columns(length(zvals), 
                             if("x" %in% names(key.args)) "top" else key.args$space))
      for (k in names(key.args)) key[k] <- key.args[k]
      if (show.strip.values && n.predictors > 2){
        for (pred in predictors[-c(x.var, z.var)]){
          x[[pred]] <- as.factor(x[[pred]])
        }
      }
      plot <- xyplot(eval(parse(
        text=paste("fit ~trans(", predictors[x.var], ")",
                   if (n.predictors > 2) paste(" |",
                                               paste(predictors[-c(x.var, z.var)], collapse="*"))))),
        strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ",
          par.strip.text=list(cex=cex.strip)),
        par.settings=list(layout.heights=list(strip=height.strip)),
        panel=function(x, y, subscripts, x.vals, rug, z, lower, upper, show.se, ...){ 
          if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
          if (rug && is.null(residuals)) lrug(trans(x.vals))
          axis.length <- diff(range(x))
          for (i in 1:length(zvals)){
            sub <- z[subscripts] == zvals[i]
            good <- !is.na(y[sub])
            if(!all(!good)){
            effect.llines(x[sub][good], y[sub][good], lwd=lwd, type='l',
                          col=colors[.modc(i)], lty=lines[.modl(i)], cex=cex, ...)
            if(show.se){
              if(ci.style == "bars"){
                os <- (i - (length(zvals) + 1)/2) * (2/(length(zvals)-1)) *
                  .01 * axis.length
                larrows(x0=x[sub][good]+os, y0=lower[subscripts][sub][good],
                        x1=x[sub][good]+os, y1=upper[subscripts][sub][good],
                        angle=90, code=3, col=eval(colors[.modc(i)]),
                        length=.125*cex/1.5)
              }
              if(ci.style == "bands"){
                panel.bands(x[sub][good], y[sub][good],
                            upper[subscripts][sub][good], lower[subscripts][sub][good],
                            fill=eval(band.colors[.modb(i)]),
                            alpha=band.transparency, use.splines=use.splines)
              }
            }
          }}
          if (has.thresholds){
            panel.abline(h=thresholds, lty=3)
            panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)),
                       thresholds, threshold.labels, adj=c(0,0), cex=0.75)
            panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                       thresholds, threshold.labels, adj=c(1,0), cex=0.75)
          }
        },
        ylim=ylim,
        xlim=suppressWarnings(trans(xlm)),
        ylab=ylab,
        xlab=if (length(xlab) == 1 && is.na(xlab)) predictors[x.var] else xlab,
        x.vals=x.vals, rug=rug,
        z=x[,z.var],
        zvals=zvals,
        main=main,
        key=key,
        #
        lower=x$lower, upper=x$upper,
        show.se=has.se && ci.style %in% c("bars", "bands"),
        #
        data=x, scales=list(y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y), 
                            x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx, cex=cex.x),
                            alternating=alternating),  ...)
      result <- update(plot, layout = if (is.null(layout)) c(0, prod(dim(plot)))
                       else layout)
      result$split <- split
      result$more <- more
      class(result) <- c("plot.eff", class(result))
    }
    return(result)
  }
  # multiplot
  ci.style <- if(is.null(ci.style) || ci.style == "auto"){
    if(is.factor(x[, x.var])) "bars" else "bands"} else ci.style
  range <- if (has.se && ci.style !="none")
    range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
  # multiplot factor
  if (is.factor(x[,x.var])){
    
    ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                               range[2] + .025*(range[2] - range[1]))
    if (!is.null(partial.residuals.range)){
      ylim[1] <- min(ylim[1], partial.residuals.range[1])
      ylim[2] <- max(ylim[2], partial.residuals.range[2])
    }
    tickmarks <- if (type == "response" && rescale.axis) make.ticks(ylim, link=trans.link,
                                                                    inverse=trans.inverse, at=ticks$at, n=ticks$n)
    else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
    
    levs <- levels(x[,x.var])
    if (show.strip.values){
      for (pred in predictors[-x.var]){
        x[[pred]] <- as.factor(x[[pred]])
      }
    }
    n.lev <- length(levs)
    x.fit <- x.data[, predictors[x.var]]
    use <- rep(TRUE, length(residuals))
    xx <- x[, predictors[-x.var], drop=FALSE]
    plot <- xyplot(eval(parse(
      text=paste("fit ~ as.numeric(", predictors[x.var], ") |",
                 paste(predictors[-x.var], collapse="*")))),
      strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ",
        par.strip.text=list(cex=cex.strip)),
      par.settings=list(layout.heights=list(strip=height.strip)),
      panel=function(x, y, subscripts, lower, upper, has.se, ...){ 
        if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
        good <- !is.na(y)
        no.points <- all(!good) # skip arrows and lines if no.points==TRUE
        if(!no.points){
        if (has.se){
          if (ci.style == "bars"){
            larrows(x0=x[good], y0=lower[subscripts][good], x1=x[good], y1=upper[subscripts][good],
                    angle=90, code=3, col=if (partial.residuals) band.colors[1] else colors[.modc(2)], 
                    length=0.125*cex/1.5)
          }
          else if(ci.style == "lines") {
            effect.llines(x[good], lower[subscripts][good], lty=2, col=colors[.modc(2)])
            effect.llines(x[good], upper[subscripts][good], lty=2, col=colors[.modc(2)])
          }
          else{ if(ci.style == "bands") {
            panel.bands(x[good], y[good], upper[subscripts][good], lower[subscripts][good],
                        fill=band.colors[1], alpha=band.transparency, use.splines=FALSE)
          }}
        }
        if (!is.null(residuals)){
          predictors <- predictors[-x.var]
          factors <- sapply(xx, is.factor)
          for (predictor in predictors){
            use <- use & if(factors[predictor]) x.all[, predictor] == xx[subscripts[1], predictor]
            else x.all[, predictor] == xx[subscripts[1], predictor]
          }
          n.in.panel <- sum(use)
          if (n.in.panel > 0){
            fitted <- y[good][as.numeric(as.factor(x.fit[use]))] 
            partial.res <- if (!rescale.axis) original.inverse(original.link(fitted) + residuals[use])
            else fitted + residuals[use]
            lpoints(jitter(as.numeric(as.factor(x.fit[use])), 0.5), partial.res, col=residuals.color, pch=residuals.pch, cex=residuals.cex)
            if (show.fitted) lpoints(x.fit[use], fitted, pch=16, col=residuals.color)  # REMOVE ME
            if (smooth.residuals && n.in.panel != 0) {
              lpoints(1:n.lev, tapply(partial.res, x.fit[use], average.resid), pch=16, cex=1.25*residuals.cex, col=residuals.color)
            }
            if (id.n > 0){
              M <- cbind(trans(x.fit[use]), partial.res)
              md <- mahalanobis(M, colMeans(M), cov(M))
              biggest <- order(md, decreasing=TRUE)[1:id.n]
              pos <- ifelse(x.fit[use][biggest] > mean(current.panel.limits()$xlim), 2, 4)
              ltext(x.fit[use][biggest], partial.res[biggest], 
                    names(partial.res)[biggest], pos=pos, col=id.col, cex=id.cex)
            }
          }
        }
        effect.llines(x[good], y[good], lwd=lwd, lty=lines, type='b', col=colors[1], pch=symbols[1], cex=cex, ...)
        if (has.thresholds){
          panel.abline(h=thresholds, lty=3)
          panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)),
                     thresholds, threshold.labels, adj=c(0,0), cex=0.75)
          panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                     thresholds, threshold.labels, adj=c(1,0), cex=0.75)
        }
      }},
      ylim=ylim,
      ylab=ylab,
      xlab=if (length(xlab) == 1 && is.na(xlab)) predictors[x.var] else xlab,
      scales=list(x=list(at=1:length(levs), labels=levs, rot=rotx, cex=cex.x),
                  y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),
                  alternating=alternating),
      main=main,
      lower=x$lower, upper=x$upper, has.se=has.se, data=x, ...)
    result <- update(plot, layout = if (is.null(layout)) c(0, prod(dim(plot))) else layout)
    result$split <- split
    result$more <- more
    class(result) <- c("plot.eff", class(result))
  }
  ### multiplot variate   ***
  else{
    effect.llines <- if(use.splines) spline.llines else effect.llines
    nm <- names(x)[x.var]
    x.vals <- x.data[, nm]
    if (nm %in% names(ticks.x)){
      at <- ticks.x[[nm]]$at
      n <- ticks.x[[nm]]$n
    }
    else{
      at <- NULL
      n <- 5
    }
    xlm <- if (nm %in% names(xlim)){
      xlim[[nm]]
    }
    else range_adj(x[nm])
    tickmarks.x <- if ((nm %in% names(transform.x)) && !(is.null(transform.x))){
      trans <- transform.x[[nm]]$trans
      make.ticks(trans(xlm), link=transform.x[[nm]]$trans, inverse=transform.x[[nm]]$inverse, at=at, n=n)
    }
    else {
      trans <- I
      make.ticks(xlm, link=I, inverse=I, at=at, n=n)
    }
    ylim <- if (!any(is.na(ylim))) ylim
    else if (is.null(residuals)) c(range[1] - .025*(range[2] - range[1]), range[2] + .025*(range[2] - range[1]))
    else if (rescale.axis) c(min(partial.residuals.range[1], range[1] - .025*(range[2] - range[1])),
                             max(partial.residuals.range[2], range[2] + .025*(range[2] - range[1])))
    else c(min(original.inverse(partial.residuals.range[1]), range[1] - .025*(range[2] - range[1])),
           max(original.inverse(partial.residuals.range[2]), range[2] + .025*(range[2] - range[1])))
    tickmarks <- if (type == "response" && rescale.axis) make.ticks(ylim, link=trans.link,
                                                                    inverse=trans.inverse, at=ticks$at, n=ticks$n)
    else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
    x.fit <- x.data[, predictors[x.var]]
    use <- rep(TRUE, length(residuals))
    xx <- x[, predictors[-x.var], drop=FALSE]
    if (show.strip.values){
      for (pred in predictors[-x.var]){
        x[[pred]] <- as.factor(x[[pred]])
      }
    }
    plot <- xyplot(eval(parse(
      text=paste("fit ~ trans(", predictors[x.var], ") |",
                 paste(predictors[-x.var], collapse="*")))),
      strip=strip.custom(strip.names=c(factor.names, TRUE), sep=" = ",
        par.strip.text=list(cex=cex.strip)),
      par.settings=list(layout.heights=list(strip=height.strip)),
      panel=function(x, y, subscripts, x.vals, rug, lower, upper, has.se, ...){
        if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
        good <- !is.na(y)
        if(!all(!good)){
        effect.llines(x[good], y[good], lwd=lwd, col=colors[1], lty=lines, ...)
        if (rug && is.null(residuals)) lrug(trans(x.vals))
        if (has.se){
          if (ci.style == "bars"){
            larrows(x0=x[good], y0=lower[subscripts][good],
                    x1=x[good], y1=upper[subscripts][good],
                    angle=90, code=3, col=if (partial.residuals) band.colors[1] else colors[.modc(2)],
                    length=.125*cex/1.5)
          }
          else if(ci.style == "lines") {
            effect.llines(x[good], lower[subscripts][good], lty=2, col=colors[.modc(2)])
            effect.llines(x[good], upper[subscripts][good], lty=2, col=colors[.modc(2)])
          }
          else if(ci.style == "bands") {
            panel.bands(x[good], y[good], upper[subscripts][good], lower[subscripts][good],
                        fill=band.colors[1], alpha=band.transparency, use.splines=use.splines)
          }
        }
        if (!is.null(residuals)){
          predictors <- predictors[-x.var]
          factors <- sapply(xx, is.factor)
          for (predictor in predictors){
            use <- use & if(factors[predictor]) x.all[, predictor] == xx[subscripts[1], predictor]
            else x.all[, predictor] == xx[subscripts[1], predictor]
          }
          n.in.panel <- sum(use)
          if (n.in.panel > 0){
            fitted <- y[good][closest(trans(x.fit[use]), x[good])]
            partial.res <- if (!rescale.axis) original.inverse(original.link(fitted) + residuals[use])
            else fitted + residuals[use]
            lpoints(trans(x.fit[use]), partial.res, col=residuals.color, pch=residuals.pch, cex=residuals.cex)
            if (show.fitted) lpoints(trans(x.fit[use]), fitted, pch=16, col=residuals.color)  # REMOVE ME
            if (smooth.residuals && n.in.panel >= 10) {
              llines(loess.smooth(x.fit[use], partial.res, span=span, family=loess.family),
                     lwd=residuals.lwd, lty=residuals.lty, col=residuals.smooth.color)
            }
            if (id.n > 0){
              M <- cbind(trans(x.fit[use]), partial.res)
              md <- mahalanobis(M, colMeans(M), cov(M))
              biggest <- order(md, decreasing=TRUE)[1:id.n]
              pos <- ifelse(trans(x.fit[use][biggest]) > mean(current.panel.limits()$xlim), 2, 4)
              ltext(trans(x.fit[use][biggest]), partial.res[biggest], 
                    names(partial.res)[biggest], pos=pos, col=id.col, cex=id.cex)
            }
          }
        }
        if (has.thresholds){
          panel.abline(h=thresholds, lty=3)
          panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)),
                     thresholds, threshold.labels, adj=c(0,0), cex=0.75)
          panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                     thresholds, threshold.labels, adj=c(1,0), cex=0.75)
        }
      }},
      ylim=ylim,
      xlim=suppressWarnings(trans(xlm)),
      ylab=ylab,
      xlab=if (length(xlab) == 1 && is.na(xlab)) predictors[x.var] else xlab,
      x.vals=x.vals, rug=rug,
      main=main,
      lower=x$lower, upper=x$upper, has.se=has.se, data=x,
      scales=list(y=list(at=tickmarks$at, labels=tickmarks$labels, rot=roty, cex=cex.y),
                  x=list(at=tickmarks.x$at, labels=tickmarks.x$labels, rot=rotx, cex=cex.x),
                  alternating=alternating), ...)
    result <- update(plot, layout = if (is.null(layout)) c(0, prod(dim(plot))) else layout)
    result$split <- split
    result$more <- more
    class(result) <- c("plot.eff", class(result))
  }
  return(result)
}

print.plot.eff <- function(x, ...){
  NextMethod(split=x$split, more=x$more, ...)
  invisible(x)
}

plot.efflist <- function(x, selection, rows, cols, ask=FALSE, graphics=TRUE, lattice, ...){
  # Next line added 8/23/17 along with lattice, also lattice arg above
  lattice <- if(missing(lattice)) list() else lattice
  if (!missing(selection)){
    if (is.character(selection)) selection <- gsub(" ", "", selection)
    return(plot(x[[selection]], lattice=lattice, ...))
  }
  effects <- gsub(":", "*", names(x))
  if (ask){
    repeat {
      selection <- menu(effects, graphics=graphics, title="Select Term to Plot")
      if (selection == 0) break
      else print(plot(x[[selection]], lattice=lattice, ...))
    }
  }
  else {
    neffects <- length(x)
    mfrow <- mfrow(neffects)
    if (missing(rows) || missing(cols)){
      rows <- mfrow[1]
      cols <- mfrow[2]
    }
    for (i in 1:rows) {
      for (j in 1:cols){
        if ((i-1)*cols + j > neffects) break
        more <- !((i-1)*cols + j == neffects)
        lattice[["array"]] <- list(row=i, col=j, nrow=rows, ncol=cols, more=more)
        print(plot(x[[(i-1)*cols + j]], lattice=lattice, ...))
      }
    }
  }
}

if (requireNamespace("carData") && require("effects")){

  # plots should show fitted values directly on plotted effect, and must be checked visually
  # numbering corresponds to effect-test-1.R
  
  data(Duncan, package="carData")
  
  mod.1 <- lm(prestige ~ type + poly(income, degree=2, raw=TRUE), data=Duncan)
  

  # (2) focal: polynomial, constant: factor
  
  print(plot(Effect(c("income"), mod.1, residuals=TRUE), show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("income"), mod.1, residual=TRUE)$fit, 
                        Effect(c("income"), mod.1, 
                               xlevels=list(income=seq(7, 81, length.out=100)))$fit)))
    stop("failed test 2 (2)")
  
  
  # (3) focal: factor*polynomial, constant: polynomial
  
  mod.2 <- lm(prestige ~ type*poly(income, degree=2, raw=TRUE) + 
                poly(education, degree=2, raw=TRUE), data=Duncan)

  print(plot(Effect(c("type", "income"), mod.2, residuals=TRUE), show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("type", "income"), mod.2, residuals=TRUE)$fit, 
                        Effect(c("type", "income"), mod.2, 
                               xlevels=list(income=seq(7, 81, length.out=100)))$fit)))
    stop("failed test 2 (3)")
  
  
  # (4) focal: polynomial, constant: factor*polynomial
  
  print(plot(Effect(c("education"), mod.2, residuals=TRUE), show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("education"), mod.2, residuals=TRUE)$fit, 
                        Effect(c("education"), mod.2, 
                               xlevels=list(education=seq(7, 100, length.out=100)))$fit)))
    stop("failed test 2 (4)")  
  
  
  # (6) focal: factor*polynomial, constant: polynomial, using predict() & orthog. polys.
  
  mod.3 <- lm(prestige ~ type*poly(income, degree=2) + poly(education, degree=2), data=Duncan)
  
  print(plot(Effect(c("type", "income"), mod.3, residuals=TRUE), show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("type", "income"), mod.3, residuals=TRUE)$fit, 
                        Effect(c("type", "income"), mod.3, 
                               xlevels=list(income=seq(7, 81, length.out=100)))$fit)))
    stop("failed test 2 (6)")  
  
  # (7) focal: factor, constant: poly*poly
  
  mod.4 <- lm(prestige ~ type + poly(income, 2)*poly(education, 2), data=Duncan)
  
  print(plot(Effect(c("income", "education"), mod.4, residuals=TRUE), show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("income", "education"), mod.4, residuals=TRUE)$fit, 
                        Effect(c("income", "education"), mod.4, 
                               xlevels=list(income=seq(7, 81, length.out=100), 
                                            education=quantile(Duncan$education, probs=seq(0.2, 0.8, by=0.2))))$fit)))
    stop("failed test 2 (7)")  
  
  # (9) focal: covariate, constant: 2 factors and 1 covariate, 3-way interaction
  
  data(Mroz, package="carData")
  mod.6 <- lm(lwg ~ inc + age*hc*wc, data=Mroz)
  inc <- range(Mroz$inc)
  age <- range(Mroz$age)

  print(plot(Effect(c("inc"), mod.6, residuals=TRUE), show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("inc"), mod.6, residuals=TRUE)$fit, 
                        Effect(c("inc"), mod.6, 
                               xlevels=list(inc=seq(inc[1], inc[2], length.out=100)))$fit)))
    stop("failed test 2 (9-1)")  
  
  print(plot(Effect(c("age", "hc", "wc"), mod.6, residuals=TRUE), show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("age", "hc", "wc"), mod.6, residuals=TRUE)$fit, 
                        Effect(c("age", "hc", "wc"), mod.6, 
                               xlevels=list(age=seq(age[1], age[2], length.out=100)))$fit)))
    stop("failed test 2 (9-2)")  
  
  # additional tests of partial residuals
  
  income <- range(na.omit(Prestige)$income)
  
  mod.7 <- lm(prestige ~ income*type + education, data=Prestige)
  print(plot(Effect(c("income", "type"), mod.7, residuals=TRUE), show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("income", "type"), mod.7, residuals=TRUE)$fit, 
                        Effect(c("income", "type"), mod.7, 
                               xlevels=list(income=seq(income[1], income[2], length.out=100)))$fit)))
    stop("failed test 2 (additional-1)")  
  
  Mroz2 <- Mroz
  Mroz2$hc <- as.numeric(Mroz$hc) - 1
  Mroz2$wc <- as.numeric(Mroz$wc) - 1
  inc <- range(Mroz2$inc)
  mod.8 <- lm(lwg ~ inc*age*k5 + hc*wc, data=Mroz2)
  print(plot(Effect(c("inc", "age", "k5"), mod.8, residuals=TRUE, xlevels=list(k5=0:1)), 
             show.fitted=TRUE))
  
  if (!isTRUE(all.equal(Effect(c("inc", "age", "k5"), mod.8, residuals=TRUE, xlevels=list(k5=0:1))$fit, 
                        Effect(c("inc", "age", "k5"), 
                               mod.8, residuals=TRUE, 
                               xlevels=list(k5=0:1, inc=seq(inc[1], inc[2], length.out=100), age=quantile(Mroz2$age, seq(.2, .8, by=.2))))$fit)))
    stop("failed test 2 (additional-2)")  
  
  print(plot(Effect(c("hc", "wc"), mod.8, residuals=TRUE, xlevels=list(hc=0:1, wc=0:1)), 
       show.fitted=TRUE, smooth.residuals=FALSE, residuals.pch="."))
}

# 2013-07-31:  extend effects to poLCA objects.  S. Weisberg
# 2013-10-15: removed effect.poLCA. J. Fox
# 2018-11-19: added focal.levels argument to predictorEffect() and predictorEffects() methods. J. Fox

#The next two functions should be exported to the namespace

allEffects.poLCA <- function(mod, ...){
  allEffects(poLCA.to.fake(mod), ...)
}

Effect.poLCA <- function(focal.predictors, mod, ...) {
  result <- Effect(focal.predictors, poLCA.to.fake(mod), ..., sources=list(type="multinom"))
  result$formula <- as.formula(formula(mod))
  result
}

predictorEffects.poLCA <- function(mod, predictors = ~.,focal.levels=50, xlevels=5,  ...){
  predictorEffects(poLCA.to.fake(mod), predictors=predictors, focal.levels=focal.levels, xlevels=xlevels, ...)
}

predictorEffect.poLCA <- function(predictor, mod, focal.levels=50, xlevels=5, ...){
  predictorEffect(predictor, poLCA.to.fake(mod), focal.levels=focal.levels, xlevels=xlevels, ...)
}

# this function makes a 'fake' multinom object or 'glm' object so
# effect.multinom  or effect.glm can be used.
# effect.multinom requires at least 3 classes, so if classes=2 use
# effect.glm
poLCA.to.fake <- function(mod) { 
  if (requireNamespace("nnet", quietly=TRUE)){
    multinom <- nnet::multinom}
  dta <- eval(mod$call$data)
  form <- as.formula(eval(mod$call$formula))
  # find the missing data:
  omit <- attr(model.frame(form, dta), "na.action")
  if(length(omit) == 0) dta$.class <- factor(mod$predclass) else{
    dta$.class <- rep(NA, dim(dta)[1])
    dta$.class[-omit] <- mod$predclass
    dta$.class <- factor(dta$.class)
  }
  # end of missing data correction
  formula1 <- update(form, .class ~ .)
  if(length(mod$P) == 2L){
    mod1 <- glm(formula1, family=binomial, data=dta)
    mod1$call$data <- dta
    mod1$call$formula <- formula1
    mod1$coef <- mod$coeff[, 1]
    mod1$vcov <- mod$coeff.V
    class(mod1) <- c("fakeglm", class(mod1)) }
  else {
    mod1 <- multinom(formula1, dta, Hess=TRUE, trace=FALSE, maxit=1)
    mod1$call$data <- dta
    mod1$call$formula <- formula1
    mod1$coeff <- mod$coeff
    mod1$coeff.V <- mod$coeff.V
    class(mod1) <- c("fakemultinom", class(mod1))
  }
  coef.fakemultinom <- function(mod){
    coef <- t(mod$coeff)
    dimnames(coef) <- list(mod$lab[-1L], mod$vcoefnames)
    coef
  }
  vcov.fakemultinom <- function(mod){mod$coeff.V}
  mod1
}

%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Predictor Effects Graphics Gallery}
%% vignette index specifications need to be *after* \documentclass{}
%%\VignetteEngine{knitr::knitr}
%%\VignetteIndexEntry{Effects Gallery}
%%\VignettePackage{effects}

\documentclass[10pt]{article}


\usepackage[left=1.25in, right=1.25in, top=1in, bottom=1in]{geometry}
\usepackage[utf8]{inputenc}
\usepackage{graphicx}
\usepackage[american]{babel}
\newcommand{\R}{{\sf R}}
\usepackage{url}
\usepackage{hyperref}
\usepackage{xcolor}
\hypersetup{
    colorlinks,
    linkcolor={red!50!black},
    citecolor={blue!50!black},
    urlcolor={blue!80!black}
}
\usepackage{alltt}
\usepackage{fancyvrb}
\usepackage{natbib}
\usepackage{amsmath}
\VerbatimFootnotes
\bibliographystyle{chicago}

\newcommand{\x}{\mathbf{x}}
\newcommand{\code}[1]{\normalfont\texttt{\hyphenchar\font45\relax #1}}
\newcommand{\lcode}[1]{\mbox{$\log($}\normalfont\texttt{\hyphenchar\font45\relax #1}\mbox{$)$}}
\newcommand{\E}{\mathrm{E}}
\newcommand{\link}[1]{#1}
\newcommand{\tild}{\symbol{126}}
\newcommand{\Rtilde}{\,\raisebox{-.5ex}{\code{\tild{}}}\,}
\newcommand{\captilde}{\mbox{\protect\Rtilde}} % use in figure captions.
\newcommand{\Rmod}[2]{\code{#1 \raisebox{-.5ex}{\tild{}} #2}}
\newcommand{\Rmoda}[2]{\code{#1} &\code{\raisebox{-.5ex}{\tild{}} #2}}
\newcommand{\Rmodb}[2]{\code{#1 &\raisebox{-.5ex}{\tild{}}& #2}}
\newcommand{\aab}[2]{\code{#1}\mbox{$*$}\code{#2}}
\newcommand{\acb}[2]{\code{#1}\mbox{$:$}\code{#2}}
\newcommand{\C}{\mathbf{C}}
\newcommand{\betahat}{\widehat{\beta}}
\newcommand{\bbetahat}{\widehat{\boldsymbol{\beta}}}
\newcommand{\bbeta}{\boldsymbol{\beta}}
\newcommand{\xbf}{\x_{\backslash{}f}}
\newcommand{\hbf}{h_{\backslash{}f}}
\newcommand{\xtb}{\x_{2\backslash{}f}}
\newcommand{\xbfi}{\x_{\backslash{}f,i}}
\newcommand{\inter}[2]{\mbox{$#1$:$#2$}}
\newcommand{\cross}[2]{\mbox{$#1$\code{*}$#2$}}
\newcommand{\N}{\mathrm{N}}
\newcommand{\fn}[1]{\texttt{#1()}}
\newcommand{\ar}{\texttt}
\newcommand{\pkg}[1]{\textbf{#1}}
\newcommand{\proglang}[1]{\textsf{#1}}
\newcommand{\yx}{\widehat{y}(\x)}
\newcommand{\lvn}[1]{\mbox{$\log(\mbox{\texttt{#1}})$}}
\newcommand{\vn}[1]{\mbox{\texttt{#1}}}
\newcommand{\level}[1]{\texttt{"#1"}}
\newcommand{\class}[1]{\texttt{"#1"}}



\begin{document}

\title{Predictor Effects Graphics Gallery}

\author{John Fox and Sanford Weisberg}

\date{2018-12-19, minor revisions 2023-02-20}

\maketitle

\tableofcontents

\begin{abstract}
Predictor effect displays visualize the response surface of complex regression models by averaging and conditioning, producing a sequence of 2D line graphs, one graph or set of graphs for each predictor in the regression problem \citep{fw19, fw19b}.  In this vignette, we give examples of effect plots produced by the \pkg{effects} package, and in the process systematically illustrate the optional arguments to functions in the package, which can be used to customize predictor effect plots.
\end{abstract}

%\centerline{\includegraphics[width=1.25in]{../inst/doc/effects-hex.pdf}}

<<setopts,echo=FALSE>>=
library("knitr")
opts_chunk$set(fig.width=5,fig.height=5,#tidy=TRUE,
               out.width="0.8\\textwidth",echo=TRUE)
#options(prompt=" ")
options(continue="+    ", prompt="R> ", width=70)
options(show.signif.stars=FALSE, scipen=3)
@

<<setup, include=FALSE, cache=FALSE, results='hide', echo=FALSE>>=
library(car)
library(effects)
render_sweave()
options(width=80, digits=5, str=list(strict.width="cut"))
strOptions(strict.width="cut")
@

\section{Introduction}\label{sec:intro}
Predictor effect plots \citep{fw19b} provide graphical summaries for fitted regression models with linear predictors, including linear models, generalized linear models, linear and generalized linear mixed models, and many others.  These graphs are an alternative to tables of fitted coefficients, which can be much harder to interpret than predictor effect plots. Predictor effect plots are implemented in \R{} in the \pkg{effects} package, documented in \citet{fw19}.  This vignette provides many examples of variations on the graphical displays that can be obtained with the \pkg{effects} package.  Many of the details, and more complete descriptions of the data sets used as examples, are provided in the references cited at the end of the vignette.



\subsection{Effects and Predictor Effect Plots}\label{sec:intro2}


We begin with an example of a multiple linear regression, using the \code{Prestige} data set in the \pkg{carData} package:
<<>>=
library("car") # also loads the carData package
Prestige$type <- factor(Prestige$type, levels=c("bc", "wc", "prof"))
lm1 <- lm(prestige ~ education + poly(women, 2) +
                     log(income)*type, data=Prestige)
@
The data, collected circa 1970, pertain to 102 Canadian occupations. The model \code{lm1} is a linear model with response \vn{prestige}, continuous predictors \vn{income}, \vn{education}, and \vn{women}, and the factor predictor \vn{type}, which has three levels. Before fitting the model, we reorder the levels of \vn{type} as \level{bc} (blue-collar), \level{wc} (white-collar),  and \level{prof} (professional and managerial).  The predictor \vn{education} represents itself in the linear model, and so it is both a predictor and a \emph{regressor}, as defined in \citet[Sec.~4.1]{fw19}.  The predictor \vn{income} is represented by the regressor \lcode{income}. The variable \vn{women}, a percentage between 0 and 100, is represented by regressors that define a polynomial of degree 2 using \fn{poly}'s default orthogonal polynomials.  The variable \vn{type} is a factor with three levels, so it is represented by two dummy regressors defined by the default contrast-generating function in \R{}, \fn{contr.treatment}.  Finally, the formula includes an interaction between \vn{income} and \vn{type}, defined by multiplying the regressor for \vn{income} (\lcode{income}) by each of the regressors that represent \vn{type}.

The usual numeric summary of the fit of \code{lm1} is a table of estimated coefficients, which we obtain via the \fn{S} function in the \pkg{car} package that is similar to, but somewhat more flexible than, the standard \R{} \fn{summary} function:
<<>>=
S(lm1)
@
\begin{itemize}

\item Interpretation of the regression coefficients is straightforward only for the predictor \vn{education}, where an increase of one year of \vn{education}, holding other predictors fixed, corresponds to an estimated expected increase in the response of \Sexpr{round(coef(lm1)[2], 3)} units.  

\item Even ignoring the interaction, the log transformation complicates the interpretation of the effect of \vn{income}.

\item The predictor \vn{women} is represented by two regressors, so the effect of \vn{women} requires examining two coefficient estimates that are interpretable only by those knowledgeable about polynomial regression analysis. Even if raw rather than orthogonal polynomial regressors were used, via \code{poly(women, 2, raw=TRUE)} in place of \code{poly(women, 2)}, interpretation of the effect of \vn{women} is complicated.  

\item Understanding the coefficients for the main effect of \vn{type} depends on the contrasts used  to define the effect.  The contrasts can be changed by the user, and the default contrasts in \R{} are different from the default contrasts used by \proglang{SAS} or other programs, so the coefficients cannot be reliably interpreted without information not present in the regression summary.  

\item Finally, the interaction further complicates the interpretation of the effect of either \vn{income} or \vn{type}, because the interaction coefficients need to be interpreted jointly with the main effect coefficients. 

\end{itemize}

\noindent Summarization of the effects of predictors using tables of coefficient estimates is often incomplete.  Effects, and particularly plots of effects, can in many instances  reveal the relationship of the response to the predictors more clearly.  This conclusion is especially true for models with linear predictors that include interactions and multiple-coefficient terms such as regression splines and polynomials, as illustrated in this vignette.

A predictor effect plot summarizes the role of a selected \emph{focal} predictor in a fitted regression model.  The \fn{predictorEffect} function is used to compute the appropriate summary of the regression, and then the \fn{plot} function may be used to graph the resulting object, as in the following example:
<<fig11,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'>>=
library("effects")
e1.lm1 <- predictorEffect("education", lm1)
plot(e1.lm1)
@

\centerline{\includegraphics[width=4in]{figure/fig11-1.pdf}}

\noindent
This graph visualizes the partial slope for \vn{education}, that for each year increase in \vn{education}, the fitted \vn{prestige} increases by \Sexpr{round(coef(lm1)[2], 3)} points, when the other predictors are held fixed.  The intercept of the line, which is outside the range of \vn{education} on the graph, affects only the height of the line, and is determined by the choices made for averaging over the fixed predictors, but for any choice of averaging method, the slope of the line would be the same. The shaded area is a pointwise confidence band for the fitted values, based on standard errors computed from the covariance matrix of the fitted regression coefficients. The rug plot at the bottom of the graph shows the location of the \vn{education} values.

The information that is needed to draw the plot is computed by the \fn{predictorEffect} function.
The minimal arguments for \fn{predictorEffect} are the quoted name of a predictor in the model followed by the fitted model object.  The essential purpose of this function is to compute fitted values from the model with \vn{education} varying and all other predictors fixed at typical values \citep[Sec.~4.3]{fw19}.  The command below displays the values of the regressors for which fitted values are computed, including a column of 1s for the intercept:
<<>>=
brief(e1.lm1$model.matrix)
@
The focal predictor \vn{education} was evaluated by default at 50 points covering the observed range of values of \vn{education}. We use the \fn{brief} function in the \pkg{car} package to show only a few of the 50 rows of the matrix. For each value of \vn{education} the remaining regressors have the same fixed values for each fitted value.  The fixed value for \lvn{income} is the logarithm of the sample mean \vn{income}, the fixed values for the regressors for \vn{women} are computed at the mean of \vn{women} in the data, and the fixed values for the regressors for \vn{type} effectively take a weighted average of the fitted values at the three levels of \vn{type}, with weights proportional to the number of cases in each level of the factor. Differences in the fitted values are due to \vn{education} alone because all the other predictors, and their corresponding regressors, are fixed.  Thus the output gives the partial effect of \vn{education} with all other predictors fixed.

The computed fitted values can be viewed by printing the \class{eff} object returned by \fn{predictorEffect}, by summarizing the object, or by converting it to a data frame. To make the printouts more compact, we recompute the predictor effect of \vn{education} with fewer values of the focal predictor by specifying the \code{focal.levels} argument (see Section~\ref{sec-focal.levels-xlevels}):
<<>>=
e1a.lm1 <- predictorEffect("education", lm1, focal.levels=5)
e1a.lm1
summary(e1a.lm1)
as.data.frame(e1a.lm1)
@
The values in the column \vn{education} are the values the focal predictor.  The remaining columns are the fitted values, their standard errors, and lower and upper end points of 95\% confidence intervals for the fitted values.
The \emph{predictor effect plot} is simply a graph of the fitted values on the vertical axis versus the focal predictor on the horizontal axis.  For a continuous focal predictor such as \vn{education}, a line, in this case, a straight line, is drawn connecting the fitted values.

We turn next to the predictor effect plot for \vn{income}.  According to the regression model, the effect of \vn{income} may depend on \vn{type} due to the interaction between the two predictors, so simply averaging over \vn{type} would be misleading.  Rather, we should allow both \vn{income} and \vn{type} to vary, fixing the other predictors at their means or other typical values.  By default, this computation would require evaluating the model at $50 \times 3 = 150$ combinations of the predictors, but to save space we will only evaluate \vn{income} at five values, again using the \ar{focal.levels} argument, thus computing only $5 \times 3 = 15$ fitted values:
<<>>=
e2.lm1 <- predictorEffect("income", lm1, focal.levels=5)
as.data.frame(e2.lm1)
@

To draw the predictor effects plot we recalculate the fitted values using the default \code{focal.levels=50} to get more accurately plotted regression curves:
<<fig12,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'>>=
plot(predictorEffect("income", lm1), 
     lines=list(multiline=TRUE))
@
Here we use both the \fn{predictorEffect} and \fn{plot} functions in the same command.

\centerline{\includegraphics[width=4in]{figure/fig12-1.pdf}}

\noindent
The focal predictor \vn{income} is displayed on the horizontal axis.  There is a separate line shown for the fitted values at each level of \vn{type}.  The lines are curved rather than straight  because \vn{income} appears in the model in log-scale but is displayed in the predictor effect plot in arithmetic (i.e., dollar) scale.  The lines in the graph are not parallel because of the interaction between \lvn{income} and \vn{type}.  For $\vn{type} = \level{prof}$, the fitted values of \vn{prestige} are relatively high for lower values of \vn{income}, and are relatively less affected by increasing values of \vn{income}.

The predictor effect plot for \vn{type} uses essentially the same fitted values as the plot for \vn{income}, but we now get five lines, one for each of the five (not 50) values of \vn{income} selected by the \fn{predictorEffect} function in this context:
<<fig13,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'>>=
plot(predictorEffect("type", lm1), lines=list(multiline=TRUE))
@

\centerline{\includegraphics[width=4in]{figure/fig13-1.pdf}}

\noindent
Because the horizontal axis is now a factor, the fitted values are displayed explicitly as points, and the lines that join the points are merely a visual aid representing  \emph{profiles} of fitted values.  Fitted \vn{prestige} increases with \vn{income} for all levels of \vn{type}, but, as we found before, when $\vn{type}=\level{prof}$, fitted \vn{prestige} is relatively high for lower \vn{income}.

These initial examples use only default arguments for \fn{predictorEffect} and \fn{plot}, apart from the \code{multiline} argument to \fn{plot} to put all the fitted lines in the same graph. We explain how to customize predictor effect plots in subsequent sections of this vignette.

\subsection{General Outline for Constructing Predictor Effect Plots}

Using the \pkg{effects} package to draw plots usually entails the following steps:
\begin{enumerate}

\item Fit a regression model with a linear predictor.  The package supports models created by \fn{lm}, \fn{glm}, \fn{lmer} and \fn{glmer} in the \pkg{lme4} package, \fn{lme} in the \pkg{nlme} package, and many other regression-modeling functions (see \code{?Effect}).

\item The regression model created in the first step is then used as input to either  \fn{predictorEffect}, to get the effects for one predictor, or \vn{predictorEffects}, to get effects for one or more predictors. These functions do the averaging needed to get fitted values that will ultimately be plotted. There are many arguments for customizing the computation of the effects. The two predictor effect functions call the more basic \fn{Effect} function, and almost all of the material in this vignette applies to \fn{Effect} as well.

\item Use the generic \fn{plot} function to draw a graph or graphs based on the object created in Step 2.

\end{enumerate}

\subsection{How \fn{predictorEffect} Chooses Conditioning Predictors}\label{sec:eff}

Suppose that you select a \emph{focal predictor} for which you want to draw a predictor effect plot.  The \fn{predictorEffect} function divides the predictors in a model formula into three groups:
\begin{enumerate}

\item The focal predictor.

\item The \emph{conditioning group}, consisting of all predictors with at least one interaction in common with the focal predictor.

\item The \emph{fixed group}, consisting of all other predictors, that is, those  with no interactions in common with the focal predictor.

\end{enumerate}

\noindent For simplicity, let's assume for the moment that all of the fixed predictors are numeric. The predictors in the fixed group are all evaluated at \emph{typical values}, usually their means, effectively averaging out the influence of these predictors on the fitted value.  Fitted values are computed for all combinations of levels of the focal predictor and the predictors in the conditioning group, with each numeric predictor in the conditioning group replaced by a few discrete values spanning the range of the predictor, for example, replacing years of \vn{education} by a discrete variable with the values 8, 12, and 16 years.

Suppose that we fit a model with \R{} formula
\begin{equation}
\Rmod{y}{x1 + x2 + x3 + x4 + x2:x3 + x2:x4}\label{eq1}
\end{equation}
or, equivalently,
\begin{equation*}
\Rmod{y}{x1 + x2*x3 + x2*x4}
\end{equation*}
There are four predictor effect plots for this model, one for each predictor selected in turn as the focal predictor:

\begin{center}
\begin{tabular}{ccc}\hline
Focal & Conditioning & Fixed\\
Predictor & Group & Group\\ \hline
\vn{x1} &  none& \vn{x2}, \vn{x3}, \vn{x4} \\
\vn{x2} & \vn{x3}, \vn{x4} & \vn{x1} \\
\vn{x3} &  \vn{x2} & \vn{x1}, \vn{x4} \\
\vn{x4} &  \vn{x2}& \vn{x1} \vn{x3} \\ \hline
\end{tabular}
\end{center}

\noindent
The predictor \vn{x1} does not interact with any of the other predictors, so its conditioning set is empty and all the remaining predictors are averaged over; \vn{x2} interacts with both \vn{x3} and \vn{x4}; \vn{x3} interacts only with \vn{x2}; and \vn{x4} interacts with \code{x2}.

\subsection{The \fn{Effect} Function}\label{sec:Effect}

Until recently, the primary function in \pkg{effects} for computing and displaying effects was the \fn{Effect} function.\footnote{The \pkg{effects} package also includes the older \fn{allEffects} function, which computes effects for each high-order term in a model with a linear predictor. As we explain in \citet{fw19b}, we prefer predictor effects to high-order term effects, and so, although its use is similar to \fn{predictorEffects}, we won't describe \fn{allEffects} in this vignette. There is also an older \fn{effect} function (with a lowercase ``\code{e}''), which is a less flexible version of \fn{Effect}, and which calls \fn{Effect} to perform computations; \fn{effect} is retained only for backwards comparability.}  Whereas the \fn{predictorEffect} function automatically determines the conditioning group and the fixed group of predictors, the \fn{Effect} function puts that burden on the user. The \fn{Effect} function doesn't distinguish between between a focal predictor and conditioning predictors, but rather only between varying (that is, focal \emph{and} conditioning) and fixed predictors.

Each call to \fn{predictorEffect} is equivalent to a specific call to the \fn{Effect} function as follows.  Suppose that \vn{m} is the fitted model produced by the formula in (\ref{eq1}); then, except for the ways in which the default levels for predictors are determined:
\begin{description}

\item[] \code{predictorEffect("x1", m)} is equivalent to \code{Effect("x1", m)};

\item[] \code{predictorEffect("x2", m)} is equivalent to \code{Effect(c("x2", "x3", "x4"), m)};

\item[] \code{predictorEffect("x3", m)} is equivalent to \code{Effect(c("x3", "x2"), m)}; and

\item[] \code{predictorEffect("x4", m)} is equivalent to \code{Effect(c("x4", "x2"), m)}.

\end{description}

The \fn{predictorEffect} function determines the correct call to \fn{Effect} based on the choice of focal predictor and on the structure of main effects and interactions in the linear predictor for the model.  It then uses the \fn{Effect} function to do the computing.  As a result, most of the arguments to \fn{predictorEffect} are documented in \code{help("Effect")} rather than in \code{help("predictorEffect")}.

\subsection{The \fn{predictorEffects} Function}

This function, whose name ends with the plural ``\code{effects}", computes the values needed for one or more predictor effect plots, and by default for \emph{all} of the predictors in the model.  For example, the following command produces all of the predictor effect plots for the model we fit to the \code{Prestige} data:
<<fig14,include=TRUE,fig.width=7,fig.height=8,fig.show='hide'>>=
eall.lm1 <- predictorEffects(lm1)
plot(eall.lm1)
@

\centerline{\includegraphics[width=0.95\textwidth]{figure/fig14-1.pdf}}

\noindent
The predictor effect plots for this model are displayed in an array of graphs.  The plots for \vn{income} and \vn{type} have a separate panel for each level of the conditioning variable because the default argument \ar{lines=list(multiline=FALSE)} was implicitly used.  Confidence bounds are shown by default when \ar{multiline=FALSE}.

The resulting object \code{eall.lm1} is a list with four elements, where \code{eall.lm1[[1]]} is the summary for the first predictor effect plot, \code{eall.lm1[[2]]} for the second plot, and so on.  The following equivalent commands draw the same array of predictor effect plots:
<<eval=FALSE>>=
plot(eall.lm1)
plot(predictorEffects(lm1))
plot(predictorEffects(lm1, ~ income + education + women + type))
@
If you want only the predictor effect plots for \vn{type} and \vn{education}, in that order, you could enter
<<eval=FALSE>>=
plot(predictorEffects(lm1, ~ type + education))
@
Similarly, the commands
<<eval=FALSE>>=
plot(predictorEffects(lm1, ~ women))
plot(predictorEffects(lm1)[[2]])
plot(predictorEffect("women", lm1))
@
all produce the same graph, the predictor effect plot for \vn{women}.

Predictor effect plots in an array can be a useful shortcut for drawing many graphs quickly, but can lead to problems with the displayed graphs. For example, the horizontal axis labels for the plot for \vn{income} are overprinted, and the labels at the top of the panels for \vn{type} with conditioning variable \vn{income} are larger than the available space.  These problems can often be fixed using optional arguments described later in this vignette or by plotting predictor effects individually.

\section{Optional Arguments for the \fn{predictorEffect} and \fn{Effect} Functions}\label{sec:peopts}

This section comprises a catalog of the arguments available to modify the behavior of the \fn{predictorEffect} and \fn{Effect} functions. These arguments may also be specified to the \fn{predictorEffects} function. The information provided by \code{help("Effect")} is somewhat more comprehensive, if terser, explaining for example exceptions applying to \class{svyglm} objects or for plotting residuals.

\subsection{\ar{focal.levels} and \ar{xlevels}: Options for the Values of the Focal Predictor and Predictors in the Conditioning Group}\label{sec-focal.levels-xlevels}

Numeric predictors in the conditioning group need to be discretized to draw a predictor effect plot.  For example the predictor effect plot for \vn{type} in model \code{lm1} consists of a separate line, or a separate panel, for each discrete value of \vn{income}:
<<fig21a,include=TRUE,fig.width=5,fig.height=4.5,fig.show='hide'>>=
e3.lm1 <- predictorEffect("type", lm1)
plot(e3.lm1, lines=list(multiline=TRUE))
@
\centerline{\includegraphics[width=4in]{figure/fig21a-1.pdf}}
<<fig21b,include=TRUE,fig.width=6,fig.height=5,fig.show='hide'>>=
plot(e3.lm1, lines=list(multiline=FALSE)) # the default
@
\centerline{\includegraphics[width=4in]{figure/fig21b-1.pdf}}

\noindent
The numeric conditioning predictor \vn{income} is evaluated by default at five
equally spaced values, when are then rounded to ``nice" numbers.  

Using the three values of 5000, 15000, 25000 for the conditioning predictor \vn{income} in this example produces a simpler graph:
<<fig22a,include=TRUE,fig.width=5,fig.height=4.5,fig.show='hide'>>=
e3.lm1 <- predictorEffect("type", lm1,
                          xlevels=list(income=c(5000, 15000, 25000)))
plot(e3.lm1, lines=list(multiline=TRUE),
     confint=list(style="bars"))
@
\centerline{\includegraphics[width=4in]{figure/fig22a-1.pdf}}
<<fig22b,include=TRUE,fig.width=5.5,fig.height=5,fig.show='hide'>>=
plot(e3.lm1,
     lines=list(multiline=FALSE), # the default
     lattice=list(layout=c(3, 1)))
@
\centerline{\includegraphics[width=4in]{figure/fig22b-1.pdf}}

\noindent
The argument \ar{xlevels} is a list of sub-arguments that control how numeric predictors are discretized when used in the conditioning group.  For example, \code{xlevels=list(x1=c(2, 4, 7), x2=6)} would use the values 2, 4, and 7 for the levels of the predictor  \code{x1}, use 6
equally spaced values for the predictor \code{x2}, and use the default of 5 values for any other numeric conditioning predictors.  Numeric predictors in the \emph{fixed} group are not affected by the \ar{xlevels} argument. We use the \ar{layout} sub-argument of the \ar{lattice} argument group to arrange the panels of the second graph in 3 columns and 1 row (see Section~\ref{sec:layout}).  See \code{help("plot.eff")} for information on the \ar{quantiles} argument, which provides an alternative method of setting \ar{xlevels} when partial residuals are displayed, as discussed in Section~\ref{sec:res}.

The points at which a numeric focal predictor is evaluated is controlled by the \ar{focal.levels} argument. The default of \vn{focal.levels=50} is recommended for drawing graphs, but if the goal is to produce a table of fitted values a smaller value such as \code{focal.levels=5} produces more compact output. The focal predictor can also be set to a vector of particular values, as in \code{focal.levels=c(30, 50, 70)}. Used with the \code{predictorEffects} function, the \ar{focal.levels} argument can be set separately for each focal predictor, similarly to the \ar{xlevels} argument; see \code{help("predictorEffects")}.    

\subsection{\ar{fixed.predictors}: Options for Predictors in the Fixed Group}

Predictors in the fixed group are replaced by ``typical" values of the predictors.  Fitted values are then computed using these typical values for the fixed group, varying the values of predictors in the conditioning group and of the focal predictor. The user can control how the fixed values are determined  by specifying the \ar{fixed.predictors} argument.  This argument takes a list of sub-arguments that allow for controlling each predictor in the fixed group individually, with different rules for factors and numeric predictors.

\subsubsection{Factor Predictors}\label{sec:facpred}

Imagine computing the fitted values evaluating a fixed factor at each of its levels.  The fitted value that is used in the predictor effects plot is a weighed average of these within-level fitted values, with weights proportional to the number of observations at each level of the factor. This is the default approach, and is an appropriate notion of ``typical" for a factor if the data at hand are viewed as a random sample from a population, and so the sample fraction at each level estimates the population fraction.

A second approach is to average the level-specific fitted values with equal weights at each level. This may be  appropriate, for example, in designed experiments in which the levels of a factor are assigned by an investigator.  The latter method is invoked by setting \code{fixed.predictors= list(given.values="equal")}.

You can construct other weighting schemes for averaging over the levels of a factor, as described on the help page for the \fn{Effect} function.

\subsubsection{Numeric Predictors}

For a numeric predictor in the fixed group the default method of selecting a typical value is to apply the \fn{mean} function to the data for the predictor.  The specification \code{fixed.predictors= list(typical=median)} would instead use the \fn{median} function; in general, \ar{typical} can be any function that takes a numeric vector as its argument and returns a single number.

Other sub-arguments to \ar{fixed.predictors} apply to the use of offsets, and to the \pkg{survey} package; see the help page for the \fn{Effect} function.

\subsection{\ar{se} and \ar{vcov.}: Standard Errors and Confidence Intervals}\label{sec:se}

Standard errors and confidence intervals for fitted values are computed by default, which corresponds to setting the argument \code{se=list(compute=TRUE, type="pointwise", level=.95)}.  Setting \code{se=FALSE} omits standard errors, \ar{type="scheffe"} uses wider Scheff\'{e} intervals that adjust for simultaneous inference, and \code{level=.8}, for example, produces 80\% intervals.

Standard errors are based by default on the ``usual" sample covariance matrix of the estimated regression coefficients. You can replace the default coefficient covariance matrix with some other estimate, such as one obtained from the bootstrap or a sandwich coefficient covariance matrix estimator, by setting the  \ar{vcov.}~argument either to a function that returns a coefficient covariance matrix, such as \fn{hccm} in the \pkg{car} package for linear models, or to a matrix of the correct size; for example:
<<fig23,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'>>=
e4.lm1 <- predictorEffect("education", lm1,
                          se=list(type="scheffe", level=.99), 
                          vcov.=hccm)
plot(e4.lm1)
@

\centerline{\includegraphics[width=4in]{figure/fig23-1.pdf}}

\noindent
This plot displays 99\% Scheff\'{e} intervals based on a robust coefficient covariance matrix computed by the sandwich method; see \code{help("hccm")}.

\subsection{\ar{residuals}: Computing Residuals for Partial Residual Plots}

The argument \ar{residuals=TRUE} computes and saves residuals, providing the basis for adding partial residuals to subsequent effect plots, a topic that we discuss in Section~\ref{sec:res}.

\section{Arguments for Plotting Predictor Effects}\label{sec:plot}

The arguments described in Section~\ref{sec:peopts} are for the \fn{predictorEffect} function or the \fn{Effect} function.  Those arguments modify the computations that are performed, such as methods for averaging and fixing predictors, and for computing standard errors.  Arguments to the \fn{plot} methods for the predictor effect and effect objects produced by the \fn{predictorEffect} and \fn{Effect} functions  are described in this section, and these change the appearance of a predictor effect plot or modify the quantities that are plotted. These optional arguments are described in more detail in \code{help("plot.eff")}.

In 2018, we reorganized the \fn{plot} method for effect objects by combining arguments into five major groups of related sub-arguments, with the goal of simplifying the specification of effect plots.  For example, the \ar{lines} argument group is a list of sub-arguments for determining line type, color, and width, whether or not multiple lines should be drawn on the same graph, and whether plotted lines should be smoothed.  The defaults for these sub-arguments are the choices we generally find the most useful, but they will not be the best choices in all circumstances.  The cost of reorganizing the arguments in this manner is the necessity of specifying arguments as lists, some of whose elements are themselves lists, requiring the user to make sure that parentheses specifying the possibly nested lists are properly balanced.

In addition to the five argument groups that we describe below, the \fn{plot} method for effect objects accepts the arguments \ar{main} for the main title of the graph and \ar{id} for identifying points in effect plots that include residuals, as discussed in Section~\ref{sec:res}.

Finally, the \fn{plot} method for effect objects retains a number of ``legacy" arguments shown in \code{help("plot.eff")}.  These arguments have been kept so existing scripts using the \pkg{effects} package would not break, but they are all duplicated as sub-arguments of the five argument groups.  The legacy arguments work but they may not be supported forever, so we encourage you to use the newer argument groups and sub-arguments.

\subsection{The \ar{axes} Group: Specify Axis Characteristics}

The \ar{axes} argument group has two major sub-arguments, \ar{x} for the horizontal axis, \ar{y} for the vertical axis, and two minor sub-arguments, the \ar{grid} argument, which adds a background grid to the plot, and the \ar{alternating} argument, for changing the placement of axis-tick labels in multi-panel plots.

\subsubsection{\ar{x}: Horizontal Axis Specification}

We introduce another linear model fit to the \code{Prestige} data set to serve as an example:
<<>>=
lm2 <- lm(log(prestige) ~ log(income) + education + type, Prestige)
@
The default predictor effect plot for \vn{income} is
<<fig30,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'>>=
plot(predictorEffects(lm2, ~ income))
@

\centerline{\includegraphics[width=4in]{figure/fig30-1.pdf}}

\noindent
The plot is curved because the predictor \vn{income} is represented by its logarithm in the model formula, but the default predictor effect plot uses the predictor \vn{income}, not the regressor \lvn{income}, on the horizontal axis.
The \ar{x} sub-argument can be used transform the horizontal axis, for example to replace \code{income} by \lcode{income}:
<<fig31,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'>>=
plot(predictorEffects(lm2, ~ income),
 axes=list(
   x=list(income=list(transform=list(trans=log, inverse=exp)))
   ))
@

\centerline{\includegraphics[width=4in]{figure/fig31-1.pdf}}

\noindent
The transformation changes the scale on the horizontal axis to log-scale, but leaves the tick labels in arithmetic scale, and the graph is now a straight line because of the change to log-scale.  This plot has several obviously undesirable features with regard to the range of the horizontal axis and over-printing of tick marks.  We show next that additional arguments to \fn{plot} can correct these defects.

A more elaborate version of the graph illustrates all the sub-arguments to \ar{x} in \ar{axis} argument group:
<<fig32,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'>>=
plot(predictorEffects(lm2, ~ income),
 main="Transformed Plot",
 axes=list(
    grid=TRUE,
    x=list(rotate=30,
           rug=FALSE,
           income=list(transform=list(trans=log, inverse=exp),
                       lab="income, log-scale",
                       ticks=list(at=c(2000, 5000, 10000, 20000)),
                       lim=c(1900, 21000))
    )))
@

\centerline{\includegraphics[width=4in]{figure/fig32-1.pdf}}

\noindent
We use the top-level argument \code{main="Transformed Plot"} to set the title of the plot. The \ar{axes} argument is a list with two sub-arguments, \ar{grid} to turn on the background grid, and \ar{x} to modify the horizontal axis.

The \ar{x} sub-argument is itself a list with three elements: The sub-arguments \code{rotate} and \ar{rug} set the rotation angle for the tick labels and suppress the rug plot, respectively.  The additional sub-argument is a list called \ar{income}, the name of the focal predictor.  If you were drawing many predictor effect plots you would supply one list named for each of the focal predictors.  All of the sub-arguments for \ar{income} are displayed in the example code above.  The sub-argument \code{transform=list(trans=log, inverse=exp)} specifies how to transform the $x$-axis.  The \code{ticks} and \code{lim} sub-arguments set the tick marks and range for the horizontal axis.

This is admittedly a complex command, but it allows you to fine-tune the graph to look the way you want.  In specifying nested argument lists, you may encounter problems getting the parentheses in the right places. Be careful, indent your code to clarify the structure of the command, and be patient!

\subsubsection{\ar{x}: Horizontal Axis Specification for Date Variables}

The functions in the \pkg{effects} package, such as \fn{Effect} and \fn{predictorEffect}, support models with numeric, factor, character, and logical predictors. Date predictors must be converted to numeric for these functions to work. 

We supply the generic function \fn{levels2dates}, with methods for \class{eff} and \class{effpoly} objects, which can be used to properly label the horizontal axes of effect and predictor effect plots by translating numeric dates back to dates for the axis tick-mark labels. \fn{levels2dates} takes several arguments:
\begin{description}

\item[\ar{effect}] An \class{eff} or \class{effpoly} object, created, e.g., by \fn{Effect} or \fn{predictorEffect}.

\item[\ar{predictor}] The quoted name of the numeric version of the date predictor.

\item[\ar{origin}] A quoted string giving the origin date (see the examples below).

\item[\ar{evenly.spaced}] If \code{TRUE} (the default), the tick marks on the horizontal axis are evenly spaced; if \code{FALSE} the tick marks are taken from the levels of the numeric date predictor in the \class{eff} or \class{effpoly} object.

\item[\ar{n}] The number of tick marks; if missing, the number of levels of the predictor in the \class{eff} or \class{effpoly} object.

\end{description}

Here are some examples:

<<figdates1,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'>>=
data("airquality", package="datasets")
airquality$Date <- with(airquality, as.Date(paste("1973", Month, Day, sep="-"), 
    format="%Y-%m-%d"))
airquality$Date.num <- as.numeric(airquality$Date)
m1.date <- lm(Ozone ~ Date.num + Solar.R + Wind + Temp, data=airquality)
eff.date.1 <- Effect("Date.num", m1.date)
plot(eff.date.1, axes=list(x=list(Date.num=list(lab="Date", 
    ticks=list(at=levels2dates(eff.date.1, "Date.num", "1970-01-01"))), 
    rotate=45)), main="Date Effect")
@

\centerline{\includegraphics[width=4in]{figure/figdates1-1.pdf}}

<<figdates2,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'>>=
plot(eff.date.1, axes=list(x=list(Date.num=list(lab="Date", 
    ticks=list(at=levels2dates(eff.date.1, "Date.num", "1970-01-01", n=4))))), 
    main="Date Effect")
@

\centerline{\includegraphics[width=4in]{figure/figdates2-1.pdf}}

<<>>=
eff.date.df <- as.data.frame(eff.date.1)
eff.date.df$Date <- as.Date(eff.date.df$Date.num, origin="1970-01-01")
eff.date.df
@

<<figdates3,include=TRUE,fig.width=7.5,fig.height=7.5,fig.show='hide'>>=
m2.date <- lm(Ozone ~ Date.num*Temp + Solar.R + Wind, data=airquality)
eff.date.2 <- Effect(c("Date.num", "Temp"), m2.date, xlevels=6)
plot(eff.date.2, axes=list(x=list(Date.num=list(lab="Date", 
  ticks=list(at=levels2dates(eff.date.2, "Date.num", "1970-01-01", n=3))), 
  rotate=45)), main="Date Effect by Temperature")
@

\centerline{\includegraphics[width=6in]{figure/figdates3-1.pdf}}


\subsubsection{\ar{y}: Vertical Axis Specification for Linear Models}

The model \code{lm2} has a transformed response \lcode{prestige}, and ``untransforming" the response to arithmetic scale may be desirable.  This can be accomplished with  the \ar{y} sub-argument, which has two sub-arguments named \vn{transform} and \vn{type} that together control the scale and labeling of the vertical axis.  

There are three options for drawing the predictor effect plot for a numeric response like\linebreak \lvn{prestige}:
<<fig33,include=TRUE,fig.width=4,fig.height=4,fig.show='hide'>>=
# default:
plot(predictorEffects(lm2, ~ education),
     main="Default log(prestige)")
# Change only tick-mark labels to arithmetic scale:
plot(predictorEffects(lm2, ~ education),
     main="log(prestige), Arithmetic Ticks",
     axes=list(y=list(transform=list(trans=log, inverse=exp),
                      lab="prestige", type="rescale")))
# Replace log(presige) by prestige:
plot(predictorEffects(lm2, ~ education),
     main="Prestige in Arithmethic Scale",
     axes=list(y=list(transform=exp, lab="prestige")))
@

\includegraphics[width=.33\textwidth]{figure/fig33-1.pdf}
\includegraphics[width=.33\textwidth]{figure/fig33-2.pdf}
\includegraphics[width=.33\textwidth]{figure/fig33-3.pdf}

\noindent
The first plot is the default, with a log-response.  In the second plot, the \ar{transform} sub-argument specifies the transformation of the response and its inverse, and the sub-argument \code{type="rescale"} changes the tick marks on the vertical axis to arithmetic scale.  In the third version, with \code{transform=exp, lab="prestige"}, the vertical axis now is in arithmetic scale, not log scale, although that may not be completely obvious in the example because $\log(x)$ is nearly linear: Look closely to see that the axis ticks marks in the second graph are unequally spaced, while those in the third graph are equally spaced and the plotted line in the latter is slightly curved.  The help page \code{?plot.eff} provides a somewhat more detailed explanation of these options.

As a second example we will reconstruct Figure~7.10 in \citet[Sec.~7.2]{fw19}.  In that section, we fit a linear mixed-effects model to data from the \code{Blackmore}  data frame in the \pkg{carData} package. \code{Blackmore} includes longitudinal data on amount of exercise for girls hospitalized for eating disorders and for similar control subjects who were not hospitalized. We transformed the response variable in the model, hours of \vn{exercise}, using a transformation in a modified Box-Cox power family that allows zero or negative responses, explained briefly by \citet[Sec.~3.4]{fw19} and more thoroughly by \citet{HawkinsWeisberg2017}. The fitted model is
<<>>=
library("lme4") # for lmer()
Blackmore$tran.exercise <- bcnPower(Blackmore$exercise, 
                                    lambda=0.25, gamma=0.1)
mm1 <- lmer(tran.exercise ~ I(age - 8)*group +
              (I(age - 8) | subject), data=Blackmore)
@
This model, with numeric  predictor \vn{age} and factor predictor \vn{group}, is a linear mixed model with random intercepts and slopes for \vn{age} that vary by \vn{subject}.  The response variable is a transformation of \vn{exercise} similar to the fourth root with adjustment for zero values; see \code{help("bcnPower")}.

The predictor effect plot for the fixed effect of \vn{age} is
<<fig33a,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'>>=
e1.mm1 <- predictorEffect("age", mm1)
plot(e1.mm1, lines=list(multiline=TRUE), confint=list(style="auto"))
@

\centerline{\includegraphics[width=4in]{figure/fig33a-1.pdf}}

\noindent
The plot clearly shows the difference in the average \vn{age} trajectory between the \level{control} and \level{patient} groups, with the fitted response for the latter having a larger slope.  The graph is hard to decode, however, because the vertical axis is approximately in the scale of the fourth-root of hours of exercise, so untransforming the response may produce a more informative plot.  Because the \fn{bcnPower} transformation is complex, the \pkg{car} package includes the function \fn{bcnPowerInverse} to reverse the transformation:
<<fig33b,include=TRUE,fig.width=5,fig.height=5,fig.show='hide'>>=
f.trans <- function(x) bcnPower(x, lambda=0.25, gamma=0.1)
f.inverse <- function(x) bcnPowerInverse(x, lambda=0.25, gamma=0.1)
plot(e1.mm1, lines=list(multiline=TRUE),
     confint=list(style="auto"),
     axes=list(x=list(age=list(lab="Age (years)")),
               y=list(transform=list(trans=f.trans, inverse=f.inverse),
                      type="response",
                      lab="Exercise (hours/week)")),
     lattice=list(key.args=list(x=.20, y=.75, corner=c(0, 0), 
                                padding.text=1.25)),
     main=""
)
@

\centerline{\includegraphics[width=4in]{figure/fig33b-1.pdf}}\label{corner}

\noindent
The response scale is now in hours per week, and we see that hours of exercise increase more quickly on average in the patient group for older subjects.  We use additional arguments in this plot to match \citet[Fig.~7.10]{fw19}, including moving the key inside of the graph (see Section~\ref{sec:key}), changing the axis labels, and removing the main title to the plot.\footnote{The code shown for this graph in \cite{fw19} uses ``legacy'' arguments, and is therefore somewhat different from the code given here. Both commands produce the same plot, however.}


\subsubsection{\ar{y}: Vertical Axis Specification for Generalized Linear Models}

Transforming the vertical axis for generalized linear models also uses the \ar{y} sub-argument to the \ar{axes} argument. You typically do not need to specify the \ar{transform} sub-argument because \fn{plot} obtains the right functions from the regression model's \ar{family} component.  The \ar{type} sub-argument has the same three possible values as for linear models, but their interpretation is somewhat different:
\begin{enumerate}

\item Predictor effect plots in \code{type="link"} scale have a predictor on the horizontal axis  and the vertical axis is in the scale of the linear predictor.  For logistic regression, for example, the vertical axis is in log-odds (logit) scale.  For Poisson regression with the log-link, the vertical axis is in log-mean (log-count) scale.

\item Predictor effect plots in \code{type="response"} or mean scale are obtained by ``untransforming" the $y$ axis using the inverse of the link function.  For the log-link, this corresponds to transforming the $y$ axis and plotting $\exp(y)$.  For logistic regression, $y = \log[p/(1-p)]$ and, solving for $p$, $p=\exp(y)/[1+\exp(y)] = 1/[1 + \exp(-y)]$, so the plot in mean scale uses $1/[1+\exp(-y)]$ on the vertical axis.

\item We also provide a third option, \code{type="rescale"}, which plots in linear predictor (e.g., logit) scale, but labels the tick marks on the vertical axis in mean (e.g., probability) scale. This third option, which retains the linear structure of the model but labels the vertical axis on the usually more familiar mean scale, is the default.

\end{enumerate}

We use the \code{Blowdown} data from the \pkg{alr4} package to provide examples.  These data concern the probability of \emph{blowdown} \vn{y}, a tree being uprooted as the result of a major straight-line wind storm in the Boundary Waters Canoe Area Wilderness in 1999, modeled as a function of the diameter \code{d} of the tree, the local severity \code{s} of the storm, and the species \code{spp} of the tree.  We fit a main-effects model and then display all three predictor effect plots:
<<>>=
data("Blowdown", package="alr4")
gm1 <- glm(y ~ log(d) + s + spp, family=binomial, data=Blowdown)
@

<<fig34,include=TRUE,fig.width=6.5,fig.height=6.5,fig.show='hide'>>=
plot(predictorEffects(gm1),
     axes=list(grid=TRUE, x=list(rug=FALSE, rotate=35)))
@

\centerline{\includegraphics[width=.9\textwidth]{figure/fig34-1.pdf}}

\noindent
The \ar{rug=FALSE} sub-argument to \ar{x} suppresses the rug plot that appears by default at the bottom of graphs for numeric predictors, and the \ar{grid} sub-argument to \ar{axes} adds background grids.  The \ar{rotate} sub-argument prints the horizontal tick labels at an angle to avoid overprinting.

Interpretation of GLM predictor effect plots in link scale is similar to predictor effect plots for linear models, and all the modifications previously described can be used for these plots.  Because the default is \code{type="rescale"}, the vertical axis is in linear predictor scale, which is the log-odds or logit for this logistic regression example, but the vertical axis labels are in mean (probability) scale, so the tick-marks are not equally spaced.

The next three graphs illustrate the possible values of the argument \ar{type}:
<<fig35,include=TRUE,fig.width=3.5,fig.height=3.5,fig.show='hide'>>=
e1.gm1 <- predictorEffect("spp", gm1)
plot(e1.gm1, main="type='rescale'",
     axes=list(y=list(type="rescale",
                      lab="logit scale, probability labels"),
               x=list(rotate=30),
               grid=TRUE))
plot(e1.gm1, main="type='link'",
     axes=list(y=list(type="link",
                      lab="logit scale, logit labels"),
               x=list(rotate=30),
               grid=TRUE))
plot(e1.gm1, main="type='response'",
     axes=list(y=list(type="response", grid=TRUE,
                      lab="probabilty scale, probability labels"),
               x=list(rotate=30),
               grid=TRUE))
@

\includegraphics[width=.33\textwidth]{figure/fig35-1.pdf}
\includegraphics[width=.33\textwidth]{figure/fig35-2.pdf}
\includegraphics[width=.33\textwidth]{figure/fig35-3.pdf}

\noindent
The first two graphs show the same plot, but in the first the tick-marks on the vertical axis are unequally spaced and are in probability scale, while in the second the tick-marks are equally spaced and are in log-odds scale.  In the third graph, the vertical axis has been transformed to probability scale, and the corresponding tick-marks are now equally spaced.

The predictor effects plot for species would be easier to understand if the levels of the factor were ordered according to the estimated log-odds of blowdown.  First, we need to recover the fitted values in link scale, which are log-odds of blowdown for a logistic model.  The fitted log-odds are stored in \code{as.data.frame(e1.gm1)\$fit} using the \code{e1.gm1} object previously computed:

<<fig36,include=TRUE,fig.width=5.5,fig.height=4.5,fig.show='hide'>>=
or <- order(as.data.frame(e1.gm1)$fit) # order smallest to largest
Blowdown$spp1 <- factor(Blowdown$spp,  # reorder levels of spp
                        levels=levels(Blowdown$spp)[or])
gm2 <- update(gm1, ~ . - spp + spp1)   # refit model
plot(predictorEffects(gm2, ~ spp1), main="type='response', ordered",
     axes=list(y=list(type="response",
                      lab="probabilty scale, probability labels"),
               x=list(rotate=30, spp=list(lab="Species")),
               grid=TRUE))
@

\centerline{\includegraphics[width=.55\textwidth]{figure/fig36-1.pdf}}

\noindent
The separation of species into two groups of lower and higher probability species is reasonably clear after ordering, with paper birch more susceptible to blowdown than the other species and possibly in a group by itself.

\subsection{The \ar{lines} Group: Specifying Plotted Lines}

The \ar{lines} argument group allows the user to specify the color, type, thickness, and smoothness of lines.  This can be useful, for example, if the colors used by \pkg{effects} by default are for some reason unacceptable, such as for publications in which only black or gray-scale lines are permitted.  The most common use of this argument group is to allow more than one line to be plotted on the same graph or panel via the \ar{multiline} sub-argument.

\subsubsection{\ar{multiline} and \ar{z.var}:  Multiple Lines in a Plot}

Default predictor effect plots with conditioning predictors generate a separate plot for each level of the conditioning variable,  or for each combination of levels if there is more than one conditioning variable. For an example, we add the \code{log(d):s} interaction to the model \code{gm1}, and generate the predictor effect plots for \vn{s} and for \vn{d}:

<<fig37,include=TRUE,fig.width=9,fig.height=12,fig.show='hide'>>=
gm3 <- update(gm2, ~ . + s:log(d)) # add an interaction
plot(predictorEffects(gm3, ~ s + d),
     axes=list(x=list(rug=FALSE, rotate=90),
               y=list(type="response", lab="Blowdown Probability")),
     lattice=list(layout=c(1, 5)))
@

\centerline{\includegraphics[width=0.75\textwidth]{figure/fig37-1.pdf}}

\noindent
Setting the sub-argument \code{type="response"} for the \ar{y} axis plots the response on the probability scale. Setting \code{layout=c(1, 5)} arranges each predictor effect plot in 1 column of 5 rows. See the description of the \ar{lattice} argument in Section~\ref{sec:lattice}.

The predictor effect plot for \vn{s} conditions on the level of \vn{d}, and displays the plot of the fitted values for \vn{y} versus \vn{s} in a separate panel for each value of \vn{d}.  Similarly, the predictor effect plot for \vn{d} displays a separate panel for each conditioning level of \vn{s}.  Confidence bands are displayed by default around each fitted line.  These two graphs are based on essentially the same fitted values, with the values of the interacting predictors \vn{s} and \vn{d} varying, and fixing the factor predictor \vn{spp} to its distribution in the data, as described in Section~\ref{sec:facpred}. Concentrating on the graph at the right for the focal predictor \vn{d}, when \vn{s} is very small the probability of blowdown is estimated to be in the range of about .05 to .3 for any value of \vn{d}, but for larger values of \vn{s}, the probability of blowdown increases rapidly with \vn{d}. Similar comments can be made concerning the predictor effect plot for \vn{s}.

Setting \code{multiline=TRUE} superimposes the lines for all the conditioning values in a single graph. In the example below, we reduce the number of levels of the conditioning variable for each predictor effect plot to three explicit values each to produce simpler graphs, although this is not required. The \ar{xlevels} argument changes the number of levels for the conditioning predictors, but does not affect the number of levels for the focal predictor.  This latter quantity could be changed with the \ar{focal.levels} argument, but the default value of 50 evaluations is appropriate for graphing effects. 
<<fig38,include=TRUE,fig.width=9,fig.height=5,fig.show='hide'>>=
plot(predictorEffects(gm3, ~ s + d, 
                      xlevels=list(d=c(5, 40, 80), s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               x=list(rug=FALSE),
               y=list(type="response", lab="Blowdown probability")),
     lines=list(multiline=TRUE))
@
\centerline{\includegraphics[width=\textwidth]{figure/fig38-1.pdf}}

\noindent
In each graph, we kept, more or less, the lowest, middle, and highest values of the conditional predictor for the interaction. We also added a grid to each graph.  Multiline plots by default omit confidence bands or intervals, but these can be included using the \ar{confint} argument discussed in Section~\ref{sec:confint}.  By default, different values of the conditioning predictor are distinguished by color, and a key is provided.  The placement and appearance of the key are controlled by the \ar{key.args} sub-argument in the \ar{lattice} group discussed in Section~\ref{sec:key}.

When the conditioning group includes two or more predictors, and certainly when it includes three or more predictors, multiline plots are almost always helpful because otherwise the resulting array of panels becomes too complicated.  Suppose that we add the \code{spp:log(d)} interaction to the illustrative model.  The predictor effect plot for \vn{d} now includes both \vn{s} and \vn{spp} in the conditioning set because \vn{d} interacts with both of these predictors:

<<fig39,include=TRUE,fig.width=7,fig.height=7,fig.show='hide'>>=
gm4 <- update(gm3, ~ . + spp:log(d))
plot(predictorEffects(gm4, ~ d, xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               y=list(type="response"),
               x=list(rug=FALSE)),
     lines=list(multiline=TRUE))
@

\centerline{\includegraphics[width=\textwidth]{figure/fig39-1.pdf}}

\noindent
This plot now displays the lines for all conditioning values of \vn{s} within the panel for each level of the conditioning factor \vn{spp}. Compare this graph to the much more confusing plot in which different lines are drawn for the nine levels of the conditioning factor \vn{spp}, obtained by using the \ar{z.var} sub-argument in the \ar{lines} group:
<<fig310,include=TRUE,fig.width=7,fig.height=5,fig.show='hide'>>=
plot(predictorEffects(gm4, ~ d, xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE, 
               y=list(type="response"), 
               x=list(rug=FALSE)),
     lines=list(multiline=TRUE, z.var="spp", lty=1:9),
     lattice=list(layout=c(3, 1)))
@

\centerline{\includegraphics[width=.7\textwidth]{figure/fig310-1.pdf}}

\noindent
The \ar{z.var} sub-argument for \ar{lines} selects the predictor that determines the lines within a panel and the remaining predictors, here just \vn{s}, distinguish the panels.  The default choice of 
\ar{z.var} is usually, but not always, appropriate.  We also use the \ar{lattice} argument to display the array of panels in 3 columns and 1 row, and differentiate the lines by line type and color using arguments discussed next. 

\subsubsection{\ar{col}, \ar{lty}, \ar{lwd}, \ar{spline}: Line Color, Type, Width, Smoothness}\label{sec:line.color.etc}

Different lines in the same plot are differentiated by default using color.  This can be modified by the sub-arguments \ar{lty}, \ar{lwd} and \ar{col} to set line types, widths, and colors, respectively.  For example, in the last graph shown you can get all black lines of different line types using \code{lines=list(multiline=TRUE, col="black", lty=1:9)}, or using a gray scale, \code{lines=}\linebreak \code{list(multiline=TRUE, col=gray((1:9)/10))}.

The \fn{plot} method for effect objects by default uses smoothing splines to interpolate between plotted points.  Smoothing can be turned off with \code{splines=FALSE} in the \ar{lines} argument, but we rarely expect this to be a good idea. The number of values at which the focal predictor is evaluated is set with the \ar{focal.levels} argument, and it defaults to 50. In any case, more than three evaluations, and possibly many more, should be used for a reasonable spline approximation.

\subsection{The \ar{confint} Group: Specifying Confidence Interval Inclusion and Style}\label{sec:confint}

The \ar{confint} argument group controls the inclusion and appearance of confidence intervals and regions.  This argument has three sub-arguments. The \ar{style} sub-argument is either \code{"bars"}, for confidence bars, typically around the estimated adjusted mean for a factor level; \code{"bands"}, for shaded confidence bands, typically for numeric focal predictors; \code{"auto"}, to let the program automatically choose between \code{"bars"} and \code{"bands"}; \code{"lines"}, to draw only the edges of confidence bands with no shading; or \code{"none"}, to suppress confidence intervals.  The default is \code{"auto"} when \code{multiline=FALSE} and \code{"none"} when \code{multiline=TRUE}. Setting \code{confint="auto"} produces bars for factors and bands for numeric predictors. For example:
<<fig311,include=TRUE,fig.width=5.5,fig.height=5.5,fig.show='hide'>>=
plot(predictorEffects(gm3, ~ d, 
                      xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               x=list(rug=FALSE),
               y=list(type="response")),
     lines=list(multiline=TRUE),
     confint=list(style="auto"))
@

\centerline{\includegraphics[width=.5\textwidth]{figure/fig311-1.pdf}}

\noindent
In this example the confidence bands are well separated, so including them in a multiline graph isn't problematic; in other cases, overlapping confidence bands produce an artistic but uninterpretable mess.  

With a factor focal predictor, we get:
<<fig312,include=TRUE,fig.width=7,fig.height=6,fig.show='hide'>>=
gm5 <- update(gm2, ~ . + spp:s)
plot(predictorEffects(gm5, ~ spp, xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               y=list(type="response"),
               x=list(rug=FALSE, rotate=30)),
     lines=list(multiline=TRUE),
     confint=list(style="auto"))
@

\centerline{\includegraphics[width=.75\textwidth]{figure/fig312-1.pdf}}

\noindent
The error bars for the various levels of \vn{s} are slightly staggered to reduce over-plotting.

Two additional arguments, \vn{col} and \vn{alpha}, control respectively the color of confidence bars and regions and the transparency of confidence regions.  Users are unlikely to need these options.  Finally, the type of confidence interval shown, either pointwise or Scheff\'{e} corrected for multiple comparisons, is controlled by the \ar{se} argument to the \fn{predictorEffect} or \fn{Effect} function (see Section~\ref{sec:se}).

\subsection{The \ar{lattice} Group: Specifying Standard \textbf{lattice} Package Arguments}\label{sec:lattice}

The \fn{plot} methods defined in the \pkg{effects} package use functions in the \pkg{lattice} package \citep{sarkar08}, such as \fn{xyplot}, to draw effect plots, which often comprise rectangular arrays of panels. In particular, the \fn{plot} method for the \class{eff} objects returned by the \fn{Effect} function are \class{trellis} objects, which can be manipulated in the normal manner. ``Printing'' a returned effect-plot object displays the plot in the current \R{} graphics device.

The \ar{lattice} group of arguments to the \fn{plot} method for effect objects may be used to specify various standard arguments for \pkg{lattice} graphics functions such as \fn{xyplot}.  In particular, you can control the number of rows and columns when panels are displayed in an array, modify the key (legend) for the graph, and specify the contents of the ``strip" displayed in the shaded region of text above each panel in a \pkg{lattice} array.  In addition, the \ar{array} sub-argument, for advanced users, controls the layout of multiple predictor effect plots produced by the \fn{predictorEffects} function.

\subsubsection{\ar{key.args}: Modifying the Key}\label{sec:key}

A user can modify the placement and appearance of the key with the \ar{key.args} sub-argument, which is itself a list.  For example:
<<fig314,include=TRUE,fig.width=8,fig.height=6,fig.show='hide'>>=
plot(predictorEffects(gm5, ~ spp, xlevels=list(s=c(0.1, 0.5, 0.9))),
     rug=FALSE,
     axes=list(grid=TRUE,
               y=list(type="response"),
               x=list(rotate=30)),
     lines=list(multiline=TRUE),
     confint=list(style="auto"),
     lattice=list(key.args=list(space="right",
                                columns=1,
                                border=TRUE,
                                fontfamily="serif",
                                cex=1.25,
                                cex.title=1.5)))
@

\centerline{\includegraphics[width=.99\textwidth]{figure/fig314-1.pdf}}

\noindent
The sub-argument \code{space="right"} moves the key to the right of the graph, overriding the default \code{space="top"}.  Alternatively the key can be placed inside the graph using the \ar{x}, \ar{y}, and \ar{corner} sub-arguments, as illustrated in the graph on page~\pageref{corner}. The choices for \ar{fontfamily} are \code{"sans"} and \code{"serif"}, and affect only the key; the rest of the plot uses \code{"sans"}.  The sub-arguments \ar{cex} and \ar{cex.title} control the relative sizes of the key entries and the key title, respectively.  Finally, any argument documented in \code{help("xyplot")} in the \code{key} section can be set with this argument. If you use the default \code{space="top"} for placement of the key, you may wish to adjust the number of columns in the key, particularly if the level names are long.

\subsubsection{\ar{layout}: Controlling Panel Placement}\label{sec:layout}

The \ar{layout} sub-argument to the \ar{lattice} argument allows a user to customize the layout of multiple panels in a predictor effect plot; for example:
<<fig313,include=TRUE,fig.width=13,fig.height=5.5,fig.show='hide'>>=
plot(predictorEffects(gm3, ~ s + d, xlevels=list(s=6, d=6)),
     axes=list(x=list(rug=FALSE, rotate=90), 
               y=list(ticks=list(at=c(.999, .99, .95, .8, .5, .2, .05)))),
     lattice=list(layout=c(3, 2)))
@

\centerline{\includegraphics[width=\textwidth]{figure/fig313-1.pdf}}

\noindent
Here, the \ar{layout} sub-argument specifies an array of 3 columns and 2 rows for each of the predictor effect plots.

\subsubsection{\ar{array}: Multiple Predictor Effect Plots}\label{sec:array}

If you create several predictor effect objects with the  \fn{predictorEffects} function, the \fn{plot} method for the resulting \class{predictorefflist} object divides the \pkg{lattice} graphics device into a rectangular array of sub-plots, so that the individual predictor effect plots, each potentially with several panels, are drawn without overlapping. An alternative is for the user to generate the predictor effect plots separately, subsequently supplying the \ar{array} sub-argument to \fn{plot} directly to create a custom meta-array of predictor effect plots; this argument is ignored, however, for \class{predictorefflist} objects produced by \fn{predictorEffects}.

Suppose, for example, that we want to arrange the two predictor effect plots for the previous example vertically rather than horizontally. One way to do that is to save the object produced by \fn{predictorEffects} and to plot each of its two components individually, specifying the \ar{position} or \ar{split} and \ar{more} arguments to the \fn{print} method for \class{trellis} objects: see \code{help("print.trellis")}. 

Another approach is to generate the plots individually using \fn{predictorEffect} and to specify the \ar{array} sub-argument to \fn{plot}, as follows:

<<fig313b,include=TRUE,fig.width=6,fig.height=10,fig.show='hide'>>=
plot(predictorEffect("s", gm3, xlevels=list(d=6)),
     axes=list(x=list(rug=FALSE, rotate=90),  
               y=list(ticks=list(at=c(.999, .99, .95, .8, .5, .2, .05)))),
     lattice=list(layout=c(3, 2),
                  array=list(row=1, col=1, nrow=2, ncol=1, more=TRUE)))
plot(predictorEffect("d", gm3, xlevels=list(s=6)),
     axes=list(x=list(rug=FALSE, rotate=90),  
               y=list(ticks=list(at=c(.999, .99, .95, .8, .5, .2, .05)))),
     lattice=list(layout=c(3, 2),
                  array=list(row=2, col=1, nrow=2, ncol=1, more=FALSE)))
@
\centerline{\includegraphics[width=.65\textwidth]{figure/fig313b-1.pdf}}

\noindent In each case, the \ar{row} and \ar{col} sub-arguments indicate the position of the current graph in the meta-array; \ar{nrow} and \ar{ncol} give the dimensions of the meta-array, here 2 rows and 1 column; and \ar{more} indicates whether there are more elements of the meta-array after the current graph.

\subsubsection{\ar{strip}: Modifying the Text at the Tops of Panels}\label{sec:strip}

Lattice graphics with more than one panel typically provide a text label at the top of each panel in an area called the \emph{strip}.  The default strip text contains the name of the conditioning predictor and the value to which it is set in the panel; if there are more than one conditioning predictor, then all of their names and corresponding values are shown. For example:
<<fig316,include=TRUE,fig.width=7,fig.height=5,fig.show='hide'>>=
plot(predictorEffects(gm4, ~ d, xlevels=list(s=c(0.1, 0.5, 0.9))),
     axes=list(grid=TRUE,
               x=list(rug=FALSE),
               y=list(type="response")),
     lines=list(multiline=TRUE, z.var="spp", lty=1:9),
     lattice=list(layout=c(3, 1),
                  strip=list(factor.names=TRUE,
                             values=TRUE,
                             cex=1.5)))
@

\centerline{\includegraphics[width=.85\textwidth]{figure/fig316-1.pdf}}

\noindent
Setting \code{factor.names=FALSE} (the default is \code{TRUE}) displays only the value, and not the name, of the conditioning predictor in each strip; usually, this is desirable only if the name is too long to fit, in which case you may prefer to rename the predictor.  Setting \code{values=FALSE} replaces the conditioning value with a line in the strip that represents the value: The line is at the left of the strip for the smallest conditioning value, at the right for the largest value, and in a proportional intermediate position in between the two extremes.  The most generally useful sub-argument is \ar{cex}, which allows you to reduce or expand the relative size of the text in the strip, in this case increasing the size to 150\% of standard size.


\subsection{\ar{symbols}: Plotting symbols}

Symbols are used to represent  adjusted means when the focal predictor is a factor. You can control the symbols used and their relative size:
<<fig315,include=TRUE,fig.width=7,fig.height=6,fig.show='hide'>>=
gm5 <- update(gm2, ~ . + spp:s)
plot(predictorEffects(gm5, ~ spp, xlevels=list(s=c(0.1, 0.5, 0.9))),
     symbols=list(pch=15:17, cex=1.5),
     axes=list(grid=TRUE,
               y=list(type="response"),
               x=list(rotate=30)),
     lines=list(multiline=TRUE),
     confint=list(style="auto"),
     lattice=list(key.args=list(cex=1.5, cex.title=1.5)))
@

\centerline{\includegraphics[width=.95\textwidth]{figure/fig315-1.pdf}}

\noindent
We use the \ar{pch} sub-argument to set the symbol number for plotted symbols;
 you can enter the commands \code{plot(1:25, pch=1:25)} and \code{lines(1:25, lty=2, type="h")} to see the 25 plotting symbols in \R{}.  The sub-argument \ar{pch} can also be a character vector, such as \code{letters[1:10]}. In this example, we set \code{cex=1.5} to increase the symbol size by the factor 1.5.  Because only one value is given, it is recycled and used for all of the symbols. We need to change the size of the symbols in the key separately, as we do here via the \ar{key.args} sub-argument to the \ar{lattice} argument (see Section~\ref{sec:key}).

\section{Displaying Residuals in Predictor Effect Plots}\label{sec:res}

\citet{fw19b} introduce methodology for adding partial residuals to a predictor effect or effect plot.  This can be desirable to display variation in data around a fitted partial regression surface or to diagnose possible lack of fit, as the resulting plots are similar to traditional component-plus-residual plots \citep[Sec.~8.4]{fw19}.

The predictor effect plot for a numeric focal predictor that does not interact with other predictors is equivalent to a standard component-plus-residual plot; for example:
<<fig51,include=TRUE,fig.width=10,fig.height=9,fig.show='hide'>>=
lm5 <- lm(prestige ~ log(income) + education + women + type,
          Prestige)
plot(predictorEffects(lm5, residuals=TRUE),
     axes=list(grid=TRUE,
               x=list(rotate=30)),
     partial.residuals=list(smooth=TRUE,
                            span=0.75,
                            lty="dashed"))
@
\centerline{\includegraphics[width=.99\textwidth]{figure/fig51-1.pdf}}

\noindent
The partial residuals to be plotted are computed using the \ar{residuals} argument to the \fn{predictorEffect}, \fn{predictorEffects}, or \fn{Effect} function.  For the numeric predictors \vn{income}, \vn{education}, and \vn{women}, the plotted points are each equal to a point on the fitted blue line, representing the partial fit, plus the corresponding residual. For \vn{income}, the fitted partial-regression line in curved because of the log transformation of the predictor, but the partial-regression function is a straight line for the other two numeric predictors.  

The dashed line produced by \code{lty="dashed"} in the same magenta color as the plotted points on the graph, is a loess nonparametric-regression smooth of the points. The sub-argument \code{smooth=TRUE} is the default if residuals are present in the effect object to be plotted. The sub-argument \code{span=0.75} adjusts the span of the loess smoother from the default of \code{2/3}---an unnecessary adjustment here specified simply to illustrate how to set the span. If the model adequately represents the data, then the dashed  magenta line should approximately match the solid blue partial-regression line, which represents the fitted model. 

For the factor \vn{type}, the points are jittered horizontally to separate them visually, because the only possible horizontal coordinates are at the three distinct factor levels. Smooths are not fit to factors and instead the conditional means of the partial residuals are plotted as solid magenta dots; in the current model, the magenta dots and the blue dots representing the fitted adjusted means of the response at the levels of \vn{name} necessarily match.

The \fn{plot} method for effect objects has a \ar{partial.residuals} argument, with several sub-arguments that control how partial residuals are displayed.  In the command above, we used the sub-argument \vn{smooth=TRUE} to add the smoother, which is the default when residuals are included in the effect object, and \ar{lty="dashed"} to change the line type for the smooth from the default solid line to a dashed line.  All the \vn{smooth} sub-arguments are described in \code{help("plot.eff")}.

For a second example, we fit a linear model with an interaction to the \code{UN} data set in the \pkg{carData} package, modelling national \vn{infantMortality} rate (infant deaths per 1000 live births) as a function of \vn{ppgdp}, per person GDP (in U.S.~dollars), and country \vn{group} (OECD nations, African nations, and other nations). The data are for roughly 200 nations of the world and are from approximately 2009 to 2011:
<<fig52,include=TRUE,fig.width=10,fig.height=5,fig.show='hide'>>=
options(scipen=10) # suppress scientific notation 
lm6 <- lm(infantMortality ~ group*ppgdp, data=UN)
plot(predictorEffects(lm6, ~ ppgdp, partial.residuals=TRUE),
     axes=list(x=list(rotate=25), 
               y=list(lim=c(0, 150))),
    id=list(n=1),
    lattice=list(layout=c(3, 1)))
@
\centerline{\includegraphics[width=.99\textwidth]{figure/fig52-1.pdf}}

\noindent
The predictor effect plot for \vn{ppgdp} conditions on the factor \ar{group} because of the interaction between these two predictors. Several problems are apparent in this plot: The \ar{id} argument is used to identify the most unusual point in each panel, as described in detail in \code{help("plot.eff")}.  Turkey has higher than predicted infant mortality for the \level{oecd} group; Afghanistan, in the \level{other} group, has infant mortality much higher than predicted; and Equatorial Guinea is clearly unusual for the \level{africa} group.  In addition, the smooths through the points do not match the fitted lines in the \level{other} and \level{africa} groups.  We use the command \code{options(scipen=10)} to suppress annoying scientific notation in the tick-mark labels on the horizontal axis, and instead rotate these labels so that they fit without over-plotting.

Log-transforming both the predictor \vn{ppgdp} and the response \vn{infantMortality} produces a better fit to the data:
<<fig53,include=TRUE,fig.width=10,fig.height=5,fig.show='hide'>>=
lm7 <- lm(log(infantMortality) ~ group*log(ppgdp), data=UN)
plot(predictorEffects(lm7, ~ ppgdp, partial.residuals=TRUE),
     axes=list(x=list(rotate=25)),
     id=list(n=1),
     lattice=list(layout=c(3, 1)))
@
\centerline{\includegraphics[width=.99\textwidth]{figure/fig53-1.pdf}}

\noindent
Equatorial Guinea is still anomalous, however. Rescaling the vertical axis to arithmetic scale produces a slightly different, but possibly useful, picture:
<<fig54,include=TRUE,fig.width=10,fig.height=5,fig.show='hide'>>=
plot(predictorEffects(lm7, ~ ppgdp, partial.residuals=TRUE),
     axes=list(x=list(rotate=25),
               y=list(transform=list(trans=log, inverse=exp),
                      type="response",
                      lab="Infant Mortality")),
     id=list(n=1),
     lattice=list(layout=c(3, 1)))
@
\centerline{\includegraphics[width=.99\textwidth]{figure/fig54-1.pdf}}

Partial residuals can be added to effect plots for linear or generalized linear models in the default link scale, and to effect plots for linear or generalized linear mixed models.

\subsection{Using the \fn{Effect} Function With Partial Residuals}

In most instances, predictor effect plots produced by \fn{predictorEffect} or \fn{predictorEffects} visualize a fitted model in the most natural manner, but sometimes in looking for lack of fit, we want to plot against arbitrary combinations of predictors. The more general \fn{Effect} function is capable of doing that.

Recall, for example, the additive model \code{lm2} fit to the \code{Prestige} data:
<<>>=
S(lm2)
@
Plotting partial residuals for the predictors \vn{income} and \vn{type} simultaneously reveals an unmodeled $\vn{income} \times \vn{type}$ interaction:
<<fig55,include=TRUE,fig.width=8,fig.height=4,fig.show='hide'>>=
plot(Effect(c("income", "type"), lm2, residuals=TRUE),
     axes=list(x=list(rotate=30)),
     partial.residuals=list(span=0.9), 
     layout=c(3, 1))
@
\centerline{\includegraphics[width=0.85\textwidth]{figure/fig55-1.pdf}}

\section{Polytomous Categorical Responses}

The \pkg{effects} package produces special graphs for ordered and unordered polytomous categorical response variables.
In an ordinal regression, the response is an ordered categorical variable with three or more levels.  For example, in a study of women's labor force participation that we introduce below, the response is not working outside the home, working part time, or working full time.  The proportional-odds model \citep[Sec.~6.9]{fw19} estimates the probability of a response in each of these three categories given a linear combination of regressors defined by a set of predictors, assuming a logit link function.  

We illustrate the proportional-odds model with the \code{Womenlf} data set in the \pkg{carData} package, for young married Canadian women's labor-force participation,  using the \fn{polr} function in the \pkg{MASS} package to fit the model:
<<>>=
library("MASS") # for polr()
Womenlf$partic <- factor(Womenlf$partic,
    levels=c("not.work", "parttime", "fulltime")) # order response levels
or1 <- polr(partic ~ log(hincome) + children, data=Womenlf)
S(or1)
@
The response variable \code{partic} initially has its levels in alphabetical order, which does not correspond to their natural ordering.  We therefore start by reordering the levels to increase from \level{not.work}, to \level{parttime} work, to \level{fulltime} work. The predictors are the numeric variable \vn{hincome} (husband's income), which enters the model in log-scale, and the dichotomous factor \vn{children}, presence of children in the household.

The model summary is relatively complex, and is explained in \citet[Sec.~6.9]{fw19}. Predictor effect plots greatly simplify interpretation of the fitted model:
<<fig41,include=TRUE,fig.width=6.5,fig.height=6.5,fig.show='hide'>>=
plot(predictorEffects(or1),
     axes=list(grid=TRUE),
     lattice=list(key.args=list(columns=1)))
@
\centerline{\includegraphics[width=.9\textwidth]{figure/fig41-1.pdf}}

\noindent
Unlike predictor effect plots for generalized linear models, the default scaling for the vertical axis is the probability scale, equivalent to \code{axes=list(y=list(type="response"))} for a GLM, and the alternative is \code{axes=list(y=list(type="logit"))}, which is analogous to \code{type="link"} for a GLM.\footnote{The logits plotted, however, correspond to the individual-level probabilities and are not the ordered logits in the definition of the proportional-odds model.}  Confidence bands are present by default, unless turned off with the argument \code{confint=list(style="none")}. Numeric focal predictors are by default evaluated at 50 points.  The plot for \vn{hincome} suggests high probability of full-time work if husband's income is low, with the probability of full-time work sharply decreasing to about \$15,000 and then nearly leveling off at about .1 to .2.  The probability of not working  rapidly increases with husband's income, while the probability of working part time is fairly flat.  A similar pattern is apparent for children present in the home, with full-time work much less prevalent and not working much more prevalent when children are present than when they are absent.

\emph{Stacked area plots} are sometimes more useful for examining polytomous response models; for example:
<<fig62,include=TRUE,fig.width=6,fig.height=4,fig.show='hide'>>=
plot(predictorEffects(or1),
     axes=list(grid=TRUE, y=list(style="stacked")),
     lattice=list(key.args=list(columns=1)))
@

\centerline{\includegraphics[width=.95\textwidth]{figure/fig62-1.pdf}}


\noindent
For each fixed value on the horizontal axis, the vertical axis ``stacks" the probabilities in the three response categories.  For example, with children absent from the household and \vn{hincome} set to its mean, nearly 30\% of women did not work outside the home, about 20\% worked part time, and the remaining approximate 50\% worked full time.  

Some ordinal-response models produced by the functions \fn{clm}, \fn{clm2}, and \fn{clmm} in the \pkg{ordinal} package can be used with the \pkg{effects} package.  To work with model objects produced by these functions, you must also load the \pkg{MASS} package.

The \pkg{effects} package can also draw similar graphs for the more general
 multinomial logit model, in which the polytomous categorical response has unordered levels \citep[see][Sec.~6.7]{fw19}.  The details of the model, its parameters, and its assumptions are different from those of the proportional-odds model and other ordered-response models, but predictor effect plots for these models are similar.

As an example, we use the \code{BEPS} data set in the \pkg{carData} package, consisting of about 1,500 observations from the 1997-2001 British Election Panel Study.  The response variable, \vn{vote}, is party choice, one of \level{Liberal Democrat}, \level{Labour}, or \level{Conservative}.  There are numerous predictors of \vn{vote} in the data set, and we fit the model
<<>>=
library("nnet") # for multinom()
mr1 <- multinom(vote ~ age + gender + economic.cond.national +
                       economic.cond.household + Blair + Hague + Kennedy +
                       Europe*political.knowledge, data=BEPS)
@
There are nine predictors, seven of which are scales with values between 0 and 5 concerning respondents' attitudes; these predictors enter the model as main effects. The remaining two predictors are scales between 0 and 3 for \code{political.knowledge} and between 1 and 11 for \code{Europe} (attitude toward European integration of the UK in the European Union, with high values representing ``Euroscepticism'', a \emph{negative} attitude toward Europe); these predictors enter the model with a two-factor interaction.  

Drawing all nine predictor effect plots simultaneously is not a good idea because the plots won't fit reasonably in a single display.  We therefore draw only a few of the plots at a time:
<<fig42,include=TRUE,fig.width=6.5,fig.height=6.5,fig.show='hide'>>=
plot(predictorEffects(mr1, ~ age + Blair + Hague + Kennedy),
     axes=list(grid=TRUE, x=list(rug=FALSE)),
     lattice=list(key.args=list(columns=1)),
     lines=list(multiline=TRUE, col=c("blue", "red", "orange")))
@

\centerline{\includegraphics[width=.9\textwidth]{figure/fig42-1.pdf}}

\noindent
We use optional arguments to get a multiline plot, with a grid and no rug plot, and to modify the key. The color specification for the lines represents the traditional colors of the three parties. Interpreting these plots is challenging: For example, the probability of voting Labour decreases with age, increases with attitude toward the Labour leader Blair, strongly decreases with attitude toward the Conservative leader Hague, and is relatively unaffected by attitude toward the Liberal Democrat leader Kennedy.  In general, a positive attitude toward a party leader increases the probability of voting for that leader's party, as one would expect. Of course, the causal direction of these relationships is unclear.

We next turn to the interaction between \vn{Europe} and \vn{political.knowledge}, this time drawing stacked area displays:

<<fig43,include=TRUE,fig.width=10,fig.height=5,fig.show='hide'>>=
plot(predictorEffects(mr1, ~ Europe + political.knowledge,
                      xlevels=list(political.knowledge=0:3,
                                   Europe=c(1, 6, 11))),
     axes=list(grid=TRUE, 
               x=list(rug=FALSE, 
                      Europe=list(ticks=list(at=c(1, 6, 11))),
                      political.knowledge=list(ticks=list(at=0:3))),
               y=list(style="stacked")),
     lines=list(col=c("blue", "red", "orange")),
     lattice=list(key.args=list(columns=1),
                  strip=list(factor.names=FALSE)))
@
\centerline{\includegraphics[width=\textwidth]{figure/fig43-1.pdf}}

\noindent
The \ar{lines} argument is used to specify the colors for the stacked areas representing the parties. Both effect plots are of nearly the same fitted values,\footnote{Not exactly the same because in each plot the focal predictor takes on 50 values and the conditioning predictor 3 or 4 values.} in the first graph with \code{Europe} varying and conditioning on \code{political.knowledge}, and in the second with \code{political.knowledge} varying and conditioning on \code{Europe}.  Setting \code{strip=} \code{list(factor.names=FALSE)} suppresses the names of the conditioning predictor in each effect plot; these names are too long for the strips at the tops of the panels. From the first graph,  preference for the Conservative Party increases with \vn{Europe} for respondents with high political knowledge, but not for those with low political knowledge. More generally, voters with high political knowledge are more likely to align their votes with the positions of the parties, Eurosceptic for the Convervatives, pro-Europe for Labour and the Liberal Democrats, than are voters with low political knowledge.

\section{The Lattice Theme for the effects Package}

The \pkg{effects} package uses the \fn{xyplot} and \fn{barchart} functions in the standard \pkg{lattice} package \citep{sarkar08} to draw effect plots.  The \pkg{lattice} package has many options for customizing the appearance of graphs that are collected into a \emph{lattice theme}.  We created a custom theme for use with the \pkg{effects} package that automatically supersedes the default Lattice theme when the \pkg{effects} package is loaded, \emph{unless the} \pkg{lattice} \emph{package has been previously loaded}.  You can invoke the \pkg{effects} package theme directly by the command
<<eval=FALSE>>=
effectsTheme()
@
You can also customize the \pkg{effects} package Lattice theme; see \code{help("effectsTheme")}. Finally, because \fn{plot} methods in the \pkg{effects} package return lattice objects, these objects can be edited and manipulated in the normal manner, for example by functions in the \pkg{latticeExtra} package \citep{SarkarAndrews2016}.


\bibliography{predictor-effects-gallery}

\end{document}

%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Regression Models Supported by the effects Package}

\documentclass[11pt]{article}


\usepackage[utf8]{inputenc}
\usepackage{graphicx}
\usepackage[american]{babel}
\newcommand{\R}{{\sf R}}
\usepackage{url}
\usepackage{hyperref}
\usepackage{alltt}
\usepackage{fancyvrb}
\usepackage{natbib}
\usepackage{amsmath}

\usepackage[margin=1in]{geometry}
\usepackage{ragged2e}

\VerbatimFootnotes
\bibliographystyle{chicago}

\newcommand{\x}{\mathbf{x}}
\newcommand{\code}[1]{\normalfont\texttt{\hyphenchar\font45\relax #1}}
\newcommand{\E}{\mathrm{E}}
\newcommand{\tild}{\symbol{126}}
\newcommand{\Rtilde}{\,\raisebox{-.5ex}{\code{\tild{}}}\,}
\newcommand{\captilde}{\mbox{\protect\Rtilde}} % use in figure captions.
\newcommand{\Rmod}[2]{\code{#1 \raisebox{-.5ex}{\tild{}} #2}}
\newcommand{\Rmoda}[2]{\code{#1} &\code{\raisebox{-.5ex}{\tild{}} #2}}
\newcommand{\Rmodb}[2]{\code{#1 &\raisebox{-.5ex}{\tild{}}& #2}}
\newcommand{\C}{\mathbf{C}}
\newcommand{\betahat}{\widehat{\beta}}
\newcommand{\bbetahat}{\widehat{\boldsymbol{\beta}}}
\newcommand{\bbeta}{\boldsymbol{\beta}}
\newcommand{\xbf}{\x_{\backslash{}f}}
\newcommand{\hbf}{h_{\backslash{}f}}
\newcommand{\xtb}{\x_{2\backslash{}f}}
\newcommand{\xbfi}{\x_{\backslash{}f,i}}
\newcommand{\inter}[2]{\mbox{$#1$:$#2$}}
\newcommand{\cross}[2]{\mbox{$#1$\code{*}$#2$}}
\newcommand{\N}{\mathrm{N}}

\newcommand{\yx}{\widehat{y}(\x)}
\newcommand{\lvn}[1]{\mbox{$\log(\mbox{\texttt{#1}})$}}

\newcommand{\fn}[1]{\code{#1()}}
\newcommand{\pkg}[1]{\textbf{#1}}
\newcommand{\proglang}[1]{\textsf{#1}}
\newcommand{\class}[1]{\texttt{"#1"}}

\usepackage{xcolor}
\newcommand{\Comment}[1]{\textbf{{\color{red}#1}}}

\begin{document}

\title{Regression Functions Supported by the \textbf{effects} Package\\
And How to Support Other Classes of Regression Models}

\author{John Fox and Sanford Weisberg}

\date{2022-07-07}

\maketitle

<<setopts,echo=FALSE>>=
library("knitr")
opts_chunk$set(fig.width=5,fig.height=5,tidy=TRUE,
               out.width="0.8\\textwidth",echo=TRUE)
options(prompt=" ")
@ 

<<echo=FALSE, results='hide', include=FALSE>>=
#options(continue="+    ", prompt="R> ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)
library(effects)
@

<<include=FALSE>>=
library(knitr)
opts_chunk$set(
tidy=FALSE,fig.width=5,fig.height=5,cache=FALSE,comment=NA, prompt=TRUE
)
render_sweave()
@


<<echo=FALSE, results='hide', include=FALSE>>=
options(continue="    ", prompt=" ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)
@

\section{Introduction}

\emph{Effect plots}, as implemented in the \pkg{effects} package, represent the ``effects'' (in the not necessarily causal sense of ``partial relationship'') of one or more predictors on a response variable,  in regression models in which the response depends on a \emph{linear predictor}---a linear combination of main effects and interactions among the predictors \citep[Sec.~4.6.3]{FoxWeisberg19}. \fn{Effect} is the basic generic function in the \pkg{effects} package; \fn{Effect} is called directly or indirectly by several other functions in the package, such as \fn{predictorEffects} and \fn{allEffects}.

Table~\ref{tab1} provides a list of regression modeling functions in \R{} that can be used with the \pkg{effects} package. This list, which is almost surely incomplete, includes functions that are directly supported by \fn{Effect} methods supplied by the \pkg{effects} package, by \fn{Effect} methods supplied by other CRAN packages, or by the default \fn{Effect} method,  which works with many classes of regression models.

\begin{table}
\caption{\R{} regression functions known to be compatible with the \fn{Effect} function.  The name before the double-colon is the package that includes the function; for example \fn{stats::lm} means that \fn{lm} is in the \pkg{stats} package. In some cases, \fn{Effect} may support only a subset of regression models fit by a particular function. Effects for mixed-effects models represent the fixed-effects part of the model.\label{tab1}}
\begin{center}
\begin{tabular}{|l|p{4.0in}|}\hline
Function & Comments \\ \hline
\multicolumn{2}{|l|}{\textbf{\code{glm}-type models}}\\ \hline
\fn{stats::lm} & Standard linear regression models fit by least-squares or weighted least-squares.  A multivariate response, generating a multivariate linear model, is permitted, and in this case effects are computed for each response separately.\\
\fn{stats::glm} & Generalized linear models.\\
\fn{nlme::lme} & Linear mixed-effects models.\\
\fn{nlme::gls} & Linear models fit by generalized least squares.\\
\fn{lmer::lmer} & Linear mixed-effects models.\\
\fn{lmer::glmer} & Generalized linear mixed-effects models.\\
\fn{survey::svyglm} & Generalized linear models for complex survey designs.\\
\fn{MASS::rlm} & Linear regression models estimated by robust M or MM regression.\\
\fn{MASS::glmmPQL} & Generalized linear mixed-effects models via partial quadratic likelihood.\\
\fn{robustlmm::rlmer} & Robust linear mixed-effects models.\\ 
\fn{betareg::betareg} & Beta regression models for rates and proportions.\\
\fn{ivreg::ivreg} & Linear regression models estimated by instrumental variables (2SLS regression). \\
\fn{glmmTMB::glmmTMB} & Generalized linear mixed-effects regression models (similar to \fn{lmer::glmer} but accommodating a broader selection of models).\\
\hline
\multicolumn{2}{|l|}{\textbf{\code{multinom}-type models}}\\
\hline
\fn{nnet::multinom} & Multinomial logistic-regression models.  If the response has $K$ categories, the response for \fn{nnet::multinom} can be a factor with $K$ levels or a matrix with $K$ columns, which will be interpreted as counts for each of $K$ categories.  Effects plots require the response to be a factor, not a matrix.\\
\fn{poLCA::poLCA} & Latent class analysis regression models for polytomous outcomes.  Latent class analysis has a similar structure to multinomial regression, except that class membership of observations is unobserved but estimated in the analysis.\\
\hline
\multicolumn{2}{|l|}{\textbf{\code{polr}-type models}}\\ \hline
\fn{MASS:polr} & Ordinal logistic (proportional-odds) and probit regression models.\\
\fn{ordinal::clm} & Cumulative-link regression models (similar to, but more extensive than, \fn{polr}).\\
\fn{ordinal::clm2}& Updated version of \fn{ordinal::clm}.\\
\fn{ordinal::clmm} & Cumulative-link regression models with random effects.\\
\hline
\end{tabular}
\end{center}
\end{table}

The most basic type of model for which \fn{Effect} is appropriate is a standard linear model fit by the \fn{lm} function; for example:

<<fig.height=4,fig.width=8>>=
library("effects")
Prestige$type <- factor(Prestige$type, c("bc", "wc", "prof")) # reorder levels
g1 <- lm(prestige ~ education + type + education:type, data = Prestige)
  # equivalent to lm(prestige ~ education*type, data = Prestige)
plot(predictorEffects(g1), lines=list(multiline=TRUE))
@

\noindent
In this example the response \code{prestige} is modeled as a linear function of years of \code{education}, the factor \code{type}, with levels blue collar (\code{"bc"}), white collar (\code{"wc"}), and professional (\code{"prof"}), and their interaction. Because of the interaction, the estimated partial relationship of \code{prestige} to \code{education} (depicted in the \emph{predictor effect plot} for \code{education}, at the left) is different for each level of \code{type}, and the partial relationship of \code{prestige} to \code{type} (depicted in the predictor effect plot for \code{type}, at the right) varies with the value \code{education}.

A linear mixed-effects model is a more complicated regression model, fit, for example, by the \fn{lmer} function in the \pkg{lme4} package \citep{Bates15}:
<<>>=
data(Orthodont, package="nlme")
g2 <- lme4::lmer(distance ~ age + Sex + (1 | Subject), data = Orthodont)
summary(g2)
@
This model has a fixed effect part, with response \code{distance} and predictors \code{age} and \code{Sex}.  The random intercept (represented by \code{1}) varies by \code{Subject}.  Effect plots for mixed-effects models are based only on the estimated fixed-effects in the model:
<<fig.height=4,fig.width=8>>=
plot(predictorEffects(g2))
@

\section{Basic Types of Regression Models in the effects Package}

The \fn{Effects} function supports three basic types of regression models:

\begin{itemize}

\item The preceding examples that use the \fn{lm} and \fn{lmer} functions are examples of \code{glm}-type models, which express, via a link function, the dependence of a discrete or continuous numeric response or of a binary response on a set of main effects and interactions among fixed-effect predictors comprising a linear predictor.  The \fn{glm} function is the prototype for this kind of model. As shown in Table~\ref{tab1}, most of the regression functions currently supported by the \pkg{effects} package are of this type.  

\item \code{multinom}-type models are multinomial regression models that arise when the response is an unordered multi-category variable, also modeled, via a suitable multivariate link function, as a linear function of fixed-effect main effects and interactions. The prototype for \code{multinom}-type models is the \fn{multinom} function in the \pkg{nnet} package \citep{VenablesRipley02}.

\item \code{polr}-type models (i.e., ordinal regression models) are used for an ordered polytomous response variable. The prototype for  \code{polr}-type models is the \fn{polr} function in the \pkg{MASS} package \citep{VenablesRipley02}. 

\end{itemize}

\section{Supporting Specific Regression Functions}

To support a specific class of regression models, say of class \code{"foo"} produced by the function \fn{foo}, one \emph{could} write a method \fn{Effect.foo} for the \proglang{S3} generic \fn{Effect} function. That approach is generally undesirable, for two reasons: (1) writing an \fn{Effect} method from scratch is a complicated endeavor; (2) the resulting object may not work properly with other functions in the \pkg{effects} package, such as \fn{plot} methods.

The \pkg{effects} package defines and exports several methods for the \fn{Effect} function, including a default method, and three specific methods corresponding to the three types of regression models introduced in the preceding section: \fn{Effect.lm} (which is also inherited by models of class \code{"glm"}), \fn{Effect.multinom}, and \fn{Effect.polr}. Moreover, \fn{Effect.default} works by setting up a call to one of the three specific \fn{Effect} methods.\footnote{There are, as well, two additional specific \fn{Effect} methods provided by the \pkg{effects} package: \fn{Effect.merMod} for models produced by the \fn{lmer} and \fn{glmer} functions in the \pkg{lme4} package; and \fn{Effect.svyglm} for models produced by the \fn{svyglm} function in the \pkg{survey} package \citep{Lumley04}. To see the code for these methods, enter the commands \code{getAnywhere("Effect.merMod")} and \code{getAnywhere("Effect.svyglm")}, after loading the \pkg{effects} package.}

The three basic \fn{Effect} methods collect information from the regression model of interest via a suitable method for the generic \fn{effects::effSources} function, and then use that information to compute effects and their standard errors.  The required information is summarized in Table~\ref{tab2}. 

\begin{table}
\caption{Values supplied by \fn{effSources} methods.  In the table, the regression model object is called \code{m}.  For functions cited in the \pkg{insight} package see \cite{insight19}.\label{tab2}}
\begin{center}
\begin{tabular}{|l|p{4.5in}|} \hline
Argument & Description \\ \hline
\code{type} & The type of the regression model: one of \code{"glm"} (the default if \code{type} isn't supplied), \code{"multinom"}, or \code{"polr"}. \\
\code{call} & The call that created the regression model, which is generally returned by either \verb+m$call+ or \verb+m@call+ or \code{insight::get\_call(m)}.   The call is used to find the usual \code{data} and \code{subset} arguments that \fn{Effect} needs to perform the computation. See the discussion of \fn{nlme:::gls} below for an example where the \code{call} must be modified.\\
formula &  The formula for the fixed-effects linear predictor, which is often returned by \code{stats::formula(m)} or \code{insight::find\_formula(m)\$conditional}.\\
\code{family} & Many \code{glm}-type models include a family, with an error distribution and a link function.  These are often returned by the default \code{stats::family(m)} or \code{insight::get\_family(m)}.\\
\code{coefficients} &  The vector of fixed-effect parameter estimates, often returned by \code{coef(m)}.  Alternatively \code{b <- insight::get\_parameters(m)} returns the coefficient estimates as a two-column matrix with parameter names in the first column, so \code{stats:setNames(b[,2], b[,1])} returns the estimates as a vector. For a \code{polr}-type model, coefficients  should return the regression coefficients excluding the thresholds.\\
\code{vcov} &  The estimated covariance matrix of the fixed-effect estimates, often given by \code{stats::vcov(m)} or \code{insight::get\_varcov(m)}. For a \code{polr}-type model, the covariance matrix should include both the regression coefficients and the thresholds, with the regression coefficients \emph{preceding} the thresholds.\\  \hline\\
\code{zeta} & The vector of estimated thresholds for a \code{polr}-type model, one fewer than the number of levels of the response. The default for a \code{polr}-type model is \code{zeta = m\$zeta}.\\
\code{method} & For a \code{polr}-type model, the name of a link supported by the \fn{MASS::polr} function: one of \code{"logistic"}, \code{"probit"}, \code{"loglog"}, \code{"cloglog"}, or \code{"cauchit"}. The default for a \code{polr}-type model is \code{method = "logistic"}.\\
\hline
\end{tabular}
\end{center}
\end{table}

The default \fn{effSources} method simply returns \code{NULL}, which corresponds to selecting all of the defaults in Table~\ref{tab2}. If that doesn't work, it usually suffices to provide a suitable \fn{effSources} method. We illustrate by a few examples.

\subsection{Examples}

The following examples, with the exception of the last, are drawn directly from the \pkg{effects} package.

\subsubsection{\texttt{glmmPQL()}}

Objects of class \code{"glmmPQL"}, produced by \fn{MASS::glmmPQL} do not respond to the generic \fn{family} function, but the name of the family can be obtained from the call; thus:
\begin{alltt}
effSources.glmmPQL <- function(mod) \{
  list(family = mod$family)
\}
\end{alltt}

\subsubsection{\texttt{gls()}}

The \code{weights} argument has different meaning for \fn{gls} in the \pkg{nlme} package \citep{nlme} and for the standard \R{} \fn{glm} function, and consequently the \code{call} must be modified to set \code{weights} to \code{NULL}:
\begin{alltt}
effSources.gls <- function(mod)\{
  cl <- mod$call
  cl$weights <- NULL
  list(call = cl)
\}
\end{alltt}

\subsubsection{\texttt{betareg()}}
The \code{betareg} function in the \pkg{betareg} package \citep{betareg} fits response data similar to a binomial regression but with beta errors. Adapting these models for use with \fn{Effect} is considerably more complex than the two previous examples:

\begin{alltt}
effSources.gls <- function(mod)\{
  coef <- mod$coefficients$mean
  vco <- vcov(mod)[1:length(coef), 1:length(coef)]
# betareg uses beta errors with mean link given in mod$link$mean.  
# Construct a family based on the binomial() family
  fam <- binomial(link=mod$link$mean)
# adjust the variance function to account for beta variance
  fam$variance <- function(mu){
    f0 <- function(mu, eta) (1-mu)*mu/(1+eta)
    do.call("f0", list(mu, mod$coefficient$precision))}
# adjust initialize
  fam$initialize <- expression({mustart <- y})
# collect arguments
  args <- list(
    call = mod$call,
    formula = formula(mod),
    family=fam,
    coefficients = coef,
    vcov = vco)
  args
\}
\end{alltt}

\subsubsection{\texttt{clm2()}}

The \fn{clm2} function in the \pkg{ordinal} package \citep{Christensen15} fits ordinal regression models, and so the aim is to create \code{polr}-type effects:

\begin{alltt}
effSources.clm2 <- function(mod)\{
  if (!requireNamespace("MASS", quietly=TRUE))
    stop("MASS package is required")
  polr.methods <- c("logistic", "probit", "loglog", 
                    "cloglog", "cauchit")
  method <- mod\$link
  if(!(method %in% polr.methods)) 
    stop("'link' must be a 'method' supported by polr; see help(polr)")
  if(is.null(mod\$Hessian))\{
     message("Re-fitting to get Hessian")
     mod <- update(mod, Hess=TRUE)
  \}
  if(mod\$threshold != "flexible") 
    stop("Effects only supports the flexible threshold")
  numTheta <- length(mod\$Theta)
  numBeta <- length(mod\$beta)
  or <- c( (numTheta+1):(numTheta + numBeta), 1:(numTheta))
  list(
    type = "polr",
    formula = mod\$call\$location,
    coefficients = mod\$beta,  
    zeta = mod\$Theta,
    method=method,
    vcov = as.matrix(vcov(mod)[or, or]))
\}
\end{alltt}

\subsubsection{\texttt{ivreg::ivreg()}}

Sometimes it doesn't suffice to define an appropriate \fn{effSources} method, but it is still possible to avoid writing a detailed \fn{Effect} method. We use the \fn{ivreg} function (for instrumental-variables regression) in the \pkg{ivreg} package \citep{ivreg} as an example; that package defines the following \fn{Effect.ivreg} method:

\begin{alltt}
Effect.ivreg <- function (focal.predictors, mod, ...) \{
  mod\$contrasts <- mod\$contrasts\$regressors
  NextMethod()
\}
\end{alltt}

\noindent Here it is sufficient to set the \code{contrasts} element of the model object to conform to the way it is defined in \class{lm} objects. That works because \class{ivreg} objects inherit from class \code{lm}, and thus \fn{Effect.lm} is called by \fn{NextMethod}.






\bibliography{functions-supported-by-effects}
\end{document}

%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Effect Displays with Partial Residuals}

\documentclass{article}

\usepackage{amsmath,amsfonts,amssymb}
\usepackage{natbib}
\bibliographystyle{abbrvnat}
\usepackage[margin=1in]{geometry}
\newcommand{\x}{\mathbf{x}}
\newcommand{\code}[1]{\normalfont\texttt{\hyphenchar\font45\relax #1}}
\newcommand{\E}{\mathrm{E}}
\newcommand{\tild}{\symbol{126}}
\newcommand{\Rtilde}{\,\raisebox{-.5ex}{\code{\tild{}}}\,}
\newcommand{\captilde}{\mbox{\protect\Rtilde}} % use in figure captions.
\newcommand{\Rmod}[2]{\code{#1 \raisebox{-.5ex}{\tild{}} #2}}
\newcommand{\Rmoda}[2]{\code{#1} &\code{\raisebox{-.5ex}{\tild{}} #2}}
\newcommand{\Rmodb}[2]{\code{#1 &\raisebox{-.5ex}{\tild{}}& #2}}
\newcommand{\C}{\mathbf{C}}
\newcommand{\betahat}{\widehat{\beta}}
\newcommand{\bbetahat}{\widehat{\boldsymbol{\beta}}}
\newcommand{\bbeta}{\boldsymbol{\beta}}
\newcommand{\xbf}{\x_{\backslash{}f}}
\newcommand{\hbf}{h_{\backslash{}f}}
\newcommand{\xtb}{\x_{2\backslash{}f}}
\newcommand{\xbfi}{\x_{\backslash{}f,i}}
\newcommand{\inter}[2]{\mbox{$#1$:$#2$}}
\newcommand{\cross}[2]{\mbox{$#1$\code{*}$#2$}}
\newcommand{\N}{\mathrm{N}}
\newcommand{\fn}{\textbf}
\newcommand{\R}{\proglang{R}}
\newcommand{\yx}{\widehat{y}(\x)}
\newcommand{\lvn}[1]{\mbox{$\log(\mbox{\texttt{#1}})$}}

\begin{document}

\title{Examples of Effect Displays with Partial Residuals\\ 
Using Contrived Regression Data}

\author{John Fox and Sanford Weisberg}

\date{2017-11-22}

\maketitle

<<include=FALSE>>=
library(knitr)
opts_chunk$set(
tidy=FALSE,fig.width=5,fig.height=5,cache=FALSE
)
@


<<echo=FALSE, results='hide', include=FALSE>>=
#options(continue="+    ", prompt="R> ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)
@

The examples developed in this vignette are meant to supplement \citet{FoxWeisberg18}.

\section{Basic Setup}
We will analyze contrived data generated according to the following setup:

\begin{itemize}

\item We sample $n = 5000$ observations from a trivariate distribution for predictors $x_1$, $x_2$, and $x_3$, with uniform margins on the interval $[-2, 2]$, and with a prespecified bivariate correlation $\rho$ between each pair of predictors. The method employed, described by \citet{Schumann15} and traceable to results reported by \citet{Pearson07}, produces predictors that are nearly linearly related. Using 5000 observations allows us to focus on essentially asymptotic behavior of partial residuals in effect plots while still being able to discern individual points in the resulting graphs.

\item We then generate the response $y$ according to the model
\begin{equation}
y = \beta_0 + h\left(\bbeta, \{x_1, x_2, x_3\}\right) + \varepsilon
\end{equation}
where $\varepsilon \Rtilde \N(0, 1.5^2)$. The regression function $h(\cdot)$ varies from example to example.

\end{itemize}

The following functions make it convenient to generate data according to this setup. These functions are more general than is strictly necessary so as to encourage further experimentation.

<<>>=
mvrunif <- function(n, R, min = 0, max = 1){
    # method (but not code) from E. Schumann,
    # "Generating Correlated Uniform Variates"
    # URL:
    # <http://comisef.wikidot.com/tutorial:correlateduniformvariates>
    # downloaded 2015-05-21
    if (!is.matrix(R) || nrow(R) != ncol(R) ||
    max(abs(R - t(R))) > sqrt(.Machine$double.eps))
    stop("R must be a square symmetric matrix")
    if (any(eigen(R, only.values = TRUE)$values <= 0))
    stop("R must be positive-definite")
    if (any(abs(R) - 1 > sqrt(.Machine$double.eps)))
    stop("R must be a correlation matrix")
    m <- nrow(R)
    R <- 2 * sin(pi * R / 6)
    X <- matrix(rnorm(n * m), n, m)
    X <- X %*% chol(R)
    X <- pnorm(X)
    min + X * (max - min)
}

gendata <- function(n = 5000, R, min = -2, max = 2, s = 1.5,
    model = expression(x1 + x2 + x3)){
    data <- mvrunif(n = n, min = min, max = max, R = R)
    colnames(data) <- c("x1", "x2", "x3")
    data <- as.data.frame(data)
    data$error <- s * rnorm(n)
    data$y <- with(data, eval(model) + error)
    data
}

R <- function(offdiag = 0, m = 3){
    R <- diag(1, m)
    R[lower.tri(R)] <- R[upper.tri(R)] <- offdiag
    R
}
@

\section{Unmodelled Interaction}

We begin with uncorrelated predictors and the true regression mean function $\E(y|\x) = x_1 + x_2x_3$, but fit the incorrect additive working model $y \Rtilde x_1 + x_2 + x_3$ to the data.
<<>>=
set.seed(682626)
Data.1 <- gendata(R = R(0), model = expression(x1 + x2 * x3))
round(cor(Data.1), 2)
summary(mod.1 <- lm(y ~ x1 + x2 + x3, data = Data.1))
@
For reproducibility, we set a known seed for the pseudo-random number generator; this seed was itself generated pseudo-randomly, and we reuse it in the examples reported below. As well, in this first example, but not for those below, we show the correlation matrix of the randomly generated data along with the fit of the working model to the data.

Effect plots with partial residuals corresponding to the terms in the working model are shown in Figure~\ref{fig-contrived-1a}:
<<fig-contrived-1a,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
library(effects)
plot(predictorEffects(mod.1, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     rows=1, cols=3)
@
In these graphs and, unless noted to the contrary, elsewhere in this vignette, the loess smooths are drawn with span 2/3. Because of the large number of points in the graphs, optional arguments to \code{plot} are specified to de-emphasize the partial residuals. To this end, the residuals are plotted as small points (\code{pch="."}) and in a translucent magenta color (\code{col="\#FF00FF80"}).

\begin{figure}[tbp]
  \caption{Effect displays with partial residuals for the individual predictors $x_1$, $x_2$, and $x_3$ in the incorrect model $y \captilde x_1 + x_2 + x_3$ fit to data generated with the mean function $\E(y|\x) = x_1 + x_2x_3$, with uncorrelated predictors.\label{fig-contrived-1a}}
  \centering
    \includegraphics[width=1\textwidth]{figure/fig-contrived-1a-1.pdf}
\end{figure}

The failure of the model is not apparent in these traditional partial residual plots, but it is clear in the term effect plot for $\{x_2, x_3\}$, corresponding to the unmodelled interaction \inter{x_2}{x_3}, and shown in the top panel of Figure~\ref{fig-contrived-1b}, generated using
<<fig-contrived-1b,include=TRUE, fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x2", "x3"), mod.1, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)))
@
Moreover, the effect plot in the bottom panel of the figure for  $\{x_1, x_2\}$, corresponding to a term \emph{not} in the true mean function, correctly indicates lack of interaction between these two predictors:
<<fig-contrived-1c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x1", "x2"), mod.1, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80"),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))
@

\begin{figure}[tbp]
  \caption{Term effect displays with partial residuals for $\{x_2, x_3 \}$, corresponding to the missing interaction \inter{x_2}{x_3}, and for $\{x_1, x_2 \}$, corresponding to an interaction not present in the model that generated the data.\label{fig-contrived-1b}}
  \centering
    \includegraphics[width=1\textwidth]{figure/fig-contrived-1b-1.pdf} \\
    \includegraphics[width=1\textwidth]{figure/fig-contrived-1c-1.pdf}
\end{figure}

As a partly contrasting example, we turn to a similar data set, generated with the same regression mean function but with moderately correlated predictors, where the pairwise predictor correlations are $\rho = 0.5$:
<<>>=
set.seed(682626)
Data.2 <- gendata(R = R(0.5), model = expression(x1 + x2 * x3))
mod.2 <- lm(y ~ x1 + x2 + x3, data = Data.2)
@
Graphs analogous to those from the preceding example appear in Figures~\ref{fig-contrived-2a} and \ref{fig-contrived-2b}:
<<fig-contrived-2a,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(predictorEffects(mod.2, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80",fig.show='hide'),
     axes=list(x=list(rotate=45)),
     rows=1, cols=3)
@
<<fig-contrived-2b,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x2", "x3"), mod.2, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)))
@
<<fig-contrived-2c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x1", "x2"), mod.2, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80",fig.show='hide'),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))
@
The predictor effect plots for  $x_2$ and $x_3$, and to a much lesser extent, for $x_1$, in the incorrect model in Figure~\ref{fig-contrived-2a} show apparent nonlinearity as a consequence of the unmodelled interaction and the correlations among the predictors. A similar phenomenon was noted in our analysis of the Canadian occupational prestige data in \citet[Section~4.2]{FoxWeisberg18}, where the unmodelled interaction between \code{type} and \code{income} induced nonlinearity in the partial relationship of \code{prestige} to \code{income}. The omitted interaction is clear in the effect plot for $\{x_2, x_3\}$, but also, to a lesser extent, contaminates the effect plot for $\{x_1,x_2\}$, which corresponds to an interaction that does not enter the model generating the data. These artifacts become more prominent if we increase the predictor correlations, say to $\rho = 0.9$ (as we invite the reader to do).

\begin{figure}[tbp]
  \caption{Predictor effect displays with partial residuals for the individual predictors $x_1$, $x_2$, and $x_3$ in the incorrect model $y \captilde x_1 + x_2 + x_3$ fit to data generated with the mean function $\E(y|\x) = x_1 + x_2x_3$, with moderately correlated predictors.\label{fig-contrived-2a}}
  \centering
    \includegraphics[width=1\textwidth]{figure/fig-contrived-2a-1.pdf}
\end{figure}
\begin{figure}[tbp]
  \caption{Term effect displays with partial residuals for $\{x_2, x_3 \}$, corresponding to the missing interaction \inter{x_2}{x_3}, and for $\{x_1, x_2 \}$, corresponding to an interaction not present in the model that generated the data.\label{fig-contrived-2b}}
  \centering
    \includegraphics[width=1\textwidth]{figure/fig-contrived-2b-1.pdf}\\
    \includegraphics[width=1\textwidth]{figure/fig-contrived-2c-1.pdf}
\end{figure}

\section{Unmodelled Nonlinearity}

We generate data as before, but from the true model $\E(y|\x) = x_1^2 + x_2 + x_3$, where the predictors are moderately correlated, with pairwise correlations $\rho = 0.5$, but fit the incorrect additive working model $y \Rtilde x_1 + x_2 + x_3$ to the data:
<<>>=
set.seed(682626)
Data.3 <- gendata(R = R(0.5), model = expression(x1^2 + x2 + x3))
mod.3 <- lm(y ~ x1 + x2 + x3, data = Data.3)
@

Effect plots with residuals for the predictors in the working model appear in Figure~\ref{fig-contrived-3a}. The unmodelled nonlinearity in the partial relationship of $y$ to $x_1$ is clear, but there is some contamination of the plots for $x_2$ and $x_3$. The contamination is much more dramatic if the correlations among the predictors are increased to, say, $\rho = 0.9$ (as the reader may verify).
<<fig-contrived-3a,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(predictorEffects(mod.3, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     rows=1, cols=3)
@

\begin{figure}[tbp]
  \caption{Predictor effect displays with partial residuals for the individual predictors $x_1$, $x_2$, and $x_3$  in the incorrect model $y \captilde x_1 + x_2 + x_3$ fit to data generated with the mean function $\E(y|\x) = x_1^2 + x_2 + x_3$, with moderately correlated predictors.\label{fig-contrived-3a}}
  \centering
    \includegraphics[width=1\textwidth]{figure/fig-contrived-3a-1.pdf}
\end{figure}

Effect plots for $\{x_1, x_2 \}$ and $\{x_2, x_3 \}$ are shown in Figure~\ref{fig-contrived-3b}:
<<fig-contrived-3b,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x2", "x3"), mod.3, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)))
@
<<fig-contrived-3c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x1", "x2"), mod.3, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80"),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))
@
Neither of these graphs corresponds to a term in the model generating the data nor in the working model, and the effect plots largely confirm the absence of \inter{x_1}{x_2} and \inter{x_2}{x_3} interactions, along with the nonlinearity of the partial effect of $x_1$, apparent in the top panel.

\begin{figure}[tbp]
  \caption{Term effect displays with partial residuals for $\{x_1, x_2 \}$ and for $\{x_2, x_3 \}$, neither of which corresponds to an interaction in the model generating the data.\label{fig-contrived-3b}}
  \centering
    \includegraphics[width=1\textwidth]{figure/fig-contrived-3c-1.pdf} \\
    \includegraphics[width=1\textwidth]{figure/fig-contrived-3b-1.pdf}
\end{figure}

\section{Simultaneous Unmodelled Nonlinearity and Interaction}

This last example also appears in \citet[Section~4.3]{FoxWeisberg18}. We consider a true model that combines nonlinearity and interaction, $\E(y|\x) = x_1^2 + x_2 x_3$; the predictors are moderately correlated, with $\rho = 0.5$. We then fit the incorrect working model $y \Rtilde x_1 + x_2 + x_3$ to the data, producing the predictor effect displays with partial residuals in Figure~\ref{fig-contrived-4a}, for the predictors $x_1$, $x_2$, and $x_3$, which appear additively in the working model, and the term effect displays in Figure~\ref{fig-contrived-4b} for $\{x_2, x_3 \}$ and $\{x_1, x_2 \}$, corresponding respectively to the incorrectly excluded \inter{x_2}{x_3} term and the correctly excluded \inter{x_1}{x_2} interaction. 

<<>>=
set.seed(682626)
Data.4 <- gendata(R = R(0.5), model = expression(x1^2 + x2 * x3))
mod.4 <- lm(y ~ x1 + x2 + x3, data = Data.4)
@
<<fig-contrived-4a,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(predictorEffects(mod.4, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     rows=1, cols=3)
@
<<fig-contrived-4b,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x2", "x3"), mod.4, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
      axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)))
@
<<fig-contrived-4c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x1", "x2"), mod.4, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80"),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))
@
The nonlinearity in the partial relationship of $y$ to $x_1$ shows up clearly. The nonlinearity apparent in the plots for $x_2$ and $x_3$ is partly due to contamination with $x_1$, but largely to the unmodelled interaction between $x_2$ and $x_3$, coupled with the correlation between these predictors. The plot corresponding to the missing \inter{x_2}{x_3} term (in the top panel of Figure~\ref{fig-contrived-4b}) does a good job of detecting the unmodelled interaction, and curvature in this plot is slight. The plot for the \inter{x_1}{x_2} term (in the bottom panel of Figure~\ref{fig-contrived-4b}), a term neither in the true model nor in the working model, primarily reveals the unmodelled nonlinearity in the partial relationship of $y$ to $x_1$.

\begin{figure}[tbp]
  \caption{Effect displays with partial residuals for the predictors $x_1$, $x_2$, and $x_3$ in the incorrect model $y \captilde x_1 + x_2 + x_3$ fit to data generated with the mean function $\E(y|\x) = x_1^2 + x_2x_3$, with moderately correlated predictors.\label{fig-contrived-4a}}
  \centering
    \includegraphics[width=1\textwidth]{figure/fig-contrived-4a-1.pdf}
\end{figure}
\begin{figure}[tbp]
  \caption{Term effect displays with partial residuals for $\{x_2, x_3 \}$ (top) and for $\{x_1, x_2 \}$ (bottom), the first of which corresponds to the missing \inter{x_2}{x_3} interaction in the model generating the data.\label{fig-contrived-4b}}
  \centering
    \includegraphics[width=1\textwidth]{figure/fig-contrived-4b-1.pdf} \\
    \includegraphics[width=1\textwidth]{figure/fig-contrived-4c-1.pdf}
\end{figure}

If we fit the correct model, $y \Rtilde{} x_1^2 + x_2*x_3$, to the data, we obtain the plots shown in Figure~\ref{fig-contrived-5}. As theory suggests, the partial residuals in these effect displays validate the model, supporting the exclusion of the \inter{x_1}{x_2} interaction, the linear-by-linear interaction between $x_1$ and $x_2$, and the quadratic partial relationship of $y$ to $x_1$.

<<fig-contrived-5a,include=TRUE,fig.width=5,fig.height=4,fig.show='hide'>>=
mod.5 <- lm(y ~ poly(x1, 2) + x2*x3, data=Data.4)
plot(Effect("x1", mod.5, partial.residuals=TRUE),
     partial.residual=list(pch=".", col="#FF00FF80", span=0.2))
@
<<fig-contrived-5b,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x2", "x3"), mod.5, partial.residuals = TRUE),
     partial.residual=list(pch=".", col="#FF00FF80"),
     axes=list(x=list(rotate=45)),
     lattice=list(layout=c(4, 1)), span=0.5)
@
<<fig-contrived-5c,include=TRUE,fig.width=12,fig.height=4,fig.show='hide'>>=
plot(Effect(c("x1", "x2"), mod.5, partial.residuals = TRUE),
    partial.residual=list(pch=".", col="#FF00FF80", span=0.35),
    axes=list(x=list(rotate=45)),
    lattice=list(layout=c(4, 1)))
@

\noindent In these graphs, we adjust the span of the loess smoother to the approximately smallest value that produces a smooth fit to the partial residuals in each case.

\begin{figure}[tbp]
  \caption{Effect displays with partial residuals for $x_1$ and  $\{x_2, x_3 \}$, which correspond to terms in the model generating \emph{and} fitted to the data, $y \captilde x_1^2 + x_2 * x_3$, and for $\{x_1, x_2 \}$, which corresponds to an interaction that is not in the model.\label{fig-contrived-5}}
  \centering
    \includegraphics[width=0.45\textwidth]{figure/fig-contrived-5a-1.pdf} \\
    \includegraphics[width=1\textwidth]{figure/fig-contrived-5b-1.pdf} \\
    \includegraphics[width=1\textwidth]{figure/fig-contrived-5c-1.pdf}
\end{figure}

\bibliography{partial-residuals}

\end{document}

%\VignetteEngine{knitr::knitr}
%\VignetteIndexEntry{Defining Effect Methods for Other Models}
%% vignette index specifications need to be *after* \documentclass{}
%%\VignetteEngine{knitr::knitr}
%%\VignetteIndexEntry{Computing Effects for Other Statistical Models}
%%\VignettePackage{effects}

\documentclass{article}


\usepackage[utf8]{inputenc}
\usepackage{graphicx}
\usepackage[american]{babel}
\newcommand{\R}{{\sf R}}
\usepackage{url}
\usepackage{hyperref}
\usepackage{alltt}
\usepackage{fancyvrb}
\usepackage{natbib}
\usepackage{amsmath}
\VerbatimFootnotes
\bibliographystyle{chicago}
%\usepackage{amsmath,amsfonts,amssymb}
%\usepackage{natbib}
%\bibliographystyle{abbrvnat}
%\usepackage[margin=1in]{geometry}

\newcommand{\x}{\mathbf{x}}
\newcommand{\code}[1]{\normalfont\texttt{\hyphenchar\font45\relax #1}}
\newcommand{\E}{\mathrm{E}}
\newcommand{\tild}{\symbol{126}}
\newcommand{\Rtilde}{\,\raisebox{-.5ex}{\code{\tild{}}}\,}
\newcommand{\captilde}{\mbox{\protect\Rtilde}} % use in figure captions.
\newcommand{\Rmod}[2]{\code{#1 \raisebox{-.5ex}{\tild{}} #2}}
\newcommand{\Rmoda}[2]{\code{#1} &\code{\raisebox{-.5ex}{\tild{}} #2}}
\newcommand{\Rmodb}[2]{\code{#1 &\raisebox{-.5ex}{\tild{}}& #2}}
\newcommand{\C}{\mathbf{C}}
\newcommand{\betahat}{\widehat{\beta}}
\newcommand{\bbetahat}{\widehat{\boldsymbol{\beta}}}
\newcommand{\bbeta}{\boldsymbol{\beta}}
\newcommand{\xbf}{\x_{\backslash{}f}}
\newcommand{\hbf}{h_{\backslash{}f}}
\newcommand{\xtb}{\x_{2\backslash{}f}}
\newcommand{\xbfi}{\x_{\backslash{}f,i}}
\newcommand{\inter}[2]{\mbox{$#1$:$#2$}}
\newcommand{\cross}[2]{\mbox{$#1$\code{*}$#2$}}
\newcommand{\N}{\mathrm{N}}
\newcommand{\fn}{\textbf}
\newcommand{\pkg}[1]{\code{#1}}
\newcommand{\proglang}[1]{\code{#1}}
\newcommand{\yx}{\widehat{y}(\x)}
\newcommand{\lvn}[1]{\mbox{$\log(\mbox{\texttt{#1}})$}}

\begin{document}

\title{Defining Effect Methods for Other Models}

\author{John Fox and Sanford Weisberg}

\date{\today}

\maketitle

<<setopts,echo=FALSE>>=
library("knitr")
opts_chunk$set(fig.width=5,fig.height=5,tidy=TRUE,
               out.width="0.8\\textwidth",echo=TRUE)
options(prompt=" ")
@ 


<<echo=FALSE, results='hide', include=FALSE>>=
#options(continue="+    ", prompt="R> ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)
@

<<include=FALSE>>=
library(knitr)
opts_chunk$set(
tidy=FALSE,fig.width=5,fig.height=5,cache=FALSE,comment=NA, prompt=TRUE
)
render_sweave()
@


<<echo=FALSE, results='hide', include=FALSE>>=
options(continue="    ", prompt=" ", width=76)
options(show.signif.stars=FALSE)
options(scipen=3)

@

The \code{effects} package in \R{} is designed primarily to draw graphs that visualize a fitted response surface of a fitted model in problems with a linear predictor.  Many modeling paradigms that can be fit with base \R{} or contributed packages fit into this framework, including methods for linear, multivariate linear, and generalized linear models fit by the standard \code{lm} and \code{glm} functions and by the \code{svyglm} function in the \code{survey} package \citep{Lumley04}; linear models fit by generalized least squares using the \code{gls} function in the \pkg{nlme} package \citep{Pinheiro16}; multinomial regression models fit by \code{multinom} in the \pkg{nnet} package \citep{VenablesRipley02};  ordinal regression models using \code{polr} from the \pkg{MASS} package \citep{VenablesRipley02} and \code{clm} and \code{clm2} from the \pkg{ordinal} package \citep{Christensen15}; linear and generalized linear mixed models using the \code{lme} function in the \pkg{nlme} package \citep{Pinheiro16} and the \code{lmer} and \code{glmer} functions in the \pkg{lme4} package \citep{Bates15}; and latent class models fit by \code{poLCA} in the \pkg{poLCA} package \citep{Linzer11}. This is hardly an exhaustive list of fitting methods that are based on a linear predictor, and we have been asked from time to time to write functions to use \code{effects} with this other fitting methods.  The mechanism for this is fairly simple.  This vignette assumes you are familiar with \R{}'s S3 methods.

The default \code{Effect.default} may work with some modeling functions, as would objects of the class \code{gls} that we describe below in Section~\ref{gls}, but as illustrated in later sections you may need to modify some of the arguments that are sent to \code{Effect.default}.  .

The \code{effect} package has five functions that create the information needed for drawing effects plots, \code{Effect}, \code{allEffects}, \code{effect} and \code{predictorEffect} and \code{predictorEffects}.  To add new modeling to the package only a new \code{Effect} needs to be written; the package will take care of all the other functions.

\section{Using \code{effects} with Other Modeling Methods, with Generalized Least Squares in the \pkg{nlme} package as an Example}\label{gls}
Applying \code{effects} to other than \code{lm} and \code{glm} objects may require writing an method for the \code{Effect} generic function for that type of model object.  For example, the \code{gls} function in the \code{nlme} package \citep{nlme} fits linear models via generalized least squares.  A call to \code{gls} creates an object of class \code{gls}.  The following function \code{Effect.gls} provides a method for \code{gls} objects by finding the information needed to draw effects plots from \code{gls} objects:

<<eval=FALSE>>=
Effect.gls <- function(focal.predictors, mod, ...){
  cl <- mod$call
  cl$weights <- NULL
  args <- list(
    type = "glm",
    call = cl,
    formula = formula(mod),
    family = NULL,
    coefficients = coef(mod),
    vcov = as.matrix(vcov(mod)),
    method=NULL)
  Effect.default(focal.predictors, mod, ..., sources=args)
}
@
The \code{Effect.gls} function has three required arguments, \code{focal.predictors} and \code{mod} that match the first two arguments of \code{Effect.default}, and  \code{...} that matches any other arguments you would like to pass to \code{Effect.default}; see \code{help(Effect)} for a list of these arguments.

The body of the function simply harvests the needed information from the \code{mod} object, and stores them in a list of named elements called \code{sources}.  The \code{sources} list is then passed as a named argument to the default \code{Effect} method.

The named elements in \code{sources} include:
\begin{description}
\item[\texttt{type}] The \code{effects} package has three basic modeling functions:  \code{type = "glm"}, the default, is used for functions with a univariate response and a linear predictor and possibly a link function.  This class includes linear models, generalized linear models, robust regression, generalized least squares fitting, linear and generalized linear mixed effects models, and many others. The \code{type = "polr"} is used for ordinal regression models, as in the \code{polr} function in the \code{MASS} package, and similar methods described below in Section~\ref{polr}.  The The \code{type = "multinom"} for multinomial log-linear models as fit by the \code{multinom} function in \code{nnet}, and to polytomous latent class models created with the \code{poLCA} function in the \pkg{poLCA} package.  The default is \code{type = "glm"}.

\item[\code{call}] The \code{Effect.default} method may use the call to set additional arguments that it needs.  For \code{type="glm"}, for example, these arguments are \code{formula}, \code{data}, \code{contrasts},  \code{subset},
\code{family}, \code{weights}, and \code{offset}, although only the \code{formula} argument is required.  The \code{gls} function includes an optional \code{weights} argument that is used differently from the \code{weights} argument for a generalized linear model and is not needed for computing effects or predictor effects plots.  In the function shown above the call is modified by setting \code{weights=NULL}.  

The default for \code{call} is \code{mod\$call} for S3 objects and \code{mod@call} for S4 objects.

\item[\code{formula}] In most cases the formula for the linear predictor is returned by \code{formula(mod)}, the default, but if this is not the case the value of this argument should be the value of the formula for fixed effects.
\item[\code{family}] The default is \code{family=NULL}.  This argument is required for GLM-like models that include a \code{family} that specifies both an error distribution and a link function only if \code{family=family(mod)} is not appropriate.  See the \code{betareg} example in Section~\ref{betareg} below for an example that includes a user-selected link function, but a fixed error distribution.
\item[\code{coefficients}] In many cases the (fixed-effect) coefficient estimates are returned by \code{coef(mod)}, the default, but if this is not the case then the value of this argument should be the estimates of the coefficients in the linear predictor. The functions in the \pkg{effects} package do not use estimates of random effects.
\item[\code{zeta}] Ordinal regression models return both a set of regression coefficients and also a set of \emph{threshholds}.  In the \code{polr} function the regression coefficients and the threshholds are stored in separate vectors, but in other ordinal regression programs, such as \code{clm} in the \code{ordinal} package they are stored as a single vector.  See Section~\ref{polr} for an example of the use of this argument for specifying the values of the threshholds.  
\item[\code{vcov}] In many cases the estimated covariance matrix of the (fixed-effect) coefficient estimates is  returned by \code{vcov(mod)}, the default, but if this is not the case then the value of this argument should be the estimated covariance matrix of the (fixed-effect) coefficient estimates in the linear predictor.
\item[method] This argument is used only for methods that use effects graphics based on the \code{polr} function, where the argument \code{method} is the name of a link function; see \code{help(polr)} for a list of the accepted links, and see Section~\ref{clm} below for an example.
\end{description}
The only non-default argument in \code{sources} in \code{Effect.gls} is the modification of the \code{call} to omit weights in the call to \code{gls}.  Had this change not been needed, there would be no need to have written the \code{Effect.gls} method, as the default method would have worked. 

<<fig.height=4,fig.width=8>>=
library(effects)
require(nlme)
g <- gls(Employed ~ GNP + Population,
         correlation=corAR1(form= ~ Year), data=longley)
plot(predictorEffects(g))
@

\section{Mixed Effects with \code{lme} (\pkg{nlme} package)}
The \code{lme} function in the \pkg{nlme} package \citep{nlme} fits linear mixed models.  The required function for fitted objects from this function is included in the \pkg{effects} package.  It is given by
<<>>=
print(Effect.lme)
@
The \code{formula}, \code{coefficients} and \code{vcov} arguments are set to non-default values.  The other arguments are automatically set to default values, with \code{call} set explicitly and other arguments set implicitly.  

<<>>=
data(Orthodont, package="nlme")
m1 <- nlme::lme(distance ~ age + Sex, data=Orthodont, 
                random= ~ 1 | Subject)
as.data.frame(Effect("age", m1))
@

\section{Mixed Effects with the \code{lmer} (\code{lme4} package)}
The \code{lme4} package \citep{Bates15} fits linear and generalized linear mixed effects models with the \code{lmer} and \code{glmer} functions, respectively.  The same \code{Effect} function can be used for \code{lmer} and \code{glmer} models.  

The following method is a little more complicated because it contains an additional argument \code{KR} to determine if the Kenward-Roger coefficient covariance matrix is to be used to compute effect standard errors.  The default is \code{FALSE} because the computation is very slow.  If \code{KR = TRUE}, the function also checks if the \pkg{pbkrtest} package is present.

<<>>=
print(Effect.merMod)
@
Because \code{lmer} is an S4 object, the default for \code{call} is \code{mod@call}, and this argument would have been set automatically had we not included it in the above method.  The fixed-effect estimates for an object created by a call to \code{lmer} or \code{glimer} are not returned by \code{coef(mod)}, so the value of \code{coefficients} is the value returned by \code{lme4::fixef(mod)}.  The \code{vcov} estimate contains its estimated variance covariance matrix of the fixed effects.  The Kenward-Roger method is used to estimate the covariance matrix for linear models if the additional argument \code{KR=TRUE}.  The default is \code{KR=FALSE} because The Kenward-Roger estimate requires a long computation; see \code{help(Effect)}. 

The \code{formula} for a mixed-effects model in the \code{lme4} package specifies linear predictors for both the mean function and the variance functions, specified by, for example  \code{(1 + age | Subject)}.  The \code{effects} code will automatically remove any terms like these in any formula, as the effects package only displays the mean function.

<<fig.height=4,fig.width=8>>=
fm2 <- lme4::lmer(distance ~ age + Sex + (1 |Subject), data
                  = Orthodont)
plot(allEffects(fm2))
@

<<>>=
data(cbpp, package="lme4")
gm1 <- lme4::glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
              data = cbpp, family = binomial)
as.data.frame(predictorEffect("period", gm1))
@

\section{Robust Linear Mixed Models (\pkg{robustlmm} package)}
The \code{rlmer} function in the \code{robustlmm} package \citep{koller16} fits linear mixed models with a robust estimation method.  As \code{rlmer} closely parallels the \code{lmer} function, an object created by \code{rlmer} is easily used with \code{effects}:
<<>>=
print(Effect.rlmerMod)
@

<<eval=FALSE,fig.height=4, fig.width=8>>=
require(lme4)
fm3 <- robustlmm::rlmer(distance ~ age * Sex + (1 |Subject), 
                        data = Orthodont)
plot(predictorEffects(fm3))
@

\section{Beta Regression}\label{betareg}
The \code{betareg} function in the \code{betareg} package \citep{betareg} fits regressions with a link function but with Beta distributed errors.
<<>>=
print(Effect.betareg)
@
Beta regression has a response $y \in [0,1]$, with the connection between the mean $\mu$ of the Beta and a set for predictors $\x$ through a link function $\x'\bbeta = g(\mu)$.  The variance function for the beta is $\mathrm{var}(y) = \mu(1-\mu)/(1+\phi)$, for a precision parameter $\phi$ estimated by \code{betareg}.

The call to \code{betareg} does not have a family argument, although it does have a link stored in \code{mod\$link\$mean}.  For use with \code{Effect.default}, the method above creates a family from the binomial family generator.  It then adjusts this family by changing from binomial variance to the variance for the beta distribution. Since the \code{glm} function expects a variance that is a function of only one parameter, we fix the value of the precision $\phi$ at its estimator from the \code{betareg} fit.  We need to replace the \code{initialize} method in the family to one appropriate for $y \in [0,1]$.  %Finally, although the \code{aic} function is not used for computing effects, it is accessed by the call to \code{glm}.  The \code{aic} function for the binomial depends on named parameters not present in the beta regression, and so we substitute a dummy function for binomial version.

<<echo=FALSE,results='hide', include=FALSE>>=
require(lme4)
@
<<fig.height=4,fig.width=8,cache=FALSE>>=
require(betareg)
require(lme4)
data("GasolineYield", package = "betareg")
gy_logit <- betareg(yield ~ batch + temp, data = GasolineYield)
summary(gy_logit)
plot(predictorEffects(gy_logit))
@

\section{Ordinal Models (\pkg{ordinal} package)}\label{polr}
Proportional odds logit and probit regression models fit with the \code{polr} function in the \code{MASS} package \citep{VenablesRipley02} are supported in the \code{effects} package.  The \code{ordinal} package, \citep{Christensen15} contains three functions that are very similar to \code{polr}.  The \code{clm} and \code{clm2} functions allow more link functions and a number of other generalizations.  The \code{clmm} function allows including random effects.

\subsection{\code{clm}}\label{clm}
<<>>=
print(Effect.clm)
@

This method first checks that the \code{MASS} package is available.  The \code{clm} function orders the parameters in the order (threshold parameters, linear predictor parameters), so the next few lines identify the elements of  \code{vcov} that are needed by \code{Effects}.  Since the \code{polr} function does not allow thresholds other than \code{flexible}, we don't allow them either. The \code{zeta} argument supplies the estimated threshholds, which are called \code{zeta} in \code{polr}, and \code{Alpha} in \code{clm}. The \code{polr} argument \code{method} is equivalent to the \code{clm} argument \code{link}, except that the \code{clm} link \code{"logit"} is equivalent to the \code{polr} method \code{"logistic"}.  

<<echo=FALSE,results='hide', include=FALSE>>=
require(ordinal)
require(MASS)
@
<<fig.height=6,fig.width=6>>=
require(ordinal)
require(MASS)
mod.wvs1 <- clm(poverty ~ gender + religion + degree + country*poly(age,3),
    data=WVS)
plot(Effect(c("country", "age"), mod.wvs1), 
     lines=list(multiline=TRUE), layout=c(2, 2))
@

\subsection{\code{clm2}}
Although the fitted madels are similar, syntax for \code{clm2} is not the same as \code{clm}, so a separate method is required.

<<>>=
print(Effect.clm2)
@

<<fig.height=6,fig.width=8>>=
v2 <- clm2(poverty ~ gender + religion + degree + country*poly(age,3),data=WVS)
plot(emod2 <- Effect(c("country", "age"), v2), 
     lines=list(multiline=TRUE), layout=c(2,2))
@

\subsection{\code{clmm}}
This function allows for random effects in an ordinal model.
<<>>=
print(Effect.clmm)
@
The first few lines of the method check for the presence of the \code{MASS} package that is needed to use \code{polr}, makes sure the link used is supported by \code{polr}, and requires that the argument \code{threshold} has its default value.  The \code{polr} and \code{clmm} functions store the fixed effects estimates of regression and threshold coefficents in different orders, so the next few lines rearrange the variance matrix to match the order that \code{polr} uses.

<<fig.height=4,fig.width=4,cache=FALSE>>=
require(ordinal)
require(MASS)
mm1 <- clmm(SURENESS ~ PROD + (1|RESP) + (1|RESP:PROD), 
        data = soup, link = "logit", threshold = "flexible")
plot(Effect("PROD", mm1),lines=list(multiline=TRUE))
@

\subsection{Others}
The \code{poLCA} function in the \code{poLCA} package \citep{Linzer11} fits polytomous variable latent class models, which uses the multinomial effects plots.

The \code{svyglm} function in the \code{survey} package \citep{Lumley04, Lumley16} fits generalized linear models using survey weights.

The \code{lm} function can also be used to create a multivariate linear model.  The \code{Effect.mlm} function, with slightly different syntax, will drow effects plots for these models, with separate plots of each response.

<<fig.height=6,fig.width=6>>=
data(Baumann, package="carData")
b1 <- lm(cbind(post.test.1, post.test.2, post.test.3) ~ group + 
    pretest.1 + pretest.2, data = Baumann)
plot(Effect("group", b1))
@




\bibliography{adding-Effect-methods}
\end{document}

\name{Prestige}
\alias{Prestige}
\docType{data}

\title{Prestige of Canadian Occupations}

\description{
The \code{Prestige} data frame has 102 rows and 6 columns.
The observations are occupations.
}

\format{
  This data frame contains the following columns:
  \describe{
    \item{education}{
      Average education (years) of occupational incumbents, in 1971. 
      }
    \item{income}{
      Average income (dollars) of incumbents, 1971.
      }
    \item{women}{
      Percentage of incumbents who are women, 1971.
      }
    \item{prestige}{
      Pineo-Porter prestige score for occupation, from a social survey
      conducted in the mid-1960s.
      }
    \item{census}{
      Canadian Census occupational code.
      }
    \item{type}{
      Type of occupation.
      A factor with levels (note: out of order):
      \code{bc}, Blue Collar;
      \code{prof}, Professional, Managerial, and Technical;
      \code{wc}, White Collar. 
      }
    }
  }

\source{
 Canada (1971)
 \emph{Census of Canada}. Vol. 3, Part 6. Statistics Canada [pp. 19-1--19-21].
 
 Personal communication from B. Blishen, W. Carroll, and
 C. Moore, Departments of Sociology, York University and
 University of Victoria.
}

\references{
  Fox, J. (1997)
  \emph{Applied Regression, Linear Models, and Related Methods.} Sage.
}


\usage{
Prestige
}

\keyword{datasets}

\name{effectsHexsticker}
\alias{effectsHexsticker}
\title{
View the Official Hex Sticker for the effects Package}
\description{
Open the official hex sticker for the effects package in your browser}
\usage{
effectsHexsticker()
}

\value{
Used for its side effect of openning the hex sticker for the effects package in your browser.
}

\author{
John Fox \email{jfox@mcmaster.ca}
}

\examples{
\dontrun{
effectsHexsticker()
}
}

\keyword{misc}

\name{Cowles}
\alias{Cowles}
\docType{data}
\title{Cowles and Davis's Data on Volunteering}
\usage{Cowles}
\description{
The \code{Cowles} data frame has 1421 rows and 4 columns.
These data come from a study of the personality determinants
of volunteering for psychological research.
}
\format{
  This data frame contains the following columns:
  \describe{
    \item{neuroticism}{scale from Eysenck personality inventory.}
    \item{extraversion}{scale from Eysenck personality inventory.}
    \item{sex}{a factor with levels: \code{female}; \code{male.}}
    \item{volunteer}{volunteeing, a factor with levels: \code{no}; \code{yes}.}
  }
}

\source{
 Cowles, M. and C. Davis (1987)
 The subject matter of psychology: Volunteers.
 \emph{British Journal of Social Psychology} \bold{26}, 97--102.
}
 
\examples{
summary(Cowles)
}

\keyword{datasets}

\name{EffectMethods}
\alias{Effect.default}
\alias{Effect.merMod}
\alias{Effect.rlmerMod}
\alias{Effect.lme}
\alias{Effect.gls}
\alias{Effect.poLCA}
\alias{Effect.clm2}
\alias{Effect.clm}
\alias{Effect.clmm}
\alias{Effect.mlm}
\alias{Effect.betareg}

\title{Functions For Constructing Effect Displays for Many Modeling Paradigms}
\description{The \code{Effect}, \code{effect} and \code{predictorEffects} methods are used to draw effects plots to visualize a fitted regression surface.  These plots can be drawn at least in principle for any model that uses a linear predictor.  Methods for modeling paradigms than the basic \code{lm}, \code{glm}, \code{multinom} and \code{polr} methods are documented here. To add methods for new classes of statistical models, see the vignette \href{../doc/adding-Effect-methods.pdf}{Defining Effect Methods for Other Models}.
}

\usage{

\method{Effect}{default}(focal.predictors, mod,  ...,
    sources=NULL)

\method{Effect}{gls}(focal.predictors, mod, ...)

\method{Effect}{clm2}(focal.predictors, mod, ...)

\method{Effect}{clmm}(focal.predictors, mod, ...)

\method{Effect}{clm}(focal.predictors, mod, ...)

\method{Effect}{merMod}(focal.predictors, mod, ...,
    KR=FALSE)

\method{Effect}{rlmerMod}(focal.predictors, mod, ...)

\method{Effect}{lme}(focal.predictors, mod, ...)

\method{Effect}{poLCA}(focal.predictors, mod, ...)

\method{Effect}{mlm}(focal.predictors, mod, response, ...)

\method{Effect}{betareg}(focal.predictors, mod,  ...)
}

\arguments{
  \item{focal.predictors}{a character vector of one or more predictors in the model in any order.}
  \item{mod}{a fitted model object of the appropriate class. }
  \item{...}{additional arguments passed to other \code{Effect}.  See \code{\link{Effect}} for all the arguments included.}
  \item{response}{for an \code{"mlm"} object, a vector containing the name(s) or indices of one or more response variable(s). The default is to use all responses in the model.}
  \item{sources}{This argument appears only in the default method for \code{Effect}, and allows the user to draw effects plots for fitting methods for which there are not existing methods in the effects package.  Seven arguments are provided:

    \describe{
  	  \item{type}{the default is \code{"glm"}, which assumes the modeling method shares characteristics with a generalized linear model, including a univariate response, a linear predictor, and possibly a error family and link function.}

      \item{call}{For S3 objects, the default is \code{ojbect$call}, returning the call that created the object.  This is used to harvest standard arguments like \code{data}, \code{subset} and \code{family}.}

      \item{formula}{the formula for the linear predictor, defaulting to \code{formula(object)}.}

      \item{family}{if the model object includes an error family, but it is not returned by \code{family(object)}, specify the family with this argument; otherwise it can be ignored.}

      \item{method}{For ordinal response models only, see the \code{method} argument to \code{\link{polr}}.}

      \item{coefficients}{The estimates of the coefficents in the linear predictor, with default \code{coef(object)}.}

      \item{vcov}{the estimated variance covariance materix to be used in computing errors in the effects plots; default is code{vcov(object)}.}
    }
  }
\item{KR}{if \code{TRUE} and the \pkg{pbkrtest} package is installed,
    use the Kenward-Roger coefficient covariance
    matrix to compute effect standard errors for linear mixed models fit with \code{\link[lme4]{lmer}}
    in the \pkg{lme4} package.  The default is \code{FALSE} because the computation can be
    very slow.}
  }

\details{
  Most of these methods simply call the \code{Effect.default} method with the appropriate values in the arguement \code{sources}.  See the vignettte Effect Methods in the vignettes for the effects package.  All the iteresting work is done by the methods described in \code{\link{Effect}}.}

\value{See \code{\link{Effect}}}


\references{
    The \href{../doc/adding-Effect-methods.pdf}{Defining Effect Methods for Other Models} vignette.
    }



\author{John Fox \email{jfox@mcmaster.ca}, Sanford Weisberg \email{sandy@umn.edu}}


\seealso{\code{\link{Effect}} and the links therein.}

\examples{
\dontrun{
# lme
require(nlme)
fm1 <- lme(distance ~ age + Sex, data = Orthodont, random = ~ 1)
plot(predictorEffects(fm1))

# gls
library(nlme)
g <- gls(Employed ~ GNP + Population,
         correlation=corAR1(form= ~ Year), data=longley)
print(predictorEffects(g))

# lmer  uses method Effect.lmerMod
if("package:nlme" %in% search()) detach(package:nlme)
require(lme4)
data("Orthodont", package="nlme")
fm2 <- lmer(distance ~ age + Sex + (1 |Subject), data = Orthodont)
plot(allEffects(fm2))

# glmer  uses method Effect.lmerMod
require(lme4)
gm1 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
              data = cbpp, family = binomial)
as.data.frame(predictorEffect("period", gm1))

# rlmer  uses method Effect.rlmerMod
require(lme4)
fm3 <- robustlmm::rlmer(distance ~ age + Sex + (1 |Subject), data = Orthodont)
plot(effect("age:Sex", fm3))
plot(predictorEffects(fm3, ~ age + Sex))

# betareg from the betareg package
library(betareg)
library(lme4)
data("GasolineYield", package = "betareg")
gy_logit <- betareg(yield ~ batch + temp, data = GasolineYield)
summary(gy_logit)
Effect("batch", gy_logit)
predictorEffects(gy_logit)

# clm in ordinal
require(ordinal)
require(MASS)
mod.wvs1 <- clm(poverty ~ gender + religion + degree + country*poly(age,3),
    data=WVS)
plot(Effect(c("country", "age"), mod.wvs1), 
     lines=list(multiline=TRUE), layout=c(2, 2))

# clm2
require(ordinal)
require(MASS)
v2 <- clm2(poverty ~ gender + religion + degree + country*poly(age,3),data=WVS)
plot(emod2 <- Effect(c("country", "age"), v2))

# clmm
require(ordinal)
require(MASS)
mm1 <- clmm(SURENESS ~ PROD + (1|RESP) + (1|RESP:PROD), 
        data = soup, link = "logit", threshold = "flexible")
plot(Effect("PROD", mm1),lines=list(multiline=TRUE))

# poLCA
library(poLCA)
data(election)
f2a <- cbind(MORALG,CARESG,KNOWG,LEADG,DISHONG,INTELG,
             MORALB,CARESB,KNOWB,LEADB,DISHONB,INTELB)~PARTY
nes2a <- poLCA(f2a,election,nclass=3,nrep=5)    # log-likelihood: -16222.32
allEffects(nes2a)

# multivariate linear model
data(Baumann, package="carData")
b1 <- lm(cbind(post.test.1, post.test.2, post.test.3) ~ group + 
    pretest.1 + pretest.2, data = Baumann))
plot(Effect("group", b1)

}
}

\keyword{hplot}
\keyword{models}

\name{BEPS}
\alias{BEPS}
\docType{data}
\title{British Election Panel Study}
\description{
  These data are drawn from the 1997-2001 British Election Panel Study (BEPS).
}
\usage{BEPS}
\format{
  A data frame with 1525 observations on the following 10 variables.
  \describe{
    \item{\code{vote}}{Party choice: \code{Conservative}, \code{Labour}, or \code{Liberal Democrat}}
    \item{\code{age}}{in years}
    \item{\code{economic.cond.national}}{Assessment of current national economic conditions, 1 to 5.}
    \item{\code{economic.cond.household}}{Assessment of current household economic conditions, 1 to 5.}
    \item{\code{Blair}}{Assessment of the Labour leader, 1 to 5.}
    \item{\code{Hague}}{Assessment of the Conservative leader, 1 to 5.}
    \item{\code{Kennedy}}{Assessment of the leader of the Liberal Democrats, 1 to 5.}
    \item{\code{Europe}}{an 11-point scale that measures respondents' attitudes
		toward European integration. High scores represent `Eurosceptic'
		sentiment.}
    \item{\code{political.knowledge}}{Knowledge of parties' positions on European integration, 0 to 3.}
    \item{\code{gender}}{\code{female} or \code{male}.}
  }
}

\references{
  J. Fox and R. Andersen (2006)
   Effect displays  for multinomial and proportional-odds logit models.
  \emph{Sociological Methodology} \bold{36}, 225--255.
}

\examples{
summary(BEPS)

\donttest{  
require(splines)  # for bs()
beps <- multinom(vote ~ age + gender + economic.cond.national + economic.cond.household 
	+ Blair + Hague + Kennedy + bs(Europe, 3)*political.knowledge, data=BEPS)
europe.knowledge <- effect("bs(Europe, 3)*political.knowledge", beps, 
	xlevels=list(Europe=seq(1, 11, length=50), political.knowledge=0:3),
  	given.values=c(gendermale=0.5))
plot(europe.knowledge)
plot(europe.knowledge, style="stacked", colors=c("blue", "red", "orange"), rug=FALSE)
}
}

\keyword{datasets}

\name{LegacyArguments}
\alias{LegacyArguments}
\alias{Legacy Arguments}

\title{Legacy Arguments for \code{plot} and \code{Effect} Methods}

\description{
Prior to verson 4.0-0 of the \pkg{effects} package, there were many (literally dozens) of
arguments to the \code{plot} methods for \code{"eff"} and \code{"effpoly"} objects. 

In version 4.0-0 of the package, we have consolidated these arguments into a much smaller
number of arguments (e.g., \code{lines}, \code{points}, \code{axes}) that take lists of
specifications. We have similarly consolidated some of the arguments to \code{Effect} methods
into the \code{confint} and \code{fixed.predictors} arguments.

For backwards compatibility, we have to the extent possible retained the
older arguments. If specified, these legacy arguments take precedence over the newer
list-style arguments.
}

\details{
Here is the correspondence between the old and new arguments.

For \code{plot} methods:

\describe{
\item{\code{multiline=TRUE/FALSE}}{\code{lines=list(multiline=TRUE/FALSE)}}

\item{\code{type=c("rescale", "link", "response")}}{For models with a link function, \code{"link"} plots in linear predictor scale, \code{"response"} plots in the response scale, and the default \code{"rescale"} plots in linear predictor scale but labels tick-marks in response scale.}

\item{\code{z.var=which.min(levels)}}{\code{lines=list(z.var=which.min(levels))} relevant only when \code{lines=list(multiline=TRUE)}}

\item{\code{colors={vector of colors}}}{\code{lines=list(col={vector of colors})}}

\item{\code{lty={vector of line types}}}{\code{lines=list(lty={vector of line types})}}

\item{\code{lwd={vector of line widths}}}{\code{lines=list(lwd={vector of line widths})}}

\item{\code{use.splines=TRUE/FALSE}}{\code{lines=list(splines=TRUE/FALSE)}}

\item{\code{cex={number}}}{\code{points=list(cex={number})}}

\item{\code{rug=TRUE/FALSE}}{\code{axes=list(x=list(rug=TRUE/FALSE)}}

\item{\code{xlab={"axis title"}}}{\code{axes=list(x=list(lab={"axis title"}))}}

\item{\code{xlim={c(min, max)}}}{\code{axes=list(x=list(lim={c(min, max)}))}}

\item{\code{rotx={degrees}}}{\code{axes=list(x=list(rot={degrees}))}}

\item{\code{ticks.x=list({tick specifications})}}{\code{axes=list(x=list(ticks=list({tick specifications})))}}

\item{\code{transform.x=list(link={function}, inverse={function})}}{\code{axes=list(x=list(transform=list({lists of transformations by predictors})))}}

\item{\code{ylab={"axis title"}}}{\code{axes=list(y=list(lab={"axis title"}))}}

\item{\code{ylim={c(min, max)}}}{\code{axes=list(y=list(lim={c(min, max)}))}}

\item{\code{roty={degrees}}}{\code{axes=list(y=list(rot={degrees}))}}

\item{\code{ticks=list({tick specifications})}}{\code{axes=list(y=list(ticks=list({tick specifications})))}}

\item{\code{alternating=TRUE/FALSE}}{\code{axes=list(alternating=TRUE/FALSE)}}

\item{\code{grid=TRUE/FALSE}}{\code{axes=list(grid=TRUE/FALSE)}}

\item{\code{ci.style="bands"/"lines"/"bars"/"none"}}{\code{confint=list(style="bands"/"lines"/"bars"/"none"})}

\item{\code{band.transparency={number}}}{\code{confint=list(alpha={number})}}

\item{\code{band.colors={vector of colors}}}{\code{confint=list(col={vector of colors})}}

\item{\code{residuals.color={color}}}{\code{partial.residuals=list(col={color})}}

\item{\code{residuals.pch={plotting character}}}{\code{partial.residuals=list(pch={plotting character})}}

\item{\code{residuals.cex={number}}}{\code{partial.residuals=list(cex={number})}}

\item{\code{smooth.residuals=TRUE/FALSE}}{\code{partial.residuals=list(smooth=TRUE/FALSE)}}

\item{\code{residuals.smooth.color={color}}}{\code{partial.residuals=list(smooth.col={color})}}

\item{\code{span={number}}}{\code{partial.residuals=list(span={number})}}

\item{\code{show.fitted=TRUE/FALSE}}{\code{partial.residuals=list(fitted=TRUE/FALSE)}}

\item{\code{factor.names=TRUE/FALSE}}{\code{lattice=list(strip=list(factor.names=TRUE/FALSE))}}

\item{\code{show.strip.values=TRUE/FALSE}}{\code{lattice=list(strip=list(values=TRUE/FALSE))}}

\item{\code{layout={lattice layout}}}{\code{lattice=list(layout={lattice layout})}}

\item{\code{key.args={lattice key args}}}{\code{lattice=list(key.args={lattice key args})}}

\item{\code{style="lines"/"stacked"}}{for \code{plot.effpoly}, \code{axes=list(y=list(style="lines"/"stacked"))}}

\item{\code{rescale.axis=TRUE/FALSE}}{\code{type="rescale"/"response"/"link"}}
}

For \code{Effect} methods:

\describe{
\item{\code{confint=TRUE/FALSE} or a list}{may be substituted for the \code{se} argument.}

\item{\code{confidence.level={number}}}{\code{se=list(level={number})}}

\item{\code{given.values={named vector}}}{\code{fixed.predictors=list(given.values={named vector})}}

\item{\code{typical={function}}}{\code{fixed.predictors=list(typical={function})}}

\item{\code{offset={function}}}{\code{fixed.predictors=list(offset={function})}}

\item{\code{partial.residuals=TRUE/FALSE}}{\code{residuals=TRUE/FALSE}}

}}

\author{John Fox \email{jfox@mcmaster.ca}}


\seealso{
\code{\link{Effect}}, \code{\link{plot.eff}}, \code{\link{plot.effpoly}}
}

\keyword{hplot}

\name{WVS}
\alias{WVS}
\docType{data}
\title{World Values Surveys}
\description{
  Data from the World Values Surveys 1995-1997 for Australia, Norway, Sweden, and the United States.
}
\usage{WVS}
\format{
  A data frame with 5381 observations on the following 6 variables.
  \describe{
    \item{\code{poverty}}{``Do you think that what the government is doing for people in poverty in
	this country is about the right amount, too much, or too little?'' (ordered): 
	\code{Too Little}, \code{About Right}, \code{Too Much}}.
    \item{\code{religion}}{Member of a religion: \code{no} or \code{yes}.}
    \item{\code{degree}}{Held a university degree: \code{no} or \code{yes}.}
    \item{\code{country}}{\code{Australia}, \code{Norway}, \code{Sweden}, or \code{USA}.}
    \item{\code{age}}{in years.}
    \item{\code{gender}}{\code{male} or \code{female}.}
  }
}

\references{
  J. Fox and R. Andersen (2006)
   Effect displays  for multinomial and proportional-odds logit models.
  \emph{Sociological Methodology} \bold{36}, 225--255.
}

\examples{
summary(WVS)

\donttest{ 
require(splines)  # for bs()
wvs <- polr(poverty ~ gender + country*(religion + degree + bs(age, 4)), data=WVS)

plot(effect("country*bs(age,4)", wvs, xlevels=list(age=18:83),
  given.values=c(gendermale=0.5)), rug=FALSE) 
plot(effect("country*bs(age,4)", wvs, xlevels=list(age=18:83),
  given.values=c(gendermale=0.5)), rug=FALSE, style="stacked")
plot(effect("country*bs(age,4)", wvs, xlevels=list(age=18:83),
  given.values=c(gendermale=0.5), latent=TRUE), rug=FALSE)
}
}

\keyword{datasets}

\name{effectsTheme}
\alias{effectsTheme}

\title{Set the lattice Theme for Effect Plots}

\description{
Set the \pkg{lattice} theme (see \code{\link[lattice]{trellis.device}}) appropriately for effect plots. This function is invoked
automatically when the \pkg{effects} package is loaded \emph{if} the \pkg{lattice} package hasn't previously been loaded. A typical
call is \code{lattice::trellis.par.set(effectsTheme())}.
}

\usage{
effectsTheme(strip.background = list(col = gray(seq(0.95, 0.5, length = 3))), 
          strip.shingle = list(col = "black"), clip = list(strip = "off"), 
          superpose.line = list(lwd = c(2, rep(1, 6))), col)
}

\arguments{
  \item{strip.background}{colors for the background of conditioning strips at the top of each panel; the default uses shades of gray and 
    makes allowance for up to three conditioning variables.}
  \item{strip.shingle}{when lines rather than numeric values are used to indicate the values of conditioning variables, the default sets the color of
    the lines to black.}
  \item{clip}{the default allows lines showing values of conditioning variables to extend slightly beyond the boundaries of 
    the strips---making the lines more visible at the extremes.}
  \item{superpose.line}{the default sets the line width of the first (of seven) lines to 2.}
  \item{col}{an optional argument specifying the colors to use for lines and symbolst: 
  if \code{col = "car"}, then the color palette for the \pkg{car} package is used (see \code{\link[car]{carPalette}});
  \code{col = "R"}, then the current R palette (ignoring the first entry which is \code{"black"} in the standard R palette) is used (see \code{\link[grDevices]{palette}});
  if \code{col = "colorblind"}, then a colorblind-friendly palette (from \url{https://jfly.uni-koeln.de/color/} but ignoring black) is used;
  if a vector of color specifications, then these are used.
  If \code{col} isn't specified then the current \pkg{lattice} colors are used.}
}

\value{
a list suitable as an argument for \code{\link[lattice]{trellis.par.set}}; current values of modified parameters are
supplied as an attribute.
}

\author{John Fox \email{jfox@mcmaster.ca}}

\seealso{\code{\link[lattice]{trellis.device}}, \code{\link[lattice]{trellis.par.set}}}

\examples{
\dontrun{
lattice::trellis.par.set(effectsTheme())
}
}


\keyword{utilities}
\keyword{device}

\name{Wells}
\alias{Wells}
\docType{data}
\title{Well Switching in Bangladesh}
\description{
  Data on whether or not households in Bangladesh changed the wells that 
  they were using.
}

\usage{Wells}
\format{
  A data frame with 3020 observations on the following 5 variables.
  \describe{
    \item{\code{switch}}{whether or not the household switched to
      another well from an unsafe well: \code{no} or \code{yes}.}
    \item{\code{arsenic}}{the level of arsenic contamination in the 
      household's original well, in hundreds of micrograms per liter; all
      are above 0.5, which was the level identified as ``safe''.}
    \item{\code{distance}}{in meters to the closest known safe well.}
    \item{\code{education}}{in years of the head of the household.}
    \item{\code{association}}{whether or not any members of the household
      participated in any community organizations: \code{no} or \code{yes}.}
  }
}

\details{
  The data are for an area of Arahazar upazila, Bangladesh.
  The researchers labelled each well with its level of arsenic and an
  indication of whether the well was ``safe'' or ``unsafe.'' Those using unsafe
  wells were encouraged to switch. After several years, it was determined
  whether each household using an unsafe well had changed its well. 
  These data are used by Gelman and Hill (2007) for a logistic-regression example.
}

\source{
  \url{http://www.stat.columbia.edu/~gelman/arm/examples/arsenic/wells.dat}.
}

\references{
 A. Gelman and J. Hill (2007)
 \emph{Data Analysis Using Regression and Multilevel/Hierarchical Models.}
 Cambridge: Cambridge University Press.
}

\examples{
summary(Wells)
}

\keyword{datasets}

\name{plot.effects}
\alias{plot.effect}
\alias{plot.effects}
\alias{plot.predictoreff}
\alias{plot.predictorefflist}
\alias{plot.eff}
\alias{plot.effpoly}
\alias{plot.efflist}
\alias{plot.mlm.efflist}
\alias{[.efflist}
\alias{levels2dates}
\alias{levels2dates.eff}
\alias{levels2dates.effpoly}


\title{Plots of Effects and Predictor Effects}

\description{
\code{plot} methods for \code{predictoreff}, \code{predictorefflist}, \code{eff}, \code{efflist} and \code{effpoly} objects created by calls other methods in the \code{effects} package.  The plot arguments were substantially changed in mid-2017. For more details and many examples, see the \href{../doc/predictor-effects-gallery.pdf}{Predictor Effects Graphics Gallery} vignette.
}

\usage{

\method{plot}{eff}(x, x.var, 
    main=paste(effect, "effect plot"),
    symbols=TRUE, lines=TRUE, axes, confint,
    partial.residuals, id, lattice, ...,
    # legacy arguments:
    multiline, z.var, rug, xlab, ylab, colors, cex, lty, lwd,
    ylim, xlim, factor.names, ci.style,
    band.transparency, band.colors, type, ticks,
    alternating, rotx, roty, grid, layout,
    rescale.axis, transform.x, ticks.x, show.strip.values,
    key.args, use.splines,
    residuals.color, residuals.pch, residuals.cex, smooth.residuals,
    residuals.smooth.color, show.fitted, span)
    
\method{plot}{efflist}(x, selection, rows, cols, ask=FALSE, graphics=TRUE, lattice, ...)

\method{plot}{predictoreff}(x, x.var, 
    main = paste(names(x$variables)[1], "predictor effect plot"), ...)

\method{plot}{predictorefflist}(x, selection, rows, cols, ask = FALSE, 
    graphics = TRUE, lattice, ...)

\method{plot}{effpoly}(x, x.var=which.max(levels),
    main=paste(effect, "effect plot"),
    symbols=TRUE, lines=TRUE, axes, confint, lattice, ...,
    # legacy arguments:
    type, multiline, rug, xlab, ylab, colors, cex, lty, lwd,
    factor.names, show.strip.values,
    ci.style, band.colors, band.transparency, style,
    transform.x, ticks.x, xlim,
    ticks, ylim, rotx, roty, alternating, grid,
    layout, key.args, use.splines)
    
\method{plot}{mlm.efflist}(x, ...)

levels2dates(effect, ...)
\method{levels2dates}{eff}(effect, predictor, origin, evenly.spaced=TRUE, n, ...)
\method{levels2dates}{effpoly}(effect, predictor, origin, evenly.spaced=TRUE, n, ...)
}

\arguments{
  \item{x}{an object of class \code{"predictoreff"}, \code{"predictorefflist"}, \code{"eff"}, \code{"effpoly"}, \code{"efflist"}, \code{"mlm.efflist"}, 	or \code{"summary.eff"}, as appropriate.}

  \item{x.var}{the index (number) or quoted name of the covariate or factor to place on the horizontal axis of each panel of the effect plot. The default is the predictor with the largest number of levels or values.  This argument is ignored with \code{predictoreff} objects.}
  
  \item{main}{the title for the plot, printed at the top; the default title is constructed from the name of the effect.}
  
  \item{symbols}{\code{TRUE}, \code{FALSE}, or an optional list of specifications for plotting symbols; if not given, symbol properties are taken from \code{superpose.symbol} in the lattice theme. See Detailed Argument Descriptions under Details for more information.}
  
  \item{lines}{\code{TRUE}, \code{FALSE}, or an optional list of specifications for plotting lines (and possibly areas); if not given, line properties are taken from \code{superpose.line} in the lattice theme. See Detailed Argument Descriptions under Details for more information.}
  
  \item{axes}{an optional list of specifications for the x and y axes; if not given, axis properties take generally reasonable default values. See Details for more information.}
  
  \item{confint}{an optional list of specifications for plotting confidence regions and intervals; if not given, generally reasonable default values are used. See Detailed Argument Descriptions under Details for more information.}
  
  \item{partial.residuals}{an optional list of specifications for plotting partial residuals for linear and generalized linear models; if not given, generally reasonable default values are used. See Detailed Argument Descriptions under Details for more information, along with the \href{../doc/partial-residuals.pdf}{Effect Displays with Partial Residuals} vignette.}
  \item{id}{an optional list of specifications for identifying points when partial residuals are plotted; if not specified, no points are labelled. See Detailed Argument Descriptions under Details for more information.}
  
  \item{lattice}{an optional list of specifications for various lattice properties, such as legend placement; if not given, generally reasonable default values are used. See Detailed Argument Descriptions under Details for more information.}
  
  \item{selection}{the optional index (number) or quoted name of the effect in an efflist object to be plotted; if not supplied, a menu of high-order terms is presented or all effects are plotted.}
  
  \item{rows, cols}{Number of rows and columns in the ``meta-array'' of plots produced for an \code{efflist} object; if either argument is missing, then the meta-layout will be computed by the \code{plot} method.}
  
  \item{ask}{if \code{selection} is not supplied and \code{ask} is \code{TRUE}, a menu of high-order terms is presented; if \code{ask} is \code{FALSE} (the default), effects for all high-order terms are plotted in an array.}
  
  \item{graphics}{if \code{TRUE} (the default), then the menu of terms to plot is presented in a dialog box rather than as a text menu.}
  
  \item{...}{arguments to be passed down. For \code{"predictoreff"} or \code{"predictorefflist"} objects, the arguments passed down can include all the arguments for \code{"eff"}.}
  
  \item{effect}{An object of class \code{"eff"} or \code{"effpoly"}.}
  \item{predictor}{The quoted name of the date variable in the effect.}
  \item{origin}{The date origin for the date variable: see Details}
  \item{evenly.spaced}{Should the dates be evenly spaced? \code{TRUE} (the default) or \code{FALSE}: see Details.}
  \item{n}{Number of tick marks for the date axis: see Details.}
  
\item{multiline, z.var, rug, xlab, ylab, colors, cex, lty, lwd,
     ylim, xlim, factor.names, ci.style,
     band.transparency, band.colors, ticks,
     alternating, rotx, roty, grid, layout,
     rescale.axis, transform.x, ticks.x, show.strip.values,
     key.args, use.splines, type,
     residuals.color, residuals.pch, residuals.cex, smooth.residuals,
     residuals.smooth.color, show.fitted, span, style}{legacy arguments retained for backwards compatibility; if specified, these will take precedence over the newer list-style arguments described above. See \code{\link{LegacyArguments}} for details.}
}

\details{
Effects plots and predictor effects plots are produced by these methods.  The plots are highly customizable using the optional arguments described here.  For example, effects in a GLM are plotted on the scale of the linear predictor, but the vertical axis is labelled on the response scale. This preserves the linear structure of the model while permitting interpretation on what is usually a more familiar scale. This approach may also be used with linear models, for example to display effects on the scale of the response even if the data are analyzed on a transformed scale, such as log or square-root.  See the \code{axes} argument details below to change the scale to response scale, or to linear predictor scale with tick marks labeled in response scale.

When a factor is on the x-axis, the \code{plot} method for \code{eff} objects connects the points representing the effect by line segments, creating a response ``profile.'' If you wish to suppress these lines, add \code{lty=0} to the \code{lines} argument to the call to \code{plot} (see below and the examples).

In a polytomous multinomial or proportional-odds logit model, by default effects are plotted on the probability scale; they may alternatively be plotted on the scale of the individual-level logits.

All of the arguments to plot objects created by \code{Effect} or \code{allEffects} can also be used with objects created by \code{predictorEffect} or \code{predictorEffects}.  

\bold{Detailed Argument Descriptions}

For more information about these arguments and many examples, see the \href{../doc/predictor-effects-gallery.pdf}{Predictor Effects Graphics Gallery} vignette.

Maximizing the flexibility of these plot commands requires inclusion of a myriad of options.  In an attempt to simplify the use of these options, they have been organized into just a few arguments that each accept a list of specifications as an argument.  In a few cases the named entries in the list are themselves lists.

  Each of the following arguments takes an optional list of specifications; any specification absent from the list
assumes its default value. Some of the list elements are themselves lists, so in complex cases, the argument can take
the form of nested lists.  All of these arguments can also be used on objects created with \code{\link{predictorEffects}}.

\describe{
\item{\code{symbols}}{\code{TRUE}, \code{FALSE}, or a list of options that controls the plotting symbols and their sizes for use with factors;
if \code{FALSE} symbols are suppressed; if \code{TRUE} default values are used:
    \describe{
    \item{\code{pch}}{ploting symbols, a vector of plotting characters, with the default taken from \code{trellis.par.get("superpose.symbol")$pch}, typically a vector of 1s (circles).}
    \item{\code{cex}}{plotting character sizes, a vector of values, with the default taken from \code{trellis.par.get("superpose.symbol")$cex}, typically a vector of 0.8s.}
    }
}
\item{\code{lines}}{\code{TRUE}, \code{FALSE}, or a list that controls the characteristics of lines drawn on a plot, and also whether or not multiple lines should be drawn in the same panel in the plot; if \code{FALSE} lines are suppressed; if \code{TRUE} default values are used:
    \describe{
    \item{\code{multiline}}{display a multiline plot in each panel; the default is \code{TRUE} if there are no standard errors
    in the \code{"eff"} object, \code{FALSE} otherwise. For an \code{"effpoly"} object \code{multline=TRUE} causes all of the response
    levels to be shown in the same panel rather than in separate panels.}
  \item{z.var}{for linear, generalized linear or mixed models,
  	the index (number) or quoted name of the covariate or factor for which
    individual lines are to be drawn in each panel of the effect plot. 
    The default is the
    predictor with the smallest number of levels or values. This argument is only
    used for multipline plots.}
    \item{\code{lty}}{vector of line types, with the default taken from \code{trellis.par.get("superpose.line")$lty}, typically a vector of 1s (solid lines).}
    \item{\code{lwd}}{vector of line widths, with the default taken from \code{trellis.par.get("superpose.line")$lwd}, typically a vector with 2 in the first position followed by 1s.}
    \item{\code{col}}{a vector of line colors, with the default taken from from \code{trellis.par.get("superpose.line")$col}, used both for lines and
    for areas in stacked area plots for \code{"effpoly"} objects; in the latter case, the default colors for an ordered response are instead generated by
    \code{\link[colorspace]{sequential_hcl}} in the \pkg{colorspace} package.}
    \item{\code{splines}}{use splines to smooth plotted effect lines; the default is \code{TRUE}.}
    }
}
\item{\code{axes}}{a list with elements \code{x}, \code{y}, \code{alternating}, and \code{grid} that control axis limits, ticks, and labels.
The \code{x} and \code{y} elements may themselves be lists.

The \code{x} entry is a list with elements named for predictors, with each predictor element itself a list with the following elements:
    \describe{
    \item{\code{lab}}{axis label, defaults to the name of the predictor; may either
    be a text string or a list with the text label (optionally named \code{label}) 
    as its first element and the named element \code{cex} as its second element.}
    \item{\code{lim}}{a two-element vector giving the axis limits, with the default determined from the data.}
    \item{\code{ticks}}{a list with either element \code{at}, a vector specifying locations for the ticks marks, or \code{n}, the number
        of tick marks.}
    \item{\code{transform}}{transformations to be applied to the horizontal axis of a numeric predictor,
        in the form of a list of two functions, with
        element names \code{trans} and \code{inverse}. The
        \code{trans} function is applied to the values of the predictor, and \code{inverse} is used for computing
        proper axis tick labels. The default is not to transform the predictor axis.}
    }

Two additional elements may appear in the \code{x} list, and apply to all predictors:
    \describe{
    \item{\code{rotate}}{angle in degrees to rotate tick labels; the default is 0.}
    \item{\code{rug}}{display a rug plot showing the marginal distribution of a numeric predictor; the default is \code{TRUE}.}
    }

The \code{y} list contains \code{lab}, \code{lim}, \code{ticks}, and \code{rotate} elements
(similar to those specified for individual predictors in the \code{x} list), along with the additional \code{type}, \code{transform}, and \code{style} elements:
    \describe{
    \item{\code{type}}{for plotting linear or generalized linear models, \code{"rescale"} (the default) plots the vertical
    axis on the link scale (e.g., the logit scale for a logit model) but labels the axis on the response
    scale (e.g., the probability scale for a logit model);
    \code{"response"} plots and labels the vertical axis on the scale of the response (e.g., the probability scale for a logit model); and
    \code{"link"} plots and labels the vertical axis on the scale of the link (e.g., the logit scale for a logit model).
    For polytomous logit models, this element is either \code{"probability"} or \code{"logit"}, with the former as the default.}
    \item{\code{transform}}{primarily for linear or linear mixed models, this argument is used to apply an arbitrary transformation to the vertical axis.  For example, if fitting a linear model with response \code{log(y)}, then setting \code{transform=exp} would plot \code{exp(log(y)) = y} on the vertical axis.  If the response were \code{1/y}, then use \code{transform=function(yt) 1/yt}, since the reciprocal is its own inverse.  The \code{transform} argument can also be a list of two functions.  For example with a response \code{log(y)}, the specification \code{transform=list(trans=log, inverse=log), type="rescale"} will plot in log-scale, but will label tick marks in arithmetic scale; see the example below.  The specification \code{transform=list(trans=log, inverse=exp), type="response"} is equivalent to \code{transform=exp}.   When \code{type="response"} the \code{lab} argument will geneally be used to get a label for the axis that matches the untransformed response.  If this argument is used with a generalized linear model or another model with a non-identity link function, the function is applied to the linear predictor, and will probably not be of interest.}
    \item{\code{style}}{for polytomous logit models, this element can take on the value \code{"lines"}
    (the default) or \code{"stacked"} for line plots or stacked-area plots, respectively.}
    }

Other elements:
    \describe{
    \item{\code{alternating}}{if \code{TRUE} (the default), the tick labels alternate by panels in
    multi-panel displays from left to right and top to bottom; if \code{FALSE}, tick labels
    appear at the bottom and on the left.}
    \item{\code{grid}}{if \code{TRUE} (the default is \code{FALSE}), add grid lines to the plot.}
    }
}

\item{\code{confint}}{specifications to add/remove confidence intervals or regions from a plot, and to set the nominal confidence level.

    \describe{
    \item{\code{style}}{one of \code{"auto"}, \code{"bars"}, \code{"lines"}, \code{"bands"}, and \code{"none"}; the default
    is \code{"bars"} for factors, \code{"bands"} for numeric predictors, and \code{"none"} for multiline plots; \code{"auto"} also produces \code{"bars"} for factors
    and \code{"bands"} for numeric predictors, even in multiline plots.}
    \item{\code{alpha}}{transparency of confidence bands; the default is 0.15.}
    \item{\code{col}}{colors; the default is taken from the line colors.}
    }
}

\item{\code{partial.residuals}}{specifications concerning the addition of partial residuals to the plot.

    \describe{
    \item{\code{plot}}{display the partial residuals;
    the default is \code{TRUE} if residuals are present in the \code{"eff"} object, \code{FALSE} otherwise.}
    \item{\code{fitted}}{show fitted values as well as residuals; the default is \code{FALSE}.}
    \item{\code{col}}{color for partial residuals; the default is the second line color.}
    \item{\code{pch}}{plotting symbols for partial residuals; the default is 1, a circle.}
    \item{\code{cex}}{size of symbols for partial residuals; the default is 1.}
    \item{\code{smooth}}{draw a loess smooth of the partial residuals; the default is \code{TRUE}.}
    \item{\code{span}}{span for the loess smooth; the default is 2/3.}
    \item{\code{smooth.col}}{color for the loess smooth; the default is the second line color.}
    \item{\code{lty}}{line type for the loess smooth; the default is the first line type, normally 1 (a solid line).}
    \item{\code{lwd}}{line width for the loess smooth; the default is the first line width, normally 2.}
    }

}

\item{\code{id}}{specifications for optional point identification when partial residuals are plotted.

  \describe{
  \item{\code{n}}{number of points to identify; default is \code{2} if \code{id=TRUE}
    and \code{0} if \code{id=FALSE}. Points are selected based on the Mahalanobis
    distances of the pairs of x-values and partial residuals from their centroid.}
  \item{\code{col}}{color for the point labels; default is the same as the color of the partial residuals.}
  \item{\code{cex}}{relative size of text for point labels; default is \code{0.75}.}
  \item{\code{labels}}{vector of point labels; the default is the names of the residual vector,
  which is typically the row names of the data frame to which the model is fit.}
  }

}

\item{\code{lattice}}{the plots are drawn with the \pkg{\link[lattice]{lattice}} package, generally by the \code{\link[lattice]{xyplot}} function.  These specifications are passed as arguments to the functions that actually draw the plots.

    \describe{
    \item{\code{layout}}{the \code{layout} argument to the \pkg{lattice} function \code{\link[lattice]{xyplot}}
  	(or, in some cases \code{\link[lattice]{densityplot}}), which
  	is used to draw the effect display; if not specified, the plot will be formatted so that
  	it appears on a single page.}
  	\item{\code{key.args}}{a key, or legend, is added to the plot if \code{multiline=TRUE}.  This argument is a list with components that determine the the placement and other characteristics of the key.  The default if not set by the user is \code{key.args = list(space="top", columns=2, border=FALSE, fontfamily="serif", cex.title=.80, cex=0.75)}.  If there are more than 6 groups in the plot, \code{columns} is set to 3. For stacked-area plots, the default is a one-column key.  In addition to the arguments shown explicitly below, any of the arguments listed in the \code{\link[lattice]{xyplot}} documentation in the \code{key} section can be used.
  	\describe{
  	\item{\code{space}}{determines the placement of the key outside the plotting area, with default \code{space="above"} for above the plot and below its title.  Setting \code{space="right"} uses space to the right of the plot for the key.}
  	\item{\code{x, y, corner}}{used to put the key on the graph itself.  For example, \code{x=.05, y=.95, corner=c(0,1)} will locate the upper-left corner of the key at (.05, .95), thinking of the graph as a unit square.}
  	\item{\code{columns}}{number of columns in the key.  If \code{space="top"}, columns should be 2, 3 or 4; if \code{space="right"}, set \code{columns=1}.}
  	\item{\code{border}}{if \code{TRUE} draw a border around the key; omit the border if \code{FALSE}.}
  	\item{\code{fontfamily}}{the default is \code{"sans"} for the sans-serif font used in the rest of the plot; the alternative is \code{"serif"} for a serif font.}
  	\item{\code{cex, cex.title}}{the default relative size of the font for labels and the title, respectively.  To save space set these to be smaller than 1.}
  	}
}
    \item{\code{strip}}{a list with three elements: \code{factor.names}, which if \code{TRUE}, the default, shows conditioning
    variable names in the panel headers;  \code{values}, which if \code{TRUE}, the default unless partial residuals are plotted,
    displays conditioning variable values in the panel headers, and \code{cex}, the relative size of the text displayed in the strip.}
    \item{\code{array}}{a list with elements \code{row}, \code{col}, \code{nrow}, \code{ncol}, and \code{more},
    used to graph an effect as part of an array of plots; \code{row}, \code{col}, \code{nrow}, and \code{ncol} are used to compose
    the \code{split} argument and \code{more} the \code{more} argument to \code{\link[lattice]{print.trellis}}.
    The \code{array} argument is automatically set by \code{plot.efflist} and will be ignored if used with that function.}
}

}
}

The \code{levels2dates} function is provided to partially accommodate \code{"Date"} variables in effect plots, as long as the date variable is on the horizontal axis of the plot. The date variable must be converted to numeric in the fitted model. The purpose of \code{levels2dates} is to reconvert the numeric version of the date variable to dates to label axis tick marks in the graph. If the argument \code{evenly.spaced} is \code{TRUE} (which is the default), then the tick marks along the horizontal axis are evenly spaced between the minimum and maximum dates of the date variable in the effect; otherwise, the levels for the variable in the effect object are used. The number of tick marks is given by the \code{n} argument; if \code{n} isn't supplied (and \code{evenly.spaced=TRUE}), then the number of tick marks is taken from the number of levels for the variable in the effect object.
}

\value{
  The \code{summary} method for \code{"eff"} objects returns a \code{"summary.eff"} object with the following components
  (those pertaining to confidence limits need not be present):
  \item{header}{a character string to label the effect.}
  \item{effect}{an array containing the estimated effect.}
  \item{lower.header}{a character string to label the lower confidence limits.}
  \item{lower}{an array containing the lower confidence limits.}
  \item{upper.header}{a character string to label the upper confidence limits.}
  \item{upper}{an array containing the upper confidence limits.}

  The \code{plot} method for \code{"eff"} objects returns a \code{"plot.eff"} object (an enhanced \code{"trellis"} object); the provided
\code{\link{print}} method plots the object.

  The \code{[} method for \code{"efflist"} objects is used to subset an \code{"efflist"} object and returns an object of the same class.
}


\author{John Fox \email{jfox@mcmaster.ca} and Jangman Hong.}


\seealso{\code{\link{LegacyArguments}}, \code{\link{effect}}, \code{\link{allEffects}}, \code{\link{effectsTheme}},
    \code{\link[lattice]{xyplot}}, \code{\link[lattice]{densityplot}}, \code{\link[lattice]{print.trellis}}, \code{\link[stats]{loess}},
	\code{\link[colorspace]{sequential_hcl}},
	and the \href{../doc/predictor-effects-gallery.pdf}{Predictor Effects Graphics Gallery} and \href{../doc/partial-residuals.pdf}{Effect Displays with Partial Residuals} vignettes.
}

\examples{
# also see examples in ?effect

# plot predictorEffects
mod <- lm(prestige ~ education + log(income)*type + women, Prestige)
plot(predictorEffects(mod, ~ income), axes=list(grid=TRUE))
plot(predictorEffects(mod, ~ income), lines=list(multiline=TRUE),
                                                 axes=list(grid=TRUE))
plot(predictorEffects(mod, ~ type), lines=list(multiline=TRUE),
                                                 axes=list(grid=TRUE),
                                                 confint=list(style="bars"))

mod.cowles <- glm(volunteer ~ sex + neuroticism*extraversion,
                  data=Cowles, family=binomial)
eff.cowles <- allEffects(mod.cowles, xlevels=list(extraversion=seq(0, 24, 6)))
eff.cowles
as.data.frame(eff.cowles[[2]]) # neuroticism*extraversion interaction

plot(eff.cowles, 'sex', axes=list(grid=TRUE,
                                  y=list(lab="Prob(Volunteer)"),
                                  x=list(rotate=90)),
                        lines=list(lty=0))

plot(eff.cowles, 'neuroticism:extraversion',
     axes=list(y=list(lab="Prob(Volunteer)",
        ticks=list(at=c(.1,.25,.5,.75,.9)))))

plot(Effect(c("neuroticism", "extraversion"), mod.cowles,
            se=list(type="Scheffe"),
            xlevels=list(extraversion=seq(0, 24, 6))),
     axes=list(y=list(lab="Prob(Volunteer)",
        ticks=list(at=c(.1,.25,.5,.75,.9)))))

\donttest{
    # change color of the confidence bands to 'black' with .15 transparency
plot(eff.cowles, 'neuroticism:extraversion',
     axes=list(y=list(lab="Prob(Volunteer)",
                      ticks=list(at=c(.1,.25,.5,.75,.9)))),
     confint=list(col="red", alpha=.3))

plot(eff.cowles, 'neuroticism:extraversion',
     lines=list(multiline=TRUE),
     axes=list(y=list(lab="Prob(Volunteer)")),
     lattice=list(key.args = list(x = 0.65, y = 0.99, corner = c(0, 1))))

# use probability scale in place of logit scale, all lines are black.
plot(eff.cowles, 'neuroticism:extraversion',
     lines=list(multiline=TRUE, lty=1:8, col="black"),
     axes=list(y=list(type="response", lab="Prob(Volunteer)")),
     lattice=list(key.args = list(x = 0.65, y = 0.99, corner = c(0, 1))),
     confint=list(style="bands"))

plot(effect('sex:neuroticism:extraversion', mod.cowles,
            xlevels=list(extraversion=seq(0, 24, 6))),
     lines=list(multiline=TRUE))

plot(effect('sex:neuroticism:extraversion', mod.cowles,
            xlevels=list(extraversion=seq(0, 24, 6))),
     lines=list(multiline=TRUE),
     axes=list(y=list(type="response")),
     confint=list(style="bands"),
     lattice=list(key.args = list(x=0.75, y=0.75, corner=c(0, 0))))
}

if (require(nnet)){
    mod.beps <- multinom(vote ~ age + gender + economic.cond.national +
                             economic.cond.household + Blair + Hague + Kennedy +
                             Europe*political.knowledge, data=BEPS)
    \donttest{
    plot(effect("Europe*political.knowledge", mod.beps,
                xlevels=list(political.knowledge=0:3)))
    }

    plot(effect("Europe*political.knowledge", mod.beps,
                xlevels=list(political.knowledge=0:3),
                fixed.predictors=list(given.values=c(gendermale=0.5))),
         axes=list(y=list(style="stacked"), x=list(rug=FALSE), grid=TRUE),
         lines=list(col=c("blue", "red", "orange")))
}

if (require(MASS)){
    mod.wvs <- polr(poverty ~ gender + religion + degree + country*poly(age,3),
                    data=WVS)
    plot(effect("country*poly(age, 3)", mod.wvs))

    \donttest{
    plot(effect("country*poly(age, 3)", mod.wvs), lines=list(multiline=TRUE))
    plot(effect("country*poly(age, 3)", mod.wvs),
         axes=list(y=list(style="stacked")),
         lines=list(col=c("gray75", "gray50", "gray25")))

    plot(effect("country*poly(age, 3)", latent=TRUE, mod.wvs))
    }
}

mod.pres <- lm(prestige ~ log(income, 10) + poly(education, 3) + poly(women, 2),
               data=Prestige)
eff.pres <- allEffects(mod.pres)
\donttest{
plot(eff.pres)
plot(eff.pres[1:2])
}
plot(eff.pres[1],
     axes=list(x=list(income=list(transform=list(
         trans=log10, inverse=function(x) 10^x),
         ticks=list(at=c(1000, 2000, 5000, 10000, 20000))))))
         
mod <- lm(log(prestige) ~ income:type + education, data=Prestige)
p1 <- predictorEffects(mod, ~ income)
# log-scale for response
plot(p1, lines=list(multiline=TRUE)) 
# log-scale, with arithmetic tick marks
plot(p1, lines=list(multiline=TRUE), 
     axes=list(y=list(transform=list(trans=log, inverse = exp), 
                      lab="prestige", type="rescale")))
# arithmetic scale and tick marks, with other arguments
plot(p1, lines=list(multiline=TRUE), grid=TRUE,
     lattice=list(key.args=list(space="right", border=TRUE)),
     axes=list(y=list(transform=exp, lab="prestige")))
     
# plotting an effect with a date variable

data("airquality", package="datasets")
airquality$Date <- with(airquality, as.Date(paste("1973", Month, Day, sep="-"),
                                            format="\%Y-\%m-\%d"))
airquality$Date.num <- as.numeric(airquality$Date)
m1.date <- lm(Ozone ~ Date.num + Solar.R + Wind + Temp, data=airquality)
eff.date.1 <- Effect("Date.num", m1.date)
plot(eff.date.1, axes=list(x=list(Date.num=list(lab="Date", 
    ticks=list(at=levels2dates(eff.date.1, "Date.num", "1970-01-01"))), 
    rotate=45)), main="Date Effect")
plot(eff.date.1, axes=list(x=list(Date.num=list(lab="Date", 
    ticks=list(at=levels2dates(eff.date.1, "Date.num", "1970-01-01", n=4))))), 
    main="Date Effect")

eff.date.df <- as.data.frame(eff.date.1)
eff.date.df$Date <- as.Date(eff.date.df$Date.num, origin="1970-01-01")
eff.date.df

m2.date <- lm(Ozone ~ Date.num*Temp + Solar.R + Wind, data=airquality)
eff.date.2 <- Effect(c("Date.num", "Temp"), m2.date, xlevels=6)
plot(eff.date.2, axes=list(x=list(Date.num=list(lab="Date", 
  ticks=list(at=levels2dates(eff.date.2, "Date.num", "1970-01-01", n=3))), 
  rotate=45)), main="Date Effect by Temperature")

}

\keyword{hplot}
\keyword{models}

\name{summary.eff}
\alias{print.eff}
\alias{print.effpoly}
\alias{print.efflatent}
\alias{print.efflist}
\alias{print.mlm.efflist}
\alias{print.summary.eff}
\alias{summary.eff}
\alias{summary.effpoly}
\alias{summary.efflatent}
\alias{summary.efflist}
\alias{summary.mlm.efflist}
\alias{as.data.frame.eff}
\alias{as.data.frame.effpoly}
\alias{as.data.frame.efflatent}
\alias{as.data.frame.efflist}
\alias{vcov.eff}

\title{Summarizing and Printing Effects}

\description{
\code{summary}, \code{print}, and \code{as.data.frame} methods for objects created using the effects package.
}

\usage{
\method{print}{eff}(x, type=c("response", "link"), ...)
\method{print}{effpoly}(x, type=c("probability", "logits"), ...)
\method{print}{efflatent}(x, ...)
\method{print}{efflist}(x, ...)
\method{print}{mlm.efflist}(x, ...)
\method{print}{summary.eff}(x, ...)
\method{summary}{eff}(object, type=c("response", "link"), ...)
\method{summary}{effpoly}(object, type=c("probability", "logits"), ...)
\method{summary}{efflatent}(object, ...)
\method{summary}{efflist}(object, ...)
\method{summary}{mlm.efflist}(object, ...)
\method{as.data.frame}{eff}(x, row.names=NULL, optional=TRUE, 
        type=c("response", "link"), ...)
\method{as.data.frame}{efflist}(x, row.names=NULL, optional=TRUE, type, ...)
\method{as.data.frame}{effpoly}(x, row.names=NULL, optional=TRUE, ...)
\method{as.data.frame}{efflatent}(x, row.names=NULL, optional=TRUE, ...)
\method{vcov}{eff}(object, ...)
}

\arguments{
  \item{x, object}{an object consisting of fitted values and other information needed to draw effects plots that is produced by functions in the \code{effects} package.}

  \item{type}{fitted values are by default printed by these functions in the \code{"response"} scale.  For models with a link function like a GLM, fitted values in the linear predictor scale are obtained by setting \code{type="link"}.  For polytomous response models setting \code{type="logits"} returns fitted values in the logit scale.}

  \item{row.names, optional}{arguments to \code{as.data.frame} not used by these methods.}
  
  \item{...}{other arguments passed on}
}

\value{
The \code{print} methods return the fitted values in tables.  The \code{summary} methods return the fitted values and 95 percent condifence intervals, also in tables.  The \code{as.data.frame} method returns fitted values, standard errors, and 95 percent confidence intervals as a data frame, or as a list of data frames for the \code{efflist} method.  The \code{vcov} method returns the covariance matrix of the fitted values.
}


\author{John Fox \email{jfox@mcmaster.ca} and Jangman Hong.}

\examples{

mod.cowles <- glm(volunteer ~ sex + neuroticism*extraversion,
                  data=Cowles, family=binomial)
eff.cowles <- predictorEffects(mod.cowles)
print(eff.cowles)
print(eff.cowles[["neuroticism"]], type="link")
summary(eff.cowles[["neuroticism"]], type="link")
as.data.frame(eff.cowles)
# covariance matrix of fitted values in linear predictor scale
vcov(eff.cowles[[1]]) 
}

\keyword{hplot}
\keyword{models}

\name{Arrests}
\alias{Arrests}
\docType{data}

\title{Arrests for Marijuana Possession}

\description{
Data on police treatment of individuals arrested in
Toronto for simple possession of small quantities of marijuana. The data are part of a larger
data set featured in a series of articles in the Toronto Star newspaper.
}

\usage{Arrests}

\format{
  A data frame with 5226 observations on the following 8 variables.
  \describe{
    \item{released}{Whether or not the arrestee was released with a summons; 
    a factor with levels:
        \code{No};
        \code{Yes}. }
    \item{colour}{The arrestee's race; a factor with levels:
        \code{Black}; 
        \code{White}. }
    \item{year}{1997 through 2002; a numeric vector.}
    \item{age}{in years; a numeric vector.}
    \item{sex}{a factor with levels:
        \code{Female};
        \code{Male}. }
    \item{employed}{a factor with levels:
        \code{No};
        \code{Yes}. }
    \item{citizen}{a factor with levels:
        \code{No};
        \code{Yes}. }
    \item{checks}{Number of police data bases (of previous
    arrests, previous convictions, parole status, etc. -- 6 in all) on which
    the arrestee's name appeared; a numeric vector}
    }
}

\source{
  Personal communication from Michael Friendly, York University.
}

\examples{
  summary(Arrests)
}

\keyword{datasets}

\name{TitanicSurvival}
\alias{TitanicSurvival}
\docType{data}
\title{Survival of Passengers on the Titanic}
\description{
  Information on the survival status, sex, age, and passenger class of 1309
  passengers in the Titanic disaster of 1912.  
}

\usage{TitanicSurvival}

\format{
  A data frame with 1309 observations on the following 4 variables.
  \describe{
    \item{\code{survived}}{\code{no} or \code{yes}.}
    \item{\code{sex}}{\code{female} or \code{male}}
    \item{\code{age}}{in years (and for some children, fractions of a year); age
      is missing for 263 of the passengers.}
    \item{\code{passengerClass}}{\code{1st}, \code{2nd}, or \code{3rd} class.}
  }
}

\details{
  This is part of a larger data set compiled by Thomas Cason. 
  Many additional details are given in the sources cited below.
}

\source{
  Data set \code{titanic3} from 
  \url{http://biostat.mc.vanderbilt.edu/twiki/bin/view/Main/DataSets}.
}

\references{
  \url{http://www.encyclopedia-titanica.org/}
  
  F. E. Harrell, Jr. (2001)
  \emph{Regression Modeling Strategies}
  New York: Springer.
}
\examples{
summary(TitanicSurvival)
  
titanic <- glm(survived ~ (passengerClass + sex + age)^2, 
                data=TitanicSurvival, family=binomial)
  
titanic.all <- allEffects(titanic, typical=median, 
	given.values=c(passengerClass2nd=1/3, passengerClass3rd=1/3, sexmale=0.5))
  	
plot(titanic.all, ticks=list(at=c(.01, .05, seq(.1, .9, by=.2), .95, .99)), 
    ask=FALSE)

plot(effect("passengerClass*sex*age", titanic, xlevels=list(age=0:65)), 
	ticks=list(at=c(.001, .005, .01, .05, seq(.1, .9, by=.2), .95, .99, .995))) 
}
\keyword{datasets}

\name{effects-deprecated}
\alias{effects-deprecated}
\alias{all.effects}

\title{Deprecated Function in effects Package}

\description{
The \code{all.effects} function is provided only for compatibility with older versions of the effects package
and may be removed; use \code{\link{allEffects}} instead.
}

\usage{
all.effects(...)
}

\arguments{
  \item{...}{arguments to be passed to \code{allEffects}.}
}

\author{John Fox \email{jfox@mcmaster.ca}.}


\seealso{\code{\link{allEffects}}}

\name{effect}
\alias{effect}
\alias{effect.default}
\alias{Effect}
\alias{Effect.default}
\alias{Effect.lm}
\alias{Effect.multinom}
\alias{Effect.merMod}
\alias{Effect.mlm}
\alias{Effect.poLCA}
\alias{Effect.polr}
\alias{Effect.svyglm}
\alias{allEffects}
\alias{allEffects.default}

\title{Functions For Constructing Effect Displays} 
\description{
  \code{Effect} and \code{effect} construct an \code{"eff"} object for a term (usually a high-order term) in a regression that models a response as a linear function of main effects and interactions of factors and covariates. These models include, among others, linear models (fit by \code{\link[stats]{lm}} and \code{\link[nlme]{gls}}), and generalized linear models (fit by \code{\link[stats]{glm}}), for which an \code{"eff"} object is created, and multinomial and proportional-odds logit models (fit respectively by \code{\link[nnet]{multinom}} and \code{\link[MASS]{polr}}), for which an \code{"effpoly"} object is created. The computed effect absorbs the lower-order terms marginal to the term in question, and averages over other terms in the model.  For multivariate linear models (of class \code{"mlm"}, fit by \code{\link[stats]{lm}}), the functions construct a list of \code{"eff"} objects, separately for the various response variables in the model. 

\code{effect} builds the required object by specifying explicitly a focal term like \code{"a:b"} for an \code{a} by \code{b} interaction.  \code{Effect} in contrast specifies the predictors in  a term, for example \code{c("a", "b")}, rather than the term itself.  \code{Effect} is consequently more flexible and robust than \code{effect}, and will succeed with some models for which \code{effect} fails.  The \code{effect} function works by constructing a call to \code{Effect} and continues to be included in \pkg{effects} so older code that uses it will not break.

The \code{Effect} and \code{effect} functions can also be used with many other models; see \code{\link{Effect.default}} and the \href{https://CRAN.R-project.org/package=effects/vignettes/functions-supported-by-effects.pdf}{Regression Models Supported by the effects Package} vignette.

\code{allEffects} identifies all of the high-order terms in a model and returns a list of \code{"eff"} or \code{"effpoly"} objects (i.e., an object of class \code{"efflist"}).

For information on computing and displaying \emph{predictor effects}, see \code{\link{predictorEffect}} and \code{\link{plot.predictoreff}}.

For further information about plotting effects, see \code{\link{plot.eff}}.
}

\usage{
effect(term, mod, vcov.=vcov, ...)

\method{effect}{default}(term, mod, vcov.=vcov, ...)

Effect(focal.predictors, mod, ...)

\method{Effect}{lm}(focal.predictors, mod, xlevels=list(),
    fixed.predictors, vcov. = vcov, se=TRUE,
    residuals=FALSE, quantiles=seq(0.2, 0.8, by=0.2),
    x.var=NULL, transformation, ...,
    #legacy arguments:
    given.values, typical, offset, confint, confidence.level,
    partial.residuals)

\method{Effect}{multinom}(focal.predictors, mod,
    xlevels=list(), fixed.predictors,
    vcov. = vcov, se=TRUE, ...,
    #legacy arguments:
    confint, confidence.level, given.values, typical)

\method{Effect}{polr}(focal.predictors, mod,
    xlevels=list(), fixed.predictors,
    vcov.=vcov, se=TRUE, latent=FALSE, ...,
    #legacy arguments:
    confint, confidence.level, given.values, typical)

\method{Effect}{svyglm}(focal.predictors, mod, fixed.predictors, ...)

\method{Effect}{merMod}(focal.predictors, mod, ..., KR=FALSE)
    
\method{Effect}{poLCA}(focal.predictors, mod, ...)

\method{Effect}{mlm}(focal.predictors, mod, response, ...)

allEffects(mod, ...)

\method{allEffects}{default}(mod, ...)
}

\arguments{
  
  \item{term}{the quoted name of a term, usually, but not necessarily, a high-order term in the model. The term must be given exactly as it appears in the printed model, although either colons (\code{:}) or asterisks (\code{*}) may be used for interactions.  If \code{term} is NULL, the function returns the formula for the linear predictor.}
  
  \item{focal.predictors}{a character vector of one or more predictors in the model in any order.}
  
  \item{mod}{a regression model object.  If no specific method exists for the class of \code{mod}, \code{Effect.default} will be called.}
  
  \item{xlevels}{this argument is used to set the number of levels for any focal numeric predictor (that is predictors that are not factors, character variables, or logical variables, all of which are treated as factors).  If \code{xlevels=NULL}, then each numeric predictor is represented by five values over its range, equally spaced and then rounded to 'nice' numbers.  If \code{xlevels=n} is an integer, then each numeric predictor is represented by \code{n} equally spaced values rounded to 'nice' numbers.  
    
    More generally, \code{xlevels} can be a named list of values at which to set each numeric predictor.  For example, \code{xlevels=list(x1=c(2, 4.5, 7), x2=4)} would use the values 2, 4.5, and 7 for \code{x1}, use 4 equally spaced values for \code{x2}, and use the default for any other numeric predictors. 
    
    If partial residuals are computed, then the focal predictor that is to appear on the horizontal axis of an effect plot is evaluated at 100 equally spaced values along its full range, and, by default, other numeric predictors are evaluated at the quantiles specified in the \code{quantiles} argument, unless their values are given explicitly in \code{xlevels}.}
  
  \item{fixed.predictors}{an optional list of specifications affecting the values at which fixed predictors for an effect are set, potentially including:

    \describe{

      \item{given.values}{\code{given.values="default"} (which is, naturally, the default) specifies averaging over levels of a non-focal factor, weighting levels of the factor in proportion to sample size.
      
      \code{given.values="equal"} computes unweighted averages over the levels of non-focal factors.  
      
      For finer control, the user can also provide a named numeric vector of weights for particular columns of the model matrix that correspond to the regressors for the factor. 
      
Character and logical predictors are treated as factors. 

For example, for a factor \code{X} with three levels \code{a}, \code{b} and \code{c}, the regressors generated using the default \code{\link[stats]{contr.treatment}} parameterization for a factor will be named \code{Xb} and \code{Xc}, as the regressor for level \code{a} is excluded as the baseline level.  The specification \code{given.values=c(Xb=1/2, Xc=1/4)} would average over the levels of \code{X} with weight 1/2 for level \code{b}, 1/4 for \code{c}, and weight 1 = 1/2 - 1/4 = 1/4 for the baseline level \code{a}.  Setting \code{given.values=c(Xb=1)} would fix \code{X} at level \code{b}.
}

  	  \item{typical}{a function to be applied to the columns of the model matrix over which the effect is "averaged"; with the exception of the \code{"svyglm"} method, the default is \code{\link{mean}}. For\code{"svyglm"} objects, the default is to use the survey-design weighted mean.}

      \item{apply.typical.to.factors}{It generally doesn't make sense to apply typical values that aren't means (e.g., medians) to the columns of the model-matrix representing contrasts for factors. This value generally defaults to \code{FALSE} except for \code{"svyglm"} objects, for which the default is \code{TRUE}, using the the survey-design weighted mean.}

      \item{offset}{a function to be applied to the offset values (if there is an offset) in a linear or generalized linear model, or a mixed-effects model fit by \code{\link[lme4]{lmer}} or \code{\link[lme4]{glmer}}; or a numeric value, to which the offset will be set. The default is the \code{\link{mean}} function, and thus the offset will be set to its mean; in the case of \code{"svyglm"} objects, the default is to use the survey-design weighted mean. \emph{Note:} Only offsets defined by the \code{offset} argument to \code{\link[stats]{lm}}, \code{\link[stats]{glm}}, \code{\link[survey]{svyglm}}, \code{\link[lme4]{lmer}}, or \code{\link[lme4]{glmer}} will be handled correctly; use of the \code{offset} function in the model formula is not supported.}
  }
}

  \item{vcov.}{Effect methods generally use the matrix returned by \code{vcov(mod)} to compute standard errors and confidence bounds.  Alternatively, the user may specify the name of a function that returns a matrix of the same dimension and structure as the matrix returned by \code{vcov(mod)}.  For example, \code{vcov. = hccm} uses the \code{\link[car]{hccm}} function from the \pkg{car} package to use a heteroscedasticity corrected covariance matrix for a linear model in place of the standard covariance estimate. This argument can be set to equal matrix of the same size and structure as the matrix returned by \code{vcov(mod)}. For example, using \code{vcov. = vcov(Boot(mod))} uses \code{\link[car]{Boot}} from the \pkg{car} package to get a bootstrap estimate of the covariance matrix for linear, generalized linear, and possibly other modeling frameworks.}
  
  \item{se}{\code{TRUE} (the default), \code{FALSE}, or a list with any or all of the following elements, controlling whether and how standard errors and confidence limits are computed for the effects: 
  \describe{
  
  \item{compute}{(default \code{TRUE}) whether or not to compute standard errors and confidence limits.} 
  
  \item{level}{(default \code{0.95}) confidence level for confidence limits.}
  
  \item{type}{one of \code{"pointwise"} (the default), \code{"Scheffe"}, or \code{"scheffe"}, whether to compute confidence limits with specified coverage at each point for an effect or to compute limits for a Scheffe-type confidence envelope. For \code{mer}, \code{merMod}, and \code{lme} objects, the normal distribution is used to get confidence limits.}
  }
}

  \item{residuals}{if \code{TRUE}, residuals for a linear or generalized linear model will be computed and saved; if \code{FALSE} (the default), residuals are suppressed. If residuals are saved, partial residuals are computed when the effect is plotted: see \code{\link{plot.eff}} and the vignette \href{https://CRAN.R-project.org/package=effects/vignettes/partial-residuals.pdf}{Effect Displays with Partial Residuals}. This argument may also be used for mixed-effects and some other models.}
  
  \item{quantiles}{quantiles at which to evaluate numeric focal predictors \emph{not} on the horizontal axis, used only when partial residuals are displayed; superseded if the \code{xlevels} argument gives specific values for a predictor.}
    
  \item{x.var}{the (quoted) name or index of the numeric predictor to define the horizontal axis of an effect plot for a linear or generalized linear model; the default is \code{NULL}, in which case the first numeric predictor in the effect will be used \emph{if} partial residuals are to be computed. This argument is intended to be used when \code{residuals} is \code{TRUE}; otherwise, the variable on the horizontal axis can be chosen when the effect object is plotted: see \code{\link{plot.eff}}.}
  
  \item{transformation}{for the \code{Effect.lm} method, an optional two-element list with \code{link} and \code{inverse} elements to transform the response (see examples); an alternative to use for graphs is to set the argument \code{axes = list(y = list(transformation = list(link = link-function, inverse = mean-function)))} (see \code{\link{plot.eff})}; the argument must be used for transforming the response in printed or summary output.}
  
  \item{latent}{if \code{TRUE}, effects in a proportional-odds logit model are computed on the scale of the latent response; if \code{FALSE}	(the default) effects are computed as individual-level probabilities and logits.}
  
  \item{x}{an object of class \code{"eff"}, \code{"effpoly"}, or \code{"efflatent"}.}
  
  \item{KR}{if \code{TRUE} and the \pkg{pbkrtest} package is installed, use the Kenward-Roger coefficient covariance matrix to compute effect standard errors for linear mixed models fit with \code{\link[lme4]{lmer}}; the default is \code{FALSE} because the computation can be time-consuming.}
  
  \item{response}{for an \code{"mlm"} object, a vector containing the (quoted) name(s) or indices of one or more response variable(s). The default is to use all responses in the model.}
    
  \item{...}{arguments to be passed down.}
  
  \item{confint, confidence.level, given.values, typical, offset, partial.residuals}{legacy arguments retained for backwards compatibility; if present, these arguments take precedence over the \code{level} element of the \code{confint} list argument and the \code{given.values}, \code{typical}, and \code{offset} elements of the \code{fixed.predictors} list argument; \code{confint} may be used in place of the \code{se} argument; \code{partial.residuals} may be used in place of the \code{residuals} argument. See \code{\link{LegacyArguments}} for details.}
}

\details{
  Normally, the functions to be used directly are \code{allEffects}, to return a list of high-order effects, and the generic \code{plot} function to plot the effects (see \code{\link{plot.efflist}}, \code{\link{plot.eff}}, and \code{\link{plot.effpoly}}). Alternatively, \code{Effect} can be used to vary a subset of predictors over their ranges, while other predictors are held to typical values.
  
Plotting methods for effect objects call the \code{\link[lattice]{xyplot}} (or in some cases, the \code{\link[lattice]{densityplot}}) function in the \pkg{lattice} package. Effects may also be printed (implicitly or explicitly via \code{print}) or summarized (using \code{summary}) (see \code{\link{print.efflist}}, \code{\link{summary.efflist}}, \code{\link{print.eff}}, \code{\link{summary.eff}}, \code{\link{print.effpoly}}, and \code{\link{summary.effpoly}}).

  If asked, the \code{effect} function will compute effects for terms that have higher-order relatives in the model, averaging over those terms (which rarely makes sense), or for terms that do not appear in the model but are higher-order relatives of terms that do. For example, for the model \code{Y ~ A*B + A*C + B*C}, one could compute the effect corresponding to the absent term \code{A:B:C}, which absorbs the constant, the \code{A}, \code{B}, and \code{C} main effects, and the three two-way interactions. In either of these cases, a warning is printed.

  See \code{\link{predictorEffects}} for an alternative paradigm for defining effects.
}

\value{

  For \code{"lm"}, \code{"glm"}, \code{"svyglm"}, \code{"lmerMod"}, \code{"glmerMod"}, and \code{"lme"}, model objects, \code{effect} and \code{Effect} return  an \code{"eff"} object, and for \code{"multinom"}, \code{"polr"}, \code{"clm"}, \code{"clmm"}, and \code{"clm2"} models, an \code{"effpoly"} object, with the components listed below. For an \code{"mlm"} object with one response specified, an \code{"eff"} object is returned, otherwise an \code{"efflist"} object is returned, containing one \code{"eff"} object for each \code{response}.
  
  \item{term}{the term to which the effect pertains.}
  
  \item{formula}{the complete model formula.}
  
  \item{response}{a character string giving the name of the response variable.}
  
  \item{y.levels}{(for \code{"effpoly"} objects) levels of the polytomous response variable.}
  
  \item{variables}{a list with information about each predictor, including its name, whether it is a factor, and its levels or values.}
  
  \item{fit}{(for \code{"eff"} objects) a one-column matrix of fitted values, representing the effect on the scale of the linear predictor; this is a raveled table, representing all combinations of predictor values.}
  
  \item{prob}{(for \code{"effpoly"} objects) a matrix giving fitted probabilities for the effect for the various levels of the the response (columns) and combinations of the focal predictors (rows).}
  
  \item{logit}{(for \code{"effpoly"} objects) a matrix giving fitted logits for the effect for the various levels of the the response (columns) and combinations of the focal predictors (rows).}
  
  \item{x}{a data frame, the columns of which are the predictors in the effect, and the rows of which give all combinations of values of these predictors.}
  
  \item{model.matrix}{the model matrix from which the effect was calculated.}
  
  \item{data}{a data frame with the data on which the fitted model was based.}
  
  \item{discrepancy}{the percentage discrepancy for the `safe' predictions of the original fit; should be very close to 0.  Note: except for \code{gls} models, this is now necessarily 0.}
  
  \item{offset}{value to which the offset is fixed; \code{0} if there is no offset.}
  
  \item{model}{(for \code{"effpoly"} objects) \code{"multinom"} or \code{"polr"}, as appropriate.}
  
  \item{vcov}{(for \code{"eff"} objects) a covariance matrix for the effect, on the scale of the linear predictor.}
  
  \item{se}{(for \code{"eff"} objects) a vector of standard errors for the effect, on the scale of the linear predictor.}
  
  \item{se.prob, se.logit}{(for \code{"effpoly"} objects) matrices of standard errors for the effect, on the probability and logit scales.}
  
  \item{lower, upper}{(for \code{"eff"} objects) one-column matrices of confidence limits, on the scale of the linear predictor.}
  
  \item{lower.prob, upper.prob, lower.logit, upper.logit}{(for \code{"effpoly"} objects) matrices of confidence limits for the fitted logits and probabilities; the latter are computed by transforming the former.}
  
  \item{confidence.level}{for the confidence limits.}
  
  \item{transformation}{(for \code{"eff"} objects) a two-element list, with element \code{link} giving the link function, and element \code{inverse} giving the inverse-link (mean) function; may be set directly via the \code{transformation} argument to \code{Effect.lm} or inferred from the model.}
  
  \item{residuals}{(working) residuals for linear or generalized linear models (and some similar models), to be used by \code{\link{plot.eff}} to compute and plot partial residuals.}
  
  \item{x.var}{the name of the predictor to appear on the horizontal axis of an effect plot made from the  returned object; will usually be \code{NULL} if partial residuals aren't computed.}
  
  \item{family}{for a \code{"glm"} model, the name of the distributional family of the model; for an \code{"lm"} model, this is \code{"gaussian"}; otherwise \code{NULL}. The \code{family} controls how partial residuals are smoothed in plots.}
  
  \item{link}{the value returned by \code{family(mod)}.  Down-stream methods may need the link, inverse link and derivative functions.}

  \code{allEffects} returns an \code{"efflist"} object, a list of \code{"eff"} or \code{"effpoly"} objects corresponding to the high-order terms of the model.

If \code{mod} is of class \code{"poLCA"} (from the \pkg{poLCA} package), representing a polytomous latent class model, effects are computed for the predictors given the estimated latent classes. The result is of class \code{"eff"} if the latent class model has 2 categories and of class \code{"effpoly"} with more than 2 categories.
}

\section{Warnings and Limitations}{

  The \code{Effect} function handles factors and covariates differently, and is likely to be confused if one is changed to the other in a model formula. Consequently, formulas that include calls to \code{as.factor}, \code{factor}, or \code{numeric} (as, e.g., in \code{y ~ as.factor(income)}) will cause errors. Instead, create the modified variables outside of the model formula (e.g., \code{fincome <- as.factor(income)}) and use these in the model formula.

The \code{effect} function doesn't work with factors that have colons in level names (e.g., \code{"level:A"}); the \code{effect} function will confuse the colons with interactions; rename levels to remove or replace the colons (e.g., \code{"level.A"}). Level names with colons are perfectly fine for use with \code{Effect}.

The functions in the \pkg{effects} package work properly with predictors that are numeric variables, factors, character variables, or logical variables; consequently, e.g., convert dates to numeric. Character predictors and logical predictors are treated as factors, the latter with "levels" \code{"FALSE"} and \code{"TRUE"}.

  Empty cells in crossed-factors are now permitted for \code{"lm"}, \code{"glm"}, and \code{"multinom"} models. For \code{"multinom"} models with two or more crossed factors with an empty cell, stacked area plots apparently do not work because of a bug in the \code{\link[lattice]{barchart}} function in the \pkg{lattice} package.  However, the default line plots do work.

  Offsets in linear and generalized linear models are supported, as are offsets in mixed models fit by \code{lmer} or \code{glmer}, but must be supplied through the \code{offset} argument to \code{lm}, \code{glm}, \code{lmer} or \code{glmer}; offsets supplied via calls to the \code{offset} function on the right-hand side of the model formula are not supported.

  Fitting ordinal mixed models using \code{\link[ordinal]{clmm}} or \code{\link[ordinal:clmmOld]{clmm2}} permits many options, including a variety of link functions, scale functions, nominal regressors, and various methods for setting thresholds.  Effects are currently generated only for the default values of the arguments \code{scale}, \code{nominal}, \code{link}, and \code{threshold}, which is equivalent to fitting an ordinal-response mixed-effects model with a logit link. \code{Effect} can also be used with objects created by \code{\link[ordinal]{clm}} or \code{\link[ordinal:clmOld]{clm2}}, fitting ordinal response models with the same links permitted by \code{\link[MASS]{polr}} in the \pkg{MASS} package, with no random effects, and with results similar to those from \code{\link[MASS]{polr}}.

  Calling any of these functions from within a user-written function may result in errors due to R's scoping rules.  See the vignette \code{embedding.pdf} in the \pkg{car} package for a solution to this problem.
  }

\references{
    Fox, J. (1987).
    Effect displays for generalized linear models.
    \emph{Sociological Methodology}
    \bold{17}, 347--361.

    Fox, J. (2003)
    Effect displays in R for generalised linear models.
    \emph{Journal of Statistical Software}
    \bold{8:15}, 1--27, \doi{10.18637/jss.v008.i15}.

  	Fox, J. and R. Andersen (2006).
   	Effect displays for multinomial and proportional-odds logit models.
  	\emph{Sociological Methodology}
  	\bold{36}, 225--255.

  	Fox, J. and J. Hong (2009).
    Effect displays in R for multinomial and proportional-odds logit models:?
    	    Extensions to the effects package.
    \emph{Journal of Statistical Software}
    \bold{32:1}, 1--24, \doi{10.18637/jss.v032.i01}.
    
    Fox, J. and S. Weisberg (2019).  \emph{An R Companion to Applied Regression, third edition},
    Thousand Oaks: Sage.

    Fox, J. and S. Weisberg (2018).
    Visualizing Fit and Lack of Fit in Complex Regression Models
        with Predictor Effect Plots with Partial Residuals.
    \emph{Journal of Statistical Software}
    \bold{87:9}, 1--27, \doi{10.18637/jss.v087.i09}.

    Hastie, T. J. (1992).
    Generalized additive models.
    In Chambers, J. M., and Hastie, T. J. (eds.)
    \emph{Statistical Models in S}, Wadsworth.

    Weisberg, S. (2014).
    \emph{Applied Linear Regression}, 4th edition, Wiley,
    \url{http://z.umn.edu/alr4ed}.
    }



\author{John Fox \email{jfox@mcmaster.ca}, Sanford Weisberg \email{sandy@umn.edu}
and Jangman Hong.}


\seealso{\code{\link{LegacyArguments}}. For information on printing, summarizing, and plotting effects:
    \code{\link{print.eff}}, \code{\link{summary.eff}}, \code{\link{plot.eff}},
    \code{\link{print.summary.eff}},
    \code{\link{print.effpoly}}, \code{\link{summary.effpoly}}, \code{\link{plot.effpoly}},
    \code{\link{print.efflist}}, \code{\link{summary.efflist}},
    \code{\link{plot.efflist}}, \code{\link[lattice]{xyplot}},
    \code{\link[lattice]{densityplot}}, and the \href{https://CRAN.R-project.org/package=effects/vignettes/predictor-effects-gallery.pdf}{Predictor Effects Graphics Gallery} and \href{https://CRAN.R-project.org/package=effects/vignettes/functions-supported-by-effects.pdf}{Regression Models Supported by the effects Package} vignettes.}

\examples{
mod.cowles <- glm(volunteer ~ sex + neuroticism*extraversion,
                  data=Cowles, family=binomial)
eff.cowles <- allEffects(mod.cowles, xlevels=list(extraversion=seq(0, 24, 6)),
                   fixed.predictors=list(given.values=c(sexmale=0.5)))
eff.cowles
as.data.frame(eff.cowles[[2]])

\donttest{
# the following are equivalent:
eff.ne <- effect("neuroticism*extraversion", mod.cowles)
Eff.ne <- Effect(c("neuroticism", "extraversion"), mod.cowles)
all.equal(eff.ne$fit, Eff.ne$fit)

plot(eff.cowles, 'sex', axes=list(y=list(lab="Prob(Volunteer)")))

plot(eff.cowles, 'neuroticism:extraversion',
     axes=list(y=list(lab="Prob(Volunteer)",
        ticks=list(at=c(.1,.25,.5,.75,.9)))))

plot(Effect(c("neuroticism", "extraversion"), mod.cowles,
            se=list(type="Scheffe"),
            xlevels=list(extraversion=seq(0, 24, 6)),
            fixed.predictors=list(given.values=c(sexmale=0.5))),
     axes=list(y=list(lab="Prob(Volunteer)",
        ticks=list(at=c(.1,.25,.5,.75,.9)))))

plot(eff.cowles, 'neuroticism:extraversion', lines=list(multiline=TRUE),
     axes=list(y=list(lab="Prob(Volunteer)")))

plot(effect('sex:neuroticism:extraversion', mod.cowles,
            xlevels=list(extraversion=seq(0, 24, 6))),
     lines=list(multiline=TRUE))
}

# a nested model:

mod <- lm(log(prestige) ~ income:type + education, data=Prestige)

plot(Effect(c("income", "type"), mod, transformation=list(link=log, inverse=exp)),
     axes=list(y=list(lab="prestige")))


if (require(nnet)){
    mod.beps <- multinom(vote ~ age + gender + economic.cond.national +
                             economic.cond.household + Blair + Hague + Kennedy +
                             Europe*political.knowledge, data=BEPS)
    \donttest{
    plot(effect("Europe*political.knowledge", mod.beps,
                xlevels=list(political.knowledge=0:3)))
    }

    plot(Effect(c("Europe", "political.knowledge"), mod.beps,
                xlevels=list(Europe=1:11, political.knowledge=0:3),
                fixed.predictors=list(given.values=c(gendermale=0.5))),
         lines=list(col=c("blue", "red", "orange")),
         axes=list(x=list(rug=FALSE), y=list(style="stacked")))

    \donttest{
    plot(effect("Europe*political.knowledge", mod.beps, # equivalent
                xlevels=list(Europe=1:11, political.knowledge=0:3),
                fixed.predictors=list(given.values=c(gendermale=0.5))),
         lines=list(col=c("blue", "red", "orange")),
         axes=list(x=list(rug=FALSE), y=list(style="stacked")))
    }
}

if (require(MASS)){
    mod.wvs <- polr(poverty ~ gender + religion + degree + country*poly(age,3),
                    data=WVS)
    \donttest{
    plot(effect("country*poly(age, 3)", mod.wvs))
    }

    plot(Effect(c("country", "age"), mod.wvs),
         axes=list(y=list(style="stacked")))

    \donttest{
    plot(effect("country*poly(age, 3)", mod.wvs),
         axes=list(y=list(style="stacked"))) # equivalent

    plot(effect("country*poly(age, 3)", latent=TRUE, mod.wvs))
    plot(effect("country*poly(age, 3)", latent=TRUE, mod.wvs,
         se=list(type="scheffe"))) # Scheffe-type confidence envelopes
    }
}


mod.pres <- lm(prestige ~ log(income, 10) + poly(education, 3) + poly(women, 2),
               data=Prestige)
eff.pres <- allEffects(mod.pres, xlevels=50)
plot(eff.pres)
plot(eff.pres[1],
     axes=list(x=list(income=list(
             transform=list(trans=log10, inverse=function(x) 10^x),
             ticks=list(at=c(1000, 2000, 5000, 10000, 20000))
    ))))
\donttest{
# linear model with log-response and log-predictor
# to illustrate transforming axes and setting tick labels
mod.pres1 <- lm(log(prestige) ~ log(income) + poly(education, 3) + poly(women, 2),
                data=Prestige)
# effect of the log-predictor
eff.log <- Effect("income", mod.pres1)
# effect of the log-predictor transformed to the arithmetic scale
eff.trans <- Effect("income", mod.pres1, transformation=list(link=log, inverse=exp))
#variations:
# y-axis:  scale is log, tick labels are log
# x-axis:  scale is arithmetic, tick labels are arithmetic
plot(eff.log)

# y-axis:  scale is log, tick labels are log
# x-axis:  scale is log, tick labels are arithmetic
plot(eff.log, axes=list(x=list(income=list(
    transform=list(trans=log, inverse=exp),
    ticks=list(at=c(5000, 10000, 20000)),
    lab="income, log-scale"))))

# y-axis:  scale is log, tick labels are arithmetic
# x-axis:  scale is arithmetic, tick labels are arithmetic
plot(eff.trans, axes=list(y=list(lab="prestige")))

# y-axis:  scale is arithmetic, tick labels are arithmetic
# x-axis:  scale is arithmetic, tick labels are arithmetic
plot(eff.trans, axes=list(y=list(type="response", lab="prestige")))

# y-axis:  scale is log, tick labels are arithmetic
# x-axis:  scale is log, tick labels are arithmetic
plot(eff.trans, axes=list(
       x=list(income=list(
            transform=list(trans=log, inverse=exp),
            ticks=list(at=c(1000, 2000, 5000, 10000, 20000)),
            lab="income, log-scale")),
        y=list(lab="prestige, log-scale")),
     main="Both response and X in log-scale")

# y-axis:  scale is arithmetic, tick labels are arithmetic
# x-axis:  scale is log, tick labels are arithmetic
plot(eff.trans, axes=list(
        x=list(
            income=list(transform=list(trans=log, inverse=exp),
                        ticks=list(at=c(1000, 2000, 5000, 10000, 20000)),
                        lab="income, log-scale")),
        y=list(type="response", lab="prestige")))
}

if (require(nlme)){ # for gls()
    mod.hart <- gls(fconvict ~ mconvict + tfr + partic + degrees, data=Hartnagel,
                    correlation=corARMA(p=2, q=0), method="ML")
    plot(allEffects(mod.hart))
    detach(package:nlme)
}

if (require(lme4)){
    data(cake, package="lme4")
    fm1 <- lmer(angle ~ recipe * temperature + (1|recipe:replicate), cake,
                REML = FALSE)
    plot(Effect(c("recipe", "temperature"), fm1))
    \donttest{
    plot(effect("recipe:temperature", fm1),
         axes=list(grid=TRUE)) # equivalent (plus grid)
    }
    if (any(grepl("pbkrtest", search()))) detach(package:pbkrtest)
    detach(package:lme4)
}

\donttest{
if (require(nlme) && length(find.package("lme4", quiet=TRUE)) > 0){
    data(cake, package="lme4")
    cake$rep <- with(cake, paste( as.character(recipe), as.character(replicate), sep=""))
    fm2 <- lme(angle ~ recipe * temperature, data=cake,
               random = ~ 1 | rep, method="ML")
    plot(Effect(c("recipe", "temperature"), fm2))
    plot(effect("recipe:temperature", fm2),
         axes=list(grid=TRUE))  # equivalent (plus grid)
    }
    detach(package:nlme)
}

\donttest{
if (require(poLCA)){
    data(election)
    f2a <- cbind(MORALG,CARESG,KNOWG,LEADG,DISHONG,INTELG,
                 MORALB,CARESB,KNOWB,LEADB,DISHONB,INTELB)~PARTY*AGE
    nes2a <- poLCA(f2a,election,nclass=3,nrep=5)
    plot(Effect(c("PARTY", "AGE"), nes2a),
         axes=list(y=list(style="stacked")))
}
}

# mlm example
if (require(heplots)) {
    data(NLSY, package="heplots")
    mod <- lm(cbind(read,math) ~ income+educ, data=NLSY)
    eff.inc <- Effect("income", mod)
    plot(eff.inc)
    eff.edu <- Effect("educ", mod)
    plot(eff.edu, axes=list(x=list(rug=FALSE), grid=TRUE))
    \donttest{
    plot(Effect("educ", mod, response="read"))
    }
    detach(package:heplots)
}

# svyglm() example (adapting an example from the survey package)

\donttest{
if (require(survey)){
  data("api")
  dstrat<-svydesign(id=~1, strata=~stype, weights=~pw,
    data=apistrat, fpc=~fpc)
  mod <- svyglm(sch.wide ~ ell + meals + mobility, design=dstrat,
    family=quasibinomial())
  plot(allEffects(mod),
    axes=list(y=list(lim=log(c(0.4, 0.99)/c(0.6, 0.01)),
      ticks=list(at=c(0.4, 0.75, 0.9, 0.95, 0.99)))))
}
}

# component + residual plot examples

\donttest{
Prestige$type <- factor(Prestige$type, levels=c("bc", "wc", "prof"))

mod.prestige.1 <- lm(prestige ~ income + education, data=Prestige)
plot(allEffects(mod.prestige.1, residuals=TRUE)) # standard C+R plots
plot(allEffects(mod.prestige.1, residuals=TRUE,
    se=list(type="scheffe"))) # with Scheffe-type confidence bands

mod.prestige.2 <- lm(prestige ~ type*(income + education), data=Prestige)
plot(allEffects(mod.prestige.2, residuals=TRUE))

mod.prestige.3 <- lm(prestige ~ type + income*education, data=Prestige)
plot(Effect(c("income", "education"), mod.prestige.3, residuals=TRUE),
     partial.residuals=list(span=1))
}

#  artificial data

set.seed(12345)
x1 <- runif(500, -75, 100)
x2 <- runif(500, -75, 100)
y <- 10 + 5*x1 + 5*x2 + x1^2 + x2^2 + x1*x2 + rnorm(500, 0, 1e3)
Data <- data.frame(y, x1, x2)
mod.1 <- lm(y ~ poly(x1, x2, degree=2, raw=TRUE), data=Data)
# raw=TRUE necessary for safe prediction
mod.2 <- lm(y ~ x1*x2, data=Data)
mod.3 <- lm(y ~ x1 + x2, data=Data)

plot(Effect(c("x1", "x2"), mod.1, residuals=TRUE)) # correct model
plot(Effect(c("x1", "x2"), mod.2, residuals=TRUE)) # wrong model
plot(Effect(c("x1", "x2"), mod.3, residuals=TRUE)) # wrong model
}

\keyword{hplot}
\keyword{models}

\name{Hartnagel}
\alias{Hartnagel}
\docType{data}

\title{Canadian Crime-Rates Time Series}

\description{
The \code{Hartnagel} data frame has 38 rows and 7 columns.
The data are an annual time-series from 1931 to 1968. There are
some missing data.
}

\format{
  This data frame contains the following columns:
  \describe{
    \item{year}{
      1931--1968.
      }
    \item{tfr}{
      Total fertility rate per 1000 women.
      }
    \item{partic}{
      Women's labor-force participation rate per 1000.
      }
    \item{degrees}{
      Women's post-secondary degree rate per 10,000.
      }
    \item{fconvict}{
      Female indictable-offense conviction rate per 100,000.
      }
    \item{ftheft}{
      Female theft conviction rate per 100,000.
      }
    \item{mconvict}{
      Male indictable-offense conviction rate per 100,000.
      }
    \item{mtheft}{
      Male theft conviction rate per 100,000.
      }
    }
  }

\details{
 The post-1948 crime rates have been adjusted to account for
 a difference in method of recording. Some of your results will differ
 in the last decimal place from those in Table 14.1 of Fox (1997) due
 to rounding of the data. Missing values for 1950 were interpolated.
}


\source{
 Personal communication from T. Hartnagel, 
 Department of Sociology, University of Alberta.
}

\references{
  Fox, J., and Hartnagel, T. F (1979)
  Changing social roles and female crime in Canada:
  A time series analysis.
  \emph{Canadian Review of Sociology and Anthroplogy}, 
  \bold{16}, 96--104.
  
  Fox, J. (2008)
  \emph{Applied Regression Analysis and Generalized Linear Models},
  Second Edition. Sage.  
}

\usage{
Hartnagel
}

\keyword{datasets}

\name{EffectMethods}
\alias{effSources}
\alias{effSources.default}
\alias{effSources.gls}
\alias{effSources.glmmPQL}
\alias{effSources.rlmerMod}
\alias{effSources.clm}
\alias{effSources.clm2}
\alias{effSources.clmm}
\alias{effSources.betareg}


\title{Functions For Constructing Effect Displays for Many Modeling Paradigms}

\description{The \pkg{effects} package can be used with many different classes of regression models beyond the basic models described in the \code{\link{Effect}} help page.  The \code{effSources} function is used to tell an \code{Effect} method where to find the information to compute effects.  Details are given in the vignette \href{https://CRAN.R-project.org/package=effects/vignettes/functions-supported-by-effects.pdf}{Regression Models Supported by the effects Package}.  Unless you are trying to extend \pkg{effects} to a different class of regression model, you will have no use for this function.  
}

\usage{

\method{effSources}{default}(mod)

\method{effSources}{gls}(mod)

\method{effSources}{glmmPQL}(mod)

\method{effSources}{rlmerMod}(mod)

\method{effSources}{clm}(mod)

\method{effSources}{clm2}(mod)

\method{effSources}{clmm}(mod)

\method{effSources}{betareg}(mod)

}

\arguments{
  \item{mod}{a fitted model object of the appropriate class. The default method works for many regression models for which there is no specific method.}
  }

\details{
\code{Effect} function methods by default expect regression objects to provide certain quantities, including a model formula, coefficient estimates, and the variance matrix of the estimates, in standard places. \code{effSources} methods return a list of the necessary quantities if they are not in standard places.}

\value{See the vignette \href{https://CRAN.R-project.org/package=effects/vignettes/functions-supported-by-effects.pdf}{Regression Models Supported by the effects Package}.}




\author{John Fox \email{jfox@mcmaster.ca}, Sanford Weisberg \email{sandy@umn.edu}}


%\examples{
%}

\keyword{internal}

\name{effects-package}
\Rdversion{1.1}
\alias{effects-package}
\alias{effects}
\docType{package}
\title{
Effect Displays for Linear, Generalized Linear, and Other Models
}
\description{
Graphical and tabular effect displays, e.g., of interactions, for 
various statistical models with linear predictors.
}
\details{
\tabular{ll}{
Package: \tab effects\cr
Version: \tab 4.2-2\cr
Date: \tab 2022-02-16\cr
Depends: \tab R (>= 3.5.0), carData\cr
Suggests: \tab pbkrtest (>= 0.4-4), nlme, MASS, poLCA, heplots, splines, ordinal, car, knitr, betareg, alr4, robustlmm\cr
Imports: \tab lme4, nnet, lattice, grid, colorspace, graphics, grDevices, stats, survey, utils, estimability, insight\cr
LazyLoad: \tab yes\cr
License: \tab GPL (>= 2)\cr
URL: \tab https://www.r-project.org, https://socialsciences.mcmaster.ca/jfox/\cr
}

This package creates effect displays for various kinds of models, as partly explained in the references.
Typical usage is \code{plot(allEffects(model))} or \code{plot(predictorEffects(model))},
where \code{model} is an appropriate fitted-model object.
Additional arguments to \code{\link{allEffects}}, \code{\link{predictorEffects}} and \code{\link[graphics]{plot}} can be used to customize the resulting
displays. The function \code{\link{effect}} can be employed to produce an effect display for a 
particular term in the model, or to which terms in the model are marginal. The function \code{\link{predictorEffect}} can be used
to construct an effect display for a particularly predictor. The function \code{\link{Effect}}
may similarly be used to produce an effect display for any combination of predictors. In any of the cases, use \code{plot}
to graph the resulting effect object. For linear and generalized linear models it is also possible to plot 
partial residuals to obtain (multidimensional) component+residual plots.
See \code{?effect}, \code{?Effect}, \code{?predictorEffect}, and \code{?plot.eff} for details.
}

\author{
John Fox, Sanford Weisberg, Brad Price, Michael Friendly, Jangman Hong, 
Robert Anderson, David Firth, Steve Taylor, and the R Core Team. 

Maintainer: John Fox <jfox@mcmaster.ca>
}

\references{
   Fox, J. and S. Weisberg (2019)
   \emph{An R Companion to Applied Regression, Third Edition}
   Sage Publications.
   
   Fox, J. (1987)
    Effect displays for generalized linear models.
    \emph{Sociological Methodology}
    \bold{17}, 347--361.
    
    Fox, J. (2003)
    Effect displays in R for generalised linear models.
    \emph{Journal of Statistical Software}
    \bold{8:15}, 1--27, \doi{10.18637/jss.v008.i15}.
    
    Fox, J. and R. Andersen (2006)
    Effect displays  for multinomial and proportional-odds logit models.
    \emph{Sociological Methodology} 
    \bold{36}, 225--255.
    
    Fox, J. and J. Hong (2009).
    Effect displays in R for multinomial and proportional-odds logit models: 
        Extensions to the effects package.
    \emph{Journal of Statistical Software}
    \bold{32:1}, 1--24, \doi{10.18637/jss.v032.i01}.
    
    Fox, J. and S. Weisberg (2018).
    Visualizing Fit and Lack of Fit in Complex Regression Models:
        Effect Plots with Partial Residuals.
    \emph{Journal of Statistical Software}
    \bold{87:9}, 1--27, \doi{10.18637/jss.v087.i09}.
}


\keyword{ package }

\name{predictorEffects}
\alias{predictorEffect}
\alias{predictorEffect.poLCA}
\alias{predictorEffect.svyglm}
\alias{predictorEffect.default}
\alias{predictorEffects}
\alias{predictorEffects.poLCA}
\alias{predictorEffects.default}

\title{
Functions For Computing Predictor Effects
}

\description{
Alternatives to the \code{Effect} and \code{allEffects} functions that use a different paradigm for conditioning in an effect display.  The user specifies one predictor, either numeric or a factor (where character and logical variables are treated as factors), for the horizontal axis of a plot, and the function determines the appropriate plot to display (which is drawn by \code{plot}). See the vignette \href{https://CRAN.R-project.org/package=effects/vignettes/predictor-effects-gallery.pdf}{Predictor Effects Graphics Gallery} for details and examples.
}

\usage{
predictorEffect(predictor, mod, focal.levels=50, xlevels=5, ...)

\method{predictorEffect}{poLCA}(predictor, mod, focal.levels=50, 
    xlevels=5, ...)

\method{predictorEffect}{svyglm}(predictor, mod, focal.levels=50, 
    xlevels=5, ...)

\method{predictorEffect}{default}(predictor, mod, focal.levels=50, 
    xlevels=5, ..., sources)

predictorEffects(mod, predictors, focal.levels=50, xlevels=5, ...)

\method{predictorEffects}{poLCA}(mod, predictors = ~ ., 
    focal.levels=50, xlevels=5, ...)

\method{predictorEffects}{default}(mod, predictors = ~ .,
    focal.levels=50, xlevels=5,  ..., sources)

}

\arguments{

  \item{mod}{A model object.  Supported models include all those described on the help page for \code{\link{Effect}}.}
  
  \item{predictor}{quoted name of the focal predictor.}
  
  \item{predictors}{If the default, \code{~ .}, a predictor effect plot is drawn for each predictor (not regressor) in a model.  Otherwise, this is a one-sided formula specifying the first-order predictors for which predictor effect plots are to be drawn.
}

  \item{focal.levels}{for \code{predictorEffect}, the number of evenly-spaced values (the default is 50) for the numeric focal predictor or a vector of values for the focal predictor. 
  
  For \code{predictorEffects}, the number of evenly-spaced values (default 50) to use for each numeric focal predictor in turn, or a named list, similar to \code{xlevels}, giving the number of values or the values themselves for each predictor individually, to be used when that predictor is the focal predictor; if a numeric focal predictor doesn't appear in the list, the default of 50 values is used.}
  
  \item{xlevels}{this argument is used to set the levels of conditioning predictors; it may either be a single number specifying the number of evenly-spaced values (the default is 5) to which each conditioning predictor is to be set,  or it may be a list with elements named for the predictors giving the number of values or a vector of values to which each conditioning predictor is to be set, as explained in the help for \code{\link{Effect}}. 
  
  If the focal predictor is included in the \code{xlevels} list, it is disregarded; if any conditioning predictor is omitted from the list, its number of values is set to 5. 
  
  The default behavior of \code{xlevels} is different when \code{residuals=TRUE}; in that case, it behaves as in \code{\link{Effect.lm}}, and is effectively set by default to the 0.2, 0.4, 0.6, and 0.8 quantiles of conditioning predictors.
  
  The \code{xlevels} argument works similarly for \code{predictorEffect} and \code{predictorEffects}.}
  
  \item{\dots}{
Additional arguments passed to \code{\link{Effect}}.}

  \item{sources}{Provides a mechanism for applying \code{predictorEffect} methods to a variety of regression models; see the vignette \href{https://CRAN.R-project.org/package=effects/vignettes/functions-supported-by-effects.pdf}{Regression Models Supported by the effects Package} for an explanation.}
}

\details{
Effect plots view a fitted regression function E(Y|X) in (sequences of) two-dimensional plots using conditioning and slicing.  The functions described here use a different method of determining the conditioning and slicing than \code{allEffects} uses.  The predictor effect of a focal predictor, say \code{x1}, is the  usual effect for the generalized interaction of \code{x1} with all the other predictors in a model.  When a predictor effect object is plotted, the focal predictor is by default plotted on the horizontal axis.

For example, in the model \code{mod} with formula \code{y ~ x1 + x2 + x3}, the predictor effect \code{p1 <- predictorEffects(mod, ~ x1)} is essentially equilavent to \code{p2 <- Effect("x1", mod)}.  When plotted, these objects may produce different graphs because \code{plot(p1)} will always put \code{x1} on the horizontal axis, while \code{plot(p2)} uses a rule to determine the horizontal axis based on the characteristics of all the predictors, e.g., preferring numeric predictors over factors.

If \code{mod} has the formula \code{y ~ x1 + x2 + x3 + x1:x2}, then \code{p1 <- predictorEffects(mod, ~ x1)} is essentially equivalent to \code{p2 <- Effect(c("x1", "x2"), mod)}.  As in the last example, the plotted versions of these objects may differ because of different rules used to determine the predictor on the horizontal axis.

If \code{mod} has the formula \code{y ~ x1 + x2 + x3 + x1:x2 + x1:x3}, then \code{p1 <- predictorEffects(mod, ~ x1)} is essentially equilavent to \code{p2 <- Effect(c("x1", "x2", "x3"), mod)}.  Again, the plotted versions of these objects may differ because of the rules used to determine the horizontal axis.
}

\value{
\code{predictorEffect} returns an object of class \code{c("predictoreff", "eff")}.  The components of the object are described in the help for \code{\link{Effect}}; \code{predictorEffects} returns an object of class \code{"predictorefflist"}, which is a list whose elements are of class \code{c("predictoreff", "eff")}.
}

\references{
See \code{\link{Effect}}.
}

\author{
S. Weisberg \email{sandy@umn.edu} and J. Fox
}

\seealso{
\code{\link{Effect}}, \code{\link{plot.predictoreff}}, the \href{https://CRAN.R-project.org/package=effects/vignettes/predictor-effects-gallery.pdf}{Predictor Effects Graphics Gallery} vignette, and the \href{https://CRAN.R-project.org/package=effects/vignettes/partial-residuals.pdf}{Effect Displays with Partial Residuals} vignette.
}

\examples{
mod <- lm(prestige ~ type*(education + income) + women, Prestige)
plot(predictorEffect("income", mod))
plot(predictorEffects(mod, ~ education + income + women))

mod.cowles <- glm(volunteer ~ sex + neuroticism*extraversion, data=Cowles, family=binomial)
plot(predictorEffects(mod.cowles, xlevels=4))
plot(predictorEffect("neuroticism", mod.cowles, xlevels=list(extraversion=seq(5, 20, by=5))),
     axes=list(grid=TRUE,
               x=list(rug=FALSE),
               y=list(lab="Probability of Vounteering")),
     lines=list(multiline=TRUE), 
     type="response")
predictorEffects(mod.cowles, focal.levels=4, xlevels=4)

# svyglm() example (adapting an example from the survey package)
\donttest{
if (require(survey)){
  data(api)
  dstrat<-svydesign(id=~1, strata=~stype, weights=~pw,
    data=apistrat, fpc=~fpc)
  mod <- svyglm(sch.wide ~ ell + meals + mobility, design=dstrat,
    family=quasibinomial())
  plot(predictorEffects(mod),
    axes=list(y=list(lim=log(c(0.4, 0.99)/c(0.6, 0.01)),
      ticks=list(at=c(0.4, 0.75, 0.9, 0.95, 0.99)))))
}
}
}

\keyword{hplot}
\keyword{models}

\name{effCoef}
\alias{effCoef}
\alias{effCoef.default}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Function to get coefficient estimates from regression models for use in the effects package.
}
\description{
This function uses the \code{\link[insight]{get_parameters}} function in the \code{insight} package to get a vector of regression coefficients for use in the effects package.  It converts the two-column \code{data.frame} returned by \code{get_parameters} to a vector of named elements.
}
\usage{

effCoef(mod, ...)

\method{effCoef}{default}(mod, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mod}{
A model object with a linear predictor representing fixed effects.  
}
\item{...}{Additional parameter passed to \code{\link[insight]{get_parameters}}.}
}
\details{
The \code{\link[insight]{get_parameters}} function can be used to retrieve the coefficient estimates corresponding to a linear predictor for many regression models, and return them as a two column \code{data.frame}, with regressor names in the first column and estimates in the second column.  This function converts this output to a named vector as is expected by the \code{effects} package.
}
\value{
A vector of coefficient estimates
%\references{
%% ~put references to the literature/web site here ~
}
\author{Sanford Weisberg \email{sandy@umn.edu}}

\seealso{\code{\link[insight]{get_parameters}}, and vignette \href{https://CRAN.R-project.org/package=effects/vignettes/functions-supported-by-effects.pdf}{Regression Models Supported by the effects Package}.}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\examples{
m1 <- lm(prestige ~ type + income + education, Duncan)
effCoef(m1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory (show via RShowDoc("KEYWORDS")):
\keyword{models}
